clear all;  close all;  clc;

% This program will loop through the transient boundary conditions files to
% perform ensemble averaging and calculate relevant statistics, then output
% the results into a set of *.csv files for use in Star-CCM+
%
% It appears that the recording times are not always consistent. Perhaps
% interpolation on the BCs in time is allowable.
%
% Blake Lance
% 29 May 2014

% Austin Parker
% 04 September 2014

% BC_Stats_Transient_I

% Initialization: First allows user to select a folder.  This section is
% where the user can save_figure, save_data, and manual_time. Save_figure
% is used to determine whether or not the plots will be saved.  save_data
% determines how to save the .csv files.  manual_time allows the user to 
% input a time step or use the average of the recorded times.

% Execute: Loads the transient data from the selected file.  This data is
% then organized to take a spatial ensemble average.  It is then
% reorganized to tatke the ensemble of the ensemble average.  The standard
% deviation and precision uncertainty is taken.  The precision uncertainty
% is used for the plots as well.

% Statistical rejection of outliers: Uses Chauvenet's criterion to
% determine which data are outliers and rejects them.  The data is modified
% for both spatially averaged data and non-spatially averaged data.  The
% variable to control whether or not to use this criterion is called 
% 'outliers' and can be found in the following section.

% Compute and Store Data for ensemble average, standard deviation, and
% precision uncertainty:
% The ensemble average, standard deviation, and precision uncertainty are
% calculated.  This data is saved to where 'folder' vaiable designates.

% Plot:
% First, plots are made for each position.  The plot shows the spatial
% average of each position and the ensemble average of each spatial average
% is shown in bold.  Precision uncertainty bands are shown over the
% ensemble curve.  The last set of plots shows the ensemble averages of
% each position with the full ensemble of the ensemble averages.  These plots
% can be saved by changing the variable 'save_figure' to 1.

% Visual Data Testing:
% The first for loop of this section of code will do 3d plot of one case of
% the wall of interest.  It is linearly interpolated and changes plots
% every tenth of a second to the next time step.  The spatial average plots
% are convenient to compare spatial average temperatures over time of
% different locations on the wall of interest.  Limits of plot should be
% input manually so the graph doesn't jump around.

% Updates: Data can now be processed for cases of different sizes.  Fixed
% the problem using weighted averages in standard deviation.  Fixed
% error in calculating full ensemble precision uncertainty.  Replaced
% multiple for loops with intrinsic functions to speed up the code.

% Version K: added t_offset so that the time shifted when manual_time ~= 0.

% Version L: only write data for time>=0 for save_data = 2
% Changes header of from precision uncertainty P to random uncertainty S

%% Initialization
folder = uigetdir('C:\Users\EFDL\Dropbox\TMCWT\Hot Film Testing',...
    'Select case folder containing position subfolders that contain transient subsubfolders');

plot_all = 0;   % Flag whether to plot all the cases (1) or just the ensemble averages (0)

% When save_figure = 1 the full ensemble vs ensemble plots will be saved.  If save_figure is 0 the
% plots will not be saved.
save_figure = 1;

% The save_data variable determines how the statistical data is saved.  If
% save_data = 0, no statistical data will be saved.  If save_data = 1, 
% the statistical data will be saved as column data
% [x,y,z,ensemble,SD,B,P,U].  If save_data = 2, it will be saved in an
% xyz,time table.  The rows will be appear like this: x, y, z, var1 [t=T1],
% var1 (t=T2]...   If save_data = 3, the data will be saved both ways.
save_data = 2;

% When the variable manual_time =/ 0, the time-step for both the plots and
% data saved will be equivalent to the value entered for manual_time.  If
% manual_time = 0 then the time step will be taken from the recorded time
% and averaged.
manual_time = 0.2;

t_offset = -1.8; % Time offset [s] to be used when manual_time ~= 0.

% When outliers = 0, the raw data will be used for the plots and writing
% the .csv files.  If outliers = 1 the Chauvenet's criterion will be used
% to eliminate outliers.  The nondimensional deviation is calculated using
% N = 2400 and the equation Probability = 1-1/(2*N).  Probability is then
% used to interpolate the nondimensional deviation.  Table A.1 of
% Experimentation, Validation, and Uncertainty Analysis for Engineers was
% used to interpolate a nondimensional deviation.
outliers = 1;
nondimensional_deviation = 3.720; % N = 2400
% nondimensional_deviation = 2.93; % N = 147
%% Execute
tic;
% Find out how many subfolders to say the number of PIV positions tested.
% First, a scan is done to see if plots, column data .csv files, and row
% data .csv files have already been saved in the folder.  If they have, the
% number of folders will be subtracted from N_pos. Note, it only scans to
% see if the Heated Wall files have been saved.  If these have manually
% been deleted but others have not, N_pos will not be correct when running
% the code again.
Scan_data = exist([folder '\HeatedWallTempEnsembleSDPU.csv'],'file');
Scan_data2 = exist([folder '\HeatedWallTemp.csv'],'file');
Scan_plots = exist([folder '\HeatedWallTemp.png'],'file');

% folder_struct with a struct of the folders and files in the folder.
folder_struct = dir(folder);
N_pos = 0; % The number of data sets/runs
% Count the number of subfolders/runs
for i=1:length(folder_struct);
    N_pos = N_pos + folder_struct(i).isdir;
end
% MATLAB counts the '.' and the '..', so correct for it
% N_pos = N_pos - 2;    
N_pos = N_pos - 1; % It seems only the '.' isdir on the real data

% folder_names is initializing the array where the names of the transient
% folders are stored.
folder_names = cell(N_pos,1);

% This was the original way to fill the variable folder_names{i}.  I have
% kept the method in case the current method is not effective.
% for i=1:N_pos
%     folder_names{i} = folder_struct(i+2).name;
% end

% Makes a cell array of the names of all the folders in the folder file.
% The if statement scans the isdir column of folder_struct beginning with
% row 3.  If isdir = 1 then it will store the name in the next cell of
% folder_names.
j = 1;
for i = 1:length(dir(folder)) - 2
    if folder_struct(i+2).('isdir') == 1
        folder_names{j} = folder_struct(i+2).name;
        j = j+1;
    end
end

% The number of transients within each case
N_trans = zeros(N_pos,1);

for i=1:N_pos   % Each case may have a unique number of transient BCs
    N_trans(i) = length(dir([folder,'\',folder_names{i}])) - 2;
end

% Now that we know the subfolder names and how many subsubfolders are
% within each, we can initialize the data arrays
HeatedWallTempData(max(N_trans),N_pos).data = struct();
HeatedWallTempData(max(N_trans),N_pos).textdata = struct();
HeatedWallTempData(max(N_trans),N_pos).colheaders = struct();

InletTempData(max(N_trans),N_pos).data = struct();
InletTempData(max(N_trans),N_pos).textdata = struct();
InletTempData(max(N_trans),N_pos).colheaders = struct();

LeftWallTempData(max(N_trans),N_pos).data = struct();
LeftWallTempData(max(N_trans),N_pos).textdata = struct();
LeftWallTempData(max(N_trans),N_pos).colheaders = struct();

RightWallTempData(max(N_trans),N_pos).data = struct();
RightWallTempData(max(N_trans),N_pos).textdata = struct();
RightWallTempData(max(N_trans),N_pos).colheaders = struct();

TopWallTempData(max(N_trans),N_pos).data = struct();
TopWallTempData(max(N_trans),N_pos).textdata = struct();
TopWallTempData(max(N_trans),N_pos).colheaders = struct();

HeatFluxData(max(N_trans),N_pos).data = struct();
HeatFluxData(max(N_trans),N_pos).textdata = struct();
HeatFluxData(max(N_trans),N_pos).colheaders = struct();

PS_VoltageData(max(N_trans),N_pos).data = struct();
PS_VoltageData(max(N_trans),N_pos).textdata = struct();
PS_VoltageData(max(N_trans),N_pos).colheaders = struct();

dPData(max(N_trans),N_pos).data = struct();
dPData(max(N_trans),N_pos).textdata = struct();
dPData(max(N_trans),N_pos).colheaders = struct();

% If statement used to scan how many folders are in the Transient data
% folder.  < 5 corresponds to the folders with 3 levels of sub folders.
% The else statement corresponds to the folders with only 2 levels of
% subfolders.  If all levels of subfolders are the same, this loop can be
% deleted.

% Loads the folders and raw data
for i=1:N_pos   % Loop through subfolders
    for j=1:N_trans(i)  % Loop through subsubfolders
        HeatedWallTempData(j,i) = ...
            importdata([folder,'\',folder_names{i},'\',int2str(j),'\HeatedWallTemp.csv']);
        InletTempData(j,i) = ...
            importdata([folder,'\',folder_names{i},'\',int2str(j),'\InletTemp.csv']);
        LeftWallTempData(j,i) = ...
            importdata([folder,'\',folder_names{i},'\',int2str(j),'\LeftWallTemp.csv']);
        RightWallTempData(j,i) = ...
            importdata([folder,'\',folder_names{i},'\',int2str(j),'\RightWallTemp.csv']);
        TopWallTempData(j,i) = ...
            importdata([folder,'\',folder_names{i},'\',int2str(j),'\TopWallTemp.csv']);
        HeatFluxData(j,i) = ...
            importdata([folder,'\',folder_names{i},'\',int2str(j),'\HeatFlux.csv']);
        PS_VoltageData(j,i) = ...
            importdata([folder,'\',folder_names{i},'\',int2str(j),'\PS_Voltage.csv']);
        dPData(j,i) = ...
            importdata([folder,'\',folder_names{i},'\',int2str(j),'\dP.csv']);
    end
    fprintf('Finished loading data from subfolder %2i of %2i at %6.1f seconds \n', i, N_pos, toc);
end

% Defines variables necessary to organize the data into arrays.
% Total number samples taken for the specific data
HWT_total = length(HeatedWallTempData(1,1).data);   
% Number of data taken per time step
HWT_data_per_sample = 229;                          

IT_total = length(InletTempData(1,1).data);
IT_data_per_sample = 15;

LWT_total = length(LeftWallTempData(1,1).data);
LWT_data_per_sample = 21;

RWT_total = length(RightWallTempData(1,1).data);
RWT_data_per_sample = 21;

TWT_total = length(TopWallTempData(1,1).data);
TWT_data_per_sample = 21;

HF_total = length(HeatFluxData(1,1).data);
HF_data_per_sample = 3;

PS_total = size(PS_VoltageData(1,1).data);
PS_total = PS_total(1);
PS_data_per_sample = 3;

dP_total = length(dPData(1,1).data);
dP_data_per_sample = 1;

% Initialize arrays to store the data for averages
HWT = NaN(HWT_data_per_sample,HWT_total/HWT_data_per_sample,max(N_trans),N_pos);
IT = NaN(IT_data_per_sample,IT_total/IT_data_per_sample,max(N_trans),N_pos);
LWT = NaN(LWT_data_per_sample,LWT_total/LWT_data_per_sample,max(N_trans),N_pos);
RWT = NaN(RWT_data_per_sample,RWT_total/RWT_data_per_sample,max(N_trans),N_pos);
TWT = NaN(TWT_data_per_sample,TWT_total/TWT_data_per_sample,max(N_trans),N_pos);
HF = NaN(HF_data_per_sample,HF_total/HF_data_per_sample,max(N_trans),N_pos);
dP = NaN(dP_data_per_sample,dP_total/dP_data_per_sample,max(N_trans),N_pos);

% Enters the data into a 4 dimensional array convenient for taking the average.
% The 4D array is organized as follows: (probe number, time step, 
% sub-subfolder, subfolder).
% HeatedWallTempData (original data was in Kelvin, here it is converted to
% degrees Celsius.
for i = 1:HWT_total/HWT_data_per_sample;
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for j = 1:HWT_data_per_sample;
                HWT(j,i,k,l) = HeatedWallTempData(k,l).data(j+(i-1)*HWT_data_per_sample,5)-273.15;
            end
        end
    end
end

% InletTempData (original data was in Kelvin, here it is converted to
% degrees Celsius)
for i = 1:IT_total/IT_data_per_sample;
    for l = 1:N_pos;
        for k = 1:N_trans(l);            
            for j = 1:IT_data_per_sample;
                IT(j,i,k,l) = InletTempData(k,l).data(j+(i-1)*IT_data_per_sample,5)-273.15;
            end
        end
    end
end

% LeftWallTempData
% RightWallTempData
% TopWallTempData (original data was in Kelvin, here it is converted to
% degrees Celsius)
% These can all be done in the same set of for loops because LWT_total =
% RWT_total = TWT_total and LWT_data_per_sample = RWT_data_per_sample =
% TWT_data_per_sample.
for i = 1:LWT_total/LWT_data_per_sample;
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for j = 1:LWT_data_per_sample;
                LWT(j,i,k,l) = LeftWallTempData(k,l).data(j+(i-1)*LWT_data_per_sample,5)-273.15;
                RWT(j,i,k,l) = RightWallTempData(k,l).data(j+(i-1)*RWT_data_per_sample,5)-273.15;
                TWT(j,i,k,l) = TopWallTempData(k,l).data(j+(i-1)*TWT_data_per_sample,5)-273.15;
            end
        end
    end
end

% HeatFluxData
for i = 1:HF_total/HF_data_per_sample;
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for j = 1:HF_data_per_sample;
                HF(j,i,k,l) = HeatFluxData(k,l).data(j+(i-1)*HF_data_per_sample,5);
            end
        end
    end
end

% dPData
for i = 1:dP_total/dP_data_per_sample;
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for j = 1:dP_data_per_sample;
                dP(j,i,k,l) = dPData(k,l).data(j+(i-1)*dP_data_per_sample,2);
            end
        end
    end
end

% PS_VoltageData
PS = NaN(3,max(N_trans),N_pos);
for i = 1:N_pos;
    for j = 1:N_trans(i)
        PS(:,j,i) = PS_VoltageData(j,i).data(:,5);
    end
end

% Initialize arrays for taking the full ensemble average and standard deviation
HWT_data = NaN(HWT_total,sum(N_trans));
IT_data = NaN(IT_total,sum(N_trans));
LWT_data = NaN(LWT_total,sum(N_trans));
RWT_data = NaN(RWT_total,sum(N_trans));
TWT_data = NaN(TWT_total,sum(N_trans));
HF_data = NaN(HF_total,sum(N_trans));
PS_data = NaN(PS_total,sum(N_trans));
dP_data = NaN(dP_total,sum(N_trans));

% Pulls the raw data into matrices organized by (total number of data
% points, sum(N_trans))
count = 1;
for j = 1:N_pos
    for i = 1:N_trans(j)
        HWT_data(:,count) = HeatedWallTempData(i,j).data(:,5);
        IT_data(:,count) = InletTempData(i,j).data(:,5);
        LWT_data(:,count) = LeftWallTempData(i,j).data(:,5);
        RWT_data(:,count) = RightWallTempData(i,j).data(:,5);
        TWT_data(:,count) = TopWallTempData(i,j).data(:,5);
        HF_data(:,count) = HeatFluxData(i,j).data(:,5);
        PS_data(:,count) = PS_VoltageData(i,j).data(:,5);
        dP_data(:,count) = dPData(i,j).data(:,2);
        count = count+1;
    end
end

%% Statistical rejection of outliers
% The outliers variable is manually changed at the top of the code.  If
% outliers == 0 then the raw data will be used and this seciton will be
% skipped.  If outliers == 1 (or anything not 0) then Chauvenet's
% criterion will be used to reject the outliers.
if outliers ~= 0;
    % Takes the true ensemble average (not spatially averaged).
    HWT_data_ensemble = mean(HWT_data,2);
    IT_data_ensemble = mean(IT_data,2);
    LWT_data_ensemble = mean(LWT_data,2);
    RWT_data_ensemble = mean(RWT_data,2);
    TWT_data_ensemble = mean(TWT_data,2);
    HF_data_ensemble = mean(HF_data,2);
    PS_data_ensemble = mean(PS_data,2);
    dP_data_ensemble = mean(dP_data,2);
    
    % Standard deviation
    HWT_data_std = std(HWT_data,0,2);
    IT_data_std = std(IT_data,0,2);
    LWT_data_std = std(LWT_data,0,2);
    RWT_data_std = std(RWT_data,0,2);
    TWT_data_std = std(TWT_data,0,2);
    HF_data_std = std(HF_data,0,2);
    PS_data_std = std(PS_data,0,2);
    dP_data_std = std(dP_data,0,2);
    
    % Any value of the raw data in each case greater than the max or lower
    % than the min will be rejected as an outlier.  Here the max and min
    % are determined for each data.
    HWT_maxmin = [HWT_data_ensemble+...
        nondimensional_deviation*HWT_data_std HWT_data_ensemble-nondimensional_deviation*HWT_data_std];
    IT_maxmin = [IT_data_ensemble+...
        nondimensional_deviation*IT_data_std IT_data_ensemble-nondimensional_deviation*IT_data_std];
    LWT_maxmin = [LWT_data_ensemble+...
        nondimensional_deviation*LWT_data_std LWT_data_ensemble-nondimensional_deviation*LWT_data_std];
    RWT_maxmin = [RWT_data_ensemble+...
        nondimensional_deviation*RWT_data_std RWT_data_ensemble-nondimensional_deviation*RWT_data_std];
    TWT_maxmin = [TWT_data_ensemble+...
        nondimensional_deviation*TWT_data_std TWT_data_ensemble-nondimensional_deviation*TWT_data_std];
    HF_maxmin = [HF_data_ensemble+...
        nondimensional_deviation*HF_data_std HF_data_ensemble-nondimensional_deviation*HF_data_std];
    PS_maxmin = [PS_data_ensemble+...
        nondimensional_deviation*PS_data_std PS_data_ensemble-nondimensional_deviation*PS_data_std];
    dP_maxmin = [dP_data_ensemble+...
        nondimensional_deviation*dP_data_std dP_data_ensemble-nondimensional_deviation*dP_data_std];
    
    % Initializes arrays for rejecting the outliers
    HWT_reject = NaN(HWT_total,sum(N_trans));
    IT_reject = NaN(IT_total,sum(N_trans));
    LWT_reject = NaN(LWT_total,sum(N_trans));
    RWT_reject = NaN(RWT_total,sum(N_trans));
    TWT_reject = NaN(TWT_total,sum(N_trans));
    HF_reject = NaN(HF_total,sum(N_trans));
    PS_reject = NaN(PS_total,sum(N_trans));
    dP_reject = NaN(dP_total,sum(N_trans));
    
    % These for loops use boolean operator to label the location of the
    % rejected data as a 1.
    % HeatedWallTemp
    for i = 1:HWT_total;
        HWT_reject(i,:) = HWT_data(i,:) > HWT_maxmin(i,1) | HWT_data(i,:) < HWT_maxmin(i,2);
    end

    % InletTemp
    for i = 1:IT_total;
        IT_reject(i,:) = IT_data(i,:) > IT_maxmin(i,1) | IT_data(i,:) < IT_maxmin(i,2);
    end
    
    % LeftWallTemp
    % RightWallTemp
    % TopWallTemp
    for i = 1:LWT_total;
        LWT_reject(i,:) = LWT_data(i,:) > LWT_maxmin(i,1) | LWT_data(i,:) < LWT_maxmin(i,2);
        RWT_reject(i,:) = RWT_data(i,:) > RWT_maxmin(i,1) | RWT_data(i,:) < RWT_maxmin(i,2);
        TWT_reject(i,:) = TWT_data(i,:) > TWT_maxmin(i,1) | TWT_data(i,:) < TWT_maxmin(i,2);
    end
    
    % HeatFlux
    for i = 1:HF_total;
        HF_reject(i,:) = HF_data(i,:) > HF_maxmin(i,1) | HF_data(i,:) < HF_maxmin(i,2);
    end
    
    % PS_Voltage
    for i = 1:PS_total;
        PS_reject(i,:) = PS_data(i,:) > PS_maxmin(i,1) | PS_data(i,:) < PS_maxmin(i,2);
    end
    
    % dP
    for i = 1:dP_total;
        dP_reject(i,:) = dP_data(i,:) > dP_maxmin(i,1) | dP_data(i,:) < dP_maxmin(i,2);
    end
    
    % Changes the data arrays to NaN wherever the reject variable is
    % equivalent to 1.  This is where the data point is greater than the
    % max or less than the min.
    HWT_data(HWT_reject == 1) = NaN; 
    IT_data(IT_reject == 1) = NaN;
    LWT_data(LWT_reject == 1) = NaN;
    RWT_data(RWT_reject == 1) = NaN;
    TWT_data(TWT_reject == 1) = NaN;
    HF_data(HF_reject == 1) = NaN;
    PS_data(PS_reject == 1) = NaN;
    dP_data(dP_reject == 1) = NaN;
    
    % Reject2 variables are for changing the data used for spatial
    % averages.  Here the reject2 array is initialized.  To plot the 
    % original data (without outliers removed) comment out lines 451-578.
    HWT_reject2 = NaN(HWT_data_per_sample,HWT_total/HWT_data_per_sample,max(N_trans),N_pos);
    IT_reject2 = NaN(IT_data_per_sample,IT_total/IT_data_per_sample,max(N_trans),N_pos);
    LWT_reject2 = NaN(LWT_data_per_sample,LWT_total/LWT_data_per_sample,max(N_trans),N_pos);
    RWT_reject2 = NaN(RWT_data_per_sample,RWT_total/RWT_data_per_sample,max(N_trans),N_pos);
    TWT_reject2 = NaN(TWT_data_per_sample,TWT_total/TWT_data_per_sample,max(N_trans),N_pos);
    HF_reject2 = NaN(HF_data_per_sample,HF_total/HF_data_per_sample,max(N_trans),N_pos);
    PS_reject2 = NaN(PS_data_per_sample,max(N_trans),N_pos);
    dP_reject2 = NaN(dP_data_per_sample,dP_total/dP_data_per_sample,max(N_trans),N_pos);
    
    % Reorganizes reject to reject2 which will be used for the spatially
    % averaged data.  
    % HeatedWallTemp
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for i = 1:HWT_total/HWT_data_per_sample;
                HWT_reject2(:,i,k,l) = HWT_reject(1+(i-1)*HWT_data_per_sample:HWT_data_per_sample+...
                    (i-1)*HWT_data_per_sample,k+sum(N_trans(1:l-1)));
            end
        end
    end
    
    % InletTemp
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for i = 1:IT_total/IT_data_per_sample;
                IT_reject2(:,i,k,l) = IT_reject(1+(i-1)*IT_data_per_sample:IT_data_per_sample+...
                    (i-1)*IT_data_per_sample,k+sum(N_trans(1:l-1)));
            end
        end
    end

    % LeftWallTemp
    % RightWallTemp
    % TopWallTemp
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for i = 1:LWT_total/LWT_data_per_sample;
                LWT_reject2(:,i,k,l) = LWT_reject(1+(i-1)*LWT_data_per_sample:LWT_data_per_sample+...
                    (i-1)*LWT_data_per_sample,k+sum(N_trans(1:l-1)));
                RWT_reject2(:,i,k,l) = RWT_reject(1+(i-1)*RWT_data_per_sample:RWT_data_per_sample+...
                    (i-1)*RWT_data_per_sample,k+sum(N_trans(1:l-1)));
                TWT_reject2(:,i,k,l) = TWT_reject(1+(i-1)*TWT_data_per_sample:TWT_data_per_sample+...
                    (i-1)*TWT_data_per_sample,k+sum(N_trans(1:l-1)));
            end
        end
    end
    
    % HeatFlux
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for i = 1:HF_total/HF_data_per_sample;
                HF_reject2(:,i,k,l) = HF_reject(1+(i-1)*HF_data_per_sample:HF_data_per_sample+...
                    (i-1)*HF_data_per_sample,k+sum(N_trans(1:l-1)));
            end
        end
    end
    
    % PS_Voltage
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            PS_reject2(:,k,l) = PS_reject(1:PS_data_per_sample,k+sum(N_trans(1:l-1)));
        end
    end
    
    % dP
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for i = 1:dP_total/dP_data_per_sample;
                dP_reject2(:,i,k,l) = dP_reject(1+(i-1)*dP_data_per_sample:dP_data_per_sample+...
                    (i-1)*dP_data_per_sample,k+sum(N_trans(1:l-1)));
            end
        end
    end
    
    % Remove all spatial averages that have a NaN. The spatial averages 
    % with a NaN are skewed since N is much smaller for the spatial
    % average.  To see the plots with only the outliers removed, comment
    % out lines 523-567.
    % HeatedWallTemp
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for i = 1:HWT_total/HWT_data_per_sample;
                if sum(HWT_reject2(:,i,k,l) > 0); HWT_reject2(1:HWT_data_per_sample,i,k,l) = 1; end
            end
        end
    end
    
    % InletTemp
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for i = 1:IT_total/IT_data_per_sample;
                if sum(IT_reject2(:,i,k,l) > 0); IT_reject2(1:IT_data_per_sample,i,k,l) = 1; end
            end
        end
    end
    
    % LeftWallTemp
    % RightWallTemp
    % TopWallTemp
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for i = 1:LWT_total/LWT_data_per_sample;
                if sum(LWT_reject2(:,i,k,l) > 0); LWT_reject2(1:LWT_data_per_sample,i,k,l) = 1; end
                if sum(RWT_reject2(:,i,k,l) > 0); RWT_reject2(1:RWT_data_per_sample,i,k,l) = 1; end
                if sum(TWT_reject2(:,i,k,l) > 0); TWT_reject2(1:TWT_data_per_sample,i,k,l) = 1; end
            end
        end
    end
    
    % HeatFlux
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            for i = 1:HF_total/HF_data_per_sample;
                if sum(HF_reject2(:,i,k,l)) > 0; HF_reject2(:,i,k,l) = 1; end
            end
        end
    end
    
    % PS_Voltage
    for l = 1:N_pos;
        for k = 1:N_trans(l);
            if sum(PS_reject2(:,k,l) > 0); PS_reject2(:,k,l) = 1; end
        end
    end
    
    % In the arrays for taking the spatial average, replaces the outliers
    % with NaN
    HWT(HWT_reject2 == 1) = NaN; 
    IT(IT_reject2 == 1) = NaN;
    LWT(LWT_reject2 == 1) = NaN;
    RWT(RWT_reject2 == 1) = NaN;
    TWT(TWT_reject2 == 1) = NaN;
    HF(HF_reject2 == 1) = NaN;
    PS(PS_reject2 == 1) = NaN;
    dP(dP_reject2 == 1) = NaN;
    
end
%% Execute (continued)

% PS_case is the label for the x axis of the PS_Voltage plot. The numbers
% 1-max(N_trans) correspond to the numbers of the case folder.
PS_case = 1:max(N_trans);

% Takes the spatial average of the data.  The data was
% previously organized.  Here the columns are being averaged.
HWT_avg = nanmean(HWT);
IT_avg = nanmean(IT);
LWT_avg = nanmean(LWT);
RWT_avg = nanmean(RWT);
TWT_avg = nanmean(TWT);
HF_avg = nanmean(HF);
PS_avg = nanmean(PS);

% Takes the ensemble average of the spatially averaged data
HWT_ensemble = nanmean(HWT_avg,3);
IT_ensemble = nanmean(IT_avg,3);
LWT_ensemble = nanmean(LWT_avg,3);
RWT_ensemble = nanmean(RWT_avg,3);
TWT_ensemble = nanmean(TWT_avg,3);
HF_ensemble = nanmean(HF_avg,3);
dP_ensemble = nanmean(dP,3);

% Initializes a matrix for taking the full ensemble average
HWT_full = NaN(sum(N_trans),HWT_total/HWT_data_per_sample);
IT_full = NaN(sum(N_trans),IT_total/IT_data_per_sample);
LWT_full = NaN(sum(N_trans),LWT_total/LWT_data_per_sample);
RWT_full = NaN(sum(N_trans),RWT_total/RWT_data_per_sample);
TWT_full = NaN(sum(N_trans),TWT_total/TWT_data_per_sample);
HF_full = NaN(sum(N_trans),HF_total/HF_data_per_sample);
dP_full = NaN(sum(N_trans),dP_total/dP_data_per_sample);

% Organizes the data to be stored as (spatially averaged data from each
% case of all the positions, time step)
count = 1;
for i = 1:N_pos;
    for j = 1:N_trans(i);
        HWT_full(count,1:HWT_total/HWT_data_per_sample) = HWT_avg(:,:,j,i);
        IT_full(count,1:IT_total/IT_data_per_sample) = IT_avg(:,:,j,i);
        LWT_full(count,1:LWT_total/LWT_data_per_sample) = LWT_avg(:,:,j,i);
        RWT_full(count,1:RWT_total/RWT_data_per_sample) = RWT_avg(:,:,j,i);
        TWT_full(count,1:TWT_total/TWT_data_per_sample) = TWT_avg(:,:,j,i);
        HF_full(count,1:HF_total/HF_data_per_sample) = HF_avg(:,:,j,i);
        dP_full(count,1:dP_total/dP_data_per_sample) = dP(:,:,j,i);
        count = count + 1;
    end
end

% Full ensemble averages
HWT_full_ensemble = nanmean(HWT_full,1);
IT_full_ensemble = nanmean(IT_full,1);
LWT_full_ensemble = nanmean(LWT_full,1);
RWT_full_ensemble = nanmean(RWT_full,1);
TWT_full_ensemble = nanmean(TWT_full,1);
HF_full_ensemble = nanmean(HF_full,1);
dP_full_ensemble = nanmean(dP_full,1);

% Initializes an array for the average time step.  Uses an average of the
% HWT(:,4,1,1) time
time_array = NaN(1,HWT_total/HWT_data_per_sample);

% If manual_time == 0 then the if statement takes all the time data from
% the first case, first position of the heated wall and averages it to get
% an average time step.  This time step is then used to make a time array
% used for plotting.  If manual_time ~= 0, time_array will be an array
% increasing by the value of manual_time.
if manual_time == 0
    avg_time_step = HeatedWallTempData(1,1).data(HWT_total,4)/(HWT_total/HWT_data_per_sample);
    for i = 1:HWT_total/HWT_data_per_sample
        time_array(i) = avg_time_step*(i-1);
    end
elseif manual_time ~= 0
    for i = 1:HWT_total/HWT_data_per_sample
        time_array(i) = (i-1)*manual_time + t_offset;
    end
end
i_t0 = find(time_array==0,1); % Find the index where t=0

% Standard deviation of spatial averages. This data is used for making the
% plots.
HWT_avg_std = nanstd(HWT_avg,0,3);
IT_avg_std = nanstd(IT_avg,0,3);
LWT_avg_std = nanstd(LWT_avg,0,3);
RWT_avg_std = nanstd(RWT_avg,0,3);
TWT_avg_std = nanstd(TWT_avg,0,3);
HF_avg_std = nanstd(HF_avg,0,3);
dP_std = nanstd(dP,0,3);

% v adjusts for the number of cases.  If the number of cases is 20 or less,
% then v will be the v(1,N_trans(i)).  If the number of cases is 21 or
% greater, v = 1.96.
v_select=[1:21;12.706 4.303 3.182 2.776 2.571 2.447 2.365 2.306 2.262 2.228 2.201 2.179...
    2.160 2.145 2.131 2.12 2.11 2.101 2.093 2.086 1.96]';
v = NaN(N_pos,1);
for i = 1:N_pos
    if N_trans(i) <= 20; v(i) = v_select(N_trans(i),2); elseif N_trans(i) >= 21; v(i) = v_select(21,2); end
end
% v_pos is used when taking the full ensemble average of the ensemble
% averages (for the plots).  It is only N_pos because it is only a
% correspondence between the number of N_pos ensemble averages.
if HWT_data_per_sample*sum(N_trans) <= 20; v_ensemble = v_select(N_pos,2); 
elseif HWT_data_per_sample*sum(N_trans) >= 21; v_ensemble = v_select(21,2); 
end
% v_sum is used for taking the true ensemble average (for recording data).
% It is a number to correlate the data for each case and position for each
% time step.
if sum(N_trans) <= 20; v_sum = v_select(sum(N_trans),2); 
elseif sum(N_trans) >= 21; v_sum = v_select(21,2); 
end

% Initialize matrices for taking the precision uncertainty
HWT_avg_pu = NaN(size(HWT_avg_std));
IT_avg_pu = NaN(size(IT_avg_std));
LWT_avg_pu = NaN(size(LWT_avg_std));
RWT_avg_pu = NaN(size(RWT_avg_std));
TWT_avg_pu = NaN(size(TWT_avg_std));
HF_avg_pu = NaN(size(HF_avg_std));
dP_pu = NaN(size(dP_std));

% Precision uncertainty of the spatial averages
for i = 1:N_pos
    HWT_avg_pu(:,:,:,i) = HWT_avg_std(:,:,:,i)*v(i)/sqrt(N_trans(i));
    IT_avg_pu(:,:,:,i) = IT_avg_std(:,:,:,i)*v(i)/sqrt(N_trans(i));
    LWT_avg_pu(:,:,:,i) = LWT_avg_std(:,:,:,i)*v(i)/sqrt(N_trans(i));
    RWT_avg_pu(:,:,:,i) = RWT_avg_std(:,:,:,i)*v(i)/sqrt(N_trans(i));
    TWT_avg_pu(:,:,:,i) = TWT_avg_std(:,:,:,i)*v(i)/sqrt(N_trans(i));
    HF_avg_pu(:,:,:,i) = HF_avg_std(:,:,:,i)*v(i)/sqrt(N_trans(i));
    dP_pu(:,:,:,i) = dP_std(:,:,:,i)*v(i)/sqrt(N_trans(i));
end

% Spacially averaged bias
HWT_avg_bias = ones(1,HWT_total/HWT_data_per_sample,1,N_pos);
IT_avg_bias = ones(1,IT_total/IT_data_per_sample,1,N_pos);
LWT_avg_bias = ones(1,LWT_total/LWT_data_per_sample,1,N_pos);
RWT_avg_bias = ones(1,RWT_total/RWT_data_per_sample,1,N_pos);
TWT_avg_bias = ones(1,TWT_total/TWT_data_per_sample,1,N_pos);
HF_avg_bias = HF_ensemble*0.05;

% Spacially averaged uncertainty
HWT_avg_u = sqrt(HWT_avg_bias.*HWT_avg_bias+HWT_avg_pu.*HWT_avg_pu);
IT_avg_u = sqrt(IT_avg_bias.*IT_avg_bias+IT_avg_pu.*IT_avg_pu);
LWT_avg_u = sqrt(LWT_avg_bias.*LWT_avg_bias+LWT_avg_pu.*LWT_avg_pu);
RWT_avg_u = sqrt(RWT_avg_bias.*RWT_avg_bias+RWT_avg_pu.*RWT_avg_pu);
TWT_avg_u = sqrt(TWT_avg_bias.*TWT_avg_bias+TWT_avg_pu.*TWT_avg_pu);
HF_avg_u = sqrt(HF_avg_bias.*HF_avg_bias+HF_avg_pu.*HF_avg_pu);

% Standard deviation of ensemble averages
HWT_ensemble_std = nanstd(HWT_full);
IT_ensemble_std = nanstd(IT_full);
LWT_ensemble_std = nanstd(LWT_full);
RWT_ensemble_std = nanstd(RWT_full);
TWT_ensemble_std = nanstd(TWT_full);
HF_ensemble_std = nanstd(HF_full);
dP_ensemble_std = nanstd(dP_full);

% Precision uncertainty of the ensemble averages
HWT_ensemble_pu = HWT_ensemble_std*v_ensemble/sqrt(sum(N_trans));
IT_ensemble_pu = IT_ensemble_std*v_ensemble/sqrt(sum(N_trans));
LWT_ensemble_pu = LWT_ensemble_std*v_ensemble/sqrt(sum(N_trans));
RWT_ensemble_pu = RWT_ensemble_std*v_ensemble/sqrt(sum(N_trans));
TWT_ensemble_pu = TWT_ensemble_std*v_ensemble/sqrt(sum(N_trans));
HF_ensemble_pu = HF_ensemble_std*v_ensemble/sqrt(sum(N_trans));
dP_ensemble_pu = dP_ensemble_std*v_ensemble/sqrt(sum(N_trans));

% Spacially averaged ensemble bias
HWT_ensemble_bias = ones(1,HWT_total/HWT_data_per_sample);
IT_ensemble_bias = ones(1,IT_total/IT_data_per_sample);
LWT_ensemble_bias = ones(1,LWT_total/LWT_data_per_sample);
RWT_ensemble_bias = ones(1,RWT_total/RWT_data_per_sample);
TWT_ensemble_bias = ones(1,TWT_total/TWT_data_per_sample);
HF_ensemble_bias = HF_full_ensemble*0.05;

% Spacially averaged ensemble uncertainty
HWT_ensemble_u = sqrt(HWT_ensemble_bias.*HWT_ensemble_bias+HWT_ensemble_pu.*HWT_ensemble_pu);
IT_ensemble_u = sqrt(IT_ensemble_bias.*IT_ensemble_bias+IT_ensemble_pu.*IT_ensemble_pu);
LWT_ensemble_u = sqrt(LWT_ensemble_bias.*LWT_ensemble_bias+LWT_ensemble_pu.*LWT_ensemble_pu);
RWT_ensemble_u = sqrt(RWT_ensemble_bias.*RWT_ensemble_bias+RWT_ensemble_pu.*RWT_ensemble_pu);
TWT_ensemble_u = sqrt(TWT_ensemble_bias.*TWT_ensemble_bias+TWT_ensemble_pu.*TWT_ensemble_pu);
HF_ensemble_u = sqrt(HF_ensemble_bias.*HF_ensemble_bias+HF_ensemble_pu.*HF_ensemble_pu);

%% Compute and Store Data for ensemble average, standard deviation, bias, precision, and total uncertainty
% Scan_data checks to see if this data has already been stored.  If the
% data has already been stored, this section will be bypassed, saving time.
if Scan_data == 0 && Scan_data2 == 0 && save_data ~= 0
    
    % Initializes arrays for making time averages
    if manual_time == 0
        Time_HWT_avg = NaN(HWT_total,sum(N_trans));
        Time_IT_avg = NaN(IT_total,sum(N_trans));
        Time_LWT_avg = NaN(LWT_total,sum(N_trans));
        Time_RWT_avg = NaN(RWT_total,sum(N_trans));
        Time_TWT_avg = NaN(TWT_total,sum(N_trans));
        Time_HF_avg = NaN(HF_total,sum(N_trans));
        Time_dP_avg = NaN(dP_total,sum(N_trans));
    elseif manual_time ~= 0
        Time_HWT_avg = NaN(HWT_total,1);
        Time_IT_avg = NaN(IT_total,1);
        Time_LWT_avg = NaN(LWT_total,1);
        Time_RWT_avg = NaN(RWT_total,1);
        Time_TWT_avg = NaN(TWT_total,1);
        Time_HF_avg = NaN(HF_total,1);
        Time_dP_avg = NaN(dP_total,1);
    end
    
    % If manual_time == 0 then the time array for the data will be filled.  
    % Later this array will be ensemble averaged.
    if manual_time == 0
        count = 1;
        for j = 1:N_pos
            for i = 1:N_trans(j)
                Time_HWT_avg(:,count) = HeatedWallTempData(i,j).data(:,4);
                Time_IT_avg(:,count) = InletTempData(i,j).data(:,4);
                Time_LWT_avg(:,count) = LeftWallTempData(i,j).data(:,4);
                Time_RWT_avg(:,count) = RightWallTempData(i,j).data(:,4);
                Time_TWT_avg(:,count) = TopWallTempData(i,j).data(:,4);
                Time_HF_avg(:,count) = HeatFluxData(i,j).data(:,4);
                Time_dP_avg(:,count) = dPData(i,j).data(:,1);
                count = count+1;
            end
        end
    end
    
    % If manual_time ~= 0, a time array will be made for the data to be
    % stored.  The array increases by the amount specified by manual_time.
    if manual_time ~= 0
        for i = 1:HWT_total/HWT_data_per_sample
            Time_HWT_avg(1+(i-1)*HWT_data_per_sample:HWT_data_per_sample*i,1) = (i-1)*manual_time + t_offset;
            Time_IT_avg(1+(i-1)*IT_data_per_sample:IT_data_per_sample*i,1) = (i-1)*manual_time + t_offset;
            Time_LWT_avg(1+(i-1)*LWT_data_per_sample:LWT_data_per_sample*i,1) = (i-1)*manual_time + t_offset;
            Time_RWT_avg(1+(i-1)*RWT_data_per_sample:RWT_data_per_sample*i,1) = (i-1)*manual_time + t_offset;
            Time_TWT_avg(1+(i-1)*TWT_data_per_sample:TWT_data_per_sample*i,1) = (i-1)*manual_time + t_offset;
            Time_HF_avg(1+(i-1)*HF_data_per_sample:HF_data_per_sample*i,1) = (i-1)*manual_time + t_offset;
            Time_dP_avg(1+(i-1)*dP_data_per_sample:dP_data_per_sample*i,1) = (i-1)*manual_time + t_offset;
        end
    end
    
    % Averages the time
    Time_HWT_avg = nanmean(Time_HWT_avg,2);
    Time_IT_avg = nanmean(Time_IT_avg,2);
    Time_LWT_avg = nanmean(Time_LWT_avg,2);
    Time_RWT_avg = nanmean(Time_RWT_avg,2);
    Time_TWT_avg = nanmean(Time_TWT_avg,2);
    Time_HF_avg = nanmean(Time_HF_avg,2);
    Time_dP_avg = nanmean(Time_dP_avg,2);
    
    % Takes the true ensemble average (not spatially averaged).
    HWT_data_ensemble = nanmean(HWT_data,2);
    IT_data_ensemble = nanmean(IT_data,2);
    LWT_data_ensemble = nanmean(LWT_data,2);
    RWT_data_ensemble = nanmean(RWT_data,2);
    TWT_data_ensemble = nanmean(TWT_data,2);
    HF_data_ensemble = nanmean(HF_data,2);
    PS_data_ensemble = nanmean(PS_data,2);
    dP_data_ensemble = nanmean(dP_data,2);
    
    % Standard deviation
    HWT_data_std = nanstd(HWT_data,0,2);
    IT_data_std = nanstd(IT_data,0,2);
    LWT_data_std = nanstd(LWT_data,0,2);
    RWT_data_std = nanstd(RWT_data,0,2);
    TWT_data_std = nanstd(TWT_data,0,2);
    HF_data_std = nanstd(HF_data,0,2);
    PS_data_std = nanstd(PS_data,0,2);
    dP_data_std = nanstd(dP_data,0,2);
    
    % Bias: the bias for the thermocouples is one degree Celcius/Kelvin.  The bias
    % for the heat flux is 5%.
    HWT_bias = ones(HWT_total,1);
    IT_bias = ones(IT_total,1);
    LWT_bias = ones(LWT_total,1);
    RWT_bias = ones(RWT_total,1);
    TWT_bias = ones(TWT_total,1);
    HF_bias = HF_data_ensemble*0.05;
    
    % Precision Uncertainty, v is determined by the number of total cases.
    HWT_data_pu = HWT_data_std*v_sum/sqrt(sum(N_trans));
    IT_data_pu = IT_data_std*v_sum/sqrt(sum(N_trans));
    LWT_data_pu = LWT_data_std*v_sum/sqrt(sum(N_trans));
    RWT_data_pu = RWT_data_std*v_sum/sqrt(sum(N_trans));
    TWT_data_pu = TWT_data_std*v_sum/sqrt(sum(N_trans));
    HF_data_pu = HF_data_std*v_sum/sqrt(sum(N_trans));
    PS_data_pu = PS_data_std*v_sum/sqrt(sum(N_trans));
    dP_data_pu = dP_data_std*v_sum/sqrt(sum(N_trans));
    
    % Uncertainty
    HWT_data_u = sqrt(HWT_bias.*HWT_bias+HWT_data_pu.*HWT_data_pu);
    IT_data_u = sqrt(IT_bias.*IT_bias+IT_data_pu.*IT_data_pu);
    LWT_data_u = sqrt(LWT_bias.*LWT_bias+LWT_data_pu.*LWT_data_pu);
    RWT_data_u = sqrt(RWT_bias.*RWT_bias+RWT_data_pu.*RWT_data_pu);
    TWT_data_u = sqrt(TWT_bias.*TWT_bias+TWT_data_pu.*TWT_data_pu);
    HF_data_u = sqrt(HF_bias.*HF_bias+HF_data_pu.*HF_data_pu);
    
    % Assembles the arrays for X position, Y position, Z position, time,
    % Ensemble, Standard Deviation, and Precision Uncertainty.
    % HeatedWallTempData(1,1).data(:,1:3) is used because no matter which
    % struct is used of HeatedWallTempData, x, y, and z are all the same.
    HWT_store = [HeatedWallTempData(1,1).data(:,1:3) Time_HWT_avg...
        HWT_data_ensemble HWT_bias HWT_data_pu HWT_data_u];
    IT_store = [InletTempData(1,1).data(:,1:3) Time_IT_avg...
        IT_data_ensemble IT_bias IT_data_pu IT_data_u];
    LWT_store = [LeftWallTempData(1,1).data(:,1:3) Time_LWT_avg...
        LWT_data_ensemble LWT_bias LWT_data_pu LWT_data_u];
    RWT_store = [RightWallTempData(1,1).data(:,1:3) Time_RWT_avg...
        RWT_data_ensemble RWT_bias RWT_data_pu RWT_data_u];
    TWT_store = [TopWallTempData(1,1).data(:,1:3) Time_TWT_avg...
        TWT_data_ensemble TWT_bias TWT_data_pu TWT_data_u];
    HF_store = [HeatFluxData(1,1).data(:,1:3) Time_HF_avg...
        HF_data_ensemble HF_bias HF_data_pu HF_data_u];
    PS_store = [PS_VoltageData(1,1).data(:,1:3) zeros(3,1)...
        PS_data_ensemble PS_data_pu];
    dP_store = [Time_dP_avg dP_data_ensemble dP_data_pu];
    
    % If save_data is 1 or 3 the boundary locations and statistical data
    % will be saved as .csv files in column format.  Save_data can be
    % changed at the top of the code.
    if save_data == 1 || save_data == 3
        % This is the array for the headers of the table to be saved
        header = {'X', 'Y', 'Z', 't','Ensemble', 'B', 'P', 'U'};
        PS_header = {'X', 'Y', 'Z', 't','Ensemble', 'P'};
        dP_header = {'t', 'Ensemble', 'P'};
        
        % Saves the store arrays with the headers  in the folder selected
        % as a .csv file.
        csvwrite_with_headers([folder '\HeatedWallTempEnsembleBPU.csv'],HWT_store,header)
        csvwrite_with_headers([folder '\InletTempEnsembleBPU.csv'],IT_store,header)
        csvwrite_with_headers([folder '\LeftWallTempEnsembleBPU.csv'],LWT_store,header)
        csvwrite_with_headers([folder '\RightWallTempEnsembleBPU.csv'],RWT_store,header)
        csvwrite_with_headers([folder '\TopWallTempEnsembleBPU.csv'],TWT_store,header)
        csvwrite_with_headers([folder '\HeatFluxEnsembleBPU.csv'],HF_store,header)
        csvwrite_with_headers([folder '\PS_VoltageEnsembleP.csv'],PS_store,PS_header)
        csvwrite_with_headers([folder '\dPEnsembleP.csv'],dP_store,dP_header)
    end
    
    % If save_data is 2 or 3 the boundary locations and statistical data
    % will be saved as .csv files in row format which can be used for CFD
    % analysis.
    if save_data == 2 || save_data == 3
        % Initialize the arrays for the row style matrix
        HWT2_store = NaN(HWT_data_per_sample,3+(HWT_total/HWT_data_per_sample-i_t0+1)*4);
        IT2_store = NaN(IT_data_per_sample,3+(IT_total/IT_data_per_sample-i_t0+1)*4);
        LWT2_store = NaN(LWT_data_per_sample,3+(LWT_total/LWT_data_per_sample-i_t0+1)*4);
        RWT2_store = NaN(RWT_data_per_sample,3+(RWT_total/RWT_data_per_sample-i_t0+1)*4);
        TWT2_store = NaN(TWT_data_per_sample,3+(TWT_total/TWT_data_per_sample-i_t0+1)*4);
        HF2_store = NaN(HF_data_per_sample,3+(HF_total/HF_data_per_sample-i_t0+1)*4);
        
        % This stores the x, y, and z locations for the row style matrix.
        % 1:3 corresponds to the x, y, and z locations.
        HWT2_store(1:HWT_data_per_sample,1:3) = HWT_store(1:HWT_data_per_sample,1:3);
        IT2_store(1:IT_data_per_sample,1:3) = IT_store(1:IT_data_per_sample,1:3);
        LWT2_store(1:LWT_data_per_sample,1:3) = LWT_store(1:LWT_data_per_sample,1:3);
        RWT2_store(1:RWT_data_per_sample,1:3) = RWT_store(1:RWT_data_per_sample,1:3);
        TWT2_store(1:TWT_data_per_sample,1:3) = TWT_store(1:TWT_data_per_sample,1:3);
        HF2_store(1:HF_data_per_sample,1:3) = HF_store(1:HF_data_per_sample,1:3);
        
        % These for loops rearrange the data from HWT_store.  The data is
        % rearanged so column 1 after the x, y, z locations is the ensemble
        % average of all the locations at time = 0.  The next column is the
        % ensemble average of the next time step until all the averages are
        % stored.  The next set of columns will be the standard deviation at time =
        % 0. This is repeated for the bias, precision, and uncertainty.
        % j is 5:9 because those are the columns where ensemble, standard
        % deviation, bias, precision, and uncertainty are stored.
        % HeatedWallTemp
        for j = 5:8
            for i = 1:HWT_total/HWT_data_per_sample-i_t0+1
                HWT2_store(1:HWT_data_per_sample,3+i+(j-5)*(HWT_total/HWT_data_per_sample-i_t0+1)) = ...
                    HWT_store(1+HWT_data_per_sample*(i-1+i_t0-1):HWT_data_per_sample*(i+i_t0-1),j);
            end
        end
        
        % InletTemp
        for j = 5:8
            for i = 1:IT_total/IT_data_per_sample-i_t0+1
                IT2_store(1:IT_data_per_sample,3+i+(j-5)*(IT_total/IT_data_per_sample-i_t0+1)) = ...
                    IT_store(1+IT_data_per_sample*(i-1+i_t0-1):IT_data_per_sample*(i+i_t0-1),j);
            end
        end
        
        % LeftWallTemp
        % RightWallTemp
        % TopWallTemp
        for j = 5:8
            for i = 1:LWT_total/LWT_data_per_sample-i_t0+1
                LWT2_store(1:LWT_data_per_sample,3+i+(j-5)*(LWT_total/LWT_data_per_sample-i_t0+1)) = ...
                    LWT_store(1+LWT_data_per_sample*(i-1+i_t0-1):LWT_data_per_sample*(i+i_t0-1),j);
                RWT2_store(1:RWT_data_per_sample,3+i+(j-5)*(RWT_total/RWT_data_per_sample-i_t0+1)) = ...
                    RWT_store(1+RWT_data_per_sample*(i-1+i_t0-1):RWT_data_per_sample*(i+i_t0-1),j);
                TWT2_store(1:TWT_data_per_sample,3+i+(j-5)*(TWT_total/TWT_data_per_sample-i_t0+1)) = ...
                    TWT_store(1+TWT_data_per_sample*(i-1+i_t0-1):TWT_data_per_sample*(i+i_t0-1),j);
            end
        end
        
        % HeatFlux
        for j = 5:8
            for i = 1:HWT_total/HWT_data_per_sample-i_t0+1
                HF2_store(1:HF_data_per_sample,3+i+(j-5)*(HF_total/HF_data_per_sample-i_t0+1)) = ...
                    HF_store(1+HF_data_per_sample*(i-1+i_t0-1):HF_data_per_sample*(i+i_t0-1),j);
            end
        end
        
        % Power supply
        PS2_store = [PS_store(:,1) PS_store(:,2) PS_store(:,3) PS_store(:,5) PS_store(:,6)];
        % Change in pressure
        dP2_store = [0 dP_store(i_t0:end,2)' dP_store(i_t0:end,3)'];
        
        % The following variables are to store the headers for the row
        % arranged data that was made above.  This step labels the first 
        % three columns X, Y, and Z.
        HWT2_header = 'X,Y,Z';
        IT2_header = 'X,Y,Z';
        LWT2_header = 'X,Y,Z';
        RWT2_header = 'X,Y,Z';
        TWT2_header = 'X,Y,Z';
        HF2_header = 'X,Y,Z';
        dP2_header = 'X';
        
        % HWT2_header and the other 2_header variables are concatenated
        % here with Ensemble average at t = 0 to follow X, Y, Z.  The
        % ensemble average at the next time step is stored and so on until
        % all the ensemble average with the respective time is stored.
        % This is then repeated for standard deviation, bias, precision,
        % and uncertianty.
        % HeatedWallTemp
        for j = 1:4
            for i = 1:HWT_total/HWT_data_per_sample-i_t0+1
                if j == 1
                    HWT2_header = [HWT2_header,...
                        ',T(K)[t=' ,num2str(Time_HWT_avg(1+HWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 2
                    HWT2_header = [HWT2_header,...
                        ',B(K)[t=' ,num2str(Time_HWT_avg(1+HWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 3
                    HWT2_header = [HWT2_header,...
                        ',S(K)[t=' ,num2str(Time_HWT_avg(1+HWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 4
                    HWT2_header = [HWT2_header,...
                        ',U(K)[t=' ,num2str(Time_HWT_avg(1+HWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                end
            end
        end
        
        % InletTemp
        for j = 1:4
            for i = 1:IT_total/IT_data_per_sample-i_t0+1
                if j == 1
                    IT2_header = [IT2_header,...
                        ',T(K)[t=' ,num2str(Time_IT_avg(1+IT_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 2
                    IT2_header = [IT2_header,...
                        ',B(K)[t=' ,num2str(Time_IT_avg(1+IT_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 3
                    IT2_header = [IT2_header,...
                        ',S(K)[t=' ,num2str(Time_IT_avg(1+IT_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 4
                    IT2_header = [IT2_header,...
                        ',U(K)[t=' ,num2str(Time_IT_avg(1+IT_data_per_sample*(i-1+i_t0-1))),'s]'];
                end
            end
        end
        
        % LeftWallTemp
        % RightWallTemp
        % TopWallTemp
        % These can all be done in the same set of for loops because
        % LWT_total = RWT_total = TWT_total and LWT_data_per_sample =
        % RWT_data_per_sample = TWT_data_per_sample.
        for j = 1:4
            for i = 1:LWT_total/LWT_data_per_sample-i_t0+1
                if j == 1
                    LWT2_header = [LWT2_header,...
                        ',T(K)[t=' ,num2str(Time_LWT_avg(1+LWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                    RWT2_header = [RWT2_header,...
                        ',T(K)[t=' ,num2str(Time_RWT_avg(1+RWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                    TWT2_header = [TWT2_header,...
                        ',T(K)[t=' ,num2str(Time_TWT_avg(1+TWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 2
                    LWT2_header = [LWT2_header,...
                        ',B(K)[t=' ,num2str(Time_LWT_avg(1+LWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                    RWT2_header = [RWT2_header,...
                        ',B(K)[t=' ,num2str(Time_RWT_avg(1+RWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                    TWT2_header = [TWT2_header,...
                        ',B(K)[t=' ,num2str(Time_TWT_avg(1+TWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 3
                    LWT2_header = [LWT2_header,...
                        ',S(K)[t=' ,num2str(Time_LWT_avg(1+LWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                    RWT2_header = [RWT2_header,...
                        ',S(K)[t=' ,num2str(Time_RWT_avg(1+RWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                    TWT2_header = [TWT2_header,...
                        ',S(K)[t=' ,num2str(Time_TWT_avg(1+TWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 4
                    LWT2_header = [LWT2_header,...
                        ',U(K)[t=' ,num2str(Time_LWT_avg(1+LWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                    RWT2_header = [RWT2_header,...
                        ',U(K)[t=' ,num2str(Time_RWT_avg(1+RWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                    TWT2_header = [TWT2_header,...
                        ',U(K)[t=' ,num2str(Time_TWT_avg(1+TWT_data_per_sample*(i-1+i_t0-1))),'s]'];
                end
            end
        end
        
        % HeatFlux
        for j = 1:4
            for i = 1:HF_total/HF_data_per_sample-i_t0+1
                if j == 1
                    HF2_header = [HF2_header,...
                        ',q"(W/m^2)[t=' ,num2str(Time_HF_avg(1+HF_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 2
                    HF2_header = [HF2_header,...
                        ',B(W/m^2)[t=' ,num2str(Time_HF_avg(1+HF_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 3
                    HF2_header = [HF2_header,...
                        ',S(W/m^2)[t=' ,num2str(Time_HF_avg(1+HF_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 4
                    HF2_header = [HF2_header,...
                        ',U(W/m^2)[t=' ,num2str(Time_HF_avg(1+HF_data_per_sample*(i-1+i_t0-1))),'s]'];
                end
            end
        end
        
        % PowerSupply
        PS2_header = ['X', ',Y', ',Z', ',E(V)[t=0]', ',P(V)[t=0]'];
        
        % Change in pressure
        for j = 1:2
            for i = 1:dP_total-i_t0+1
                if j == 1
                    dP2_header = [dP2_header,...
                        ',P(mmHg)[t=' ,num2str(Time_dP_avg(1+dP_data_per_sample*(i-1+i_t0-1))),'s]'];
                elseif j == 2
                    dP2_header = [dP2_header,...
                        ',P_S(mmHg)[t=' ,num2str(Time_dP_avg(1+dP_data_per_sample*(i-1+i_t0-1))),'s]'];
                end
            end
        end
        
        % These funcitons write the row style matrices to .csv files.
        csvwrite_with_headers([folder '\HeatedWallTemp.csv'],HWT2_store,strsplit(HWT2_header,','))
        csvwrite_with_headers([folder '\InletTemp.csv'],IT2_store,strsplit(IT2_header,','))
        csvwrite_with_headers([folder '\LeftWallTemp.csv'],LWT2_store,strsplit(LWT2_header,','))
        csvwrite_with_headers([folder '\RightWallTemp.csv'],RWT2_store,strsplit(RWT2_header,','))
        csvwrite_with_headers([folder '\TopWallTemp.csv'],TWT2_store,strsplit(TWT2_header,','))
        csvwrite_with_headers([folder '\HeatFlux.csv'],HF2_store,strsplit(HF2_header,','))
        csvwrite_with_headers([folder '\PS_Voltage.csv'],PS2_store,strsplit(PS2_header,','))
        csvwrite_with_headers([folder '\dP.csv'],dP2_store,strsplit(dP2_header,','))
    end
end
toc;
%% Plot
% Stores the user's monitor size to prevent cascading from displaying graphs
% off the screen
scrsize = get(0,'ScreenSize');

if plot_all == 1
    % Plots the spatial average for each case as well as the ensemble average
    % for all cases.  Multiple plots will be displayed if data was taken for
    % multiple positions.
    % HeatedWallTemp plot
    for l = 1:N_pos
        for k = 1:N_trans(l)
            figure(l); plot(time_array,HWT_avg(1,:,k,l));
            % Cascades figures to make it more simple to move. 
            % Forces plot to be 576 by 512 pixels (default) and will cascade depending on the user's screen
            % size and the number of plots.
            set(figure(l),'Position',[16+((l-1)*floor((scrsize(3)-576)/(N_pos))),...
                (scrsize(4)-604)-((l-1)*floor((scrsize(4)-604)/(N_pos))),576,512])
            hold all
        end
        plot(time_array,HWT_ensemble(1,:,1,l),'k','LineWidth',3); hold on;
        % Errorbars only plot every 5th point beginning with the third point.
        % It begins on the third so the scale doesn't show anything to the left
        % of time = 0s.  Plotting every point makes it difficult to see the
        % other plots.
        errorbar(time_array(3:5:end),HWT_ensemble(1,3:5:end,1,l),HWT_avg_u(:,3:5:end,1,l),'k.')
        xlabel('Time [s]'); ylabel('Temperature [C]'); title(['Heated Wall Temperature ',folder_names(l,1)])
        if N_trans(l) < 15; legend('location','best'); end
        hold off
    end
    
    % InletTemp plot
    input('Press enter to clear Heated Wall Temp plots and plot Inlet Temp ');
    for l = 1:N_pos
        for k = 1:N_trans(l)
            figure(l); plot(time_array,IT_avg(1,:,k,l)); hold all
        end
        plot(time_array,IT_ensemble(1,:,1,l),'k','LineWidth',3); hold on
        errorbar(time_array(3:5:end),IT_ensemble(1,3:5:end,1,l),IT_avg_u(:,3:5:end,1,l),'k.')
        xlabel('Time [s]'); ylabel('Temperature [C]'); title(['Inlet Temperature ',folder_names(l,1)])
        if N_trans(l) < 15; legend('location','best'); end
        hold off
    end
    
    % LeftWallTemp plot
    input('Press enter to clear Inlet Temp plots and plot Left Wall Temp ');
    for l = 1:N_pos
        for k = 1:N_trans(l)
            figure(l); plot(time_array,LWT_avg(1,:,k,l)); hold all
        end
        plot(time_array,LWT_ensemble(1,:,1,l),'k','LineWidth',3); hold on
        errorbar(time_array(3:5:end),LWT_ensemble(1,3:5:end,1,l),LWT_avg_u(:,3:5:end,1,l),'k.')
        xlabel('Time [s]'); ylabel('Temperature [C]'); title(['Left Wall Temperature ',folder_names(l,1)])
        if N_trans(l) < 15; legend('location','best'); end
        hold off
    end
    
    % RightWallTemp plot
    input('Press enter to clear Left Wall Temp plots and plot Right Wall Temp ');
    for l = 1:N_pos
        for k = 1:N_trans(l)
            figure(l); plot(time_array,RWT_avg(1,:,k,l)); hold all
        end
        plot(time_array,RWT_ensemble(1,:,1,l),'k','LineWidth',3); hold on
        errorbar(time_array(3:5:end),RWT_ensemble(1,3:5:end,1,l),RWT_avg_u(:,3:5:end,1,l),'k.')
        xlabel('Time [s]'); ylabel('Temperature [C]'); title(['Right Wall Temperature ',folder_names(l,1)])
        if N_trans(l) < 15; legend('location','best'); end
        hold off
    end
    
    % TopWallTemp plot
    input('Press enter to clear Right Wall Temp plots and plot Top Wall Temp ');
    for l = 1:N_pos
        for k = 1:N_trans(l)
            figure(l); plot(time_array,TWT_avg(1,:,k,l)); hold all
        end
        plot(time_array,TWT_ensemble(1,:,1,l),'k','LineWidth',3); hold on
        errorbar(time_array(3:5:end),TWT_ensemble(1,3:5:end,1,l),TWT_avg_u(:,3:5:end,1,l),'k.')
        xlabel('Time [s]'); ylabel('Temperature [C]'); title(['Top Wall Temperature ',folder_names(l,1)])
        if N_trans(l) < 15; legend('location','best'); end
        hold off
    end
    
    % HeatFlux plot
    input('Press enter to clear Top Wall Temp plots and plot Heat Flux ');
    for l = 1:N_pos
        for k = 1:N_trans(l)
            figure(l); plot(time_array,HF_avg(1,:,k,l)); hold all
        end
        plot(time_array,HF_ensemble(1,:,1,l),'k','LineWidth',3); hold on
        errorbar(time_array(3:5:end),HF_ensemble(1,3:5:end,1,l),HF_avg_u(:,3:5:end,1,l),'k.')
        xlabel('Time [s]'); ylabel('Heat Flux [W/m^2]'); title(['Heat Flux ',folder_names(l,1)])
        if N_trans(l) < 15; legend('location','best'); end
        hold off
    end
    input('Press enter to clear Heat Flux plots and plot the change in Pressure (dP)');
    
    % Change in Pressure (dP) plot
    for l = 1:N_pos
        for k = 1:N_trans(l)
            figure(l); plot(time_array,dP(1,:,k,l)); hold all
        end
        plot(time_array,dP_ensemble(1,:,1,l),'k','LineWidth',3); hold on
        errorbar(time_array(3:5:end),dP_ensemble(1,3:5:end,1,l),dP_pu(:,3:5:end,1,l),'k.')
        xlabel('Time [s]'); ylabel('Change in Pressure [mmHg]'); title(['dP ',folder_names(l,1)])
        if N_trans(l) < 15; legend('location','best'); end
        hold off
    end
    input('Press enter to clear Pressure change plots and plot Power Supply Voltage');
    
    % Power Supply plot
    for l = 1:N_pos
        for i = 1:3
            figure(l); plot(PS_case(1:N_trans(l)),PS(i,1:N_trans(l),l),'.'); hold all
        end
        plot(PS_case(1:N_trans(l)),PS_avg(1,1:N_trans(l),l),'k*','LineWidth',2)
        xlabel('Case'); ylabel('Voltage [V]'); title(['Power Supply Voltage ',folder_names(l,1)])
        legend('X = 0.162','X = 0.778','X = 1.394','Average','location','best')
        hold off
    end
    input('Press enter to clear Pressure change plots and plot the Full Ensemble Averages');
    close all
end

% Plots the ensemble averages of all cases along with a full average of the
% ensemble averages.  The bolded line is the full ensemble average.
% Heated Wall Temp.  If save_figure variable == 1 then these figures will
% be saved in the selected file.
legend_names2 = [folder_names' char('ensemble')];

% HeatedWallTemp
for l = 1:N_pos
    figure(1); plot(time_array,HWT_ensemble(1,:,1,l))
    % Cascades the figures to make it more simple to move.
    % Forces plot to be 576 by 512 pixels(default) and will cascade depending on the user's screen size and
    % the number of plots.
    set(figure(1),'Position',[16+((1-1)*floor((scrsize(3)-576)/(7))),...
        (scrsize(4)-604)-((1-1)*floor((scrsize(4)-604)/(7))),576,512])
    hold all
end
plot(time_array,HWT_full_ensemble(1,:,1,1),'k','LineWidth',3); hold on
    % Errorbars only plot every 5th point beginning with the third point.
    % It begins on the third so the scale doesn't show anything to the left
    % of time = 0s.  Plotting every point makes it difficult to see the
    % other plots.  Only plots the errorbars if there is more than one
    % position folder.  
if N_pos > 1; 
    errorbar(time_array(3:5:end),HWT_full_ensemble(1,(3:5:end),1,1),HWT_ensemble_u(3:5:end),'k.'); 
end
xlabel('Time [s]'); ylabel('Temperature [C]'); title('Heated Wall Temperature Ensemble')
if save_figure == 1 && Scan_plots ~= 2
    saveas(figure(1),[folder,'\HeatedWallTemp'],'png')
end
legend(legend_names2,'location', 'best')
hold off

% InletTemp
for l = 1:N_pos
    figure(2); plot(time_array,IT_ensemble(1,:,1,l))
    set(figure(2),'Position',[16+((2-1)*floor((scrsize(3)-576)/(7))),...
        (scrsize(4)-604)-((2-1)*floor((scrsize(4)-604)/(7))),576,512])
    hold all
end
plot(time_array,IT_full_ensemble(1,:,1,1),'k','LineWidth',3); hold on
if N_pos > 1; errorbar(time_array(3:5:end),IT_full_ensemble(1,(3:5:end),1,1),IT_ensemble_u(3:5:end),'k.'); end
xlabel('Time [s]'); ylabel('Temperature [C]'); title('Inlet Temperature Ensemble')
if save_figure == 1 && Scan_plots ~= 2
    saveas(figure(2),[folder,'\InletTemp'],'png')
end
legend(legend_names2,'location', 'best')
hold off

% LeftWallTemp
for l = 1:N_pos
    figure(3); plot(time_array,LWT_ensemble(1,:,1,l))
    set(figure(3),'Position',[16+((3-1)*floor((scrsize(3)-576)/(7))),...
        (scrsize(4)-604)-((3-1)*floor((scrsize(4)-604)/(7))),576,512])
    hold all
end
plot(time_array,LWT_full_ensemble(1,:,1,1),'k','LineWidth',3); hold on
if N_pos > 1; 
    errorbar(time_array(3:5:end),LWT_full_ensemble(1,(3:5:end),1,1),LWT_ensemble_u(3:5:end),'k.'); 
end
xlabel('Time [s]'); ylabel('Temperature [C]'); title('Left Wall Temperature Ensemble')
if save_figure == 1 && Scan_plots ~= 2
    saveas(figure(3),[folder,'\LeftWallTemp'],'png')
end
legend(legend_names2,'location', 'best')
hold off

% RightWallTemp
for l = 1:N_pos
    figure(4); plot(time_array,RWT_ensemble(1,:,1,l))
    set(figure(4),'Position',[16+((4-1)*floor((scrsize(3)-576)/(7))),...
        (scrsize(4)-604)-((4-1)*floor((scrsize(4)-604)/(7))),576,512])
    hold all
end
plot(time_array,RWT_full_ensemble(1,:,1,1),'k','LineWidth',3); hold on
if N_pos > 1; 
    errorbar(time_array(3:5:end),RWT_full_ensemble(1,(3:5:end),1,1),RWT_ensemble_u(3:5:end),'k.'); 
end
xlabel('Time [s]'); ylabel('Temperature [C]'); title('Right Wall Temperature Ensemble')
if save_figure == 1 && Scan_plots ~= 2
    saveas(figure(4),[folder,'\RightWallTemp'],'png')
end
legend(legend_names2,'location', 'best')
hold off

% TopWallTemp
for l = 1:N_pos
    figure(5); plot(time_array,TWT_ensemble(1,:,1,l))
    set(figure(5),'Position',[16+((5-1)*floor((scrsize(3)-576)/(7))),...
        (scrsize(4)-604)-((5-1)*floor((scrsize(4)-604)/(7))),576,512])
    hold all
end
plot(time_array,TWT_full_ensemble(1,:,1,1),'k','LineWidth',3); hold on
if N_pos > 1; 
    errorbar(time_array(3:5:end),TWT_full_ensemble(1,(3:5:end),1,1),TWT_ensemble_u(3:5:end),'k.'); 
end
xlabel('Time [s]'); ylabel('Temperature [C]'); title('Top Wall Temperature Ensemble')
if save_figure == 1 && Scan_plots ~= 2
    saveas(figure(5),[folder,'\TopWallTemp'],'png')
end
legend(legend_names2,'location', 'best')
hold off

% HeatFlux
for l = 1:N_pos
    figure(6); plot(time_array,HF_ensemble(1,:,1,l))
    set(figure(6),'Position',[16+((6-1)*floor((scrsize(3)-576)/(7))),...
        (scrsize(4)-604)-((6-1)*floor((scrsize(4)-604)/(7))),576,512])
    hold all
end
plot(time_array,HF_full_ensemble(1,:,1,1),'k','LineWidth',3); hold on
if N_pos > 1; errorbar(time_array(3:5:end),HF_full_ensemble(1,(3:5:end),1,1),HF_ensemble_u(3:5:end),'k.'); end
xlabel('Time [s]'); ylabel('[W/m^2]'); title('Heat Flux Ensemble')
if save_figure == 1 && Scan_plots ~= 2
    saveas(figure(6),[folder,'\HeatFlux'],'png')
end
legend(legend_names2,'location', 'best')
hold off

% Change in Pressure (dP)
for l = 1:N_pos
    figure(7); plot(time_array,dP_ensemble(1,:,1,l))
    set(figure(7),'Position',[16+((7-1)*floor((scrsize(3)-576)/(7))),...
        (scrsize(4)-604)-((7-1)*floor((scrsize(4)-604)/(7))),576,512])
    hold all
end
plot(time_array,dP_full_ensemble(1,:,1,1),'k','LineWidth',3); hold on
if N_pos > 1; 
    errorbar(time_array(3:5:end),dP_full_ensemble(1,(3:5:end),1,1),dP_ensemble_pu(3:5:end),'k.'); 
end
xlabel('Time [s]'); ylabel('Change in Pressure [mmHg]'); title('dP Ensemble')
if save_figure == 1 && Scan_plots ~= 2
    saveas(figure(7),[folder,'\dP'],'png')
end
legend(legend_names2,'location', 'best')
hold off

input('Press enter to clear the Full Ensemble Averages plots');
close all

%% Visual Data Testing
% 3D surface map of the temperature of the Heated wall.  The plot changes
% every 0.1 seconds to the next 0.1 second interval of the temperature
% plot.

% for i = 1:HWT_total/HWT_data_per_sample
%     surf_temp = ...
%       reshape(HeatedWallTempData(1,1).data(6+(i-1)*HWT_data_per_sample:i*HWT_data_per_sample,5),7,32);
%     surf_z = HeatedWallTempData(1,1).data(6:12,3);
%     surf_x = HeatedWallTempData(1,1).data(6:7:229,1);
% 
%     figure(1)
%     surf(surf_x,surf_z,surf_temp)
%     xlim([0, 2])
%     ylim([-0.2, 0.2])
%     zlim([380, 420])
%     set(gca,'plotboxaspectratio',[1 .4 1])
%     pause(0.1)
% end

% HeatedWallTemp spatial Average temp for z = -0.138, 0, and 0.138
% extremes_temp_z1 = (32,125);
% extremes_temp_z2 = zeros(32,125);
% extremes_temp_z3 = zeros(32,125);
% for i = 1:HWT_total/HWT_data_per_sample
%     extremes_temp_z1(:,i) = HeatedWallTempData(1,1).data(6+(i-1)*229:7:i*229,5);
%     extremes_temp_z2(:,i) = HeatedWallTempData(1,1).data(9+(i-1)*229:7:i*229,5);
%     extremes_temp_z3(:,i) = HeatedWallTempData(1,1).data(12+(i-1)*229:7:i*229,5);
% end
% extremes_temp_z1 = mean(extremes_temp_z1);
% extremes_temp_z2 = mean(extremes_temp_z2);
% extremes_temp_z3 = mean(extremes_temp_z3);
% extremes_time = (1:HWT_total/HWT_data_per_sample)*.1;
%
% % Figure(1) below is convenient for comparing three average temperatures
% % that have similar values
% figure(1)
% plot(extremes_time,extremes_temp_z1)
% hold on
% plot(extremes_time,extremes_temp_z2,'k')
% plot(extremes_time,extremes_temp_z3,'r')
% xlabel('Time [s]')
% ylabel('spatial Average Temperature [K]')
% title('Extreme vs. Centerline Temperature')
% legend('z = -0.138','z = 0','z = 0.138','location','best')
%
% % Figure 2, 3, and 4 below are convenient for plotting curves with greater
% % ranges of temperatures.  figure 2 will show one curve and figure 3 will
% % show 2 curves on the same plot with different axes.
% figure(2)
% plot(extremes_time,extremes_temp_z2,'k')
% figure(3)
% plotyy(extremes_time,extremes_temp_z1,extremes_time,extremes_temp_z3)
%
% % Figure  4 plots all three temperature averages on the same graph with two
% % different scales of axes.
% figure(4)
% [hAx,hLine1,hLine2] = ...
%   plotyy(extremes_time,extremes_temp_z2,[extremes_time',extremes_time'],...
%   [extremes_temp_z1',extremes_temp_z3']);