clear all;  close all;  clc;

% Profiles are taken from Stats.mat files for inlet data.

% Input - A Stats.mat file.  Run this code several times to append each of
% the input planes to the output files.

% Output - 1 *.xls file and 6 *.csv files.  The first includes one y-z
% plane at x=0 and includes x, y, z, u, v, w, u'u', v'v', w'w', Uu, Uv, Uw,
% Uu'u', Uv'v', Uu'v'.  The second file is the same plane and includes x,
% y, z, and epsilon.  The third file includes x, y, z, k (TKE).  The
% fourth file is the same as the first file.  The fifth file is the same
% as the first file, but at proloc+1.  The last is at proloc-1.
% This will allow an analyst to compute any relevant turbulence parameters
% at the inlet (specifically, epsilon).

% Requires csvwrite_with_headers at
% http://www.mathworks.com/matlabcentral/fileexchange/29933-csv-with-column-headers

% ----------------------------------------------
% Updates:

% Jul 11 2014 - Jeff Harris - Changed vxy2=uvt instead of uvuvt, Uuv_p,
% Uuv_m also corrected to use Uuv_plus and minus

% Jun 12 2014 - Jeff Harris - Improved comments

% May 22 2014 - Jeff Harris - Modified to output 6 .xls files.  The first includes one y-z
% plane at x=0 and includes x, y, z, u, v, w, u'u', v'v', w'w', Uu, Uv, Uw,
% Uu'u', Uv'v', Uu'v'.  The second file is the same plane and includes x,
% y, z, and epsilon.  The third file includes x, y, z, k (TKE).  The
% fourth file is the same as the first file.  The fifth file is the same
% as the first file, but at proloc+1.  The last is at proloc-1.

% Version 6_0:
% Sept. 15 2014 - Blake Lance - Removed BL variable
% Removed extra complications in the code, especially taking the 2D data from Stats, converting to 1D, then
% converting back to 2D.  This also removes the need to transpose and transpose data back.
% The epsilon calculations were updated with correct spatial definitions with small difference on the output.
% Updated plots with uncertainty bands
% Exported scalar values at the wall positions (these had previously not been specified)
% Corrected fliplr of Xshift when Opposed==1 and mask is assymetric
% Code now automatically checks if the output files exist and appends the data if they do
% Added headers to k and epsilon output files
% Code now automatically records the prolocs to a file
% epsilon is now calculated by an empirical correlation from a CFD text
% Code now smooths SD by a robust local regression, set by the option Smooth
% Write uncertainty of k and epsilon to file
% Sets the minus uncertainty of SD, k, and epsilon to no more than their magnitude since these values are
%   exclusively positive
% Assigns the z coordinate by the last character in the Serial string (1-9)

% Version 6_1:
% May 26 2015 - Blake Lake
% Corrected uncertainty on TKE as it was overestimated
% Added span on smoothing option

%% User Input
proloc = 36;        % j-index for epsilon profile (increasing from top)
Serial = '7.4.9';   % Name of the sheet to be used for the excel file for the Stats
Export = 0;         % Export data in *.csv files? (1 for Yes/0 for No)
ExportPlanes = 1;   % Additionally export data in three *.csv files (1 for Yes/0 for No)
Smooth = 0;         % Choose whether to smooth SD and subsequently k and epsilon
Span = 16;          % Percent of total number of data points (Span) for smoothing
Opposed = 1;        % For opposed orientation, switch y-coordinate direction when writing output
Nth = 5;            % Plot every Nth uncertainty band

% nu = 1.632E-5;  % kinematic viscosity at 1400 m elevation

fprintf('Select the MATLAB Stats file \n');
[f1,p] = uigetfile('.mat');  % Ask user to choose Stat file
tic;    load([p,f1]);

% Assign the z coordinate for the data plane based on the last number of Serial (1-9) [m]
if strcmp(Serial(1),'4') % z-array for series 4 data
    z_array = [0; 0.0762; 0.127; 0.13589; 0.14478; -0.14478; -0.13589; -0.127; -0.0762];
elseif strcmp(Serial(1),'6') || strcmp(Serial(1),'7') % z-array for series 6 and 7 data
    z_array = [-0.14478; -0.13589; -0.127; -0.0762; 0.0; 0.0762; 0.127; 0.13589; 0.14478];
end
z = z_array(str2double(Serial(end)));

%% Allocate and initialize

w = size(u_ave,2);
u = v_ave;  u(isnan(u)) = 0;    % v_ave in DaVis is u in global coordinates
v = u_ave;  v(isnan(v)) = 0;    % u_ave in DaVis is v in global coordinates
U_u = Uv;   U_u(isnan(U_u)) = 0;    % The time averaged u (global) velocity uncertainty
U_v = Uu;   U_v(isnan(U_v)) = 0;    % The time averaged v (global) velocity uncertainty
U_w = Uu;   U_w(isnan(U_w)) = 0;    % The time averaged w (global) velocity uncertainty, assumed U_w=U_v

u_SD = sqrt(vvt);   u_SD(isnan(u_SD)) = 0; % The standard deviation of u in global coordinates
v_SD = sqrt(uut);   v_SD(isnan(v_SD)) = 0;  % The standard deviation of v in global coordinates
SD = sqrt( u_SD.^2 + v_SD.^2 + v_SD.^2 ); % The magnitude of the standard deviation

uu = vvt;   uu(isnan(uu)) = 0;  % u'u' in global coordinates
vv = uut;   vv(isnan(vv)) = 0;  % v'v' in global coordinates
ww = uut;   ww(isnan(ww)) = 0;  % w'w' in global coordinates, assumed w'w'=v'v'
uv = uvt;   uv(isnan(uv)) = 0;  % u'v' in global coordinates

U_uu_p = Uvv_plus;  U_uu_p(isnan(U_uu_p)) = 0;  % Plus uncertainty of u'u' in global coordinates
U_uu_m = Uvv_minus; U_uu_m(isnan(U_uu_m)) = 0;  % Minus uncertainty of u'u' in global coordinates
U_vv_p = Uuu_plus;  U_vv_p(isnan(U_vv_p)) = 0;  % Plus uncertainty of v'v' in global coordinates
U_vv_m = Uuu_minus; U_vv_m(isnan(U_vv_m)) = 0;  % Minus uncertainty of v'v' in global coordinates
U_uv_p = Uuv_plus;  U_uv_p(isnan(U_uv_p)) = 0;  % Plus uncertainty of u'v' in global coordinates
U_uv_m = Uuv_minus; U_uv_m(isnan(U_uv_m)) = 0;  % Minus uncertainty of u'v' in global coordinates

U_SD_p = sqrt( U_uu_p + U_vv_p + U_vv_p );  % Plus uncertainty in the standard deviation
U_SD_m = sqrt( U_uu_m + U_vv_m + U_vv_m );  % Minus uncertainty in the standard deviation

du_dx = zeros(w,1);
du_dy = zeros(w,1);
dv_dx = zeros(w,1);
dv_dy = zeros(w,1);

if Opposed == 1 % Change the sign
    u = -u;
    v = -v;
end

%% Adaptive trimming to masked area
[imin,imax,jmin,jmax] = TrimToMask(u);

xlower = X(1,jmin);
xupper = X(1,jmax);
ylower = Y(imin,1);
yupper = Y(imax,1);

Xshift = X - xlower;    % Shift x_2D array so y=0 is at the left
xupper = xupper - xlower;   % Just changed this order
xlower = xlower - xlower;

%% Flip Xshift if Opposed==1, also shift values if the masked area isn't symmetric
if Opposed==1
    dx = abs( X(1,1) - X(1,2) );  % Spacing between vectors in mm given by DaVis
    Xshift = fliplr(Xshift);
    TrimR = w - jmax + 1;   TrimL = jmin;
    % Add or subtract values so assymetric Xshift will have consistent values
    Xshift = Xshift - dx*(TrimR - TrimL);
end

%% Calculate the Turbulence Kinetic Energy

if Smooth == 1; 
    SD(proloc,:) = smooth(SD(proloc,:),Span,'rlowess');
    uu(proloc,:) = smooth(uu(proloc,:),Span,'rlowess');
    vv(proloc,:) = smooth(vv(proloc,:),Span,'rlowess');
end;
for i=1:w   % Set the minus uncertainty of SD to SD so the uncertainty bands stay positive
    if U_SD_m(proloc,i) > SD(proloc,i);   U_SD_m(proloc,i) = SD(proloc,i);  end;
    if U_uu_m(proloc,i) > uu(proloc,i);   U_uu_m(proloc,i) = uu(proloc,i);  end;
    if U_vv_m(proloc,i) > vv(proloc,i);   U_vv_m(proloc,i) = vv(proloc,i);  end;
end

% TKE = 0.5*SD(proloc,:).^2;
% U_TKE_p = (SD(proloc,:).*U_SD_p(proloc,:))';
% U_TKE_m = (SD(proloc,:).*U_SD_m(proloc,:))';
TKE = 0.5*( uu(proloc,:) + vv(proloc,:) + vv(proloc,:) );
U_TKE_p = 0.5*sqrt( U_uu_p(proloc,:).^2 + U_vv_p(proloc,:).^2 + U_vv_p(proloc,:).^2 );
U_TKE_m = 0.5*sqrt( U_uu_m(proloc,:).^2 + U_vv_m(proloc,:).^2 + U_vv_m(proloc,:).^2 );
for i=1:w      % Set the minus uncertainty of k to k so the uncertainty bands stay positive
    if U_TKE_m(i) > TKE(i);   U_TKE_m(i) = TKE(i);  end;
end

%% Calculate epsilon based on gradients of fluctuating terms
% 
% % We realized that this section is not the correct method for epsilon.  The SD is a time-averaged quantity,
% % but the arguments in the gradients need to be instantaneous values.  The gradients should be calculated for
% % each measurement and time-averaged over the data set.

% dx = abs( X(1,1) - X(1,2) );  % Spacing between vectors in mm given by DaVis
% % dx = 304.8./(jmax-jmin+2);  % Spacing between vectors in mm given by geometry
% 
% du_dx(1:w) = 1000.*( u_SD(proloc+1,1:w) - u_SD(proloc-1,1:w) )/(2.*dx);
% dv_dx(1:w) = 1000.*( v_SD(proloc+1,1:w) - v_SD(proloc-1,1:w) )/(2.*dx);
% du_dy(2:w-1) = 1000.*( u_SD(proloc,3:w) - v_SD(proloc,1:w-2) )/(2.*dx);
% dv_dy(2:w-1) = 1000.*( v_SD(proloc,3:w) - v_SD(proloc,1:w-2) )/(2.*dx);
% du_dy(1) = 1000.*( -u_SD(proloc,3) + 4.*u_SD(proloc,2) - 3.*u_SD(proloc,1) )/(2.*dx);
% du_dy(w) = 1000.*( 3.*u_SD(proloc,w) - 4.*u_SD(proloc,w-1) + u_SD(proloc,w-2) )/(2.*dx);
% dv_dy(1) = 1000.*( -v_SD(proloc,3) + 4.*v_SD(proloc,2) - 3.*v_SD(proloc,1) )/(2.*dx);
% dv_dy(w) = 1000.*( 3.*v_SD(proloc,w) - 4.*v_SD(proloc,w-1) + v_SD(proloc,w-2) )/(2.*dx);
% 
% % Sharp and Adrian's approximation for the epsilon profile
% epsilon = nu*( 2*du_dx.^2 + 2*dv_dy.^2 + 3*du_dy.^2 + 3*dv_dx.^2 + 2*du_dy.*dv_dx );

%% Calculate epsilon on an empirical correlation

% An empirical equation for epsilon is given by Versteeg and Malalaskekera in "An Introduction to
% Computational Fluid Dynamics," page 77.
epsilon = 0.09^0.75*TKE.^1.5/(0.07*0.3048);
U_epsilon_p = 1.5*0.09^0.75*U_TKE_p.^0.5/(0.07*0.3048);
U_epsilon_m = 1.5*0.09^0.75*U_TKE_m.^0.5/(0.07*0.3048);
for i=1:w      % Set the minus uncertainty of epsilon to epsilon so the uncertainty bands stay positive
    if U_epsilon_m(i) > epsilon(i);   U_epsilon_m(i) = epsilon(i);  end;
end

%% Plotting

% Plot contour of velocity magnitude
figure('Name','Velocity Magnitude'); contourf(Xshift,Y,uv_ave,'edgecolor','none'); hold on;
plot([Xshift(proloc,1) Xshift(proloc,w)],[Y(proloc,1) Y(proloc,w)],'r');
uv_avemax = max(max(uv_ave(imax:imin,jmin:jmax)));    % Max value in plotted domain
uv_avemin = min(min(uv_ave(imax:imin,jmin:jmax)));    % Min value in plotted domain
h1 = colorbar('EastOutside');
caxis([uv_avemin uv_avemax]); %shading interp
set(get(h1,'ylabel'),'String','U [m/s]')
axis equal; axis([xlower,xupper,ylower,yupper])
xlabel('y [mm]');   ylabel('x [mm]');

% Plot contour of SD
figure('Name','Standard Deviation'); contourf(Xshift,Y,SD,'edgecolor','none'); hold on;
plot([Xshift(proloc,1) Xshift(proloc,w)],[Y(proloc,1) Y(proloc,w)],'r');
SDxymax = max(max(SD(imax:imin,jmin:jmax)));    % Max value in plotted domain
SDxymin = min(min(SD(imax:imin,jmin:jmax)));    % Min value in plotted domain
h1 = colorbar('EastOutside');
caxis([SDxymin SDxymax]); %shading interp
set(get(h1,'ylabel'),'String','S_U [m/s]')
axis equal; axis([xlower,xupper,ylower,yupper])
xlabel('y [mm]');   ylabel('x [mm]');

% Plot velocity magnitude profile
figure('Name','Velocity Profile');    plot(Xshift(proloc,:),u(proloc,:),'k','LineWidth',2); hold on;
errorbar(Xshift(proloc,1:Nth:end),u(proloc,1:Nth:end),U_u(proloc,1:Nth:end),'.','MarkerSize',0.1);
axis tight; xlim([xlower xupper]);  xlabel('y [mm]');   ylabel('u [m/s]');

% Plot SD profile
figure('Name','Standard Deviation Profile'); plot(Xshift(proloc,:),SD(proloc,:),'k','LineWidth',2);   hold on;
if Smooth == 1
    SD_orig = sqrt( u_SD(proloc,:).^2 + v_SD(proloc,:).^2 + v_SD(proloc,:).^2 );
    plot(Xshift(proloc,:),SD_orig,'r','LineWidth',2);
    legend('Smoothed S_U','Original S_U','location','best');
end
errorbar(Xshift(proloc,1:Nth:end),SD(proloc,1:Nth:end),U_SD_m(proloc,1:Nth:end),U_SD_p(proloc,1:Nth:end),...
    '.','MarkerSize',0.1);
axis tight; xlim([xlower xupper]);  xlabel('y [mm]');   ylabel('S_U [m/s]');

% Plot TKE profile
figure('Name','TKE Profile'); plot(Xshift(proloc,:),TKE,'k','LineWidth',2);  hold on;
if Smooth == 1
    TKE_orig = 0.5*SD_orig.^2;
    plot(Xshift(proloc,:), TKE_orig,'r','LineWidth',2);
    legend('Smoothed k','Original k','location','best');
end
errorbar(Xshift(proloc,1:Nth:end),TKE(1:Nth:end),U_TKE_m(1:Nth:end),U_TKE_p(1:Nth:end),...
    '.','MarkerSize',0.1);
axis tight; xlim([xlower xupper]);  xlabel('y [mm]');   ylabel('k [m^2/s^2]');

% Plot epsilon profile
figure('Name','Epsilon Profile'); plot(Xshift(proloc,:),epsilon,'k','LineWidth',2); hold on;
if Smooth == 1
    epsilon_orig = 0.09^0.75*TKE_orig.^1.5/(0.07*0.3048);
    plot(Xshift(proloc,:), epsilon_orig,'r','LineWidth',2);
    legend('Smoothed \epsilon','Original \epsilon','location','best');
end
errorbar(Xshift(proloc,1:Nth:end),epsilon(1:Nth:end),U_epsilon_m(1:Nth:end),U_epsilon_p(1:Nth:end),...
    '.','MarkerSize',0.1);
axis tight; xlim([xlower xupper]);  xlabel('y [mm]');   ylabel('\epsilon [m^2/s^3]');

if Export == 1

    %% Write u, v, and w to *.csv files
    
    M_Uxy = zeros(3,jmax-jmin+1,21);
%     N_Uxy = zeros(3,jupper-jlower+1,21);
    
    xspace = 0.001*(Y(proloc,2)-Y(proloc+1,2));
    M_Uxy(1,:,1) = -xspace;
    M_Uxy(3,:,1) = xspace;
    
    for i=1:3
        plane = [1;0;-1];
        
        M_Uxy(i,:,2) = 0.001*Xshift(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,3) = z;
        M_Uxy(i,:,4) = u(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,5) = v(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,6) = v(proloc+plane(i),jmin:jmax);  % Assume w=v
        M_Uxy(i,:,7) = uu(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,8) = vv(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,9) = ww(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,10) = uv(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,11) = U_u(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,12) = U_v(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,13) = U_w(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,14) = U_uu_p(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,15) = U_uu_m(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,16) = U_vv_p(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,17) = U_vv_m(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,18) = U_uv_p(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,19) = U_uv_m(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,20) = U_vv_p(proloc+plane(i),jmin:jmax);
        M_Uxy(i,:,21) = U_vv_m(proloc+plane(i),jmin:jmax);
        
        % dlmwrite('Inlet-uvw.csv',M_Uxy,'delimiter',',','precision',4,'-append')
        
        % Commented the following lines to try and not use the cross profiles
        % N_Uxy(i,:,2) = z+0.1525;     % Swap y and z coordinates and write profiles again
        % N_Uxy(i,:,3) = 0.001*x_2D(proloc+plane(i),jlower+1:jupper+1)-0.1525;
        % %N_Uxy(:,3) = 0.001*(dx:dx:305.-dx)-0.1525;  % Spread by geometry
        % N_Uxy(i,:,4) = u(proloc+plane(i),jlower:jupper);  % v from DaVis is actually u in Star
        % N_Uxy(i,:,5) = v(proloc+plane(i),jlower:jupper);  % u from DaVis is actually v in Star
        % N_Uxy(i,:,6) = v(proloc+plane(i),jlower:jupper);  % we must assume that w=v
        % N_Uxy(i,:,7) = uu(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,8) = vv(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,9) = ww(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,10) = uv(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,11) = U_u(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,12) = U_v(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,13) = U_w(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,14) = U_uu_p(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,15) = U_uu_m(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,16) = U_vv_p(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,17) = U_vv_m(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,18) = U_uv_p(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,19) = U_uv_m(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,20) = U_vv_p(proloc+plane(i),jlower:jupper);
        % N_Uxy(i,:,21) = U_vv_m(proloc+plane(i),jlower:jupper);
    end
    M_Uxy_1(:,:) = M_Uxy(2,:,:);
    % N_Uxy_1(:,:) = N_Uxy(2,:,:);
    M_Uxy_2(:,:) = M_Uxy(1,:,:);
    % N_Uxy_2(:,:) = N_Uxy(1,:,:);
    M_Uxy_3(:,:) = M_Uxy(3,:,:);
    % N_Uxy_3(:,:) = N_Uxy(3,:,:);
    
    NewFile = exist('Inlet-uvw.csv','file');    % Check if the file exists
    % Write the one plane file
    if NewFile == 0 % File does not exist
        csvwrite_with_headers('Inlet-uvw.csv',M_Uxy_1,{'X', 'Y', 'Z', 'u', 'v',...
            'w', 'u`u`', 'v`v`', 'w`w`','u`v`', 'Uu', 'Uv', 'Uw', 'Uuup', 'Uuum',...
            'Uvvp' ,'Uvvm' ,'Uuvp', 'Uuvm','Uwwp', 'Uwwm'})
    else
        dlmwrite('Inlet-uvw.csv',M_Uxy_1,'delimiter',',','precision',4,'-append')
    end
    
    if ExportPlanes==1  % Write three files that contain all three planes
        if NewFile == 0 % File does not exist, write the header
            csvwrite_with_headers('Inlet-uvw3plane1.csv',M_Uxy_1,{'X', 'Y', 'Z', 'u', 'v',...
                'w', 'u`u`', 'v`v`', 'w`w`','u`v`', 'Uu', 'Uv', 'Uw', 'Uuup', 'Uuum',...
                'Uvvp' ,'Uvvm' ,'Uuvp', 'Uuvm','Uwwp', 'Uwwm'})
            csvwrite_with_headers('Inlet-uvw3plane2.csv',M_Uxy_2,{'X', 'Y', 'Z', 'u', 'v',...
                'w', 'u`u`', 'v`v`', 'w`w`','u`v`', 'Uu', 'Uv', 'Uw', 'Uuup', 'Uuum',...
                'Uvvp' ,'Uvvm' ,'Uuvp', 'Uuvm','Uwwp', 'Uwwm'})
            csvwrite_with_headers('Inlet-uvw3plane3.csv',M_Uxy_3,{'X', 'Y', 'Z', 'u', 'v',...
                'w', 'u`u`', 'v`v`', 'w`w`','u`v`', 'Uu', 'Uv', 'Uw', 'Uuup', 'Uuum',...
                'Uvvp' ,'Uvvm' ,'Uuvp', 'Uuvm','Uwwp', 'Uwwm'})
        else    % Don't write the header
            dlmwrite('Inlet-uvw3plane1.csv',M_Uxy_1,'delimiter',',','precision',4,'-append')
            dlmwrite('Inlet-uvw3plane2.csv',M_Uxy_2,'delimiter',',','precision',4,'-append')
            dlmwrite('Inlet-uvw3plane3.csv',M_Uxy_3,'delimiter',',','precision',4,'-append')
        end
    end
    
    %% Write k to *.csv file
    
    % Write the measured profile
    K1 = zeros(jmax-jmin+1,6);
    K1(:,2) = 0.001*Xshift(proloc,jmin:jmax);
    K1(:,3) = z;
    K1(:,4) = TKE(jmin:jmax);
    K1(:,5) = U_TKE_p(jmin:jmax);
    K1(:,6) = U_TKE_m(jmin:jmax);
    if NewFile == 0
        csvwrite_with_headers('Inlet-k.csv',K1,{'X', 'Y', 'Z', 'k', 'U_k_p', 'U_k_m'});
    else
        dlmwrite('Inlet-k.csv',K1,'delimiter',',','precision',4,'-append')
    end
    
%     % Write the measured profile to an orthogonal plane
%     K2 = zeros(jmax-jmin+1,4);
%     K2(:,2) = z+0.1524;
%     K2(:,3) = 0.001*Xshift(proloc,jmin+1:jmax+1)-0.1524;
%     K2(:,4) = TKE(jmin:jmax);
%     dlmwrite('Inlet-k.csv',K2,'delimiter',',','precision',4,'-append')
    
    %% Write epsilon to *.csv file
    
    % Write the measured profile
    eps1 = zeros(jmax-jmin+1,6);
    eps1(:,2) = 0.001*Xshift(proloc,jmin:jmax);
    eps1(:,3) = z;
    eps1(:,4) = epsilon(jmin:jmax);
    eps1(:,5) = U_epsilon_p(jmin:jmax);
    eps1(:,6) = U_epsilon_m(jmin:jmax);
    if NewFile == 0
        csvwrite_with_headers('Inlet-epsilon.csv',eps1,{'X', 'Y', 'Z', 'epsilon', 'U_eps_p', 'U_eps_m'});
    else
        dlmwrite('Inlet-epsilon.csv',eps1,'delimiter',',','precision',4,'-append')
    end
    
%     % Write the measured profile to an orthogonal plane
%     eps2 = zeros(jmax-jmin+1,4);
%     eps2(:,2) = z+0.1524;
%     eps2(:,3) = 0.001*Xshift(proloc,jmin+1:jmax+1)-0.1524;
%     eps2(:,4) = epsilon(jmin:jmax);
%     dlmwrite('Inlet-epsilon.csv',eps2,'delimiter',',','precision',4,'-append')
    
    %% Write ProfilesFromStats.xlsx file
    % Writes data to different sheets
    
    % Write Header
    xlswrite('ProfilesFromStats.xlsx',{'Position' 'u_ave' 'Uu' 'uut' ...
        'Uuu_plus' 'v_ave' 'Uv' 'vvt' 'Uvv_plus' 'uvt' 'Uuv_plus' 'Position2'...
        'u_ave2' 'Uu2' 'uut2' 'Uuu_plus2' 'v_ave2' 'Uv2' 'vvt2' 'Uvv_plus2'...
        'uvt2' 'Uuv_plus2'},Serial,'A1:V1'), warning off MATLAB:xlswrite:AddSheet;
    
    % Write the Offset equation for skipping uncertainty bands with Tecplot
    xlswrite('ProfilesFromStats.xlsx',{'=OFFSET(A$2,5*(ROW()-2),0)'},Serial,'L2:L2'),...
        warning off MATLAB:xlswrite:AddSheet;
    
    profilewhole=[Xshift(proloc,jmin:jmax)',u(proloc,jmin:jmax)',U_u(proloc,jmin:jmax)',...
        uu(proloc,jmin:jmax)',U_uu_p(proloc,jmin:jmax)',v(proloc,jmin:jmax)',...
        U_v(proloc,jmin:jmax)',vv(proloc,jmin:jmax)',U_vv_p(proloc,jmin:jmax)',...
        uv(proloc,jmin:jmax)',U_uv_p(proloc,jmin:jmax)'];
    
    % Write the data
    xlswrite('ProfilesFromStats.xlsx',profilewhole,Serial,['A2:K',int2str(jmax-jmin+2)]),...
        warning off MATLAB:xlswrite:AddSheet;
    
    %% Write the prolocs (Profile Locations) to a file

    fid = fopen('Inlet-prolocs.csv','at');
    if NewFile == 0 % If file doesn't exist, write the header
        fprintf(fid,'Stats File,Serial,z [m],proloc,Smoothed \n');
    end
    fprintf(fid,[f1,',',Serial,',',num2str(z),',',int2str(proloc),',',int2str(Smooth),'\n']);
    fclose(fid);
end

toc;