clear all;  close all;  clc;

% Profiles are taken from Stats.mat files for inlet data.

% Input - A Stats.mat file.  Run this code several times to append each of
% the input planes to the output files.

% Output - 1 *.xls file and 6 *.csv files.  The first includes one y-z
% plane at x=0 and includes x, y, z, u, v, w, u'u', v'v', w'w', Uu, Uv, Uw,
% Uu'u', Uv'v', Uu'v'.  The second file is the same plane and includes x,
% y, z, and epsilon.  The third file includes x, y, z, k (TKE).  The
% fourth file is the same as the first file.  The fifth file is the same
% as the first file, but at proloc+1.  The last is at proloc-1.
% This will allow an analyst to compute any relevant turbulence parameters
% at the inlet (specifically, epsilon).

% Requires csvwrite_with_headers at
% http://www.mathworks.com/matlabcentral/fileexchange/29933-csv-with-column-headers

% ----------------------------------------------
% Updates:

% Jul 11 2014 - Jeff Harris - Changed vxy2=uvt instead of uvuvt, Uuv_p,
% Uuv_m also corrected to use Uuv_plus and minus

% Jun 12 2014 - Jeff Harris - Improved comments

% May 22 2014 - Jeff Harris - Modified to output 6 .xls files.  The first includes one y-z
% plane at x=0 and includes x, y, z, u, v, w, u'u', v'v', w'w', Uu, Uv, Uw,
% Uu'u', Uv'v', Uu'v'.  The second file is the same plane and includes x,
% y, z, and epsilon.  The third file includes x, y, z, k (TKE).  The
% fourth file is the same as the first file.  The fifth file is the same
% as the first file, but at proloc+1.  The last is at proloc-1.

% Version 6_0:
% Sept. 15 2014 - Blake Lance - Removed BL variable
% Removed extra complications in the code, especially taking the 2D data from Stats, converting to 1D, then
% converting back to 2D.  This also removes the need to transpose and transpose data back.
% The epsilon calculations were updated with correct spatial definitions with small difference on the output.
% Updated plots with uncertainty bands
% Exported scalar values at the wall positions (these had previously not been specified)
% Corrected fliplr of Xshift when Opposed==1 and mask is assymetric
% Code now automatically checks if the output files exist and appends the data if they do
% Added headers to k and epsilon output files
% Code now automatically records the prolocs to a file
% epsilon is now calculated by an empirical correlation from a CFD text
% Code now smooths SD by a robust local regression, set by the option Smooth
% Write uncertainty of k and epsilon to file
% Sets the minus uncertainty of SD, k, and epsilon to no more than their magnitude since these values are
%   exclusively positive
% Assigns the z coordinate by the last character in the Serial string (1-9)

% Version Transient_6_1:
% The transient version of this code will read in many Stats files for a given PIV location or 'case' and
%   output the typical six *.csv files. It will not output the *.xlsx file at this time. The time information
%   is appended horizontally. The user selects the data set folders to run and the first *Stats.m file. This
%   requires that the paths between the data set folders and the *Stats.m files are the same.
% The input format for Star-CCM+ is as follows:
%   X Y Z u(m/s)[t=0s] u(m/s)[t=1s] u(m/s)[t=2s] v(m/s)[t=0s] v(m/s)[t=1s] v(m/s)[t=2s] w(m/s)[t=0s]...
%   1 2 3 4.5 2.25 0 0.1 0.05 0 0.1 0.05 0
%   This formating requirement is used for Inlet-uvw.csv, Inlet-k.csv, and Inlet-epsilon.csv.
% The maximum of the first phase is 005, so only 005-101 should be used.

% Version 6_2:
% Added option to export to Star-CCM+. This only writes u, v, w, k, and epsilon and saves disk space.
% Added span on smoothing option

% Version 6_3:
% Added t_offset specification to shift the time

% Version 6_4:
% Found and corrected uncertainty on TKE. The new uncertainty is much lower.

%% User Input
proloc = 38;        % j-index for the row of data to use in profile (increasing from top)
Serial = '8.5';     % Name of the sheet to be used for the excel file for the Stats
Export = 1;         % Export data in *.csv files? (1 for Yes/0 for No)
ExportPlanes = 1;   % Additionally export data in three *.csv files (1 for Yes/0 for No)
ExportToStar = 0;   % Option to export to Star-CCM+, which only includes u, v, w, k, and epsilon
                        % (1 for Yes/0 for No)
Smooth = 1;         % Choose whether to smooth SD and subsequently k and epsilon
Span = 16;          % Percent of total number of data points (Span) for smoothing
Opposed = 1;        % For opposed orientation, switch y-coordinate direction when writing output
Nth = 5;            % Plot every Nth uncertainty band
t_offset = -1.8;    % Time offset [s]

CaseDir = uigetfile_n_dir('H:\','Select all the data set folders to investigate');
[StatsFile,StatsDir] = uigetfile('*Stats.mat','Select the first Stats file',CaseDir{1});

tic;

Nphases = length(CaseDir);
SlashLoc = strfind(CaseDir{1},'\'); % Find the locations of '\' in the data sets
SlashLoc = SlashLoc(end);   % Use the last one
phase = CaseDir{1};
phase = num2str(phase(SlashLoc+1:end));   % Get the data set name
j_0 = str2double(phase); % The actual index the PIV starts
t = linspace(0.2*(j_0-1)+t_offset,0.2*(Nphases+j_0-2)+t_offset,Nphases); % Define the time vector

% Assign the z coordinate for the data plane based on the last number of Serial (1-5) [m]
z_array = [-0.13589; -0.1016; 0.0; 0.1016; 0.13589];
z = z_array(str2double(Serial(end)));

% The loop for phases must start here
for j=1:Nphases
    
    % Replace the case name for the current phase
    SlashLoc_new = strfind(CaseDir{j},'\'); % Find the locations of '\' in the data sets
    SlashLoc_new = SlashLoc_new(end);   % Use the last one
    phase_new = CaseDir{j};
    phase_new = num2str(phase_new(SlashLoc_new+1:end));   % Get the data set name
    
    StatsDir_new = strrep(StatsDir,CaseDir{1},CaseDir{j});
    StatsFile_new = strrep(StatsFile,phase,phase_new);
    load([StatsDir_new,StatsFile_new]);
    
    %% Allocate and initialize
    
    w = size(u_ave,2);
    u = v_ave;  u(isnan(u)) = 0;    % v_ave in DaVis is u in global coordinates
    v = u_ave;  v(isnan(v)) = 0;    % u_ave in DaVis is v in global coordinates
    U_u = Uv;   U_u(isnan(U_u)) = 0;    % The time averaged u (global) velocity uncertainty
    U_v = Uu;   U_v(isnan(U_v)) = 0;    % The time averaged v (global) velocity uncertainty
    U_w = Uu;   U_w(isnan(U_w)) = 0;    % The time averaged w (global) velocity uncertainty, assumed U_w=U_v
    
    u_SD = sqrt(vvt);   u_SD(isnan(u_SD)) = 0; % The standard deviation of u in global coordinates
    v_SD = sqrt(uut);   v_SD(isnan(v_SD)) = 0;  % The standard deviation of v in global coordinates
    SD = sqrt( u_SD.^2 + v_SD.^2 + v_SD.^2 ); % The magnitude of the standard deviation
    
    uu = vvt;   uu(isnan(uu)) = 0;  % u'u' in global coordinates
    vv = uut;   vv(isnan(vv)) = 0;  % v'v' in global coordinates
    ww = uut;   ww(isnan(ww)) = 0;  % w'w' in global coordinates, assumed w'w'=v'v'
    uv = uvt;   uv(isnan(uv)) = 0;  % u'v' in global coordinates
    
    U_uu_p = Uvv_plus;  U_uu_p(isnan(U_uu_p)) = 0;  % Plus uncertainty of u'u' in global coordinates
    U_uu_m = Uvv_minus; U_uu_m(isnan(U_uu_m)) = 0;  % Minus uncertainty of u'u' in global coordinates
    U_vv_p = Uuu_plus;  U_vv_p(isnan(U_vv_p)) = 0;  % Plus uncertainty of v'v' in global coordinates
    U_vv_m = Uuu_minus; U_vv_m(isnan(U_vv_m)) = 0;  % Minus uncertainty of v'v' in global coordinates
    U_uv_p = Uuv_plus;  U_uv_p(isnan(U_uv_p)) = 0;  % Plus uncertainty of u'v' in global coordinates
    U_uv_m = Uuv_minus; U_uv_m(isnan(U_uv_m)) = 0;  % Minus uncertainty of u'v' in global coordinates
    
    U_SD_p = sqrt( U_uu_p + U_vv_p + U_vv_p );  % Plus uncertainty in the standard deviation
    U_SD_m = sqrt( U_uu_m + U_vv_m + U_vv_m );  % Minus uncertainty in the standard deviation
    
    du_dx = zeros(w,1);
    du_dy = zeros(w,1);
    dv_dx = zeros(w,1);
    dv_dy = zeros(w,1);
    
    if Opposed == 1 % Change the sign
        u = -u;
        v = -v;
    end
    
    %% Adaptive trimming to masked area
    [imin,imax,jmin,jmax] = TrimToMask(u);
    
    xlower = X(1,jmin);
    xupper = X(1,jmax);
    ylower = Y(imin,1);
    if imax==0; imax = 1; end;
    yupper = Y(imax,1);
    
    Xshift = X - xlower;    % Shift x_2D array so y=0 is at the left
    xupper = xupper - xlower;   % Just changed this order
    xlower = xlower - xlower;
    
    %% Flip Xshift if Opposed==1, also shift values if the masked area isn't symmetric
    if Opposed==1
        dx = abs( X(1,1) - X(1,2) );  % Spacing between vectors in mm given by DaVis
        Xshift = fliplr(Xshift);
        TrimR = w - jmax + 1;   TrimL = jmin;
        % Add or subtract values so assymetric Xshift will have consistent values
        Xshift = Xshift - dx*(TrimR - TrimL);
    end
    
    %% Calculate the Turbulence Kinetic Energy
    
    if Smooth == 1; SD(proloc,:) = smooth(SD(proloc,:),Span,'rlowess');    end;
    for i=1:w   % Set the minus uncertainty of SD to SD so the uncertainty bands stay positive
        if U_SD_m(proloc,i) > SD(proloc,i);   U_SD_m(proloc,i) = SD(proloc,i);  end;
    end
    
%     TKE = 0.5*SD(proloc,:).^2;
%     U_TKE_p = (SD(proloc,:).*U_SD_p(proloc,:))';
%     U_TKE_m = (SD(proloc,:).*U_SD_m(proloc,:))';
    TKE = 0.5*( uu(proloc,:) + vv(proloc,:) + vv(proloc,:) );
    U_TKE_p = 0.5*sqrt( U_uu_p(proloc,:).^2 + U_vv_p(proloc,:).^2 + U_vv_p(proloc,:).^2 );
    U_TKE_m = 0.5*sqrt( U_uu_m(proloc,:).^2 + U_vv_m(proloc,:).^2 + U_vv_m(proloc,:).^2 );
    for i=1:w      % Set the minus uncertainty of k to k so the uncertainty bands stay positive
        if U_TKE_m(i) > TKE(i);   U_TKE_m(i) = TKE(i);  end;
    end
    
    %% Calculate epsilon on an empirical correlation
    
    % An empirical equation for epsilon is given by Versteeg and Malalaskekera in "An Introduction to
    % Computational Fluid Dynamics," page 77.
    epsilon = 0.09^0.75*TKE.^1.5/(0.07*0.3048);
    U_epsilon_p = 1.5*0.09^0.75*U_TKE_p.^0.5/(0.07*0.3048);
    U_epsilon_m = 1.5*0.09^0.75*U_TKE_m.^0.5/(0.07*0.3048);
    for i=1:w      % Set the minus uncertainty of epsilon to epsilon so the uncertainty bands stay positive
        if U_epsilon_m(i) > epsilon(i);   U_epsilon_m(i) = epsilon(i);  end;
    end
    
    %% Plotting
    
    if j==1 % Only plot for the first phase
        % Plot contour of velocity magnitude
        figure('Name','Velocity Magnitude'); contourf(Xshift,Y,uv_ave,'edgecolor','none'); hold on;
        plot([Xshift(proloc,1) Xshift(proloc,w)],[Y(proloc,1) Y(proloc,w)],'r');
        uv_avemax = max(max(uv_ave(imax:imin,jmin:jmax)));    % Max value in plotted domain
        uv_avemin = min(min(uv_ave(imax:imin,jmin:jmax)));    % Min value in plotted domain
        h1 = colorbar('EastOutside');
        caxis([uv_avemin uv_avemax]); %shading interp
        set(get(h1,'ylabel'),'String','U [m/s]')
        axis equal; axis([xlower,xupper,ylower,yupper])
        xlabel('y [mm]');   ylabel('x [mm]');
        
        % Plot contour of SD
        figure('Name','Standard Deviation'); contourf(Xshift,Y,SD,'edgecolor','none'); hold on;
        plot([Xshift(proloc,1) Xshift(proloc,w)],[Y(proloc,1) Y(proloc,w)],'r');
        SDxymax = max(max(SD(imax:imin,jmin:jmax)));    % Max value in plotted domain
        SDxymin = min(min(SD(imax:imin,jmin:jmax)));    % Min value in plotted domain
        h1 = colorbar('EastOutside');
        caxis([SDxymin SDxymax]); %shading interp
        set(get(h1,'ylabel'),'String','S_U [m/s]')
        axis equal; axis([xlower,xupper,ylower,yupper])
        xlabel('y [mm]');   ylabel('x [mm]');
        
        % Plot velocity magnitude profile
        figure('Name','Velocity Profile');    plot(Xshift(proloc,:),u(proloc,:),'k','LineWidth',2); hold on;
        errorbar(Xshift(proloc,1:Nth:end),u(proloc,1:Nth:end),U_u(proloc,1:Nth:end),'.','MarkerSize',0.1);
        axis tight; xlim([xlower xupper]);  xlabel('y [mm]');   ylabel('u [m/s]');
        
        % Plot SD profile
        figure('Name','Standard Deviation Profile'); plot(Xshift(proloc,:),SD(proloc,:),'k','LineWidth',2);   hold on;
        if Smooth == 1
            SD_orig = sqrt( u_SD(proloc,:).^2 + v_SD(proloc,:).^2 + v_SD(proloc,:).^2 );
            plot(Xshift(proloc,:),SD_orig,'r','LineWidth',2);
            legend('Smoothed S_U','Original S_U','location','best');
        end
        errorbar(Xshift(proloc,1:Nth:end),SD(proloc,1:Nth:end),U_SD_m(proloc,1:Nth:end),U_SD_p(proloc,1:Nth:end),...
            '.','MarkerSize',0.1);
        axis tight; xlim([xlower xupper]);  xlabel('y [mm]');   ylabel('S_U [m/s]');
        
        % Plot TKE profile
        figure('Name','TKE Profile'); plot(Xshift(proloc,:),TKE,'k','LineWidth',2);  hold on;
        if Smooth == 1
            TKE_orig = 0.5*SD_orig.^2;
            plot(Xshift(proloc,:), TKE_orig,'r','LineWidth',2);
            legend('Smoothed k','Original k','location','best');
        end
        errorbar(Xshift(proloc,1:Nth:end),TKE(1:Nth:end),U_TKE_m(1:Nth:end),U_TKE_p(1:Nth:end),...
            '.','MarkerSize',0.1);
        axis tight; xlim([xlower xupper]);  xlabel('y [mm]');   ylabel('k [m^2/s^2]');
        
        % Plot epsilon profile
        figure('Name','Epsilon Profile'); plot(Xshift(proloc,:),epsilon,'k','LineWidth',2); hold on;
        if Smooth == 1
            epsilon_orig = 0.09^0.75*TKE_orig.^1.5/(0.07*0.3048);
            plot(Xshift(proloc,:), epsilon_orig,'r','LineWidth',2);
            legend('Smoothed \epsilon','Original \epsilon','location','best');
        end
        errorbar(Xshift(proloc,1:Nth:end),epsilon(1:Nth:end),U_epsilon_m(1:Nth:end),U_epsilon_p(1:Nth:end),...
            '.','MarkerSize',0.1);
        axis tight; xlim([xlower xupper]);  xlabel('y [mm]');   ylabel('\epsilon [m^2/s^3]');
    end

    if Export == 1
        %% Write u, v, and w to *.csv files
        
        if j==1; % Only initialize with the first phase
            if ExportToStar~=1
                M_Uxy = zeros(3,jmax-jmin+1,3+18*Nphases); % Three for X, Y, Z + 18*Nphases for Transient data
                K = zeros(jmax-jmin+1,3+3*Nphases);
                eps = zeros(jmax-jmin+1,3+3*Nphases);
            else
                M_Uxy = zeros(3,jmax-jmin+1,3+3*Nphases);  % Three for X, Y, Z + 3*Nphases for Transient data
                K = zeros(jmax-jmin+1,3+Nphases);
                eps = zeros(jmax-jmin+1,3+Nphases);
            end
            K(:,2) = 0.001*Xshift(proloc,jmin:jmax);
            K(:,3) = z;
            eps(:,2) = 0.001*Xshift(proloc,jmin:jmax);
            eps(:,3) = z;
        end
        
        xspace = 0.001*(Y(proloc,2)-Y(proloc+1,2));
        % Write the y-locations off the vertical center at proloc
        M_Uxy(1,:,1) = -xspace;
        M_Uxy(3,:,1) = xspace;
        
        plane = [1;0;-1];
        for i=1:3
            M_Uxy(i,:,2) = 0.001*Xshift(proloc+plane(i),jmin:jmax);
            M_Uxy(i,:,3) = z;
            if ExportToStar~=1
                M_Uxy(i,:,4+18*(j-1)) = u(proloc+plane(i),jmin:jmax); % These indeces need to change with j
                M_Uxy(i,:,5+18*(j-1)) = v(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,6+18*(j-1)) = v(proloc+plane(i),jmin:jmax);  % Assume w=v
                M_Uxy(i,:,7+18*(j-1)) = uu(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,8+18*(j-1)) = vv(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,9+18*(j-1)) = ww(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,10+18*(j-1)) = uv(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,11+18*(j-1)) = U_u(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,12+18*(j-1)) = U_v(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,13+18*(j-1)) = U_w(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,14+18*(j-1)) = U_uu_p(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,15+18*(j-1)) = U_uu_m(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,16+18*(j-1)) = U_vv_p(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,17+18*(j-1)) = U_vv_m(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,18+18*(j-1)) = U_vv_p(proloc+plane(i),jmin:jmax); %
                M_Uxy(i,:,19+18*(j-1)) = U_vv_m(proloc+plane(i),jmin:jmax); % Changed order here
                M_Uxy(i,:,20+18*(j-1)) = U_uv_p(proloc+plane(i),jmin:jmax); %
                M_Uxy(i,:,21+18*(j-1)) = U_uv_m(proloc+plane(i),jmin:jmax); %
            else
                M_Uxy(i,:,4+3*(j-1)) = u(proloc+plane(i),jmin:jmax); % These indeces need to change with j
                M_Uxy(i,:,5+3*(j-1)) = v(proloc+plane(i),jmin:jmax);
                M_Uxy(i,:,6+3*(j-1)) = v(proloc+plane(i),jmin:jmax);  % Assume w=v
            end
        end
        M_Uxy_1(:,:) = M_Uxy(2,:,:);    % This is the nominal one at proloc
        M_Uxy_2(:,:) = M_Uxy(1,:,:);
        M_Uxy_3(:,:) = M_Uxy(3,:,:);
        
        if ExportToStar~=1
            K(:,4+3*(j-1)) = TKE(jmin:jmax);
            K(:,5+3*(j-1)) = U_TKE_p(jmin:jmax);
            K(:,6+3*(j-1)) = U_TKE_m(jmin:jmax);
            eps(:,4+3*(j-1)) = epsilon(jmin:jmax);
            eps(:,5+3*(j-1)) = U_epsilon_p(jmin:jmax);
            eps(:,6+3*(j-1)) = U_epsilon_m(jmin:jmax);
        else
            K(:,4+(j-1)) = TKE(jmin:jmax);
            eps(:,4+(j-1)) = epsilon(jmin:jmax);
        end
    end
end

if Export == 1
    % Write all combinations of scalar and time columns
    NewFile = exist('Inlet-uvw.csv','file');    % Check if the file exists
    if NewFile == 0 % File does not exist
        % Assign the header with X, Y, Z, u(m/s)[t=0s], u(m/s)[t=0.1s], ...
        uvw_header = 'X,Y,Z';   % Initialize the header
        for i=1:length(t)   % Append the columns
            if ExportToStar~=1
                uvw_header = [uvw_header,',',...
                    'u(m/s)[t=',num2str(t(i)),'s],','v(m/s)[t=',num2str(t(i)),'s],',...
                    'w(m/s)[t=',num2str(t(i)),'s],','u''u''(m^2/s^2)[t=',num2str(t(i)),'s],',...
                    'v''v''(m^2/m^2)[t=',num2str(t(i)),'s],','w''w''(m^2/s^2)[t=',num2str(t(i)),'s],',...
                    'u''v''(m^2/m^2)[t=',num2str(t(i)),'s],','Uu(m/s)[t=',num2str(t(i)),'s],',...
                    'Uv(m/s)[t=',num2str(t(i)),'s],','Uw(m/s)[t=',num2str(t(i)),'s],',...
                    'Uuup(m^2/s^2)[t=',num2str(t(i)),'s],','Uuum(m^2/s^2)[t=',num2str(t(i)),'s],',...
                    'Uvvp(m^2/s^2)[t=',num2str(t(i)),'s],','Uvvm(m^2/s^2)[t=',num2str(t(i)),'s],',...
                    'Uwwp(m^2/s^2)[t=',num2str(t(i)),'s],','Uwwm(m^2/s^2)[t=',num2str(t(i)),'s],',...
                    'Uuvp(m^2/s^2)[t=',num2str(t(i)),'s],','Uuvm(m^2/s^2)[t=',num2str(t(i)),'s]'];
            else
                uvw_header = [uvw_header,',',...
                    'u(m/s)[t=',num2str(t(i)),'s],','v(m/s)[t=',num2str(t(i)),'s],',...
                    'w(m/s)[t=',num2str(t(i)),'s]'];
            end
        end
        csvwrite_with_headers('Inlet-uvw.csv',M_Uxy_1,strsplit(uvw_header,','))
    else
        dlmwrite('Inlet-uvw.csv',M_Uxy_1,'delimiter',',','precision',4,'-append')
    end
    
    if ExportPlanes==1  % Write three files that contain all three planes
        if NewFile == 0 % File does not exist, write the header
            csvwrite_with_headers('Inlet-uvw-plane1.csv',M_Uxy_1,strsplit(uvw_header,','));
            csvwrite_with_headers('Inlet-uvw-plane2.csv',M_Uxy_2,strsplit(uvw_header,','));
            csvwrite_with_headers('Inlet-uvw-plane3.csv',M_Uxy_3,strsplit(uvw_header,','));
        else    % Don't write the header
            dlmwrite('Inlet-uvw-plane1.csv',M_Uxy_1,'delimiter',',','precision',4,'-append')
            dlmwrite('Inlet-uvw-plane2.csv',M_Uxy_2,'delimiter',',','precision',4,'-append')
            dlmwrite('Inlet-uvw-plane3.csv',M_Uxy_3,'delimiter',',','precision',4,'-append')
        end
    end
    
    %% Write k to *.csv file
    
    % Write the measured profile
    k_header = 'X,Y,Z';   % Initialize the header
    for i=1:length(t)   % Append the columns
        if ExportToStar~=1
            k_header = [k_header,',',...
                'k(J/kg)[t=',num2str(t(i)),'s],',...
                'U_k_p(J/kg)[t=',num2str(t(i)),'s],','U_k_m(J/kg)[t=',num2str(t(i)),'s]'];
        else
            k_header = [k_header,',','k(J/kg)[t=',num2str(t(i)),'s]'];
        end
    end    
    if NewFile == 0
        csvwrite_with_headers('Inlet-k.csv',K,strsplit(k_header,','));
    else
        dlmwrite('Inlet-k.csv',K,'delimiter',',','precision',4,'-append')
    end
    
    %% Write epsilon to *.csv file
    
    % Write the measured profile
    eps_header = 'X,Y,Z';   % Initialize the header
    for i=1:length(t)   % Append the columns
        if ExportToStar~=1
            eps_header = [eps_header,',',...
                'epsilon(m^2/s^3)[t=',num2str(t(i)),'s],',...
                'U_eps_p(m^2/s^3)[t=',num2str(t(i)),'s],','U_eps_m(m^2/s^3)[t=',num2str(t(i)),'s]'];
        else
            eps_header = [eps_header,',','epsilon(m^2/s^3)[t=',num2str(t(i)),'s]'];
        end
    end
    if NewFile == 0
        csvwrite_with_headers('Inlet-epsilon.csv',eps,strsplit(eps_header,','));
%         csvwrite_with_headers('Inlet-epsilon.csv',eps,{'X', 'Y', 'Z', 'epsilon', 'U_eps_p', 'U_eps_m'});
    else
        dlmwrite('Inlet-epsilon.csv',eps,'delimiter',',','precision',4,'-append')
    end
    
    %% Write ProfilesFromStats.xlsx file
%     % Writes data to different sheets
%     
%     % Write Header
%     xlswrite('ProfilesFromStats.xlsx',{'Position' 'u_ave' 'Uu' 'uut' ...
%         'Uuu_plus' 'v_ave' 'Uv' 'vvt' 'Uvv_plus' 'uvt' 'Uuv_plus' 'Position2'...
%         'u_ave2' 'Uu2' 'uut2' 'Uuu_plus2' 'v_ave2' 'Uv2' 'vvt2' 'Uvv_plus2'...
%         'uvt2' 'Uuv_plus2'},Serial,'A1:V1'), warning off MATLAB:xlswrite:AddSheet;
%     
%     % Write the Offset equation for skipping uncertainty bands with Tecplot
%     xlswrite('ProfilesFromStats.xlsx',{'=OFFSET(A$2,5*(ROW()-2),0)'},Serial,'L2:L2'),...
%         warning off MATLAB:xlswrite:AddSheet;
%     
%     profilewhole=[Xshift(proloc,jmin:jmax)',u(proloc,jmin:jmax)',U_u(proloc,jmin:jmax)',...
%         uu(proloc,jmin:jmax)',U_uu_p(proloc,jmin:jmax)',v(proloc,jmin:jmax)',...
%         U_v(proloc,jmin:jmax)',vv(proloc,jmin:jmax)',U_vv_p(proloc,jmin:jmax)',...
%         uv(proloc,jmin:jmax)',U_uv_p(proloc,jmin:jmax)'];
%     
%     % Write the data
%     xlswrite('ProfilesFromStats.xlsx',profilewhole,Serial,['A2:K',int2str(jmax-jmin+2)]),...
%         warning off MATLAB:xlswrite:AddSheet;
    
    %% Write the prolocs (Profile Locations) to a file

    fid = fopen('Inlet-prolocs.csv','at');
    if NewFile == 0 % If file doesn't exist, write the header
        fprintf(fid,'Stats File,Serial,z [m],proloc,Smoothed \n');
    end
    fprintf(fid,[StatsFile,',',Serial,',',num2str(z),',',int2str(proloc),',',int2str(Smooth),'\n']);
    fclose(fid);
end

toc;