clear all; close all; clc;

% This code will read all instantenous vc7 files, compute time means and subsuquent instantenous deviations of
% velocity, and plot them in histograms.  Turbulence intermittency may be determined by the shape of the
% histogram.  The velocity point is determined by the vertical center and horizontal peak of u'u'.  The input
% structure of the code is borrowed from PeakLocking.m
%
% Blake Lance, 1 July 2015

%% Inputs

HFS = 2;    % Determine the HFS location (1, 2, or 3)
Opposed = 0; % Whether the orientation is opposed
yp = 3;     % Maximum y+ value for linear fit
Fit = 10;   % The initial number of near-wall velocity data to fit for shear calc
% rho = 1;    % Simple guess for fluid density [kg/m^3]
% mu
bins = 30; % The number of bins in the histogram
FontSize = 11;

% Get the folder containing vc7 files for the data set
folder = uigetdir('D:\','Select folder containing vector files');
tic;

%% Prepare for Import
folder_struct = dir(folder);

% Read vc7 FileNames
count = 0;
FileNames = cell(1);
for i=1:length(folder_struct)
    [~,~,ext] = fileparts([folder,'\',folder_struct(i).name]);
    if strcmp(ext,'.vc7')
        count = count + 1;
        FileNames{count} = folder_struct(i).name;
    end
end

% Find out what the size of the arrays are for defining U and V
davis = readimx([folder,'\',FileNames{1}]);
Data.vcNum = 1;
U0 = davis.Frames{Data.vcNum}.Components{1}.Planes{1}';
% Active_Choice = davis.Frames{Data.vcNum}.Components{9}.Planes{1}';
[ny,nx] = size(U0);
N = length(FileNames);

% Create 3D arrays U and V
u = zeros(ny,nx,N);
v = zeros(ny,nx,N);
Uv = zeros(ny,nx,N);

%% Import Data into U and V
for i=1:N
    davis = readimx([folder,'\',FileNames{i}]);
    % Find the scaling between pixels and m/s
    C = davis.Frames{1}.Components{1}.Scale.Slope;
    
    U0 = davis.Frames{Data.vcNum}.Components{1}.Planes{1}';
    V0 = davis.Frames{Data.vcNum}.Components{2}.Planes{1}';
    U1 = davis.Frames{Data.vcNum}.Components{3}.Planes{1}';
    V1 = davis.Frames{Data.vcNum}.Components{4}.Planes{1}';
    U2 = davis.Frames{Data.vcNum}.Components{5}.Planes{1}';
    V2 = davis.Frames{Data.vcNum}.Components{6}.Planes{1}';
    U3 = davis.Frames{Data.vcNum}.Components{7}.Planes{1}';
    V3 = davis.Frames{Data.vcNum}.Components{8}.Planes{1}';
    Active_Choice = davis.Frames{Data.vcNum}.Components{9}.Planes{1}';
    % Check if there is a U4 and U5 and output error if there is
    if max(Active_Choice(:)) == 4  || max(Active_Choice(:)) == 5
        error('Active Choice had a 4 or 5')
    end
    AC0 = zeros(size(U0));  AC1 = AC0;  AC2 = AC0;  AC3 = AC0;
    AC0(Active_Choice==0) = 1;
    AC1(Active_Choice==1) = 1;
    AC2(Active_Choice==2) = 1;
    AC3(Active_Choice>2) = 1;
    u(:,:,i) = U0.*AC0 + U1.*AC1 + U2.*AC2 + U3.*AC3;
    v(:,:,i) = V0.*AC0 + V1.*AC1 + V2.*AC2 + V3.*AC3;
    Uv(:,:,i) = davis.Frames{1}.Components{24}.Planes{1}';
    
    Eval = double(davis.Frames{Data.vcNum}.Components{10}.Planes{1}');
    Mask = double(davis.Frames{1}.Components{12}.Planes{1}');
    u(:,:,i) = C*double(u(:,:,i).*Eval.*Mask);
    v(:,:,i) = C*double(v(:,:,i).*Eval.*Mask);
    Uv(:,:,i) = C*double(Uv(:,:,i).*Eval.*Mask);
end

%% Calculations

% Compute time means
u_m = sum(u,3)./sum(u~=0,3); u_m(isnan(u_m)) = 0;
v_m = sum(v,3)./sum(v~=0,3); v_m(isnan(v_m)) = 0;

% It seems the streamwise velocity has the wrong sign
v = -v;
v_m = -v_m;

% Define x vector
x0 = davis.Frames{1}.Scales.X.Offset;
dx = davis.Frames{1}.Scales.X.Slope*davis.Frames{1}.Grids.X;
x = linspace(x0,x0+dx*nx,nx);

% Adaptive trimming to masked area
[imin,imax,jmin,jmax] = TrimToMask(v_m);
if jmin == 0; jmin = 1; end
xlower = x(jmin);
Xshift = x - xlower;    % Shift x_2D array so y=0 is at the left
            
% Flip Xshift if Opposed, also shift values if the masked area isn't symmetric
if Opposed == 1
    u = -u; u_m = -u_m; % Buoyancy-opposed, change sign
    v = -v; v_m = -v_m;
    
    Xshift = fliplr(Xshift);
    TrimR = size(v_m,2) - jmax + 1;   TrimL = jmin;
    % Add or subtract values so assymetric Xshift will have consistent values
    Xshift = Xshift - dx*(TrimR - TrimL);
end

% Compute Reynolds stress
UU = var(u,1,3);
VV = var(v,1,3);

% Determine the profile location at the vertical center
proloc = floor(0.5*(imin-imax+1));

% Determine index of maximum streamwise Reynolds stress
[~,VVjmax] = max(VV(proloc,:));

% Calculate the instantenous velocities at proloc and VVjmax
Up = zeros(N,1);
Vp = zeros(N,1);
for i=1:N
    Up(i) = u(proloc,VVjmax,i) - u_m(proloc,VVjmax);
    Vp(i) = v(proloc,VVjmax,i) - v_m(proloc,VVjmax);
end

%% Define Heated Wall Temperatures
% These are the measured temperatures for the Mixed case (S4 and S7)
Temp = [408.82,411.55,410.79;408.76,410.64,410.42]; % [K]
U_Temp = [1.31,1.11,1.05;1,1,1]; % [K]

% Just use the data for the current position
Temp = Temp(HFS,Opposed+1);
U_Temp = U_Temp(HFS,Opposed+1);

%% Calculate Air Properties
% See Tsilingiris2007 and Melling97
T0 = 23.4 + 273.15;   % Room temperature [K]
P = 86696; % Atmospheric pressure [Pa]
phi0 = 0.336;  % Relative humidity at room temperature

% Saturation pressure at room temperature [Pa]
P_sat0 = 1000*( 1.901888 - 2.1424081E-1*(T0-273.15) + 1.2057211E-2*(T0-273.15)^2 - ...
    1.3867581E-4*(T0-273.15)^3 + 1.3888951E-6*(T0-273.15)^4 );
omega = 0.622*phi0*P_sat0/( P - phi0*P_sat0 ); % Specific humidity at any temperature
R_mix = 8.31434/((omega+1)/((omega/0.01802)+(1/0.02897))); % Compute the Mixture Gas Constant [m^2/(K*s^2)]
rho = P./(Temp*R_mix);

% Compute Dynamic Viscosity using Sutherland's Law
a = 1.458E-6; b = 110.4;
mu = a*Temp.^(3/2)./( Temp + b ); % [kg/(m*s)]
U_mu = U_Temp*a.*sqrt(Temp).*(Temp+3*b)./(2*(Temp+b).^2);

%% Calculate Wall Shear for Each Shot

Shear = zeros(N,1);
U_Shear = zeros(N,1);
FitOut = zeros(N,1);
for i=1:N
    % Calculate velocity gradient and its uncertainty
    if Opposed == 0 % Force wall to be on right
        [ y0, FitOut(i), dudy, U_dudy ] = PIV_dudy( fliplr(Xshift), ...
            fliplr(v(proloc,:,i)),fliplr(Uv(proloc,:,i)), rho, mu, yp, Fit );
    elseif Opposed == 1 % Wall is on right
        [ y0, FitOut(i), dudy, U_dudy ] = PIV_dudy( Xshift,v(proloc,:,i),Uv(proloc,:,i), rho, mu, yp, Fit );
    end
        
    % Calculate shear and its uncertainty
    Shear(i) = mu*dudy;
    U_Shear(i) = sqrt( (dudy*U_mu)^2 + (mu*U_dudy)^2 );
    
%     % Calculate u_tau
%     if j == 2 && i == 3; u_tau = sqrt(Shear./rho); end
end

% % Plot velocity data and fit
% %             if j == 2 && i == 2
% if j == 2
%     figure(5); hold on;
%     set(gcf,'Units','inches','Position',[10 6 3.5 2.75]);
%     %                 leg_h3(count) = plot(Xshift(proloc,:),v_ave(proloc,:),'LineWidth',1.25);
%     h5(i,1) = plot(Xshift(proloc,:),v_ave(proloc,:),...
%         'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25);
%     % Appearantly MATLAB sets the width of the uncertainty bands to 2% of the x range, so use less
%     errorbar(Xshift(proloc,275:2:jmax),v_ave(proloc,275:2:jmax),...
%         Uv(proloc,275:2:jmax),'Color',linecolor{i},'LineStyle','none','LineWidth',1.25);
%     xlabel('$y$ [mm]','Interpreter','latex');
%     ylabel('$\overline{u}$ [m/s]','Interpreter','latex');
%     if Opposed == 0
%         h5(i,2) = plot([Xshift(proloc,jmin), Xshift(proloc,jmin+FitOut-1)],...
%             [y0, y0+dudy*(Xshift(proloc,jmax)-Xshift(proloc,jmax-FitOut+1))/1000],...
%             'Color',[0.25 0.25 0.25],'LineWidth',1.25);
%     elseif Opposed == 1
%         h5(i,2) = plot([Xshift(proloc,jmax), Xshift(proloc,jmax-FitOut+1)],...
%             [y0, y0-dudy*(Xshift(proloc,jmax)-Xshift(proloc,jmax-FitOut+1))/1000],...
%             'Color',[0.25 0.25 0.25],'LineWidth',1.25);
%     end
%     axis([axislim1(1) 2.1 axislim1(3) 1.4]);
%     if i == 3
%         %                     axis([axislim1(1) 2.1 axislim1(3) 1.4]);
%         leg5 = legend(h5(:),'$x_1$ Opp','$x_2$ Opp','$x_3$ Opp','Line Fit');
%         set(leg5,'Interpreter','latex','Location','northwest');
%         set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex','Box','on');
%         set(findall(gcf,'type','axes'),'FontSize',FontSize);
%         set(findall(gcf,'type','text'),'FontSize',FontSize);
%         set(findall(gcf,'type','legend'),'FontSize',FontSize-1);
%         tightfig;
%     end
% end

%% Plots

% Plot u'u' and location for histograms
figure(1); set(gcf,'Units','inches','Position',[4 6 3.5 3]);
plot(Xshift(jmin:jmax-1),VV(proloc,jmin:jmax-1)); hold on;
plot(Xshift(VVjmax),VV(proloc,VVjmax),'x');
xlabel('$y$ [m]','Interpreter','latex');
ylabel('$\overline{u''u''}$ [m$^2$/s$^2$]','Interpreter','latex');
xlim([0 inf]);

% Plot v' histogram
figure(2); set(gcf,'Units','inches','Position',[8 6 3.5 3]);
h1 = histogram(Up,linspace(-0.5,0.5,bins));
xlabel('$v''$ [m/s]','Interpreter','latex');
ylabel('Counts','Interpreter','latex');

% Plot u' histogram
figure(3); set(gcf,'Units','inches','Position',[12 6 3.5 3]);
histogram(Vp,linspace(-2,2,bins));
xlabel('$u''$ [m/s]','Interpreter','latex');
ylabel('Counts','Interpreter','latex');

% Plot u'v' scatter
figure(4); set(gcf,'Units','inches','Position',[8 2 0.75*[3.5 3]]);
plot(Vp,Up,'.');
xlabel('$u''$ [m/s]','Interpreter','latex');
ylabel('$v''$ [m/s]','Interpreter','latex');
axis([-1.25 1.25 -1.25 1.25]);

% Plot shear histogram
figure(5); set(gcf,'Units','inches','Position',[12 2 3.5 3]);
histogram(Shear);
xlabel('$\tau_s$ [Pa]','Interpreter','latex');
ylabel('Counts','Interpreter','latex');

for i=1:5
    figure(i);
    set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
    set(findall(gcf,'type','axes'),'FontSize',FontSize);
    set(findall(gcf,'type','text'),'FontSize',FontSize);
    set(findall(gcf,'type','legend'),'FontSize',FontSize-1);
    tightfig;
end

toc;