%LAST UPDATED: 10/29/2014

%% VECTOR STATISTICS FUNCTION 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 03-18-13  Added mean Particle Image diameter and density output
% 06-25-13  Corrected equations for Uu, Uv, and Uuv
% 10-27-14  Calculates RAM requirements to run
%           r_low and r_high values are commented out (not needed)
%           Fixed bug that included  NaN, Inf, or Complex
%           

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Input: from PIVuncertainty (ie. B00001.mat)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Bias_k      - bias uncertainty [m/s]
    DiaRatio    - Ratio of estimated particle image diameters (D_X/D_Y)
    Eval        - Marks removed/null vectors (0 = good, 1 = removed)
    P_Dens      - Particle image density [particles/(32x32)]
    P_Dia       - Particle image diameter [pix]
    RR_95       - defined to calculate the 95% confidence on the Reynolds stresses [m/s]^2
    R_high      - Upper random uncertainty (95% confidence interval) [m/s]
    R_low       - Lower random uncertainty [m/s]
    Ravg        - mean diameter ratio
    U           - u velocity [m/s]
    V           - v velocity [m/s]
    X           - x location [mm]
    Y           - y location [mm]
    dudy        - shear term [pixel displacement / pixel]
    dvdx        - shear term [pixel displacement / pixel]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Output: *_Stats.mat
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    N           - Samples per interogation region
Location:
    X           - x location [mm]
    Y           - y location [mm]
Velocity
    u_ave       - Time averaged u velocity [m/s]
    uv_ave      - Time averaged velocity magnitude [m/s]
    v_ave       - Time averaged v velocity [m/s]
Variance/Covariance:
    uut         - u variance (u'u'bar) [m/s]^2
    uvt         - covariance (u'v'bar) [m/s]^2
    vvt         - v variance (v'v'bar) [m/s]^2
    uvuvt       - velocity magnitude variance (uv'uv'bar) [m/s]^2
Precision:
    Puu         - Precision Uncertainty in Variance (u'u'bar) [m/s]^2
    Puv         - Precision Uncertainty in covariance (u'v'bar) [m/s]^2 
    Puvuv       - Precision Uncertainty in Velocity magnitude variance (uv'uv'bar) [m/s]^2
    Pvv         - Precision Uncertainty in Variance (v'v'bar) [m/s]^2
Uncertainties:
    Uu          - Time averaged u velocity uncertaity *(Eq 9)
    Uuv         - Time averaged velocity magnitude uncertainty [m/s]
    Uv          - Time averaged v velocity uncertainty [m/s]
    Uuu (+,-)   - Uncertainty in Variance (u'u'bar) [m/s]^2
    Uuv(+,-)    - Uncertainty in covariance (u'v'bar) [m/s]^2 
    Uuvuv(+,-)  - Uncertainty in Velocity magnitude variance (uv'uv'bar) [m/s]^2
    Uvv (+,-)   - Uncertainty in Variance (v'v'bar) [m/s]^2
Other Outputs:    
    dudy_ave    - Average Shear term [pixel displacement / pixel]
    dvdx_ave    - Average Shear term [pixel displacement / pixel]
    Den_ave     - Average Paricle image Density (particle/32x32)
    Dia_ave     - Average Particle image Diameter (pixels)
    uv_std      - uv magnitude standard deviation sqrt(uv'uv'bar) 
    u_std       -u standard deviation sqrt(u'u'bar)
    v_std       -v standard deviation sqrt(v'v'bar)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Other intermediate variables:
    uv          -velocity magnitude
    b_k_u       -Bias in u
    b_k_v       -Bias in v
    r_low_u     -upper u 95% coverage interval 
    r_low_v 	-upper v 95% coverage interval 
    r_high_u	-lower u 95% coverage interval 
    r_high_v	-lower v 95% coverage interval 
    rr_high_u	-upper u (r^2) 95% coverage interval 
    rr_high_v	-upper v (r^2) 95% coverage interval 
    

% * Equations from "Taylor-Series and Monte-Carlo-Method
    Uncertainty Estimation of the Width of a Probability Distribution
    Based on Varying Bias and Random Error" by Wilson and Smith are noted
    with by * (Eq #)

Update: 18 December 2014, Blake Lance
    Added historgrams of diameter, density, and shear
    Also finds the extrema of parameters from the uncertainty surface to add bands to plots
%}

function PIV_Stats(Data)

%input/output directory
if ispc
    pltdirec=[Data.outdirec '\'];
else
    pltdirec=[Data.outdirec '/'];
end

FileName=[pltdirec Data.batchname '_Stats.mat'];

tf=cputime;
fprintf('\n----------------------------------------------------\n')
fprintf('Processing Vector Statistics\n')
fprintf('----------------------------------------------------\n')

%% Read in all data
t1=cputime;
fprintf('reading data...                  ')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Preallocate Matrices
%Total Number of image pair
n = length(str2double(Data.datafstart):str2double(Data.datafstep):str2double(Data.datafend));
%Load First image pair
fname = [pltdirec Data.outbase sprintf(['%0.' Data.datazeros 'i.mat'],str2double(Data.datafstart))];
a=load(fname);
%Convert to vector to matrix form if necessary
if (size(a.U,2) == 1)
    [a.X,a.Y,a.U,a.V,a.Eval] = five_matrixform(a.X,a.Y,a.U,a.V,a.Eval);
end

%Check RAM requirements to load data
[~,sys] = memory;
AvailableRAM=sys.PhysicalMemory.Available/1024/1024/1024;
GB=(size(a.U,1)*size(a.U,2)*n*8*(11)+size(a.U,1)*size(a.U,2)*8*30)/1000000000;
fprintf('\nStats needs %2.4f GB of RAM \n',GB)
if GB>AvailableRAM
    fprintf('Continue anyway?')
    choice = input('[y/n]\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n','s');
    if choice == 'y'
    else
        error('Program was ended by user')
    end 
end


%Preallocate matrices
u = zeros([size(a.U),n]);   % u-velocity
v = u;                      % v-velocity
eval = u;                   % matrix of acceptable values
b_k_u = u;                  % Bias in u
b_k_v = u;                  % Bias in v
% r_low_u = u;                % upper u 95% coverage interval 
% r_low_v = u;                % upper v 95% coverage interval 
% r_high_u = u;               % lower u 95% coverage interval 
% r_high_v = u;               % lower v 95% coverage interval 
rr_high_u = u;              % upper u (r^2) 95% coverage interval 
rr_high_v = u;              % upper v (r^2) 95% coverage interval 
Dudy  = single(u);          % Shear
Dvdx  = Dudy;               % Shear
PDen = Dudy;
PkD = Dudy;
PDia  = Dudy;
X=double(a.X);
Y=double(a.Y);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Load all *.mat files and store in three dimensional array
n = 1;
for i = str2double(Data.datafstart):str2double(Data.datafstep):str2double(Data.datafend)
    fname = [pltdirec Data.outbase sprintf(['%0.' Data.datazeros 'i.mat'],i)];
    a=load(fname);
    if (size(a.U,2) == 1)
        [a.X,a.Y,a.U,a.V,a.Eval] = five_matrixform(a.X,a.Y,a.U,a.V,a.Eval);
    end
    u(:,:,n) = a.U;
    v(:,:,n) = a.V;
    %-----------------------------------
    %Add part to remove values of low density (add input to GU for minDen)
    if Data.denlimit=='1'
    a.Eval(a.P_Dens<str2double(Data.minDen))=1;
    end
    %-----------------------------------
    eval(:,:,n) = a.Eval;
    b_k_u(:,:,n) = abs(a.Bias_k(:,:,1));
    b_k_v(:,:,n) = abs(a.Bias_k(:,:,2));
%     r_low_u(:,:,n) = a.R_low(:,:,1);
%     r_low_v(:,:,n) = a.R_low(:,:,2);
%     r_high_u(:,:,n) = a.R_high(:,:,1);
%     r_high_v(:,:,n) = a.R_high(:,:,2);
    rr_high_u(:,:,n) = a.RR_95(:,:,1);
    rr_high_v(:,:,n) = a.RR_95(:,:,2);
    Dudy(:,:,n) = a.dudy;
    Dvdx(:,:,n) = a.dvdx;
    PDen(:,:,n) = a.P_Dens;
    PkD(:,:,n) = a.PkD;
    PDia(:,:,n) = a.P_Dia;
    
    n = n + 1 ;
end
n = n - 1;

% Load the uncertainty surface to find the range of parameters
surf = load([Data.surfdirec,'\',Data.surfname]);

clear a Data fname i pltdirec GB AvailableRAM
%Print Time to load data
eltime=cputime-t1;
fprintf('%0.2i:%0.2i.%0.0f\n',floor(eltime/60),floor(rem(eltime,60)),rem(eltime,60)-floor(rem(eltime,60)))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Plot histograms of Diameter, Density, and Shear
close all;

% figure(1); hist(PDia(:).*eval(:),100);
figure(1); hist(PDia(:),100); hold on;
plot([min(surf.Key(:,3)) min(surf.Key(:,3))],ylim,'r','LineWidth',1.5);
plot([max(surf.Key(:,3)) max(surf.Key(:,3))],ylim,'r','LineWidth',1.5);
xlabel('Diameter [pixels]'); ylabel('Counts');

figure(2); hist(PkD(:),100); hold on;
plot([min(surf.Key(:,4)) min(surf.Key(:,4))],ylim,'r','LineWidth',1.5);
plot([max(surf.Key(:,4)) max(surf.Key(:,4))],ylim,'r','LineWidth',1.5);
xlabel('Density [particles/(32x32)]'); ylabel('Counts');

figure(3); hist(abs(Dudy(:)),100); hold on;
plot([min(surf.Key(:,1)) min(surf.Key(:,1))],ylim,'r','LineWidth',1.5);
plot([max(surf.Key(:,1)) max(surf.Key(:,1))],ylim,'r','LineWidth',1.5);
xlabel('|du/dy| [pixels/pixel]'); ylabel('Counts');

figure(4); hist(abs(Dvdx(:)),100); hold on;
plot([min(surf.Key(:,1)) min(surf.Key(:,1))],ylim,'r','LineWidth',1.5);
plot([max(surf.Key(:,1)) max(surf.Key(:,1))],ylim,'r','LineWidth',1.5);
xlabel('|dv/dx| [pixels/pixel]'); ylabel('Counts');

figure(5); close;

%% compute the vector statistics and uncertainty
t1=cputime;
fprintf('computing statistics...          ')

%Make good values 1 and bad values 0
N=sum(eval,3);
eval(N==1)=0;N(N==1)=0;
N(N==0)=Inf;

%Average u velocity
Dia_ave=double(mean(PDia.*eval,3).*(n./N));
%Average v velocity
Den_ave=double(mean(PDen.*eval,3).*(n./N));
%Average Shear
dudy_ave=double(mean(Dudy.*eval,3).*(n./N));
dvdx_ave=double(mean(Dvdx.*eval,3).*(n./N));
clear PDia PDen Dudy Dvdx

%-----------------------------------

%AVERAGE U VELOCITY
u_ave=mean(u.*eval,3).*(n./N);
%u variance (u'u'bar)
uut=1./(N-1).*sum((u.*eval).^2,3)-N./(N-1).*u_ave.^2;
uut(isnan(uut))=0;
%u standard deviation sqrt(u'u'bar)(DaVis RMS Vx)
u_std=sqrt(uut);
% u velocity 95% coverage interval *(Eq 9)
Uu=sqrt(1./N.*(1.96*u_std).^2+((1./N).*sum(b_k_u.*eval,3)).^2);
%Precision Uncertainties
Puu=1.96*sqrt(2./(N-1)).*u_std.^2;
%Uncertainty in Variance (u'u'bar)
Uuu_minus=sqrt(1./((N-1).^2).*sum(sqrt((rr_high_u.*eval).^2+(b_k_u.*eval).^4),3).^2 +  Puu.^2);
Uuu_plus=Puu;

%-----------------------------------

%AVERAGE V VELOCITY
v_ave=mean(v.*eval,3).*(n./N);
%v variance v'v'bar
vvt=1./(N-1).*sum((v.*eval).^2,3)-N./(N-1).*v_ave.^2;
vvt(isnan(vvt))=0;
%v standard deviation sqrt(v'v'bar) (DaVis RMS Vy)
v_std=sqrt(vvt);
% v velocity 95% coverage interval *(Eq 9)
Uv=sqrt(1./N.*(1.96*v_std).^2+((1./N).*sum(b_k_v.*eval,3)).^2);
%Precision Uncertainties
Pvv=1.96*sqrt(2./(N-1)).*v_std.^2;
%Uncertainty in Variance (v'v'bar)
Uvv_minus=sqrt(1./((N-1).^2).*sum(sqrt((rr_high_v.*eval).^2+(b_k_v.*eval).^4),3).^2 + Pvv.^2);
Uvv_plus=Pvv;

%-----------------------------------

%COVARIANCE (u'v'bar)
uvt=1./(N-1).*sum((u.*v.*eval),3)-N./(N-1).*u_ave.*v_ave;
uvt(isnan(uvt))=0;
%Precision Uncertainties
Puv=1.96*sqrt(2./(N-1)).*u_std.*v_std;
%Uncertainty in covariance (u'v'bar)      
Uuv_minus=sqrt(1./((N-1).^2).*sum(sqrt((rr_high_u.*eval).^2+(b_k_u.*eval).^4),3).*sum(((rr_high_v.*eval).^2+(b_k_v.*eval).^4),3) + Puv.^2);
Uuv_plus=Puv;

%-----------------------------------

%VELOCITY MAGNITUDE
uv = (u.*u+v.*v).^(0.5);
clear u v
%Mean velocity magnitude
uv_ave=mean(uv.*eval,3).*(n./N);
%Velocity magnitude variance (uv'uv'bar)
uvuvt=1./(N-1).*sum((uv.*eval).^2,3)-N./(N-1).*uv_ave.^2;
uvuvt(isnan(uvuvt))=0;
clear uv
%Velocity magnitude standard deviation sqrt(uv'uv'bar)
uv_std=sqrt(uvuvt);
%Bias magnitude
b_k_uv=((b_k_v.^2+b_k_u.^2).^(0.5));
clear b_k_u b_k_v
%"Taylor-Series and Monte-Carlo... " paper (Eq.29)
%Lower velocity magnitude 95% coverage interval
% r_low_uv=(max((r_low_u.^2+r_low_v.^2+2*r_low_u.*r_low_v).^(0.5),(r_low_u.^2+r_high_v.^2+2*r_low_u.*r_high_v).^(0.5)));
%Upper velocity magnitude 95% coverage interval
% r_high_uv=(max((r_high_u.^2+r_high_v.^2+2*r_high_u.*r_high_v).^(0.5),(r_high_u.^2+r_low_v.^2+2*r_high_u.*r_low_v).^(0.5)));
%R^2 velocity magnitude 95% coverage interval
rr_high_uv=(rr_high_u.^2+rr_high_v.^2+2*rr_high_u.*rr_high_v).^(0.5);
% velocity magnitude 95% coverage interval *(Eq 9)
Uuv=sqrt(1./N.*(1.96*uv_std).^2+((1./N).*sum(b_k_uv.*eval,3)).^2);
Puvuv=1.96*sqrt(2./(N-1)).*uv_std.^2;
%Uncertainty in Velocity magnitude variance (uv'uv'bar)        
Uuvuv_minus=sqrt(1./((N-1).^2).*sum(sqrt((rr_high_uv.*eval).^2+(b_k_uv.*eval).^4),3).^2+ Puvuv.^2);
Uuvuv_plus=Puvuv;

%-----------------------------------

eltime=cputime-t1;
fprintf('%0.2i:%0.2i.%0.0f\n',floor(eltime/60),floor(rem(eltime,60)),rem(eltime,60)-floor(rem(eltime,60)))

%write the vector statistics
save(FileName,'Den_ave','Dia_ave','dudy_ave','dvdx_ave','N','Puu','Puv','Puvuv','Pvv','u_ave','u_std','Uu','uut','Uuu_minus','Uuu_plus','Uuv','Uuv_minus','Uuv_plus','Uuvuv_minus','Uuvuv_plus','Uv','uv_ave','uv_std','uvt','uvuvt','Uvv_minus','Uvv_plus','v_ave','v_std','vvt','X','Y'); 
%write_plt_stats([pltdirec 'Vector_Stats.plt'],X,Y,u_ave,u_std,u_plus,u_minus,v_ave,v_std,v_plus,v_minus,uv_ave,uv_std,R_Ave,N,'Vector Statistics')
X;Y;u_ave;Uu;v_ave;Uv;uv_ave;Uuv;dudy_ave;dvdx_ave;uvuvt;N;uut;Uuu_minus;Uuu_plus;vvt;Uvv_minus;Uvv_plus;uvt;Uuv_minus;Uuv_plus;uvuvt;Uuvuv_minus;Uuvuv_plus;Puu;Pvv;Puv;Puvuv;Dia_ave;Den_ave;

eltime=cputime-tf;
fprintf('total frame time...              %0.2i:%0.2i.%0.0f\n\n',floor(eltime/60),floor(rem(eltime,60)),rem(eltime,60)-floor(rem(eltime,60)))
fprintf('\n----------------------------------------------------\n')

% %% Write Statistics plt
% function []=write_plt_stats(fname,X,Y,Uave,Ustd,Uplus,Uminus,Vave,Vstd,Vplus,Vminus,UVave,UVstd,Rave,N,title)
% %output text
% fprintf('saving...                        ')
% t1=cputime;
% %convert to matrix if necessary
% if size(X,2)==1
%     [X,Y,Uave,Ustd,Uplus,Uminus,Vave,Vstd,Vplus,Vminus,N]=matrixform(X,Y,Uave,Ustd,Uplus,Uminus,Vave,Vstd,Vplus,Vminus,N);
% end
% %find I,J for plt
% S = size(Uave);
% %generate text file
% fid = fopen(fname,'w');
% if fid==-1
%     error(['error creating file ',fname])
% end
% %header lines
% fprintf(fid,['TITLE        = "' title '"\n']);
% fprintf(fid,'VARIABLES    = "X", "Y", "Uave", "Ustd", "Ur+", "Ur-", "Vave", "Vstd", "Vr+", "Vr-", "UVave", "UVstd", "Rave", "N"\n');
% %fprintf(fid,'ZONE T="Time=%0.6f" I=%i J=%i C=BLACK\n',0,S(2),S(1));
% fprintf(fid,'ZONE T="Time=%0.6f", I=%i, J=%i\n',0,S(2),S(1));
% %write data
% for i=1:S(1)
%     for j=1:S(2)
%         if isnan(Uave(i,j)) || isnan(Vave(i,j))
%             %second check to ensure no nans present
%             fprintf(fid,'%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n',X(i,j),Y(i,j),0,0,0,0,0,0,0,0,0,0);
%         else
%             fprintf(fid,'%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n',X(i,j),Y(i,j),Uave(i,j),Ustd(i,j),Uplus(i,j),Uminus(i,j),Vave(i,j),Vstd(i,j),Vplus(i,j),Vminus(i,j),UVave(i,j),UVstd(i,j),Rave(i,j),N(i,j));
%         end
%     end
% end
% fclose(fid);
% %output text
% eltime=cputime-t1;
% fprintf('%0.2i:%0.2i.%0.0f\n',floor(eltime/60),floor(rem(eltime,60)),rem(eltime,60)-floor(rem(eltime,60)))

%% VECTOR TO MATRIX SUBFUNCTION
function [X,Y,U,V,Eval]=five_matrixform(x,y,u,v,eval)

%find unique x and y grid points
a=sort(unique(x));
b=sort(unique(y));
N=length(x);

%initialize matrices
U=nan(length(b),length(a));
V=nan(length(b),length(a));
Eval=nan(length(b),length(a));

%generate grid matrix
[X,Y]=meshgrid(a,b);

%generate variable matrices (nans where no data available)
for n=1:N
    I=find(b==y(n));
    J=find(a==x(n));
    U(I,J) = u(n);
    V(I,J) = v(n);
    Eval(I,J) = eval(n);
end