clear all; close all; clc; dbstop if error

%% Plot SRQ Data from Stats.m files
% Plot time-mean and Reynolds stress profiles from Stats.m files.  Also calculates wall shear stress by
% directly measuring velocity gradient at the wall.  Also exports profiles of SRQ data to include in a
% database.  Based off ProfilesfromStats4_2.m.

% Version 1_1, Blake Lance, 4 June 2015
%   Changed from plots of u to plotyy of u and u'u'
% Version 1_2, Blake Lance, 4 June 2015
%   Changed from plotyy to manually adding second y axis
% Version 2, Blake Lance, 16 June 2015
%   Added loop for aided and opposed since most functions are repeated
% Version 2_1, Blake Lance, 16 June 2015
%   Added double-y plots for difference of u and u'u' from buoyancy
% Version 2_2, Blake Lance, 16 June, 2015
%   Added shear calculation by calls to external functions
% Version 2_3, Blake Lance, 17 June 2015
%   Added option to export data to csv files.  Folders for aided and opposed cases will be created.  Files
%       are uvw and shear.  Both will contain X,Y,Z.  The uvw file contains
%       u,v,u'u',v'v',u'v',Uu,Uv,Uuup,Uuum,Uvvp,Uvvm,Uuvp,Uuvm.  The shear file contains X,Y,Z,shear,Ushear.
% Version 2_4, Blake Lance, 3 July 2015
%   Added turbulence intensity profile plot
% Version 2_5, Blake Lance, 30 July 2015
%   Now imports more data for repeatability analysis and plots
%   Always uses the vertical center for proloc
% Version 2_6, Blake Lance, 31 July 2015
%   Added repeatability plots for Reynolds normal stress
% Version 2_7, Blake Lance, 10 August 2015
%   Change normalization quantity to be set in user input section so bulk velocity at inlet may be used
% Version 2_8, Blake Lance, 15 August 2015
%   Added calculations for the momentum thickness and its Reynolds number
%   Added option to include repeatability study and plots

%% User input defined by proloc and serial number for the Uncerstainty
% proloc = 65;% The profile location (can be commented out for middle center to be used)
% u_bulk = 4.31;   % Inlet bulk velocity used for normalization [m/s]
u_bulk = 2.44;   % Inlet bulk velocity used for normalization [m/s]
Rep = 1;    % Whether to input and plot repeatability data
Trim = 0;   % Number of extra values to trim off free-stream edge
dy = 1.25;  % Plot uncertainty bands dy apart [mm]
yp = 3;     % Maximum y+ value for linear fit
Fit = 10;   % The initial number of near-wall velocity data to fit for shear calc
Export = 0; % Option to export data to csv files
nu = 2.097E-5;  % Kinematic viscosity at 80C [m^2/s]

x = [0.1617; 0.7777; 1.3936]; % x-coordinate array [m]
FigPos = [0.5,6,3,2.75;4,6,3,2.75]; % Figure position vectors
axislim1 = [0 14.5 0 1.175]; % limits for u axes
axislim2 = [0 14.5 0 0.042]; % limits for u'u' axes
arrow_pos_x1 = [0.6,0.5;0.3,0.4]; % x-position for text arrows for axis 1 (left)
arrow_pos_x2 = [0.55,0.45;0.55,0.45]; % x-position for text arrows for axis 2 (right)
arrow_pos_y1 = [0.775,0.8;0.8,0.75]; % y-position for text arrows for axis 1 (left)
arrow_pos_y2 = [0.325,0.275;0.55,0.5]; % y-position for text arrows for axis 2 (right)
linestyle = {'-';'-.';'--'};
linecolor = {[0.000,  0.447,  0.741]; [0.850,  0.325,  0.098]; [0.929,  0.694,  0.125]};
marker = {'x','o','+'};
marker_repeat = {'*','d';'+',''};
FontSize = 11; % Font size for figure text
SRQdir = {'SRQs-Aided','SRQs-Opposed'};

%% Open the Stats file from PIV_Uncertainty

% Determine if data selection file exists and prompt the user to select files if not
if exist('SteadySRQ_PlotSelection.mat','file') ~= 0
    load('SteadySRQ_PlotSelection.mat');
else % Select multiple Stats files
    p(1,:) = uigetfile_n_dir(pwd,'Select the Buoyancy-aided Stats file(s), or hit cancel');
    p(2,:) = uigetfile_n_dir(pwd,'Select the Buoyancy-opposed Stats file(s), or hit cancel');
    if Rep == 1
        p_r(1,:) = ...
            uigetfile_n_dir(pwd,...
            'Select the Buoyancy-aided first repeatability Stats file(s), or hit cancel');
        p_r(2,:) = ...
            uigetfile_n_dir(pwd,...
            'Select the Buoyancy-aided second repeatability Stats file(s), or hit cancel');
        p_r(3,:) = uigetfile_n_dir(pwd,...
            'Select the Buoyancy-opposed repeatability Stats file(s), or hit cancel');
        save('SteadySRQ_PlotSelection.mat','p','p_r');
    else
        save('SteadySRQ_PlotSelection.mat','p');
    end    
end
tic;

%% Define Heated Wall Temperatures
% These are the measured temperatures for the Mixed case (S4 and S7)
Temp = [408.82,411.55,410.79;408.76,410.64,410.42]; % [K]
U_Temp = [1.31,1.11,1.05;1,1,1]; % [K]

%% Calculate Air Properties
% See Tsilingiris2007 and Melling97
T0 = 23.4 + 273.15;   % Room temperature [K]
P = 86696; % Atmospheric pressure [Pa]
phi0 = 0.336;  % Relative humidity at room temperature

% Saturation pressure at room temperature [Pa]
P_sat0 = 1000*( 1.901888 - 2.1424081E-1*(T0-273.15) + 1.2057211E-2*(T0-273.15)^2 - ...
    1.3867581E-4*(T0-273.15)^3 + 1.3888951E-6*(T0-273.15)^4 );
omega = 0.622*phi0*P_sat0/( P - phi0*P_sat0 ); % Specific humidity at any temperature
R_mix = 8.31434/((omega+1)/((omega/0.01802)+(1/0.02897))); % Compute the Mixture Gas Constant [m^2/(K*s^2)]
rho = P./(Temp*R_mix);

% Compute Dynamic Viscosity using Sutherland's Law
a = 1.458E-6; b = 110.4;
mu = a*Temp.^(3/2)./( Temp + b ); % [kg/(m*s)]
U_mu = U_Temp*a.*sqrt(Temp).*(Temp+3*b)./(2*(Temp+b).^2);

%% Perform Plotting and Calculations in Loop
Figs = 0;
ylabh = zeros(2,1);
for j=1:2 % Loop over aided and opposed cases
    if j == 1
        Opposed = 0;
    elseif j == 2
        Opposed = 1;
    end
    if isempty(p(j,:)) == 0 % Only enter loop if files were selected
        Ni = length(p(j,:));
        ax1 = zeros(Ni,1);
        ax2 = zeros(Ni,1);
        h1 = zeros(Ni,1); % Handle array
        h2 = zeros(Ni,1); % Handle array
        Figs = Figs + 1;
        for i=1:Ni % Loop through cells in p(j)
            load(p{j,i});
            dx = X(1,2) - X(1,1); % Vector spacing [mm]
            Nth_y = round(dy/dx); % Plot every Nth_y uncertainty band
            
            %% Overwrite NaNs with 0s
            v_ave(isnan(v_ave))=0;
            Uv(isnan(Uv))=0;
            vvt(isnan(vvt))=0;
            Uvv_plus(isnan(Uvv_plus))=0;
            Uvv_minus(isnan(Uvv_minus))=0;
            
            u_ave(isnan(u_ave))=0;
            Uu(isnan(Uu))=0;
            uut(isnan(uut))=0;
            Uuu_plus(isnan(Uuu_plus))=0;
            Uuu_minus(isnan(Uuu_minus))=0;
            
            uv_ave(isnan(uv_ave))=0;
            Uuv(isnan(Uuv))=0;
            uvt(isnan(uvt))=0;
            Uuv_plus(isnan(Uuv_plus))=0;
            Uuv_minus(isnan(Uuv_minus))=0;
            
            %% Adaptive trimming to masked area
            [imin,imax,jmin,jmax] = TrimToMask(v_ave);
            
            if jmin == 0; jmin = 1; end

            xlower = X(1,jmin);
            Xshift = X - xlower;    % Shift x_2D array so y=0 is at the left
            
            %% Define proloc as the center of vertical height
            proloc = floor(0.5*(imin-imax+1));
            
            %% Flip Xshift if Opposed, also shift values if the masked area isn't symmetric
            if Opposed == 1
                u_ave = -u_ave; v_ave = -v_ave; uv_ave = -uv_ave; % Buoyancy-opposed, change sign
                
                dx = abs( X(1,1) - X(1,2) );  % Spacing between vectors in mm given by DaVis
                Xshift = fliplr(Xshift);
                TrimR = size(v_ave,2) - jmax + 1;   TrimL = jmin;
                % Add or subtract values so assymetric Xshift will have consistent values
                Xshift = Xshift - dx*(TrimR - TrimL);
            end

            %% Calculate BL Thicknesses
            if Opposed == 1
                % Eq 5-6 in Kays and Crawford, assuming constant density
                delta2(j,i) = -trapz(0.001*Xshift(proloc,:),...
                    v_ave(proloc,:)/v_ave(proloc,1).*(1-v_ave(proloc,:)/v_ave(proloc,1)));
                Re_delta2(j,i) = v_ave(proloc,1)*delta2(j,i)/nu;
                % Determine virtual origin (from Eq 11-22 in Kays and Crawford)
                xVO(j,i) = delta2(j,i)^1.25*v_ave(proloc,1)^0.25/(0.0157*nu^0.25); % [m]
            else
                delta2(j,i) = trapz(0.001*Xshift(proloc,:),...
                    v_ave(proloc,:)/v_ave(proloc,end).*(1-v_ave(proloc,:)/v_ave(proloc,end)));
                Re_delta2(j,i) = v_ave(proloc,end)*delta2(j,i)/nu;
                xVO(j,i) = delta2(j,i)^1.25*v_ave(proloc,end)^0.25/(0.0157*nu^0.25); % [m]
            end
%             figure; plot(Xshift(proloc,:),v_ave(proloc,:)); % Check if free-stream is measured
%             xlabel('$y$ [mm]','Interpreter','latex');
%             ylabel('$\overline{u}$ [m/s]','Interpreter','latex');
%             set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');

            %% Plot the Profiles
            
            % Determine the index where the data are within the desired plotting area
            Nx = find( Xshift(1,:) > axislim1(2) ,1);
            
            % Plot u and u'u' for all three locations
            figure(j);
            if i == 1; set(gcf,'Units','inches','Position',FigPos(j,:)); end;
            
            % Start first axes on left
            ax1(i) = axes; hold on;
            if Opposed == 0 % The indeces are different if Opposed or not
                h1(i) = plot(Xshift(proloc,:),v_ave(proloc,:)/u_bulk);
                errorbar(Xshift(proloc,1:Nth_y:Nx),v_ave(proloc,1:Nth_y:Nx)/u_bulk,...
                    Uv(proloc,1:Nth_y:Nx)/u_bulk,'Color',linecolor{i},'LineStyle','none','LineWidth',1.25);
            elseif Opposed == 1
                h1(i) = plot(Xshift(proloc,:),v_ave(proloc,:)/u_bulk);
                errorbar(Xshift(proloc,Nx:Nth_y:end),v_ave(proloc,Nx:Nth_y:end)/u_bulk,...
                    Uv(proloc,Nx:Nth_y:end)/u_bulk,'Color',linecolor{i},'LineStyle','none','LineWidth',1.25);
            end
            if i == 1;
                annotation('textarrow',arrow_pos_x1(j,:),arrow_pos_y1(j,:),...
                    'String','$\overline{u}/\overline{u}_\mathrm{bulk}$','Interpreter','latex','FontSize',FontSize);
            end
            
            % Start second axes on right
            ax2(i) = axes; hold on;
            if Opposed == 0 % The indeces are different if Opposed or not
                h2(i) = plot(Xshift(proloc,:),vvt(proloc,:)/u_bulk^2);
                errorbar(Xshift(proloc,1:Nth_y:Nx),vvt(proloc,1:Nth_y:Nx)/u_bulk^2,...
                    Uvv_minus(proloc,1:Nth_y:Nx)/u_bulk^2,...
                    Uvv_plus(proloc,1:Nth_y:Nx)/u_bulk^2,'Color',linecolor{i},...
                    'LineStyle','none','LineWidth',1.25);
            elseif Opposed == 1
                h2(i) = plot(Xshift(proloc,:),vvt(proloc,:)/u_bulk^2);
                errorbar(Xshift(proloc,Nx:Nth_y:end),vvt(proloc,Nx:Nth_y:end)/u_bulk^2,...
                    Uvv_minus(proloc,Nx:Nth_y:end)/u_bulk^2,...
                    Uvv_plus(proloc,Nx:Nth_y:end)/u_bulk^2,'Color',linecolor{i},...
                    'LineStyle','none','LineWidth',1.25);
            end
            if i == 1;
                annotation('textarrow',arrow_pos_x2(j,:),arrow_pos_y2(j,:),'String',...
                    '$\overline{u''u''}/\left(\overline{u}_\mathrm{bulk}\right)^2$',...
                    'Interpreter','latex','FontSize',FontSize);
            end
            
            % Fix up appearance
            if i ~= 1
                set(ax1(i),'Box','off','Color','none'); % Clear background color
            else
                set(ax1(i),'Box','off'); % Have the first axes background color default
            end
            set(ax2(i),'Box','off','Color','none','YAxisLocation','right',...
                'XAxisLocation','top','XTickLabel',[]);
            if i == 1
                xlabel(ax1(i),'$y$ [mm]','Interpreter','latex');
                ylabel(ax1(i),'$\overline{u}/\overline{u}_\mathrm{bulk}$','Interpreter','latex');
                ylabh(j) = ylabel(ax2(i),'$\overline{u''u''}/\left(\overline{u}_\mathrm{bulk}\right)^2$',...
                    'Interpreter','latex');
                pos1 = get(ax1(i),'Position'); % Get and store position of ax1
            else
                set(ax1(i),'XTickLabel',[],'YTickLabel',[]);
                set(ax2(i),'XTickLabel',[],'YTickLabel',[]);
            end
            set(h1(i),'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25);
            set(h2(i),'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25);
            axis(ax1(i),axislim1);
            axis(ax2(i),axislim2);
            set(ax1(i),'Position',pos1);
            set(ax2(i),'Position',pos1);
            
            % Plot turbulence intensity in streamwise direction (see Kays and Crawford, Ch. 11)
            TIx = 100*sqrt(vvt(proloc,:))./v_ave(proloc,:); TIx(isnan(TIx)) = 0;
            figure(6); hold on;
%             if j == 1 && i == 1; Figs = Figs + 1; end
            set(gcf,'Units','inches','Position',[7.5,2,3.5,3]);
%             plot(Xshift(proloc,:),TIx,'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25);
            line_fewer_markers(Xshift(proloc,:),TIx,15,'LineStyle',linestyle{i},...
                'Color',linecolor{i},'LineWidth',1.25,'Marker',marker{j},'MarkerSize',6);
            if j == 2 && i == 3
                axis([axislim1(1),axislim1(2),0,45]);
                xlabel('$y$ [mm]','Interpreter','latex');
                ylabel('$TI_x$ [\%]','Interpreter','latex');
                leg6 = legend('$x_1$ Aid','$x_2$ Aid','$x_3$ Aid','$x_1$ Opp','$x_2$ Opp','$x_3$ Opp');
                set(leg6,'Interpreter','latex','Location','northeast');
                set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex','Box','on');
                set(findall(gcf,'type','axes'),'FontSize',FontSize);
                set(findall(gcf,'type','text'),'FontSize',FontSize);
                set(findall(gcf,'type','legend'),'FontSize',FontSize-1);
                tightfig;
            end
            
            %% Calculate Wall Shear
            
            % Calculate velocity gradient and its uncertainty
            Uv_fit = Uv;
            Uv_fit(Uv_fit<0.01 & Uv_fit~=0) = 0.01; % Enforce a floor on uncertainty
%             Uv_fit(Uv_fit<0.01) = 0.01; % Enforce a floor on uncertainty
            if Opposed == 0 % Force wall to be on right
                [ y0, FitOut, dudy, U_dudy ] = PIV_dudy( fliplr(Xshift(proloc,:)), ...
                    fliplr(v_ave(proloc,:)),fliplr(Uv_fit(proloc,:)),...
                    rho(j,i), mu(j,i), yp, Fit );
            elseif Opposed == 1 % Wall is on right
                [ y0, FitOut, dudy, U_dudy ] = PIV_dudy( Xshift(proloc,:), ...
                    v_ave(proloc,:),Uv_fit(proloc,:), rho(j,i), mu(j,i), yp, Fit );
            end
            
            % Plot velocity data and fit
%             if j == 2 && i == 2
            if j == 2
                figure(5); hold on;
                set(gcf,'Units','inches','Position',[7.5 6 3.5 2.75]);
                %                 leg_h3(count) = plot(Xshift(proloc,:),v_ave(proloc,:),'LineWidth',1.25);
                h5(i,1) = plot(Xshift(proloc,:),v_ave(proloc,:),...
                    'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25);
                % Appearantly MATLAB sets the width of the uncertainty bands to 2% of the x range, so use less
                errorbar(Xshift(proloc,275:2:jmax),v_ave(proloc,275:2:jmax),...
                    Uv_fit(proloc,275:2:jmax),'Color',linecolor{i},'LineStyle','none','LineWidth',1.25);
                xlabel('$y$ [mm]','Interpreter','latex'); 
                ylabel('$\overline{u}$ [m/s]','Interpreter','latex');
                if Opposed == 0
                    h5(i,2) = plot([Xshift(proloc,jmin), Xshift(proloc,jmin+FitOut-1)],...
                        [y0, y0+dudy*(Xshift(proloc,jmax)-Xshift(proloc,jmax-FitOut+1))/1000],...
                        'Color',[0.25 0.25 0.25],'LineWidth',1.25);
                elseif Opposed == 1
                    h5(i,2) = plot([Xshift(proloc,jmax), Xshift(proloc,jmax-FitOut+1)],...
                        [y0, y0-dudy*(Xshift(proloc,jmax)-Xshift(proloc,jmax-FitOut+1))/1000],...
                        'Color',[0.25 0.25 0.25],'LineWidth',1.25);
                end
                axis([axislim1(1) 2.1 axislim1(3) 1.4]);
                if i == 3
%                     axis([axislim1(1) 2.1 axislim1(3) 1.4]);
                    leg5 = legend(h5(1:4),'$x_1$','$x_2$','$x_3$','Line Fit');
                    set(leg5,'Interpreter','latex','Location','northwest');
                    set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex','Box','on');
                    set(findall(gcf,'type','axes'),'FontSize',FontSize);
                    set(findall(gcf,'type','text'),'FontSize',FontSize);
                    set(findall(gcf,'type','legend'),'FontSize',FontSize-1);
                    tightfig;
                end
            end
            
            % Calculate shear and its uncertainty
            Shear(j,i) = mu(j,i)*dudy;
            U_Shear(j,i) = sqrt( (dudy*U_mu(j,i))^2 + (mu(j,i)*U_dudy)^2 );
            
            % Calculate u_tau
            if j == 2 && i == 3; u_tau = sqrt(Shear./rho); end
            
            %% Export Data to csv Files
            if Export == 1
                % Check if Aided and Opposed folders exist, create if they don't
                if exist(SRQdir{1},'dir') == 0
                    mkdir(SRQdir{1}); mkdir(SRQdir{2});
                end
                
                % Prepare headers
                if j == 1 && i == 1
                    head_uvw = {'x[m]','y[m]','z[m]','u[m/s]','v[m/s]',...
                        'u''u''[m^2/s^2]','v''v''[m^2/s^2]','u''v''[m^2/s^2]',...
                        'Uu[m/s]','Uv[m/s]','Uuup[m^2/s^2]','Uuum[m^2/s^2]','Uvvp[m^2/s^2]',...
                        'Uvvm[m^2/s^2]','Uuvp[m^2/s^2]','Uuvm[m^2/s^2]'};
                    head_shear = {'x[m]','y[m]','z[m]','shear[Pa]','Ushear[Pa]'};
                end
                
                % Compile data to export
                if Opposed == 0
                    Duvw = [repmat(x(i),length(N)-jmin+1,1),0.001*Xshift(proloc,jmin:end)',...
                        zeros(length(N)-jmin+1,1),...
                        v_ave(proloc,jmin:end)',u_ave(proloc,jmin:end)',...
                        vvt(proloc,jmin:end)',uut(proloc,jmin:end)',uvt(proloc,jmin:end)',...
                        Uv(proloc,jmin:end)',Uu(proloc,jmin:end)',...
                        Uvv_plus(proloc,jmin:end)',Uvv_minus(proloc,jmin:end)',...
                        Uuu_plus(proloc,jmin:end)',Uuu_minus(proloc,jmin:end)',...
                        Uuv_plus(proloc,jmin:end)',Uuv_minus(proloc,jmin:end)'];
                elseif Opposed == 1
                    Duvw = [repmat(x(i),jmax,1),0.001*Xshift(proloc,1:jmax)',zeros(jmax,1),...
                        v_ave(proloc,1:jmax)',u_ave(proloc,1:jmax)',...
                        vvt(proloc,1:jmax)',uut(proloc,1:jmax)',uvt(proloc,1:jmax)',...
                        Uv(proloc,1:jmax)',Uu(proloc,1:jmax)',...
                        Uvv_plus(proloc,1:jmax)',Uvv_minus(proloc,1:jmax)',...
                        Uuu_plus(proloc,1:jmax)',Uuu_minus(proloc,1:jmax)',...
                        Uuv_plus(proloc,1:jmax)',Uuv_minus(proloc,1:jmax)'];
                    Duvw = flipud(Duvw);
                end
                if i == 3 % Only write shear when all values are available
                    Dshear = [x,zeros(3,1),zeros(3,1),Shear(j,:)',U_Shear(j,:)'];
                end
                
                % Export data
                csvwrite_with_headers([SRQdir{j},'\uvw_x',int2str(i),'.csv'],Duvw,head_uvw);
                if i == 3
                    csvwrite_with_headers([SRQdir{j},'\shear.csv'],Dshear,head_shear);
                end
            end
            
            %% Store Data for Comparison
            if j == 1 && i == 1;
                Xshift_store = Xshift(proloc,:); % Store the x-data from the first set
                v_ave_store(j,i,:) = v_ave(proloc,:);
                vvt_store(j,i,:) = vvt(proloc,:);
            else % Interpolate subsequent data for common x-values
                v_ave_store(j,i,:) = interp1(Xshift(proloc,:),v_ave(proloc,:),Xshift_store);
                vvt_store(j,i,:) = interp1(Xshift(proloc,:),vvt(proloc,:),Xshift_store);
            end
        end

        % Fix up current figure
        figure(j); leg = legend(h1,'$x_1$','$x_2$','$x_3$'); 
        if Opposed == 0
            set(leg,'Location','East','Interpreter','latex');
        elseif Opposed == 1
            legpos = get(leg,'Position');
            set(leg,'Position',[0.25,0.2,0.75*legpos(3),0.75*legpos(4)],'Interpreter','latex');
        end
        set(ax1(1),'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
        set(ax2(1),'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
        set(findall(gcf,'type','axes'),'FontSize',FontSize);
        set(findall(gcf,'type','text'),'FontSize',FontSize);
        set(findall(gcf,'type','legend'),'FontSize',FontSize);
        tightfig;
    end
end

%% Plot Comparisons

% Plot u comparison
figure(Figs+1); hold on; Figs = Figs + 1;
set(gcf,'Units','inches','Position',[0.5,2,3.5,3]);
for i=1:Ni
    for j=1:2 % Plot velocity profiles
         line_fewer_markers(Xshift_store,squeeze(v_ave_store(j,i,:)),15,...
            'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25,...
            'Marker',marker{j},'MarkerSize',6);
    end
    line_fewer_markers(Xshift_store,... % Plot difference
        squeeze(v_ave_store(1,i,:)-v_ave_store(2,i,:)),15,...
        'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25,...
        'Marker',marker{3},'MarkerSize',6);
end
xlabel('$y$ [mm]','Interpreter','latex');
ylabel('$\overline{u}$ [m/s]','Interpreter','latex');
set(gca,'Box','on');
axis([axislim1(1),axislim1(2),axislim1(3),3.95]);
legstr = {'$x_1$ Aid','$x_1$ Opp','$x_1$ Diff',...
    '$x_2$ Aid','$x_2$ Opp','$x_2$ Diff',...
    '$x_3$ Aid','$x_3$ Opp','$x_3$ Diff'};
[leg2,leg3,~,~] = columnlegend(3,legstr,'Location','north');
set(findall(leg3,'Type','text'),'Interpreter','latex','FontSize',FontSize-1);
legpos = get(leg2,'Position'); 
set(leg2,'Position',[1.1*legpos(1) legpos(2) 0.75*legpos(3) 0.925*legpos(4)]);
set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
set(findall(gcf,'type','axes'),'FontSize',FontSize);
set(findall(gcf,'type','text'),'FontSize',FontSize);
set(findall(gcf,'type','legend'),'FontSize',FontSize);
tightfig2_1(gcf);

% Plot u'u' comparison
figure(Figs+1); hold on; Figs = Figs + 1;
set(gcf,'Units','inches','Position',[4,2,3.5,3]);
for i=1:Ni
    for j=1:2 % Plot velocity profiles
         line_fewer_markers(Xshift_store,squeeze(vvt_store(j,i,:)),15,...
            'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25,...
            'Marker',marker{j},'MarkerSize',6);
    end
    line_fewer_markers(Xshift_store,... % Plot difference
        squeeze(vvt_store(1,i,:)-vvt_store(2,i,:)),15,...
        'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25,...
        'Marker',marker{3},'MarkerSize',6);
end
xlabel('$y$ [mm]','Interpreter','latex');
ylabel('$\overline{u''u''}$ [m$^2$/s$^2$]','Interpreter','latex');
set(gca,'Box','on');
axis([axislim1(1),axislim1(2),-0.1,0.25]);
legstr = {'$x_1$ Aid','$x_1$ Opp','$x_1$ Diff',...
    '$x_2$ Aid','$x_2$ Opp','$x_2$ Diff',...
    '$x_3$ Aid','$x_3$ Opp','$x_3$ Diff'};
[leg2,leg3,~,~] = columnlegend(3,legstr,'Location','north');
set(findall(leg3,'Type','text'),'Interpreter','latex','FontSize',FontSize-1);
legpos = get(leg2,'Position'); 
set(leg2,'Position',[1.1*legpos(1) legpos(2) 0.7*legpos(3) 0.925*legpos(4)]);
set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
set(findall(gcf,'type','axes'),'FontSize',FontSize);
set(findall(gcf,'type','text'),'FontSize',FontSize);
set(findall(gcf,'type','legend'),'FontSize',FontSize);
tightfig2_1(gcf);

% Rotate right-hand ylabel on double-y plots
set(ylabh,'Rotation',-90,'Units','normalized','Position',[1.25 0.5]);

%% Now Repeat Some of the Previous Work for Repeatability Plots
if Rep == 1
    % Remove repeated values in p_r
    [~,ia,~] = unique(p_r);
    count = 0;
    for j=1:size(p_r,1) % j goes to 3 since there are two repeats for the aided case (j=1,2)
        for i=1:size(p_r,2)
            count = count + 1;
            if ~ismember(count,ia)
                p_r{i,j} = [];
            end
        end
    end
    
    % Plot original velocity profiles
    figure(7); set(gcf,'Units','inches','Position',[11,6,3.5,3]); hold on;
    figure(8); set(gcf,'Units','inches','Position',[11,2,3.5,3]); hold on;
    for i=1:Ni
        for j=1:2 % Plot velocity profiles
            if j == 1; figure(7); elseif j == 2; figure(8); end
            line_fewer_markers(Xshift_store,squeeze(v_ave_store(j,i,:)),15,...
                'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25,...
                'Marker',marker{j},'MarkerSize',6);
        end
    end
    
    % Plot original Reynolds stress profiles
    figure(9); set(gcf,'Units','inches','Position',[14.5,6,3.5,3]); hold on;
    figure(10); set(gcf,'Units','inches','Position',[14.5,2,3.5,3]); hold on;
    for i=1:Ni
        for j=1:2 % Plot velocity profiles
            if j == 1; figure(9); elseif j == 2; figure(10); end
            line_fewer_markers(Xshift_store,squeeze(vvt_store(j,i,:)),15,...
                'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25,...
                'Marker',marker{j},'MarkerSize',6);
        end
    end
    
    for j=1:size(p_r,1) % j goes to 3 since there are two repeats for the aided case (j=1,2)
        if j == 1 || j == 2
            Opposed = 0;
        elseif j == 3
            Opposed = 1;
        end
        for i=1:size(p_r,2) % This number of x locations
            if isempty(p_r{j,i}) == 0 % Only enter loop if files were selected
                load(p_r{j,i});
                dx = X(1,2) - X(1,1); % Vector spacing [mm]
                
                %% Overwrite NaNs with 0s
                v_ave(isnan(v_ave))=0;
                Uv(isnan(Uv))=0;
                vvt(isnan(vvt))=0;
                Uvv_plus(isnan(Uvv_plus))=0;
                Uvv_minus(isnan(Uvv_minus))=0;
                
                u_ave(isnan(u_ave))=0;
                Uu(isnan(Uu))=0;
                uut(isnan(uut))=0;
                Uuu_plus(isnan(Uuu_plus))=0;
                Uuu_minus(isnan(Uuu_minus))=0;
                
                uv_ave(isnan(uv_ave))=0;
                Uuv(isnan(Uuv))=0;
                uvt(isnan(uvt))=0;
                Uuv_plus(isnan(Uuv_plus))=0;
                Uuv_minus(isnan(Uuv_minus))=0;
                
                %% Adaptive trimming to masked area
                [imin,imax,jmin,jmax] = TrimToMask(v_ave);
                
                if jmin == 0; jmin = 1; end
                
                xlower = X(1,jmin);
                Xshift = X - xlower;    % Shift x_2D array so y=0 is at the left
                
                %% Define proloc as the center of vertical height
                proloc = floor(0.5*(imin-imax+1));
                
                %% Flip Xshift if Opposed, also shift values if the masked area isn't symmetric
                if Opposed == 1
                    u_ave = -u_ave; v_ave = -v_ave; uv_ave = -uv_ave; % Buoyancy-opposed, change sign
                    
                    dx = abs( X(1,1) - X(1,2) );  % Spacing between vectors in mm given by DaVis
                    Xshift = fliplr(Xshift);
                    TrimR = size(v_ave,2) - jmax + 1;   TrimL = jmin;
                    % Add or subtract values so assymetric Xshift will have consistent values
                    Xshift = Xshift - dx*(TrimR - TrimL);
                end
                
                %% Add Results to Previous Mean Velocity Plots
                if j == 1 || j == 2;
                    figure(7); % Call up the correct figure
                    % Set the repeat value (R=1 was plotted earlier)
                    if j == 1; R = 2; elseif j == 2; R = 3; end
                elseif j == 3;
                    figure(8); R = 2; % Call up the correct figure
                end
                line_fewer_markers(Xshift(proloc,:),v_ave(proloc,:),15,...
                    'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25,...
                    'Marker',marker_repeat{R-1,Opposed+1},'MarkerSize',6);
                if i == 1 && j == 2 || i == 3 && j == 3
                    xlabel('$y$ [mm]','Interpreter','latex');
                    ylabel('$\overline{u}$ [m/s]','Interpreter','latex');
                    set(gca,'Box','on');
                    axis([axislim1(1),axislim1(2),axislim1(3),3.95]);
                    if j == 2
                        legstr = {'$x_1$ R1','$x_2$ R1','$x_3$ R1',...
                            '$x_1$ R2','$x_2$ R2','$x_3$ R2',...
                            '$x_1$ R3'};
                        [leg2,leg3,~,~] = columnlegend(3,legstr,'Location','north');
                    elseif j == 3
                        legstr = {'$x_1$ R1','$x_2$ R1','$x_3$ R1',...
                            '$x_1$ R2','$x_2$ R2','$x_3$ R2'};
                        [leg2,leg3,~,~] = columnlegend(2,legstr,'Location','north');
                    end
                    set(findall(leg3,'Type','text'),'Interpreter','latex','FontSize',FontSize-1);
                    legpos = get(leg2,'Position');
                    set(leg2,'Position',[1.1*legpos(1) legpos(2) 0.8*legpos(3) 0.95*legpos(4)]);
                    set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
                    set(findall(gcf,'type','axes'),'FontSize',FontSize);
                    set(findall(gcf,'type','text'),'FontSize',FontSize);
                    set(findall(gcf,'type','legend'),'FontSize',FontSize);
                    tightfig2_1(gcf);
                end
                
                %% Add Results to Previous Reynolds Stress Plots
                if j == 1 || j == 2;
                    figure(9); % Call up the correct figure
                    % Set the repeat value (R=1 was plotted earlier)
                    if j == 1; R = 2; elseif j == 2; R = 3; end
                elseif j == 3;
                    figure(10); R = 2; % Call up the correct figure
                end
                line_fewer_markers(Xshift(proloc,:),vvt(proloc,:),15,...
                    'LineStyle',linestyle{i},'Color',linecolor{i},'LineWidth',1.25,...
                    'Marker',marker_repeat{R-1,Opposed+1},'MarkerSize',6);
                if i == 1 && j == 2 || i == 3 && j == 3
                    xlabel('$y$ [mm]','Interpreter','latex');
                    ylabel('$\overline{u''u''}$ [m$^2$/s$^2$]','Interpreter','latex');
                    set(gca,'Box','on');
                    axis([axislim1(1),axislim1(2),axislim1(3),0.225]);
                    if j == 2
                        legstr = {'$x_1$ R1','$x_2$ R1','$x_3$ R1',...
                            '$x_1$ R2','$x_2$ R2','$x_3$ R2',...
                            '$x_1$ R3'};
                        [leg2,leg3,~,~] = columnlegend(3,legstr,'Location','north');
                    elseif j == 3
                        legstr = {'$x_1$ R1','$x_2$ R1','$x_3$ R1',...
                            '$x_1$ R2','$x_2$ R2','$x_3$ R2'};
                        [leg2,leg3,~,~] = columnlegend(2,legstr,'Location','north');
                    end
                    set(findall(leg3,'Type','text'),'Interpreter','latex','FontSize',FontSize-1);
                    legpos = get(leg2,'Position');
                    set(leg2,'Position',[1.1*legpos(1) legpos(2) 0.8*legpos(3) 0.95*legpos(4)]);
                    set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
                    set(findall(gcf,'type','axes'),'FontSize',FontSize);
                    set(findall(gcf,'type','text'),'FontSize',FontSize);
                    set(findall(gcf,'type','legend'),'FontSize',FontSize);
                    tightfig2_1(gcf);
                end
            end
        end
    end
end

toc;