close all; clear all; clc; dbstop if error;

% Code to interpolate the PIV data to a fine grid for input to Star-CCM+.
% This code reads the uvw, epsilon, and k files generated from the Inlet
% Analysis code and interpolates the data to a grid defined in the code.

% Updates:
% June 12, 2014 - Jeff Harris - updated comments
% Sept 17, 2014 - Blake Lance - Added option to pad the sides with zeros
    % Simplified the selection of files
    % Removed padding with zeros
    % Improved plots
    % Changed the definition of z since the incoming data won't be mapped orthogonally
    % Reinstated the orthogonal mapping and changed the operation if Opposed
    % Changed TriScatteredInterp function to scatteredInterpolant for future use
    % Changed to map the top wall data before the interpolation (still has dimpling effect that need to be
    %   corrected, and is only taking the correct data for Opposed)
    % Removed the omega2 option for the transition model
% version 4_2, Sept 25, 2014 - Blake Lance
    % Added section to manually set the velocity profile.  This is to test for Inlet having positive and
    %   negative u velocity such as when in the Opposed orientation and under natural convection.
    % Changed interpolation extrapolation to 'nearest' (removes zeros in the corners)
% version 4_3, Jan 12, 2015 - Blake Lance
    % Since PIV near walls is often biased high, added option to remove BL data and linearly interpolate
% version 5_0, Jan 12, 2015 - Blake Lance
    % Altered the previous code to work with Transient data. It automatically detects the large number of
    %   columns associated with Transient files.  This will only use the nominal values for use with StarCCM+
    % (no uncertainties). The data should be in the form 
    %       X,Y,Z,u(m/s)[t=0s],v(m/s)[t=0s],w(m/s)[t=0s],u(m/s)[t=0.2s],v(m/s)[t=0.2s],w(m/s)[t=0.2s],...
    % Compiled the figures into one and saves them to a movie.
% version 5_1, Jan 14, 2015 - Blake Lance
    % Data set 8_5 has unique PIV calibration and has 3 extra data points. This code will interpolate the data
    %   to share the same number of points as the other data sets.
    % Changed colorbar max to 75% of the real max. This allows the spikes to look more similar.
% version 5_2, Jan 19, 2015 - Blake Lance
    % The transient output data are too fine for the CFD (interpolation in StarCCM+ takes a long time). The
    % option is added to subsample the PIV data for the mapping and the output.
% version 5_3, Jan 26, 2015 - Blake Lance
    % Removed omega from plot and added plot of bulk velocity through the Transient

%% User Input

Export = 0;     % Export the output files?
Opposed = 1;    % If buoyancy opposed, set opposed to 1
Manual = 0;     % Whether to manually set the u velocity
Orthog = 0.0;   % Specify the portion of the data points to map orthogonally to the sides
BLdel = 5;      % The number of data points to delete in the boundary layer (BL) - Can be 0
Nth = 4;        % Use every Nth data point
i0 = 7;        % Index to start movie

%% Read in the uvw, k, and epsilon files to be interpolated

% Ask user to choose velocity file
[f,p] = uigetfile('*uvw.csv','Select the *.csv file for the average velocity');
uvwdata_temp = importdata([p,f]);
uvwdata = uvwdata_temp.data;
y0 = uvwdata(:,2); z_0 = unique(uvwdata(:,3));

[f,~] = uigetfile('*k.csv','Select the *.csv file for k',p);  % Ask user to choose k file
kdata_temp = importdata([p,f]);
kdata = kdata_temp.data;
if size(kdata,2) > 6    % Automatically determine if the files have Transient data
    Trans = 1;
else
    Trans = 0;
end;

[f,~] = uigetfile('*epsilon.csv','Select the *.csv file for epsilon',p);  % Ask user to choose epsilon file
epsdata_temp = importdata([p,f]);
epsdata = epsdata_temp.data;

tic;

% Calculate the bulk velocity from u
u_bulk = mean(uvwdata(:,4:3:end));

%% Count how many vectors are in a single profile
N = 1; i = 1;
if Opposed == 1
    while y0(i) > y0(i+1)
        N = N + 1;
        i = i + 1;
    end
else
    while y0(i) < y0(i+1)
        N = N + 1;
        i = i + 1;
    end
end

% Count how many columns in the data that are not X,Y,Z
Cols = length(kdata_temp.colheaders) - 3;

% Get the time-stamp strings
Left = cell2mat(strfind(kdata_temp.colheaders,'=')); % Find left[ brackets
Right = cell2mat(strfind(kdata_temp.colheaders,'s')); % Find right ] brackets
t_s = cell(1,Cols);
t_s_d = zeros(1,Cols);
for i=1:Cols
    temp = kdata_temp.colheaders{i+3};
    t_s{i} = temp(Left(i)+1:Right(i)-1);
    t_s_d(i) = str2double(temp(Left(i)+1:Right(i)-1));
end

%% Define y, z, and Fine Grid

% Define the y coordinates
% N0 = size(y_o,1);
N0 = size(y0,1) - 3; % Manually remove three extra points from 8_5
y1 = y0(1:N);

% Define the z coordinates
z_1 = y1 - max(y1)/2;

% Use the vector spacing of the PIV data, no fine interpolation
y = y1;
z = z_1;

%% Perform Interpolation for 8_5 data

Nplanes = round(N0/N); % The number of measured planes in the incoming data
uvwdata(Nplanes*N-N+1:Nplanes*N,:) = interp1(y0(Nplanes*N-N+1:end),uvwdata(Nplanes*N-N+1:end,:),y);
kdata(Nplanes*N-N+1:Nplanes*N,4:end) = interp1(y0(Nplanes*N-N+1:end),kdata(Nplanes*N-N+1:end,4:end),y);
epsdata(Nplanes*N-N+1:Nplanes*N,4:end) = interp1(y0(Nplanes*N-N+1:end),epsdata(Nplanes*N-N+1:end,4:end),y);
uvwdata = uvwdata(1:N0,:); % Crop last 3 rows
kdata = kdata(1:N0,:); % Crop last 3 rows
epsdata = epsdata(1:N0,:); % Crop last 3 rows

%% Define omega

omega(1:N0,:) = epsdata;   % Assign the coordinates to omega
omega(1:N0,4:end) = epsdata(1:N0,4:end)./kdata(1:N0,4:end);    % Then assign omega
omega(isnan(omega)) = 0;  % Replace NaNs with zeros

%% Remove PIV data near walls that is biased

% Specify how much of the measured data to write (must be more than the relative BL thickness)
Npart = floor(Orthog*N);

BLi = ones(length(uvwdata),1);  % Boolean array for data to be interpolated
if BLdel > 0
    for i=1:Nplanes
        BLi(2+N*(i-1):BLdel+N*(i-1)+1) = 0;
        BLi((2+N*(i-1):BLdel+N*(i-1)+1)+N-BLdel-2) = 0;
    end
end

%% Map the data near the top wall orthogonally to the two side walls

if Npart == 0   % Manually add zeros to side walls
    uvwdata(N0+1:N0+2*N,:) = 0; % Expand the data a little
    uvwdata(N0+1:N0+N,2) = y; % Write y for the right wall
    uvwdata(N0+1:N0+N,3) = z(1); % Write z for the right wall
    uvwdata(N0+N+1:N0+2*N,2) = y; % Write y for the left wall
    uvwdata(N0+N+1:N0+2*N,3) = z(end); % Write z for the left wall
    
    kdata(N0+1:N0+2*N,:) = 0; % Expand the data a little
    kdata(N0+1:N0+N,2) = y; % Write y for the right wall
    kdata(N0+1:N0+N,3) = z(1); % Write z for the right wall
    kdata(N0+N+1:N0+2*N,2) = y; % Write y for the left wall
    kdata(N0+N+1:N0+2*N,3) = z(end); % Write z for the left wall
    
    epsdata(N0+1:N0+2*N,:) = 0; % Expand the data a little
    epsdata(N0+1:N0+N,2) = y; % Write y for the right wall
    epsdata(N0+1:N0+N,3) = z(1); % Write z for the right wall
    epsdata(N0+N+1:N0+2*N,2) = y; % Write y for the left wall
    epsdata(N0+N+1:N0+2*N,3) = z(end); % Write z for the left wall
    
    omega(N0+1:N0+2*N,:) = 0; % Expand the data a little
    omega(N0+1:N0+N,2) = y; % Write y for the right wall
    omega(N0+1:N0+N,3) = z(1); % Write z for the right wall
    omega(N0+N+1:N0+2*N,2) = y; % Write y for the left wall
    omega(N0+N+1:N0+2*N,3) = z(end); % Write z for the left wall
    
    BLi(N0+1:N0+2*N) = 1; % Expand used indeces too
else % Run through the data files and map the top wall data to the left and right walls
    uvwdata(N0+1:N0+2*Npart*Nplanes,1) = 0;   % The x-values are all 0
    kdata(N0+1:N0+2*Npart*Nplanes,1) = 0;   % The x-values are all 0
    epsdata(N0+1:N0+2*Npart*Nplanes,1) = 0;   % The x-values are all 0
    omega(N0+1:N0+2*Npart*Nplanes,1) = 0;   % The x-values are all 0
    for i=1:Nplanes
        TWi = 1+N*(i-1):Npart+N*(i-1);  % Top wall (unheated) indeces for input
        LWo = N0+1+2*Npart*(i-1):N0+Npart+2*Npart*(i-1);    % Left wall (unheated) indeces for output
        RWo = N0+1+Npart+2*Npart*(i-1):N0+2*Npart+2*Npart*(i-1);    % Right wall (unheated) indeces for output
        
        BLi(N0+2+2*Npart*(i-1):N0+BLdel+2*BLdel*(i-1)+4*(i-1)+1) = 0;
        BLi((N0+2+2*Npart*(i-1):N0+BLdel+2*BLdel*(i-1)+4*(i-1)+1)+Npart-BLdel-2) = 0;
        BLi(N0+1+Npart+2*Npart*(i-1)+1:N0+2*BLdel+2*BLdel*(i-1)+4*(i-1)+3) = 0;
        BLi((N0+1+Npart+2*Npart*(i-1)+1:N0+2*BLdel+2*BLdel*(i-1)+4*(i-1)+3)+Npart-BLdel-2) = 0;
        
        uvwdata(LWo,2) = -uvwdata(TWi,3) + max(y1)/2;  % Map z onto y on left
        uvwdata(LWo,3) = -uvwdata(TWi,2) + max(y1)/2;  % Map y onto z on left
        uvwdata(LWo,4:end) = uvwdata(TWi,4:end);    % Copy the velocity data over to the left
        uvwdata(RWo,2) = uvwdata(TWi,3) + max(y1)/2;   % Map z onto y on right
        uvwdata(RWo,3) = uvwdata(TWi,2) - max(y1)/2;   % Map y onto z on right
        uvwdata(RWo,4:end) = uvwdata(TWi,4:end);    % Copy the velocity data over to the right
        
        kdata(LWo,2) = -kdata(TWi,3) + max(y1)/2;  % Map z onto y on left
        kdata(LWo,3) = -kdata(TWi,2) + max(y1)/2;  % Map y onto z on left
        kdata(LWo,4:end) = kdata(TWi,4:end);    % Copy the k data over to the left
        kdata(RWo,2) = kdata(TWi,3) + max(y1)/2;   % Map z onto y on right
        kdata(RWo,3) = kdata(TWi,2) - max(y1)/2;   % Map y onto z on right
        kdata(RWo,4:end) = kdata(TWi,4:end);    % Copy the k data over to the right
        
        epsdata(LWo,2) = -epsdata(TWi,3) + max(y1)/2;  % Map z onto y on left
        epsdata(LWo,3) = -epsdata(TWi,2) + max(y1)/2;  % Map y onto z on left
        epsdata(LWo,4:end) = epsdata(TWi,4:end);    % Copy the epsilon data over to the left
        epsdata(RWo,2) = epsdata(TWi,3) + max(y1)/2;   % Map z onto y on right
        epsdata(RWo,3) = epsdata(TWi,2) - max(y1)/2;   % Map y onto z on right
        epsdata(RWo,4:end) = epsdata(TWi,4:end);    % Copy the epsilon data over to the right
        
        omega(LWo,2) = -omega(TWi,3) + max(y1)/2;  % Map z onto y on left
        omega(LWo,3) = -omega(TWi,2) + max(y1)/2;  % Map y onto z on left
        omega(LWo,4:end) = omega(TWi,4:end);    % Copy the omega data over to the left
        omega(RWo,2) = omega(TWi,3) + max(y1)/2;   % Map z onto y on right
        omega(RWo,3) = omega(TWi,2) - max(y1)/2;   % Map y onto z on right
        omega(RWo,4:end) = omega(TWi,4:end);    % Copy the omega data over to the right
    end
end

%% Interpolate the data to a fine grid

N1 = length(1:Nth:N);   % Specify the new length
u = zeros(N1,N1,Cols);  v = zeros(N1,N1,Cols);      w = zeros(N1,N1,Cols);
k1 = zeros(N1,N1,Cols); eps1 = zeros(N1,N1,Cols);   omega1 = zeros(N1,N1,Cols);

BLi2 = find(BLi==1);    % Find indeces where good data are located
[qx,qy] = meshgrid(z,y);
y2 = linspace(y(1),y(N),N1); % Determine new subsampled values
z2 = linspace(z(1),z(N),N1); % Determine new subsampled values
[qx1,qy1] = meshgrid(z2,y2,N1);

% Prepare video
writerObj = VideoWriter('Inlet-Transient.mp4','MPEG-4');
writerObj.FrameRate = 5;
open(writerObj);

% Prepare Figure
scrsize = get(0,'ScreenSize');
figure(1); set(gcf,'Position',scrsize,'ToolBar','none','MenuBar','none');
set(gca,'nextplot','replacechildren');
set(gcf,'Renderer','zbuffer');

for i=i0:Cols
    Fu = scatteredInterpolant(uvwdata(BLi2,3),uvwdata(BLi2,2),uvwdata(BLi2,1+3*i),'linear','nearest');
    u(:,:,i) = interp2(qx,qy,Fu(qx,qy),qx1,qy1); % Interpolate onto subsampled grid
    u(1,:,i) = 0; u(N1,:,i) = 0; u(:,1,i) = 0; u(:,N1,i) = 0; % Enforce data = 0 on edges
    subplot(2,2,1); surf(1000*z2,1000*y2,-u(:,:,i),'edgecolor','none'); shading interp;
    if i == i0
        ind_z_PIV = zeros(length(z_0),1);
        for j=1:length(z_0)
            ind_z_PIV(j) = find(qx1(1,:) > z_0(j), 1, 'last'); % Find index of PIV planes in z
        end
        hold on;
        for j=1:length(z_0)
            plot3(1000*z_0(j)*ones(N1,1),1000*y2,-u(:,ind_z_PIV(j),i),'k','linewidth',2);
        end
        hold off;
    end
    h1 = colorbar('EastOutside'); set(get(h1,'ylabel'),'String','$-u$ [m/s]','Interpreter','latex');
    axis([-inf inf -inf inf -5 1]);
    xlabel('$z$ [mm]','Interpreter','latex');
    ylabel('$y$ [mm]','Interpreter','latex'); 
    zlabel('$-u$ [m/s]','Interpreter','latex');
    title('Streamwise Velocity','Interpreter','latex');
    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on','ZMinorTick','on');
    set(h1,'TickLabelInterpreter','latex');
    
    % Plot the bulk velocity through the Transient and the current value
    subplot(2,2,2); plot(t_s_d,u_bulk,'Linewidth',1.5); hold on;
    plot(t_s_d(i),u_bulk(i),'r+','MarkerSize',15,'Linewidth',1.5);
    plot(t_s_d(i),u_bulk(i),'r.','MarkerSize',20,'Linewidth',1.5);
    plot([-2 19],[0 0],'--','Color',[0.5 0.5 0.5]);
    axis([-2 19 -0.5 4.75]);
    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on');
    xlabel('$t$ [s]','Interpreter','latex'); ylabel('$u_{bulk}$ [m/s]','Interpreter','latex');
    title('Transient Bulk Velocity','Interpreter','latex');
    
    text(-6,-0.85,'$g$','Interpreter','latex');
    
    Fv = scatteredInterpolant(uvwdata(BLi2,3),uvwdata(BLi2,2),uvwdata(BLi2,2+3*i),'linear','nearest');
    v(:,:,i) = interp2(qx,qy,Fv(qx,qy),qx1,qy1); % Interpolate onto subsampled grid
    v(1,:,i) = 0; v(N1,:,i) = 0; v(:,1,i) = 0; v(:,N1,i) = 0; % Enforce data = 0 on edges

    Fw = scatteredInterpolant(uvwdata(BLi2,3),uvwdata(BLi2,2),uvwdata(BLi2,3+3*i),'linear','nearest');
    w(:,:,i) = interp2(qx,qy,Fw(qx,qy),qx1,qy1); % Interpolate onto subsampled grid
    w(1,:,i) = 0; w(N1,:,i) = 0; w(:,1,i) = 0; w(:,N1,i) = 0; % Enforce data = 0 on edges

    Fk = scatteredInterpolant(kdata(BLi2,3),kdata(BLi2,2),kdata(BLi2,3+i),'linear','nearest');
    k1(:,:,i) = interp2(qx,qy,Fk(qx,qy),qx1,qy1); % Interpolate onto subsampled grid
    k1(1,:,i) = 0; k1(N1,:,i) = 0; k1(:,1,i) = 0; k1(:,N1,i) = 0; % Enforce data = 0 on edges
    subplot(2,2,3); surf(1000*z2,1000*y2,k1(:,:,i),'edgecolor','none'); shading interp;
    if i == 1
        hold on;
        for j=1:length(z_0)
            plot3(1000*z_0(j)*ones(N1,1),1000*y2,k1(:,ind_z_PIV(j),i),'k','linewidth',2);
        end
        hold off;
    end
    caxis([0 0.75*max(max(k1(:,:,i)))]); % Use a portion of the max
    h1 = colorbar('EastOutside'); set(get(h1,'ylabel'),'String','$k$ [m$^2$/s$^2$]','Interpreter','latex')
    set(h1,'TickLabelInterpreter','latex');
    axis([-inf inf -inf inf 0 0.5]);
    xlabel('$z$ [mm]','Interpreter','latex'); 
    ylabel('$y$ [mm]','Interpreter','latex');
    zlabel('$k$ [m$^2$/s$^2$]','Interpreter','latex');
    title('Turbulence Kinetic Energy','Interpreter','latex');
    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on','ZMinorTick','on');

    Feps = scatteredInterpolant(epsdata(BLi2,3),epsdata(BLi2,2),epsdata(BLi2,3+i),'linear','nearest');
    eps1(:,:,i) = interp2(qx,qy,Feps(qx,qy),qx1,qy1); % Interpolate onto subsampled grid
    eps1(1,:,i) = 0; eps1(N1,:,i) = 0; eps1(:,1,i) = 0; eps1(:,N1,i) = 0; % Enforce data = 0 on edges
    subplot(2,2,4); surf(1000*z2,1000*y2,eps1(:,:,i),'edgecolor','none'); shading interp;
    if i == 1
        hold on;
        for j=1:length(z_0)
            plot3(1000*z_0(j)*ones(N1,1),1000*y2,eps1(:,ind_z_PIV(j),i),'k','linewidth',2);
        end
        hold off;
    end
    caxis([0 0.75*max(max(eps1(:,:,i)))]); % Use a portion of the max
    h1 = colorbar('EastOutside'); 
    set(get(h1,'ylabel'),'String','$\epsilon$ [m$^2$/s$^3$]','Interpreter','latex')
    set(h1,'TickLabelInterpreter','latex');
    axis([-inf inf -inf inf 0 5]);
    xlabel('$z$ [mm]','Interpreter','latex'); 
    ylabel('$y$ [mm]','Interpreter','latex');
    zlabel('$\epsilon$ [m$^2$/s$^3$]','Interpreter','latex');
    title('Turbulence Dissipation Rate','Interpreter','latex');
    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on','ZMinorTick','on');

    Fomega = scatteredInterpolant(omega(BLi2,3),omega(BLi2,2),omega(BLi2,3+i),'linear','nearest');
    omega1(:,:,i) = interp2(qx,qy,Fomega(qx,qy),qx1,qy1); % Interpolate onto subsampled grid
    omega1(1,:,i) = 0; omega1(N1,:,i) = 0; omega1(:,1,i) = 0; omega1(:,N1,i) = 0; % Enforce data = 0 on edges

%     h = annotation('textarrow',[0.5,0.5],[0.6,0.5],'String','g');
    h = annotation('arrow',[0.5,0.5],[0.6,0.5]);
    set(h,'LineWidth',2); %,'background','w','edgecolor','k');
    
%     h = suptitle(['{\itt} = ',t_s{i},' s']); set(h,'background','w','edgecolor','k');
    h = suptitle(['$t$ = ',num2str(t_s_d(i),'%5.1f'),' s']); 
    set(h,'background','w','edgecolor','k','Interpreter','latex');
%     set(findall(gcf,'type','axes'),'FontWeight','normal','FontSize',16,'FontName','Times New Roman');
%     set(findall(gcf,'type','text'),'FontWeight','normal','FontSize',16,'FontName','Times New Roman');
    
    set(findall(gcf,'type','axes'),'FontSize',16);
    set(findall(gcf,'type','text'),'FontSize',16);
    
    writeVideo(writerObj,getframe(gcf));
end
close(writerObj);

u(isnan(u))=0;      v(isnan(v))=0;          w(isnan(w))=0;
k1(isnan(k1))=0;    eps1(isnan(eps1))=0;    omega1(isnan(omega1))=0;

%% Manually set the velocity profile (if Manual == 1)
if Manual == 1
    u_max = 1;  % Maximum u velocity [m/s]
    u_min = -1; % Minimum u velocity [m/s]
    u_mean = 0.5;   % Mean u velocity [m/s]
    u_vector = (u_min:(u_max-u_min)/(N-1):u_max)' + u_mean;  % Linear profile from u_max to u_min
    u = repmat(u_vector,1,N);   % Copy the column vector to a 2D matrix
    
    if Opposed == 1 % Change the sign so the wall-side has a negative sign
        u = -u;
    end
    
    v = zeros(N,N);
    w = zeros(N,N);
    
    figure; contourf(1000*z,1000*y,u,'edgecolor','none');
    h1 = colorbar('EastOutside');
    set(get(h1,'ylabel'),'String','u [m/s]')
    axis equal; axis tight; xlabel('z [mm]');   ylabel('y [mm]');   title('Manual u Assignment');
end

%% Export Files

if Export == 1
    Vel =       zeros(N1*N1,length(uvwdata_temp.colheaders));
    turb =      zeros(N1*N1,length(kdata_temp.colheaders));
    epsilon =   zeros(N1*N1,length(epsdata_temp.colheaders));
    omega =     zeros(N1*N1,length(kdata_temp.colheaders));
    
    % Arrange 3D data into output form of X, Y, Z, data,...
    count=0;
    for i=1:N1
        for j=1:N1
            count = count + 1;
            for k=1:Cols
                if k == 1   % Compile the X,Y,Z coordinates
                    Vel(count,1:6) =        [0,y2(i),z2(j),u(i,j,k),v(i,j,k),w(i,j,k)];
                else % Just append the data to the right
                    Vel(count,1+3*k:3+3*k) =[u(i,j,k),v(i,j,k),w(i,j,k)];
                end
            end
            turb(count,:) =     [0,y2(i),z2(j),reshape(k1(i,j,:),1,Cols)];
            epsilon(count,:) =  [0,y2(i),z2(j),reshape(eps1(i,j,:),1,Cols)];
            omega(count,:) =    [0,y2(i),z2(j),reshape(omega1(i,j,:),1,Cols)];
        end
        fprintf('Completed copying plane %3i of %3i \n', i, N);
    end
    
    % Write the velocity inlet file
    if Trans == 1
        csvwrite_with_headers('Inlet-uvw-interp.csv',Vel,uvwdata_temp.colheaders);
    else
        fid = fopen('Inlet-uvw-interp.csv', 'w+');
        fprintf(fid, '%s \n', 'X,Y,Z,u,v,w'); fclose(fid);
        dlmwrite('Inlet-uvw-interp.csv',Vel,'delimiter',',','precision',4,'-append')
    end
    fprintf('Completed writing Inlet-uvw-interp.csv \n');
    
    % Write the k file for the inlet
    if Trans == 1
        csvwrite_with_headers('Inlet-k-interp.csv',turb,kdata_temp.colheaders);
    else
        fid = fopen('Inlet-k-interp.csv', 'w+');
        fprintf(fid, '%s \n', 'X,Y,Z,k'); fclose(fid);
        dlmwrite('Inlet-k-interp.csv',turb,'delimiter',',','precision',4,'-append')
    end
    fprintf('Completed writing Inlet-k-interp.csv \n');
    
    % Write the epsilon file for the inlet
    if Trans == 1
        csvwrite_with_headers('Inlet-epsilon-interp.csv',epsilon,epsdata_temp.colheaders);
    else
        fid = fopen('Inlet-epsilon-interp.csv', 'w+');
        fprintf(fid, '%s \n', 'X,Y,Z,epsilon'); fclose(fid);
        dlmwrite('Inlet-epsilon-interp.csv',epsilon,'delimiter',',','precision',4,'-append')
    end
    fprintf('Completed writing Inlet-epsilon-interp.csv \n');

    % Write the omega files for the inlet
    if Trans == 1
        csvwrite_with_headers('Inlet-w-interp.csv',omega,strrep(kdata_temp.colheaders,'k(J/kg)','w(1/s)'));
    else
        fid = fopen('Inlet-w-interp.csv', 'w+');     % Omega 1 formulation (w = e/k)
        fprintf(fid, '%s \n', 'X,Y,Z,w'); fclose(fid);
        dlmwrite('Inlet-w-interp.csv',omega,'delimiter',',','precision',4,'-append')
    end
    fprintf('Completed writing Inlet-w-interp.csv \n');
end

toc;