clear all; close all; clc;

% This code will take the transient PIV images from each run and organize them into folders for each phase.
% The folder must only contain the subfolders for the data sets and the *.set files.  The output will be ready
% to open in DaVis without the need to import.
%
% Blake Lance
% 21 October 2014
%
% Revision 2:  Since data will be acquired with 2 dts and appended together, the copied data range can be
% specified to reduce disk usage and processing time.
% Revision 2_1:  Creates a *.set file for the new folder containing all the data sets
% Revision 2_2:  Sorts the subfolders and subsequent data by numerical value (requires sort_nat.m function)
% Revision 2_3:  Compensates for PIV Delay from a recorded file
    % The data from the two sets with different dt should be copied into the same folder(1:3).  This requires
    % that the serial number in the form 8_x_x be used as the first 5 characters in the folder name.
    % Check for the count vector saved in a file and starts naming the im7s using the next numbers available.
    % This method assumes use process the small dt data set first (earlier phases used)

%% Input

% Ask the user to select the folder containing the data sets for each run
folder = uigetdir('H:\','Select folder containing the data sets');
[DelayFile,p] = uigetfile('*.csv','Select the PIV Delay file for this case');

% Check the particle displacement in pixels within DaVis to choose ImageStart and ImageEnd
ImageStart = 29; % The first image to copy (if ~= 1, should have displacements just under 32 pixels)
ImageEnd = 101; % The last image to copy (if ~= 101, should have displacements just over 8 pixels)

tic;

%% Adjust for PIV Delay

% Load the PIV Delay Data
data = importdata([p,DelayFile]);
Delay = data.data(:,2);

% Adjust the ImageStart or ImageEnd with the PIV Delay of the first loop
if ImageStart == 1
    ImageEnd = ImageEnd + Delay(1) - min(Delay);
else
    ImageStart = ImageStart + Delay(1) - min(Delay);
end

%% Read the Folder Contents

folder_struct = dir(folder);
N_runs = 0; % The number of data sets/runs
% Count the number of subfolders/runs
for i=1:length(folder_struct);
    N_runs = N_runs + folder_struct(i).isdir;
end
N_runs = N_runs - 2;    % MATLAB counts the '.' and the '..', so correct for it

% Read the subfolder names
count = 0;
subfolders = cell(N_runs,1);
for i=3:length(folder_struct)
    if folder_struct(i).isdir
        count = count + 1;
        subfolders{count} = folder_struct(i).name;
    end
end

% Order subfolders by numerical value
subfolders = sort_nat(subfolders);

% Read how many *.im7 files are in each subfolder
N_images = zeros(N_runs,1);
for i=1:N_runs
    N_images(i) = length(dir([folder,'\',subfolders{i},'\*.im7']));
end

% Find the first five characters (serial number) of the data set
folder_index = strfind(folder,'\');
folder_out = folder(1:folder_index(end)+3);

%% Create New Folder and Subfolders

if ImageStart == 1  % We are using the starting phases
    LoopStart = ImageStart + min(Delay);
    LoopEnd = ImageEnd + max(Delay);
else    % We are using later phases
    % The first phase has a delay, but we may have good data from earlier phases (less delay)
    LoopStart = ImageStart + min(Delay);
    LoopEnd = ImageEnd; % There is likely data from later phases, but it's not useful
end

% Create subfolders, one for each phase
for i=LoopStart:LoopEnd
    mkdir([folder_out,' - Organized\',num2str(i,'%03d')]);
end

%% Create *.set File for the New Folder

folder2 = strrep(folder,'\','/');   % Replace '\' with '/' for output

fid = fopen([folder_out,' - Organized.set'],'wt');

fprintf(fid,['// set file <',folder2,' - Organized.set> created by DaVis at ',...
    datestr(clock,'dd.mm.yy HH:MM:SS')]);
fprintf(fid,'\n'); fprintf(fid,'\n');
fprintf(fid,'#GROUP Sets \n');
fprintf(fid,'SetType = 16384; \n');
fprintf(fid,'SetGroups = ""; \n');
fprintf(fid,['SetTime = "',datestr(clock,'ddd mmm dd HH:MM:SS yy'),'"; \n']);
fprintf(fid,'SetComments = ""; \n');
fprintf(fid,'SetStart = 1; \n');
fprintf(fid,'SetInc = 1; \n');
fprintf(fid,'SetSourceSet = ""; \n');
fprintf(fid,'SetViewCallback = ""; \n');
fprintf(fid,'SetLoadCallback = ""; \n');
fprintf(fid,'VeryLargeSet_Size = 10000; \n');

fclose(fid);

%% Create *.set File for Each New Subfolder

for i=LoopStart:LoopEnd
    fid = fopen([folder_out,' - Organized\',num2str(i,'%03d'),'.set'],'wt');
    
    fprintf(fid,['// set file <',folder2,' - Organized/',num2str(i,'%03d'),'.set> created by DaVis']);
    fprintf(fid,'\n'); fprintf(fid,'\n');
    fprintf(fid,'#GROUP Sets \n');
    fprintf(fid,'SetType = 256; \n');
    fprintf(fid,'SetGroups = ""; \n');
    fprintf(fid,['SetTime = "',datestr(clock,'ddd mmm dd HH:MM:SS yy'),'"; \n']);
    fprintf(fid,'SetComments = ""; \n');
    fprintf(fid,'SetStart = 1; \n');
    fprintf(fid,'SetInc = 1; \n');
    fprintf(fid,'SetSourceSet = ""; \n');
    fprintf(fid,'SetViewCallback = ""; \n');
    fprintf(fid,'SetLoadCallback = ""; \n');
    
    fclose(fid);
end

%% Loop Through the Data and Copy to New Subfolders

if ImageStart == 1; % Make a count array to assign names to copied data
    count = zeros(max(N_images),1); 
else % Use the count vector in a file if not using the first phases
    count = importdata(['Transient - Phase Count',folder_out(end-3:end),'.mat']);
end;

for i=1:N_runs  % Loop through runs
    if ImageStart == 1  % We are using the starting phases
        CopyEnd = ImageEnd;
    else    % We are using later phases
        CopyEnd = ImageEnd - Delay(i);
    end

    for j=ImageStart:CopyEnd    % Loop through phases
        count(j+Delay(i)) = count(j+Delay(i)) + 1;  % Count the images where they are being written
        copyfile([folder,'\',subfolders{i},'\B',num2str(j,'%05d'),'.im7'],...
            [folder_out,' - Organized\',num2str(j+Delay(i),'%03d'),'\B',num2str(count(j+Delay(i)),'%05d'),'.im7']);
    end
    fprintf('Finished copying run %3i of %3i at %6.1f seconds \n', i, N_runs, toc);
end

%% Write the Count Vector so the Second Run Doesn't Overwrite the Im7s

if ImageStart == 1
    save(['Transient - Phase Count',folder_out(end-3:end),'.mat'],'count');
else % Plot the total image count for the current case
    figure; bar(count);
    xlim([0 max(N_images)]); xlabel('Phase'); ylabel('Image Count');
    save(['Transient - Phase Count',folder_out(end-3:end),'_Total.mat'],'count');
end

toc;