clear all;  close all;  clc;

% This program is designed to solve for the conduction losses of the
% fine-wire thermocouple probe used in the Rotatable Buouyancy Tunnel for
% heat flux measurements.  It will use an estimated temperature profile
% from earlier experiments.  The geometry of the probe is a wishbone shape
% similar to that of a hot wire where the sensing section is in a constant
% temperature region and the leads span a section with a large air temperature
% gradient that is the source of the heat loss.  The temperature at the
% corner is initially guessed to numerically solve for the temperature
% profile within the leads, then the heat loss in the sensing section will
% be calculated by a fin equation and compared to the numerical result.
% Then the corner temperature can be corrected and the process repeated.
% The final result is the error of the sensing portion.  This result can be
% used to correct for experimental readings.
%
%  Sensor section
% O--------------  <--Corner
%              //
%             //
%            //
%           //
%          //
%         //
%        // Lead section
%       //
%      //
%     //
%    //
%   //
%  //
% //
%||
%
% Blake Lance
% 21 January 2014
% Version 2, Blake Lance, 20 July 2015
%   Updated with flosser TC Probe geometry and data

%% Initialize
tic;

HFS = 1;    % Which HFS location in x to use
Ny = 201;   % Number of discretized nodes in the lead section
Nz = 100;   % Number of points in the z-direction to plot
% omega = 1;    % Relaxation factor
y_TC = 0.0; % The y location of the thermocouple sensor [mm] (doesn't have to be at wall)
T_b = 100;  % Guess the temperature of the corner [deg C] (temperature of the base)
Case = 'Mixed';    % The case to test with experimental velocity and temperature profiles
% Case = 'Forced';    % The case to test with experimental velocity and temperature profiles

k_chromel = 18.6;   % Thermal conductivity of chromel [W/mK]
k_alumel = 31;      % Thermal conductivity of alumel [W/mK]
k_TC = k_alumel;    % Choose the thermal conductivity to use
D = 5.08E-2;        % Diameter of TC wire [mm] (0.002 in)
L_lead = 22.86;     % Length of lead wire [mm] (0.9 in)
% L_lead = 40;      % Length of lead wire [mm] (1 in)
% L_sensor = 6.35;    % Length from TC junction to corner [mm] (1/4 in)
% L_sensor = 3.58;    % Length from TC junction to corner [mm] (As-built length of first sensor)
% L_sensor = 9.53;    % Length from TC junction to corner [mm] (As-built length of first sensor)
L_sensor = 7.65;    % Length from TC junction to corner [mm] (As-built length flosser head sensor)

dy = L_lead/(Ny-1);  % Spacing between nodes [mm]
y = (y_TC:dy:L_lead+y_TC)';
dz = L_sensor/(Nz-1);
z = (0:dz:L_sensor)';

%% Read in all types of data

% Read in temperature data
% data = importdata([Case,'-125C-ApproxTemp.txt']);
data = importdata(['T_',Case,'_x',int2str(HFS),'.csv']);
y_exp = 1000*data.data(:,2);
T_exp = data.data(:,4);
T_inf_b = interp1(y_exp,T_exp,y_TC,'linear','extrap');  % Base/wall temperature
T_inf_fs = interp1(y_exp,T_exp,y_TC+L_lead,'linear');  % Free-stream air temperature

% Read in velocity profile
data = importdata(['6-',Case,'-HFS3-u.txt']);
y_PIV = data(:,1);
u_PIV = data(:,2);
u_b = interp1(y_PIV,u_PIV,y_TC,'linear');   % Velocity at TC

% Read in kinematic viscosity of air
data = importdata('Air Kinematic Viscosity.txt');
T_nu = data(:,1);   % Temperature [C]
nu = data(:,2);     % Air kinematic viscosity [m^2/s]
nu_b = interp1(T_nu,nu,T_inf_b,'linear');    % nu at the base

% Read in Prandtl number of air
data = importdata('Air Prandtl Number.txt');
T_Pr = data(:,1);
Pr = data(:,2);
Pr_b = interp1(T_Pr,Pr,T_inf_b,'linear');    % Pr at the base

% Read in thermal conductivity of air
data = importdata('Air Thermal Conductivity.txt');
T_k = data(:,1);
k_air = data(:,2);
k_b = interp1(T_k,k_air,T_inf_b,'linear');    % k at the base

%% Perform calculations in a loop until the answer stops changing

% Do fin analysis on sensor section for base heat transfer rate
Re_b = u_b*(D/1000)/nu_b;   % Reynolds number at the base
h_b = k_b/(D/1000)*( 0.3 + 0.62*Re_b^(1/2)*Pr_b^(1/3)/(1 + (0.4/Pr_b)^(2/3) )^(1/4)*...
    ( 1 + (Re_b/282000)^(5/8) )^(4/5));
% h_b = 0.5*h_b;  % Conservatively reduce the heat transfer coefficient
% h_b = 10;    % Guess a really low heat transfer coefficient

% Populate properties on the discretized grid
T_inf_y = interp1(y_exp,T_exp,y,'spline');
k_inf_y = interp1(T_k,k_air,T_inf_y,'spline');
u_y = interp1(y_PIV,u_PIV,y,'spline');
nu_y = interp1(T_nu,nu,T_inf_y,'spline');
Re_y = u_y*(D/1000)./nu_y;
Pr_y = interp1(T_Pr,Pr,T_inf_y,'spline');
h_y = k_inf_y./(D/1000).*( 0.3 + 0.62.*Re_y.^(1/2).*Pr_y.^(1/3)./(1 + (0.4./Pr_y).^(2/3) ).^(1/4).*...
    ( 1 + (Re_y./282000).^(5/8) ).^(4/5));

% h_y = 10*ones(Ny,1);  % Predict lower h with heat shrink for insulation

% Solve the temperature profile in the lead section until the answer stops changing
T_y = T_b*ones(Ny,1);
T_y_old = T_b*ones(Ny,1);
T_b_old = T_b;

for k=1:10000
    M = sqrt(h_b*k_TC*pi^2*(D/1000)^3/4)*( T_b - T_inf_b);   % M = sqrt(h_b*P*k*A_c*(T-T_inf))
    m = sqrt(4*h_b/((D/1000)*k_TC)); % m = sqrt(h*P/(k*A_c))
    q_b_sensor = -M*tanh(m*(L_sensor/1000)); % heat transfer at the corner [W]
    % Calculate resulting temperature profile in sensor region
    T_sensor = T_inf_b + ( T_b - T_inf_b )*cosh(m.*((L_sensor-z)/1000))/cosh(m*L_sensor/1000);
    
    T_y(1) = T_b;   % The base (corner temperature)
    T_y(Ny) = T_inf_fs;
    for i=1:10000
        for j=2:Ny-1
            T_y(j) = ( ( T_y(j+1) + T_y(j-1) )/(dy/1000)^2 + 4*h_y(j)*T_inf_y(j)/(k_TC*(D/1000)) )...
                /( 2/(dy/1000)^2 + 4*h_y(j)/(k_TC*(D/1000)) );
        end
%         T_y(2:Ny-1) = ( ( T_y(3:Ny) + T_y(1:Ny-2) )/(dy/1000)^2 + 4*h_y(2:Ny-1).*T_inf_y(2:Ny-1)/(k_TC*(D/1000)) )...
%                 ./( 2/(dy/1000)^2 + 4*h_y(2:Ny-1)/(k_TC*(D/1000)) );

%         T_y(2:Ny-1) = T_y_old(2:Ny-1) + omega*( T_y(2:Ny-1) - T_y_old(2:Ny-1) );
        
        % Calculate RMS error, exit if small
        RMS_T_y = sqrt(sum(( T_y - T_y_old ).^2)/Ny);
        if RMS_T_y < 1.0E-16;  break;  end;
        T_y_old = T_y;
    end
    
    % Estimate the heat loss to the corner from the lead side
    dT_dy = ( -T_y(3) + 4*T_y(2) - 3*T_y(1) )/(2*(dy/1000));
    q_b_lead = -pi*(D/1000)^2/4*k_TC*dT_dy; % [W]
    
    % Correct T_b based on the difference of the heat losses
    T_b = T_b + 1000*( q_b_sensor - q_b_lead );
    
    % Exit loop when T_b stops changing
    RMS_T_b = ( T_b - T_b_old ).^2;
    RMS_T_b = sqrt(RMS_T_b);
    if RMS_T_b < 1.0E-16;    break;  end;
    T_b_old = T_b;
    fprintf('Outer iteration = %6i \t Inner iterations = %6i \t T_b = %18.15f \n',k, i, T_b);
end

% Append temperature profiles together
T_append = [flipud(T_sensor);T_y(2:Ny)];
x_append = [z;z(Nz)+y(2:Ny)-y_TC];

%% Assess temperature error in the thermocouple

T_e = T_inf_b - T_sensor(end);
fprintf('Temperature error = %6.4f \n',T_e);

%% Plot results

% Plot experimental temperature profile
% figure; plot(y_exp,T_exp,'*');
% xlabel('y [mm]');   ylabel('T [\circC]');
% title('Experimental Temperature Profile');
% 
% % Plot experimental velocity profile
% figure; plot(y_PIV,u_PIV,'*');
% xlabel('y [mm]');   ylabel('u [m/s]');
% title('Experimental Velocity Profile');
% 
% % figure; plotyy(y_exp,T_exp,y_PIV,u_PIV);
% 
% Plot air kinematic viscosity
% figure; plot(T_nu,nu,'*');
% xlabel('T [\circC]');   ylabel('\nu [m^2/s]');
% title('Air Kinematic Viscosity');
% 
% Plot air kinematic viscosity as a function of space
% figure; plot(y,nu_y)
%
% % Plot air Prandtl number
% figure; plot(T_Pr,Pr,'*');
% xlabel('T [\circC]');   ylabel('Pr');
% title('Air Prandtl Number');
% 
% % Plot air thermal conductivity
% figure; plot(T_k,k,'*');
% xlabel('T [\circC]');   ylabel('k [W/mK]');
% title('Air Thermal Conductivity');
% 
% % Plot the temperature profile result from the fin equation
% figure; plot(z,T_sensor);
% xlabel('z [mm]');   ylabel('T [\circC]');
% title('Temperature Profile Along the Sensor Section');
% 
% % Plot the temperature profile result from the numerical section
% figure; plot(y,T_y);    hold on;
% plot(y,T_inf_y,'r');
% xlabel('y [mm]');   ylabel('T [\circC]');
% legend('T_{TC}','T_{\infty}','location','best');
% title('Temperature Profile Along the Lead Section');
% axis([0 y(end) -inf inf]);

% Plot the appended temperature profile for both regions
figure; plot(x_append,T_append); hold on;
plot(x_append(Nz:Nz+Ny-1),T_inf_y,'r');
plot([0 x_append(Nz)],[T_inf_b T_inf_b],'r');
plot([x_append(Nz) x_append(Nz)],[min(T_append) max(T_append)],'--','Color',[0.5 0.5 0.5]);
% axis([0 x_append(end) -inf inf]);
xlabel('Distance from TC sensor [mm]');   ylabel('T [\circC]');
legend('T_{TC}','T_{\infty}','location','best');

toc;