%LAST UPDATED: 10/29/2014

function PIVuncertaintyCode(Data)
%
% Uncertainty analysis tool based upon input structure "Data"
% generated using the PIVuncertainty.m and PIVuncertainty.fig GUI file.
% 
% Required GUI inputs include: 
%       -Image files (including DaVis *.im7 files)
%       -Position and displacement vectors (*.mat / *.vc7 files)
%       -Pulse separation - dt (µs), Length conversion (µm/pix)
% History:
%   -Code originated as Prana.m (a PIV code)
%   -Uncertainty calculations added by B. Timmins and renamed 'PIVadvanced2Code.m'
%   -Uncertainty code separated 'PIVadvanced2code.m', expanded, and renamed
%    'PIVuncertaintyCode.m'.
%   -As part of the expansion, more accurate methods were implemented for 
%    calculating diameter and density
%
% Update History:
% 2-19-13   U and V uncertainties calculated using DiaX and DiaY,
%           respectively.
% 2-20-13   Added error message if 3C data is read in.
% 2-25-13   Feature to load data from various folders and save output files
%           from each in a single output folder in consecutive number. This
%           enables stats to be calculated on the data from the various
%           folders collectively.
%           Job file autosaves into output directory for future reference. 
% 3-11-13   Fixed bug when trying to do the background subrations on stereo
%           images.
% 6-12-13   Bug fixed in computing particle image diameter.
% 6-13-13   Added comments. Code now takes roughly 1/3 the time to run.
% 9-07-13   Adjusted the multifolder option so it accounts for step size 
%           account (ie step size 2, 1st set 1,3,5. 2nd set starts at 7, not 6)
% 10-27-14  Readimx update. "compute_bias" function is much faster. Replaced
%           matlabpool by parpool (MATLAB R2014a and newer). The newer
%           readimx does not use IType, so there may be issues depending on
%           the IType. Updated Diaden to Diaden2. DiaDen2 is faster and
%           uses the same code used for the paper: "Autocorrelation-based 
%           estimate of particle image density for diffraction-limited 
%           particle images" by Warner and Smith. "Fill" function updated.
% 12-18-14  Changed the background subtraction from 1% to 0.5% for all but 10
%           and 12-bit cameras.
% 12-22-14  Changed the density for uncertainty to peak counting (line 460).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Data.vcNum=str2double(Data.vcNum);
Data.imfstart=str2double(Data.imfstart);
Data.imfstep=str2double(Data.imfstep);
Data.imfend=str2double(Data.imfend);
Data.imNum=str2double(Data.imNum);
Data.par=str2double(Data.par);
Data.parprocessors=str2double(Data.parprocessors);
Data.datafstart=str2double(Data.datafstart);
Data.datafstep=str2double(Data.datafstep);
Data.datafend=str2double(Data.datafend);
Data.tag=0;

fprintf(['Current Job: ' Data.batchname '\n'])
%PC or Mac
if ispc
    pltdirec=[Data.outdirec '\'];
else
    pltdirec=[Data.outdirec '/'];
end
    
% This feature allows for these uncertainty to be calculated from data in
% separate folders and then be saved in a single folder for stats calc.
% (each job file is a folder).
d= dir([pltdirec Data.outbase '0' '*.mat']);    
if Data.multifolders=='1'    
    if isempty(d)
        Data.fileend=0-Data.imfstart+1;
    else
        d2=d(size(d,1),1);
        lastfile=d2.name;
        %Fine values between Data.imbase and .mat
        Data.fileend=str2double(lastfile(strfind(lastfile,Data.outbase)+size(Data.outbase,2):strfind(lastfile,'.mat')-1))-Data.imfstart+1;        
    end
else
    Data.fileend=0;
end

%Grid Size - set based on GUI input
if Data.DiaGS=='1'
    Data.GSdia=32;
elseif Data.DiaGS=='2'
    Data.GSdia=64;
elseif Data.DiaGS=='3'
    Data.GSdia=128;
elseif Data.DiaGS=='4'
    Data.GSdia=256;
end

if Data.bkgd =='1'
%% Calculate BackGround:
%{
    Noise is introduced through stationary objects, reflections, and a 
    non-zero backgrond. The background is calculated by comparing all 
    images in the set and taking the minimum value for each pixel.
%}
fprintf('\n-------------- Calculating Background --------------\n')

%Image file location and base (ie. C:\Folder\File-prefix)
if ispc
    imbase=[Data.imdirec '\' Data.imbase];
else
    imbase=[Data.imdirec '/' Data.imbase];
end

%Image sets
I1 = Data.imfstart:Data.imfstep:Data.imfend;

if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
    %Load single image to determin image size
    Davis=readimx([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(1))]);
    im2=double(Davis.Frames{Data.imNum}.Components{1}.Planes{1}');
else
    %All other image files (Same method as for DaVis *im7 files [above])
    im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(1))]));
end

%Pre-allocate matrices for the background and to calculate the mean/stdv
Data.bkgspc=100;%length(I1);
bkgd = ones([size(im2),2])*(2^16);
bkgrd2=zeros([size(im2), ceil(length(I1)/Data.bkgspc)]);
%Load every image pair 
j=0;
for q=1:length(I1)
    if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
        %DaVis Image File
        Davis=readimx([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(q))]);
        im2=double(Davis.Frames{Data.imNum}.Components{1}.Planes{1}');
    else
        %All other image files (Same method as for DaVis *im7 files [above])
        im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(q))]));
    end
    %Assign current image to 2nd slot in matrix
    bkgd(:,:,2) = im2;
    %Compare the current min the the new image, keep lowest value
    bkgd(:,:,1) = min(bkgd,[],3);

    %Sum the individual values and squared values for mean and stdv
    % imgSum = imgSum + im2;
    % imgSum2 = imgSum2 + im2.^2;
    % Imax=max(Imax,max(im2(:)));
    if (q/Data.bkgspc-floor(q/Data.bkgspc))==0 || q==length(I1)
        j=j+1;
        bkgrd2(:,:,j)= bkgd(:,:,1);
        bkgd = ones([size(im2),2])*(2^16);
    end
end
%Store background in Data structure for future use
Data.bkgrd = bkgrd2;%bkgd(:,:,1);

%Calculate and store mean
% Data.imgMean = (imgSum./q);
%Calculate and store standard deviation using: 
%http://en.wikipedia.org/wiki/Computational_formula_for_the_variance
% Data.imgStd = sqrt(imgSum2./(q-1)-(q/(q-1))*((imgSum./q).^2));
%{
%Uncomment to make the following plots:
map = gray(2^8);

%Shows Original Image
figure(1)
imshow(im2,map)   

%Shows time minimum background
figure(2)
imshow(Data.bkgrd,map) 

%Shows Figure (1) without background
figure(3)
Temp2=im2-Data.bkgrd;
Temp2(Temp2<0)=0;
imshow(Temp2,map) 
%}
%End Calculage Backgroud
end

%% Run Jobs in Parallel
%{
    If the "Run in Parallel" box is checked in the PIVuncertainty GUI, 
    this section opens matlabpool which enables parallel processing.
    Images are devided between the total number of processors.
%}
if Data.par
    fprintf('\n--- Initializing Processor Cores for Parallel Job ----\n')
    poolopen=1;
    
    %Don't open more processors than there are image pairs
    if length(Data.imfstart:Data.imfstep:Data.imfend) < Data.parprocessors
        Data.parprocessors=length(Data.imfstart:Data.imfstep:Data.imfend);
    end
    
    try
        %Open Matlab Pool
%         matlabpool('open','local',Data.parprocessors);
        parpool('local',Data.parprocessors);
    catch
        try
            %If First attepmt doesn't work, close and retry
%             matlabpool close
            delete(gcp('nocreate'));
%             matlabpool('open','local',Data.parprocessors);
            parpool('local',Data.parprocessors);
        catch
            %If Second attempt doesn't work, continue with single processor
            beep
            disp('Error Running Job in Parallel - Defaulting to Single Processor')
            poolopen=0;
            fprintf('\n-------------- Processing Dataset ------------------\n')
            USMprocessing(Data)
            fprintf('---------------- Job Completed ---------------------\n')
        end
    end
    if poolopen
        %Pool successfully open, divide sets
        I1=Data.imfstart:Data.imfstep:Data.imfend;
        Idata = Data.datafstart:Data.datafstep:Data.datafend;

            fprintf('\n--------------- Processing Dataset -------------------\n')
            spmd
                verstr=version('-release');
                if str2double(verstr(1:4))>=2010
                    I1dist=getLocalPart(codistributed(I1,codistributor('1d',2)));
                    Idatdist=getLocalPart(codistributed(Idata,codistributor('1d',2)));
                else
                    I1dist=localPart(codistributed(I1,codistributor('1d',2),'convert'));
                    Idatdist=localPart(codistributed(Idata,codistributor('1d',2),'convert'));
                end
                USMprocessing(Data,I1dist,Idatdist);
            end
            fprintf('----------------- Job Completed ----------------------\n')
            
        if poolopen
%             matlabpool close
            delete(gcp('nocreate'));
        end
    end
else
    %"Run in Parallel" not selected - Default to single processor
    fprintf('\n-------------- Processing Dataset ------------------\n')
    USMprocessing(Data)
    fprintf('---------------- Job Completed ---------------------\n')
end


function USMprocessing(Data,I1,I1data)
    
tic
%%Image file location/base (ie. C:\Folder\File-prefix) and output directory
if ispc
    imbase=[Data.imdirec '\' Data.imbase];
    database=[Data.datadirec '\' Data.database];
    pltdirec=[Data.outdirec '\'];
    Usurf=[Data.surfdirec '\' Data.surfname];
else
    imbase=[Data.imdirec '/' Data.imbase];
    database=[Data.datadirec '/' Data.database];
    pltdirec=[Data.outdirec '/'];
    Usurf=[Data.surfdirec '/' Data.surfname];
end

if nargin<3 %(Not runing in parallel)
%Image indices
I1 = Data.imfstart:Data.imfstep:Data.imfend;
%Data indices
I1data = Data.datafstart:Data.datafstep:Data.datafend;
end

%Physical parameters
Mag = str2double(Data.wrmag);   %(µm/pix)
dt = str2double(Data.wrsep);    %(µs)

%Preallocate "time remaining" variable
frametime=zeros(length(I1));

%% EVALUATE IMAGE PAIRS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for q=1:length(I1)
    %Record Start time
    tf=cputime;
    %Set being analyzed
    title=['Frame' sprintf(['%0.' Data.imzeros 'i'],I1(q)) ' and Frame' sprintf(['%0.' Data.imzeros 'i'],I1(q))];
    
    %LOAD IMAGE PAIRS
    if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
        %DaVis Image File
        Davis=readimx([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(q))]);
        im2=double(Davis.Frames{Data.imNum}.Components{1}.Planes{1}');
    else
        %All other image files (bmp, gif, jpg, png, tif, etc)
        im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(q))]));
    end
   
% Subtract background and a small % of bit-depth to compensate for additional noise.
% One could also hard code a value to compensate additional noise, or use
% other image correction. Subtracting Data.dkgrd, only helps with
% constant noise (ie from a structure in the image) This does not help
% with intermittent noise. Image correction only effects DiaDen.  
    bitdepth = ceil(log(max(im2(:)))/log(2));
    
    if Data.bkgd =='1'
        kk=ceil(I1(q)/Data.bkgspc);
        if bitdepth ==8
            im2 = im2-Data.bkgrd(:,:,kk)-5; % 2% of bit depth
        elseif bitdepth == 12
            im2 = im2-Data.bkgrd(:,:,kk)-20;% 0.5% of bit depth
        else
            im2 = im2-Data.bkgrd(:,:,kk)-(0.005*2^bitdepth);% 0.5% of bit depth
        end
    end

    
%     if Data.bkgd =='1'
%         if bitdepth ==8
%             im2 = im2-Data.bkgrd-5; % 2% of bit depth
%         elseif bitdepth == 12
%             im2 = im2-Data.bkgrd-20;% 0.5% of bit depth
%         else
%             im2 = im2-Data.bkgrd-(0.0048828125*2^bitdepth);
%         end
%     end
%     if Data.batchname(1)=='J';
%         im2=im2(:,1:298); %for J
%     elseif Data.batchname(1)=='F';
%         im2=im2(:,1:404); %for F
%     elseif Data.batchname(1)=='B';
%         im2=im2(:,1:192); %for B
%     end
    %Any negative values set to zero
    im2(im2<0)=0;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Load Position and Displacement values (*.mat/*.VC7 files only)
    if strcmp(Data.dataext,'vc7')==1 || strcmp(Data.dataext,'VC7')==1
    %Create Eval matrix to locate 0 values (not including mask 0s)
        %pre-processed vector field
        davis=readimx([database sprintf(['%0.' Data.datazeros 'i.' Data.dataext],I1data(q))]);
        Grid=davis.Frames{Data.vcNum}.Grids.X;
        [Nx, Ny]= size(davis.Frames{Data.vcNum}.Components{1}.Planes{1});
        
        X=Grid/2:Grid:Grid*Nx;
        Y=Grid/2:Grid:Grid*Ny;
        [X,Y]=meshgrid(X,Y);
      
        [files1] = intersect(davis.Frames{Data.vcNum}.ComponentNames,'W0');
        if strcmp(files1,'W0')
            error('VC7 FILES ARE STILL IN 3C FORMAT! CONVERT TO 2x2D')
        end

        [files1] = intersect(davis.Frames{Data.vcNum}.ComponentNames,'U3');
        if strcmp(files1,'U3')
            U0=davis.Frames{Data.vcNum}.Components{1}.Planes{1}';
            V0=davis.Frames{Data.vcNum}.Components{2}.Planes{1}';
            U1=davis.Frames{Data.vcNum}.Components{3}.Planes{1}';
            V1=davis.Frames{Data.vcNum}.Components{4}.Planes{1}';
            U2=davis.Frames{Data.vcNum}.Components{5}.Planes{1}';
            V2=davis.Frames{Data.vcNum}.Components{6}.Planes{1}';
            U3=davis.Frames{Data.vcNum}.Components{7}.Planes{1}';
            V3=davis.Frames{Data.vcNum}.Components{8}.Planes{1}';
            Active_Choice=davis.Frames{Data.vcNum}.Components{9}.Planes{1}';

            AC0=zeros(size(U0));  AC1=AC0;  AC2=AC0;  AC3=AC0;
            AC0(Active_Choice==0)=1;
            AC1(Active_Choice==1)=1;
            AC2(Active_Choice==2)=1;
            AC3(Active_Choice==4)=1;

            U=U0.*AC0 + U1.*AC1 + U2.*AC2 + U3.*AC3;
            V=V0.*AC0 + V1.*AC1 + V2.*AC2 + V3.*AC3;

            Eval=double(davis.Frames{Data.vcNum}.Components{10}.Planes{1}');
            U= double(U.*Eval);
            V= double(V.*Eval);
        else
            error('LET SCOTT KNOW IF THIS HAPPENS')
        end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    else %All other files
        %Based on format:
            %TITLE = "B00002"
            % VARIABLES = "x", "y", "Vx", "Vy"
            % ZONE T="Frame 0", I=128, J=128
            % 8.000000 8.000000 1.500675 -0.172555
            % 24.000000 8.000000 0.328738 1.074726
            % 40.000000 8.000000 0.000000 0.000000
            % 56.000000 8.000000 0.629379 -0.249157
        
        %This does not seem to work for latest PRANA output    
        D=importdata([database sprintf(['%0.' Data.datazeros 'i.' Data.dataext],I1data(q))], ' ');%,nheaderlines);
        X=D.data(:,1);
        Y=D.data(:,2);
        U=D.data(:,3);
        V=D.data(:,4);
        Eval =zeros(size(U));
        %Only works if bad vectors are not replaced.
        Eval(U==0)=1;
        [X,Y,U,V,Eval]=matrixform(X,Y,U,V,Eval);
        U = double(flipud(U));
        V = double(flipud(V));                
    end
    
    % Remove NaN from Prana (DaVis should already have this done)
    U(isnan(U))=0;
    V(isnan(V))=0;
    
    [dt,Mag]=dt_sf(Davis,davis);
    %Convert from physical units to pixles (if able)
    if X(1,1)-floor(X(1,1))==0 %(0 if pixel units, decimal if physical units)
        % Results are in pixel untis, Correct
    else
        %If im7/vc7 files are used/convert to pixels based on data.
        if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
            %Same values entered as found in Files
            if floor(Davis.ScaleX(1)*1000)==floor(Mag)
                % Results are in physical units, convert to Pixel Units
                X= (X-Davis.ScaleX(2))/Davis.ScaleX(1);
                Y= (Y-Davis.ScaleY(2))/Davis.ScaleY(1);  
                if isnan(dt)||dt==0
                   error('Images or PIV results are not in pixel units. No calibration information avaliable in DaVis files.')
                end
                if Data.stereo=='0'    
                U= U/Mag*dt;
                V= V/Mag*dt;
                end
            else
                error('Error:PIVuncertaintyCode','!!!!! Images or PIV results are not in pixel units!!!!!\n(Values entered for conversion to pixel units do not match those found in the im7 files.') 
            end
        else
        error('!!!!! Images or PIV results are not in pixel units !!!!!')    
        end
        
        %Check conversion to pixel units
        if X(1,1)-floor(X(1,1))~=0
            if X(1,1)-floor(X(1,1))<=0.001
               X=floor(X);
               Y=floor(Y);
            else
            error('!!!!! Conversion to pixel units failed !!!!!')
            end
        end
    end
    bcalc = cputime-tf;
    
    % Calculate Particle Diameter and Density (sub-function "FindDia")
    tc=cputime;
    [P_Dia, P_Dens, DiaRatio, P_DiaX, P_DiaY,~,PkD, PkD2] = DiaDen2(im2,Data,X,Y);
    
    %Time to calculate particle diameter/density
    tcalc = cputime-tc;
    t1=cputime;
    
    %Average bad vectors for shear calculation (sub-function "Fill")
    [U2,V2]=Fill(U,V,Eval,10);
   
    %Compute shear and uncertainty values
%     [dudy,dvdx,Bias_k,R_low,R_high,RR_95,Mark,MarkEval] = compute_bias(P_Dens,P_DiaX,P_DiaY,X,Y,U2,V2,Usurf,Eval);
        [dudy,dvdx,Bias_k,R_low,R_high,RR_95,Mark,MarkEval] = compute_bias(PkD,P_DiaX,P_DiaY,X,Y,U2,V2,Usurf,Eval);

    %Time to compute shear and bias
    corrtime=cputime-t1;

    %Write Results (sub-function "write_plt_val")
    t2=cputime;
    time=(q-1);%/Freq;
    
    %Convert to physical units if applicable
    if Mag==1 && dt==1 || Mag==1000 %Values are to remain in pixels     
    else     
        %If vc7 are used and the value read into um/pix matches Davis.ScaleX
        if strcmp(Data.dataext,'vc7')==1 || strcmp(Data.dataext,'VC7')==1 && floor(davis.ScaleX(1)*1000)==floor(Mag)
            X=X*davis.Frames{Data.vcNum}.Scales.X.Slope+davis.Frames{Data.vcNum}.Scales.X.Offset; 
            Y=Y*davis.Frames{Data.vcNum}.Scales.Y.Slope+davis.Frames{Data.vcNum}.Scales.Y.Offset; 
        else
            %Otherwise, scales dont match, use entered scale
            X=X*Mag/1000; %[mm]
            Y=Y*Mag/1000; %[mm];
        end
        U=U*Mag/dt;
        V=V*(-Mag)/dt;
        Bias_k=Bias_k*Mag/dt;
        R_low=R_low*Mag/dt;
        R_high=R_high*Mag/dt;
        RR_95=RR_95*(Mag/dt)^2;
    end
      
    %Output data in *.PLT file
    if Data.pltout == '1'
        if Data.fileend==0
        write_plt_val([pltdirec char(Data.outbase) sprintf(['%0.' Data.imzeros 'i.plt' ],Data.fileend+I1data(q))],X,Y,U,V,Eval,MarkEval,Bias_k,R_low,R_high,RR_95,dudy,dvdx,P_Dia,P_Dens,PkD, PkD2,DiaRatio,time,title);
        else
        write_plt_val([pltdirec char(Data.outbase) sprintf(['%0.' Data.imzeros 'i.plt' ],Data.fileend+(Data.datafstep-1)+I1data(q))],X,Y,U,V,Eval,MarkEval,Bias_k,R_low,R_high,RR_95,dudy,dvdx,P_Dia,P_Dens,PkD, PkD2,DiaRatio,time,title);
        end
    end
    X=single(X); Y=single(Y);  Eval=uint8(Eval); MarkEval=uint8(MarkEval);
    dudy=single(dudy); dvdx=single(dvdx); P_Dia=single(P_Dia); 
    P_Dens=single(P_Dens); PkD=single(PkD); PkD2 =single(PkD2); DiaRatio=single(DiaRatio);   
    %Output data in*.MAT file
    if Data.matout == '1'
        if Data.fileend==0
        save([pltdirec Data.outbase sprintf(['%0.' Data.imzeros 'i.mat'],Data.fileend+I1data(q))],'X','Y','U','V','Eval','Mark','MarkEval','Bias_k','R_low','R_high','RR_95','dudy','dvdx','P_Dia','P_Dens','PkD', 'PkD2','DiaRatio','dt','Mag');
        else   
        save([pltdirec Data.outbase sprintf(['%0.' Data.imzeros 'i.mat'],Data.fileend+(Data.datafstep-1)+I1data(q))],'X','Y','U','V','Eval','Mark','MarkEval','Bias_k','R_low','R_high','RR_95','dudy','dvdx','P_Dia','P_Dens','PkD', 'PkD2','DiaRatio','dt','Mag');
        end
    end
    %compute save time
    savetime=cputime-t2;
    %compute total frame time
    eltime=cputime-tf;
    
    %output status
    fprintf('\n----------------------------------------------------\n')
    fprintf(['Job: ',Data.batchname,'\n'])
    fprintf([title ' Completed (' num2str(q) '/' num2str(length(I1)) ')\n'])
    fprintf('----------------------------------------------------\n')
    fprintf('load data...                     %0.2i:%0.2i.%0.0f\n',floor(bcalc/60),floor(rem(bcalc,60)),rem(bcalc,60)-floor(rem(bcalc,60)))
    fprintf('dia/dens calculation...          %0.2i:%0.2i.%0.0f\n',floor(tcalc/60),floor(rem(tcalc,60)),rem(tcalc,60)-floor(rem(tcalc,60)))
    fprintf('bias calculation...              %0.2i:%0.2i.%0.0f\n',floor(corrtime/60),floor(rem(corrtime,60)),rem(corrtime,60)-floor(rem(corrtime,60)))
    fprintf('save time...                     %0.2i:%0.2i.%0.0f\n',floor(savetime/60),floor(rem(savetime,60)),rem(savetime,60)-floor(rem(savetime,60)))   
    fprintf('total frame time...              %0.2i:%0.2i.%0.0f\n',floor(eltime/60),floor(rem(eltime,60)),rem(eltime,60)-floor(rem(eltime,60)))
    %estimate job completion time
    frametime(q)=eltime;
    comptime=mean(frametime(frametime~=0))*(length(I1)-q);
    fprintf('estimated job completion time... %0.2i:%0.2i:%0.2i\n\n',floor(comptime/3600),floor(rem(comptime,3600)/60),floor(rem(comptime,60)))
end
        
        
%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           END MAIN FUNCTION                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% VECTOR TO MATRIX SUBFUNCTION
function [X,Y,U,V,Eval]=matrixform(x,y,u,v,eval)
%If PIV outputs position/velocity values in vector, convert to matrix form

%find unique x and y grid points
a=sort(unique(x));
b=sort(unique(y));
N=length(x);

%initialize matrices
U=nan(length(b),length(a));
V=nan(length(b),length(a));
Eval=-1*ones(length(b),length(a));

%generate grid matrix
[X,Y]=meshgrid(a,b);

%generate variable matrices (nans where no data available)
for n=1:N
    I=find(b==y(n));
    J=find(a==x(n));
    U(I,J) = u(n);
    V(I,J) = v(n);
    Eval(I,J) = eval(n);
end


%% PLT WRITER SUBFUNCTION (Needed sub-function: matrixform)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function []=write_plt_val(fname,X,Y,U,V,Eval,MarkEval,Bias_k,R_low,R_high,RR_95,dudy,dvdx,P_Dia,P_Dens,PkD,PkD2,DiaRatio,T,title)

Ur_minus = (Bias_k.*Bias_k+R_low.*R_low).^0.5;
Ur_plus = (Bias_k.*Bias_k+R_high.*R_high).^0.5;

%find I,J for plt
S = size(U);

%generate text file
fid = fopen(fname,'w');
if fid==-1
    error(['error creating file ',fname])
end

%header lines
fprintf(fid,['TITLE        = "' title '"\n']);
fprintf(fid,'VARIABLES    = "X", "Y", "U", "V", "Eval", "MarkEval", "b_k_x", "b_k_y","b_rr_x", "b_rr_y", "r_low_x", "r_low_y", "r_high_x", "r_high_y", "U_r-_x", "U_r-_y", "U_r+_x", "U_r+_y", "du/dy", "dv/dx", "Diameter", "Density","PkD","PkD2", "DiaRatio"\n');
fprintf(fid,'ZONE T="Time=%0.6f" I=%i J=%i C=BLACK\n',T,S(2),S(1));

%write data
for i=1:S(1)
    for j=1:S(2)
        if isnan(U(i,j)) || isnan(V(i,j))
            %second check to ensure no nans present
            fprintf(fid,'%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n',...
                X(i,j),Y(i,j),0,0,0,-1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
        else
            %valid data points
            fprintf(fid,'%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n',...
                X(i,j),Y(i,j),U(i,j),V(i,j),Eval(i,j),MarkEval(i,j),Bias_k(i,j,1),Bias_k(i,j,2),RR_95(i,j,1),RR_95(i,j,2),R_low(i,j,1),R_low(i,j,2),R_high(i,j,1),R_high(i,j,2),Ur_minus(i,j,1),Ur_minus(i,j,2),Ur_plus(i,j,1),Ur_plus(i,j,2),dudy(i,j),dvdx(i,j),P_Dia(i,j),P_Dens(i,j),PkD(i,j),PkD2(i,j),DiaRatio(i,j));
        end
    end
end
fclose(fid);

%% PLT WRITER SUBFUNCTION (Needed sub-function: matrixform)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function []=write_plt_val2(fname,X,Y,U,V,Eval,Bias_k,R_low,R_high,title)
% 
% Ur_minus = (Bias_k.*Bias_k+R_low.*R_low).^0.5;
% Ur_plus = (Bias_k.*Bias_k+R_high.*R_high).^0.5;
% Ur_minus(Ur_minus<0.012)=0.012;
% Ur_plus(Ur_plus<0.012)=0.012;
% Ux_minus=Ur_minus(:,:,1);
% Uy_minus=Ur_minus(:,:,2);
% Ux_plus=Ur_plus(:,:,1);
% Uy_plus=Ur_plus(:,:,2);
% 
% Ux_minus(U==0)=0;
% Uy_minus(U==0)=0;
% Ux_plus(U==0)=0;
% Uy_plus(U==0)=0;
% 
% %remove nans from data, replace with zeros
% U(Eval<0)=0;
% V(Eval<0)=0;
% 
% %find I,J for plt
% S = size(U);
% 
% %generate text file
% fid = fopen(fname,'w');
% if fid==-1
%     error(['error creating file ',fname])
% end
% 
% %header lines
% fprintf(fid,['TITLE        = "' title '"\n']);
% fprintf(fid,'VARIABLES    = "x", "y", "u", "v", "ex-", "ey-", "ex+", "ey+"\n');
% fprintf(fid,['ZONE T="' title '" I=%i J=%i\n'],S(2),S(1));
% 
% %write data
% for i=1:S(1)
%     for j=1:S(2)
%         if isnan(U(i,j)) || isnan(V(i,j))
%             %second check to ensure no nans present
%             fprintf(fid,'%4.0f %4.0f %8.6f %8.6f %8.6f %8.6f %8.6f %8.6f\n',...
%                 X(i,j),Y(i,j),0,0,0,0,0,0,0,0);
%         else
%             %valid data points
%             fprintf(fid,'%4.0f %4.0f %8.6f %8.6f %8.6f %8.6f %8.6f %8.6f\n',...
%                 X(i,j),Y(i,j),U(i,j),V(i,j),Ux_minus(i,j),Uy_minus(i,j),Ux_plus(i,j),Uy_plus(i,j));
%         end
%     end
% end
% fclose(fid);

%% Bias Estimation  (Needed sub-functions: tps, derivative_tps, matrixform)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [u_shear,v_shear,B_k,R_l,R_u,RR_95,Mark,MarkEval] = compute_bias(dens,diaX,diaY,x,y,u,v,Usurf,Eval)

 %OLD CODE (FOR LOOPS)   
%%Loop: estimates Shear
%     for i = 1:size(x,2)
%         for j = 1:size(x,1)
%             %Generate grid to feed into shear calculation.
%             grid_size = 3;              
%             if (i == 1 && j == 1) %Top left Corner
%                 X = x(1:grid_size,1:grid_size);
%                 Y = y(1:grid_size,1:grid_size);
%                 U = u(1:grid_size,1:grid_size);
%                 V = v(1:grid_size,1:grid_size);
%             elseif (i == 1 && j == size(x,1))%Top right corner
%                 X = x(size(x,1)-grid_size+1:size(x,1),1:grid_size);
%                 Y = y(size(x,1)-grid_size+1:size(x,1),1:grid_size);
%                 U = u(size(x,1)-grid_size+1:size(x,1),1:grid_size);
%                 V = v(size(x,1)-grid_size+1:size(x,1),1:grid_size);
%             elseif (i == size(x,2) && j == 1) %Bottom left corner
%                 X = x(1:grid_size,size(x,2)-grid_size+1:size(x,2));
%                 Y = y(1:grid_size,size(x,2)-grid_size+1:size(x,2));
%                 U = u(1:grid_size,size(x,2)-grid_size+1:size(x,2));
%                 V = v(1:grid_size,size(x,2)-grid_size+1:size(x,2));
%             elseif (i == size(x,2) && j == size(x,1)) %Bottom right corner
%                 X = x(size(x,1)-grid_size+1:size(x,1),size(x,2)-grid_size+1:size(x,2));
%                 Y = y(size(x,1)-grid_size+1:size(x,1),size(x,2)-grid_size+1:size(x,2));
%                 U = u(size(x,1)-grid_size+1:size(x,1),size(x,2)-grid_size+1:size(x,2));
%                 V = v(size(x,1)-grid_size+1:size(x,1),size(x,2)-grid_size+1:size(x,2));
%             elseif (i == 1 && j ~= 1 && j ~= size(x,1)) %Top edge
%                 X = x(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,1:grid_size);
%                 Y = y(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,1:grid_size);
%                 U = u(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,1:grid_size);
%                 V = v(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,1:grid_size);
%             elseif (j == 1 && i ~= 1 && i ~= size(x,2)) %Left Edge
%                 X = x(1:grid_size,i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 Y = y(1:grid_size,i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 U = u(1:grid_size,i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 V = v(1:grid_size,i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%             elseif (i == size(x,2) && j ~= 1 && j ~= size(x,1)) %Bottom edge
%                 X = x(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,size(x,2)-grid_size+1:size(x,2));
%                 Y = y(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,size(x,2)-grid_size+1:size(x,2));
%                 U = u(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,size(x,2)-grid_size+1:size(x,2));
%                 V = v(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,size(x,2)-grid_size+1:size(x,2));
%             elseif (j == size(x,1) && i ~= 1 && i ~= size(x,2)) %Right Edge
%                 X = x(size(x,1)-grid_size+1:size(x,1),i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 Y = y(size(x,1)-grid_size+1:size(x,1),i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 U = u(size(x,1)-grid_size+1:size(x,1),i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 V = v(size(x,1)-grid_size+1:size(x,1),i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%             else %Internal
%                 grid_size = (grid_size-1)*0.5;
%                 X = x(j-grid_size:j+grid_size,i-grid_size:i+grid_size);
%                 Y = y(j-grid_size:j+grid_size,i-grid_size:i+grid_size);
%                 U = u(j-grid_size:j+grid_size,i-grid_size:i+grid_size);
%                 V = v(j-grid_size:j+grid_size,i-grid_size:i+grid_size);
%             end
% 
%             %Estimate the Gradient using TPS (sub-functions "tps" and "derivative_tps")
%             [X,Y,coef_matu,coef_matv,~,~]=tps(X(:),Y(:),U(:),V(:));
%             [~,dudy,dvdx,~]=derivative_tps(i,j,X,Y,coef_matu,coef_matv);
%             u_shear0(j,i) = dudy;
%             v_shear0(j,i) = dvdx;
%         end
%     end   
%     u_shear0=u_shear0.*Eval;v_shear0=v_shear0.*Eval;
% clearvars -except u_shear0 v_shear0 dens diaX diaY x y u v Usurf Eval    


w = 1; %interrogation for shear is (1+w+w)x(1+w+w)
[i,~,j] = meshgrid(1:size(x,2),1,1:size(x,1));
ipos=1:size(x,2);
jpos=1:size(x,1);

%Particle size interrogation (out of bounds set to bounds)
imin = max(ipos-w,1);
imax = min(ipos+w,size(x,2));
jmin = max(jpos-w,1);
jmax = min(jpos+w,size(x,1));

%Make edge particles size bigger to make the same size as internal particles
imax(imin==1)=1+w+w;
imin(imax==size(x,2))=size(x,2)-w-w;
jmax(jmin==1)=1+w+w;
jmin(jmax==size(x,1))=size(x,1)-w-w; 

u=permute(u',[3 1 2]);
v=permute(v',[3 1 2]);
x=permute(x',[3 1 2]);
y=permute(y',[3 1 2]);

U=[u(1,imin,jmin);u(1,imin,jmin+1);u(1,imin,jmin+2);u(1,imin+1,jmin);u(1,imin+1,jmin+1);u(1,imin+1,jmin+2);u(1,imin+2,jmin);u(1,imin+2,jmin+1);u(1,imin+2,jmin+2);];
V=[v(1,imin,jmin);v(1,imin,jmin+1);v(1,imin,jmin+2);v(1,imin+1,jmin);v(1,imin+1,jmin+1);v(1,imin+1,jmin+2);v(1,imin+2,jmin);v(1,imin+2,jmin+1);v(1,imin+2,jmin+2);];
X=[x(1,imin,jmin);x(1,imin,jmin+1);x(1,imin,jmin+2);x(1,imin+1,jmin);x(1,imin+1,jmin+1);x(1,imin+1,jmin+2);x(1,imin+2,jmin);x(1,imin+2,jmin+1);x(1,imin+2,jmin+2);];
Y=[y(1,imin,jmin);y(1,imin,jmin+1);y(1,imin,jmin+2);y(1,imin+1,jmin);y(1,imin+1,jmin+1);y(1,imin+1,jmin+2);y(1,imin+2,jmin);y(1,imin+2,jmin+1);y(1,imin+2,jmin+2);];

[coef_matu,coef_matv]=tps2(X,Y,U,V);
[~,u_shear,v_shear,~]=derivative_tps2(i,j,X,Y,coef_matu,coef_matv,Eval);

clear x y imin imax jmin jmax ipos jpos w X Y U V coef_matu coef_matv i j

u=(permute(u,[2 3 1])').*Eval;
v=(permute(v,[2 3 1])').*Eval;

%Pre-allocate Arrays (DONT NEED?)
            B_k     = zeros([size(u),2])+0.5;
            R_l     = zeros([size(u),2])+0.5;
            R_u     = zeros([size(u),2])+0.5;
            RR_95   = zeros([size(u),2]);

%Load Uncertainty Surface Data
s=load(Usurf);

%"Mark" AN OUTPUT, SIMILAR TO "Eval", shows values that were cropped
Mark = int8(zeros(size(diaX)));
Mark(dens>max(s.density(:))) = 1;
Mark(dens<min(s.density(:))) = -1;
Mark(diaX>max(s.diameter(:))) = 2;
Mark(diaX<min(s.diameter(:))) = -2;
Mark(diaY>max(s.diameter(:))) = 2;
Mark(diaY<min(s.diameter(:))) = -2;
Mark(abs(u_shear)>max(s.shear(:))) = 3;
Mark(abs(u_shear)<min(s.shear(:))) = -3;
Mark(abs(v_shear)>max(s.shear(:))) = 3;
Mark(abs(v_shear)<min(s.shear(:))) = -3;

MarkEval=Mark;
MarkEval(MarkEval~=0)=-1;
MarkEval=MarkEval+1;

    %Locate/mark values outside uncertainty surface (limits pulled from
    %uncertainty surface are used.)
    dens(dens>max(s.density(:)))=max(s.density(:)); 
    dens(dens<min(s.density(:)))=min(s.density(:));
    diaX(diaX>max(s.diameter(:)))=max(s.diameter(:));
    diaX(diaX<min(s.diameter(:)))=min(s.diameter(:)); 
    diaY(diaY>max(s.diameter(:)))=max(s.diameter(:));
    diaY(diaY<min(s.diameter(:)))=min(s.diameter(:));
    u_shear(abs(u_shear)>max(s.shear(:)))=max(s.shear(:));
    u_shear(abs(u_shear)<min(s.shear(:)))=min(s.shear(:));
    v_shear(abs(v_shear)>max(s.shear(:)))=max(s.shear(:));
    v_shear(abs(v_shear)<min(s.shear(:)))=min(s.shear(:));

    %Sub-pixel displacement (no assumed symmetry: values used 0±1.0)
    dx = abs(u-fix(u));
    dy = abs(v-fix(v));
fo=ones(size(dx))*0;
    %Interpolate Bias Uncertainties from Uncertainty surface
if length(size(s.b_k))==5    
    %Calculate the bias uncertainty (u_bar-u_true)
    B_k(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.b_k,dens,diaX,abs(u_shear),dx,fo,'linear',0.5);
    B_k(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.b_k,dens,diaY,abs(v_shear),dy,fo,'linear',0.5);
    %Calculate the r_low uncertainty (u_bar-x_2.5%)
    R_l(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.r_low,dens,diaX,abs(u_shear),dx,fo,'linear',0.5);
    R_l(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.r_low,dens,diaY,abs(v_shear),dy,fo,'linear',0.5);
    %Calculate the r_high uncertainty (x_97.5%-u_bar)
    R_u(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.r_high,dens,diaX,abs(u_shear),dx,fo,'linear',0.5);
    R_u(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.r_high,dens,diaY,abs(v_shear),dy,fo,'linear',0.5); 
    %Calculate the rr uncertainty for the fluctuations (x_95%-u_bar) [introduced by Brandon]
    RR_95(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.rr95,dens,diaX,abs(u_shear),dx,fo,'linear',0.5);
    RR_95(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.rr95,dens,diaY,abs(v_shear),dy,fo,'linear',0.5); 
elseif length(size(s.b_k))==4
    %Calculate the bias uncertainty (u_bar-u_true)
    B_k(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.b_k,dens,diaX,abs(u_shear),dx,'linear',0.5);
    B_k(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.b_k,dens,diaY,abs(v_shear),dy,'linear',0.5);
    %Calculate the r_low uncertainty (u_bar-x_2.5%)
    R_l(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.r_low,dens,diaX,abs(u_shear),dx,'linear',0.5);
    R_l(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.r_low,dens,diaY,abs(v_shear),dy,'linear',0.5);
    %Calculate the r_high uncertainty (x_97.5%-u_bar)
    R_u(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.r_high,dens,diaX,abs(u_shear),dx,'linear',0.5);
    R_u(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.r_high,dens,diaY,abs(v_shear),dy,'linear',0.5); 
    %Calculate the rr uncertainty for the fluctuations (x_95%-u_bar) [introduced by Brandon]
    RR_95(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.rr95,dens,diaX,abs(u_shear),dx,'linear',0.5);
    RR_95(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.rr95,dens,diaY,abs(v_shear),dy,'linear',0.5);
end

%% TPS                                 (Used by sub-function: compute_bias)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [X,Y,coef_matu,coef_matv,U,V]=tps(X,Y,U,V)
%         P = zeros(length(X),3);
%         for i=1:length(X)
%             P(i,:)=[1,X(i),Y(i)];
%         end
%         B = zeros(length(X),length(X));
%         for i=1:length(X)
%             for j=1:length(X)% index for columns
%                dummy=((X(i)-X(j))^2 +(Y(i)-Y(j) )^2);
%                 B(i,j)=dummy*log(sqrt(dummy+eps));
%             end
%         end
%   G=[B,P;
%      transpose(P),zeros(size(P,2),size(P,2))];
%   coef_matu=G\[U;zeros(size(P,2),1)];
%   coef_matv=G\[V;zeros(size(P,2),1)];

function [coef_matu,coef_matv]=tps2(X,Y,U,V)
P = ones([size(X,1),3,size(X,2),size(X,3)]);
P(:,2,:,:)=X;
P(:,3,:,:)=Y;

dummy=[0,1,4,1,2,5,4,5,8;1,0,1,2,1,2,5,4,5;4,1,0,5,2,1,8,5,4;1,2,5,0,1,4,1,2,5;2,1,2,1,0,1,2,1,2;5,2,1,4,1,0,5,2,1;4,5,8,1,2,5,0,1,4;5,4,5,2,1,2,1,0,1;8,5,4,5,2,1,4,1,0]*(Y(2,1,1)-Y(1,1,1))^2;
B=dummy.*log(sqrt(dummy+eps));
B=repmat(permute(B,[1 2 3 4]),[1 1 size(U,2) size(U,3) ]);

G=  [B,P;
    permute(P,[2 1 3 4]),zeros(size(P,2),size(P,2),size(U,2),size(U,3))];

tempU=[U;zeros(size(P,2), size(P,3),size(P,4))];
tempV=[V;zeros(size(P,2), size(P,3),size(P,4))];

coef_matu=zeros(size(tempU));
coef_matv=coef_matu;
%Takes the most time:
for i=1:size(X,2)
    for j=1:size(X,3)
    coef_matu(:,i,j)=G(:,:,i,j)\tempU(:,i,j);
    coef_matv(:,i,j)=G(:,:,i,j)\tempV(:,i,j);
    end
end


%% Derivative TPS                      (Used by sub-function: compute_bias)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [dudx,dudy,dvdx,dvdy]=derivative_tps(x,y,X,Y,coef_matu,coef_matv)
% % Functions should be called after computing the coefficient matrices 
% % Compute the gradients at x, y for the interpolation over X, Y 
% 
% dudx_interp=0;dvdx_interp=0;dudy_interp=0;dvdy_interp=0;
% 
% for k=1:length(X);
%     dummy=((x-X(k))^2 +(y-Y(k) )^2);
%     dummy1=(1+log(sqrt(dummy+eps)));
%     dudx_interp=(x-X(k))*dummy1*coef_matu(k)+dudx_interp;
%     dudy_interp=(y-Y(k))*dummy1*coef_matu(k)+dudy_interp;
%     dvdx_interp=(x-X(k))*dummy1*coef_matv(k)+dvdx_interp;
%     dvdy_interp=(y-Y(k))*dummy1*coef_matv(k)+dvdy_interp;
% end
% 
% Pddx=[0,1,0];Pddy=[0,0,1];%,x^2,x*y,y^2];%,x^2,x*y,y^2];
% dudx=dudx_interp+Pddx*coef_matu(end-size(Pddx,2)+1:end);
% dvdx=dvdx_interp+Pddx*coef_matv(end-size(Pddx,2)+1:end);  
% dudy=dudy_interp+Pddy*coef_matu(end-size(Pddy,2)+1:end);  
% dvdy=dvdy_interp+Pddy*coef_matv(end-size(Pddy,2)+1:end);  

function [dudx,dudy,dvdx,dvdy]=derivative_tps2(x,y,X,Y,coef_matu,coef_matv,Eval)
% Functions should be called after computing the coefficient matrices 
% Compute the gradients at x, y for the interpolation over X, Y 
x=permute(x,[2 3 1]);
y=permute(y,[2 3 1]);
X=permute(X,[2 3 1]);
Y=permute(Y,[2 3 1]);
coef_matu=permute(coef_matu,[2 3 1]);
coef_matv=permute(coef_matv,[2 3 1]);

dummy=bsxfun(@minus,x,X).^2+bsxfun(@minus,y,Y).^2;
dummy1=(1+log(sqrt(dummy+eps)));
dudx_interp=sum(bsxfun(@minus,x,X).*dummy1.*coef_matu(:,:,1:size(X,3)),3);
dudy_interp=sum(bsxfun(@minus,y,Y).*dummy1.*coef_matu(:,:,1:size(X,3)),3);
dvdx_interp=sum(bsxfun(@minus,x,X).*dummy1.*coef_matv(:,:,1:size(X,3)),3);
dvdy_interp=sum(bsxfun(@minus,y,Y).*dummy1.*coef_matv(:,:,1:size(X,3)),3);
    
dudx=((dudx_interp+coef_matu(:,:,end-1))').*Eval;
dvdx=((dvdx_interp+coef_matv(:,:,end-1))').*Eval;
dudy=((dudy_interp+coef_matu(:,:,end))').*Eval;
dvdy=((dvdy_interp+coef_matv(:,:,end))').*Eval;

%% Average Scheme
 function [u,v]=Fill(u,v,eval,maxct)

%find unique x and y grid points
i=size(u,1);
j=size(u,2);

%Keep originals
U=u.*eval;
V=v.*eval;


for k=1:maxct
    %Average all 4 neighbors
    I=zeros([size(u) 4]);
    I(:,2:j,1)=u(:,1:j-1);
    I(:,1:j-1,2)=u(:,2:j);
    I(2:i,:,3)=u(1:i-1,:);
    I(1:i-1,:,4)=u(2:i,:);
    Temp=sum(I,3)./4;
    %Remove all original U
    Temp=Temp.*(eval-1)*(-1);
    %Fill up touching original U
    u=U+Temp;
    
    %Average all 4 neighbors
    I=zeros([size(v) 4]);
    I(:,2:j,1)=v(:,1:j-1);
    I(:,1:j-1,2)=v(:,2:j);
    I(2:i,:,3)=v(1:i-1,:);
    I(1:i-1,:,4)=v(2:i,:);
    Temp=sum(I,3)./4;
    %Remove all original V
    Temp=Temp.*(eval-1)*(-1);
    %Fill up touching original V
    v=V+Temp;
end


function [dt,sf]=dt_sf(Davis,VC7)

%SF
sf=Davis.Frames{1,1}.Scales.X.Slope*1000;
sf_check=VC7.Frames{1,1}.Scales.X.Slope*1000;

if sf_check~=sf
    error(['Can not determine sf automatically. ' num2str(sf) ' not equal ' num2str(sf_check) ')\n'])
end

%DT
t0='empty';
t1=t0;
t2=t0;
if strcmp(Davis.Frames{1,1}.Attributes{2,1}.Name,'AcqTime')
    t0= Davis.Frames{1,1}.Attributes{2,1}.Value;
else
    i=0;
    while strcmp(t0,'empty')
        i=i+1;
        if strcmp(Davis.Frames{1,1}.Attributes{i,1}.Name,'AcqTime')
          t0= Davis.Frames{1,1}.Attributes{i,1}.Value;
        end
    end
end

if strcmp(Davis.Frames{2,1}.Attributes{2,1}.Name,'AcqTime')
    t1= Davis.Frames{2,1}.Attributes{2,1}.Value;
else
    i=0;
    while strcmp(t1,'empty')
       i=i+1;
       if strcmp(Davis.Frames{2,1}.Attributes{i,1}.Name,'AcqTime')
          t1= Davis.Frames{2,1}.Attributes{i,1}.Value;
       end
    end
end
value0 = textscan(t0, '%s',1);
value1 = textscan(t1, '%s',1);
dt=str2double(value1{1,1})-str2double(value0{1,1});

i=0;
if strcmp(VC7.Frames{1,1}.Attributes{3,1}.Name,'FrameDt')
    t2=    VC7.Frames{1,1}.Attributes{3,1}.Value;
else
    while strcmp(t2,'empty')
       i=i+1;
       if strcmp(VC7.Frames{1,1}.Attributes{i,1}.Name,'FrameDt')
          t2=    VC7.Frames{1,1}.Attributes{i,1}.Value;
       end            
    end
end
value2 = textscan(t2, '%s',1);
dt_check=str2double(value2{1,1});

if dt_check~=dt
    error(['Can not determine dt automatically. ' num2str(dt) ' not equal ' num2str(dt_check) ')\n'])
end


%%
% diff=1; 
% uNew=u;
% vNew=v;
% ct=0;
% while   ct<maxct && diff > 10^-3
%     ct=ct+1;
%     uOld=uNew;
%     vOld=vNew;
%     for i = 1:b
%         for j = 1:a
%             if eval(i,j)~=1
%                  % North Face (u = 0)
%                  if i==1    
%                      %Northwest Corner
%                      if j==1         
%                         u(i,j)=(u(i+1,j)+u(i,j+1))/2;
%                         v(i,j)=(v(i+1,j)+v(i,j+1))/2;
%                      %North East Corner
%                      elseif j==a    
%                         u(i,j)=(u(i+1,j)+u(i,j-1))/2;
%                         v(i,j)=(v(i+1,j)+v(i,j-1))/2;
%                      %North Face (non-corner)
%                      else
%                         u(i,j)=(u(i+1,j)+u(i,j+1)+u(i,j-1))/3;
%                         v(i,j)=(v(i+1,j)+v(i,j+1)+v(i,j-1))/3;
%                      end
%                  % South Face (u = 0)
%                  elseif i==b 
%                      %Southwest Corner
%                      if j==1    
%                         u(i,j)=(u(i-1,j)+u(i,j+1))/2;
%                         v(i,j)=(v(i-1,j)+v(i,j+1))/2;
%                      %Southeast Corner
%                      elseif j==a
%                         u(i,j)=(u(i-1,j)+u(i,j-1))/2;
%                         v(i,j)=(v(i-1,j)+v(i,j-1))/2;
%                      %South Face (non-corner)
%                      else
%                         u(i,j)=(u(i-1,j)+u(i,j-1)+u(i,j+1))/3;
%                         v(i,j)=(v(i-1,j)+v(i,j-1)+v(i,j+1))/3;
%                      end    
%                  else
%                      %West Face
%                      if j==1
%                          u(i,j)=(u(i+1,j)+u(i-1,j)+u(i,j+1))/3;
%                          v(i,j)=(v(i+1,j)+v(i-1,j)+v(i,j+1))/3;
%                      %East Face
%                      elseif j==a
%                          u(i,j)=(u(i+1,j)+u(i-1,j)+u(i,j-1))/3;
%                          v(i,j)=(v(i+1,j)+v(i-1,j)+v(i,j-1))/3;
%                      else
%                          u(i,j)=(u(i+1,j)+u(i-1,j)+u(i,j-1)+u(i,j+1))/4;
%                          v(i,j)=(v(i+1,j)+v(i-1,j)+v(i,j-1)+v(i,j+1))/4;
%                      end
%                  end
%             end
%         end
%     end
%     %Estimate of T0
%     uNew=u;    
%     vNew=u;
%     %keep iterating until residual < 10^-6
%     diffu=abs(uNew-uOld);
%     diffv=abs(vNew-vOld);
%     diff=max(max([diffu diffv]));
% end           



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pad Array                                                               %
%   This function replicates the matrix 'a' so that it becomes size       %
%   'padSize'                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                             
% function b = ReplicatePad(a, padSize)
% numDims = numel(padSize);
% idx   = cell(1,numDims);
% for k = 1:numDims
%   M = size(a,k);
%   p = padSize(k);
%   onesVector = ones(1,p);
%   idx{k}   = [1:M M*onesVector];
% end
%  b = a(idx{:});

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             END OF CODE                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%{
% LATEST VERSION UPDATE: 10-13-14

function PIVuncertaintyCode(Data)
%
% Uncertainty analysis tool based upon input structure "Data"
% generated using the PIVuncertainty.m and PIVuncertainty.fig GUI file.
% 
% Required GUI inputs include: 
%       -Image files (including DaVis *.im7 files)
%       -Position and displacement vectors (*.mat / *.vc7 files)
%       -Pulse separation - dt (µs), Length conversion (µm/pix)
% History:
%   -Code originated as Prana.m (a PIV code)
%   -Uncertainty calculations added by B. Timmins and renamed 'PIVadvanced2Code.m'
%   -Uncertainty code separated 'PIVadvanced2code.m', expanded, and renamed
%    'PIVuncertaintyCode.m'.
%   -As part of the expansion, more accurate methods were implemented for 
%    calculating diameter and density
%
% Update History:
% 2-19-13   U and V uncertainties calculated using DiaX and DiaY,
%           respectively.
% 2-20-13   Added error message if 3C data is read in.
% 2-25-13   Feature to load data from various folders and save output files
%           from each in a single output folder in consecutive number. This
%           enables stats to be calculated on the data from the various
%           folders collectively.
%           Job file autosaves into output directory for future reference. 
% 3-11-13   Fixed bug when trying to do the background subrations on stereo
%           images.
% 6-12-13   Bug fixed in computing particle image diameter.
% 6-13-13   Added comments. Code now takes roughly 1/3 the time to run.
% 9-07-13   Adjusted the multifolder option so it accounts for step size 
%           account (ie step size 2, 1st set 1,3,5. 2nd set starts at 7, not 6)
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(['Current Job: ' Data.batchname '\n'])
%PC or Mac
if ispc
    pltdirec=[Data.outdirec '\'];
else
    pltdirec=[Data.outdirec '/'];
end
    
% This feature allows for these uncertainty to be calculated from data in
% separate folders and then be saved in a single folder for stats calc.
% (each job file is a folder).
d= dir([pltdirec Data.outbase '0' '*.mat']);    
if Data.multifolders=='1'    
    if isempty(d)
        Data.fileend=0-str2double(Data.imfstart)+1;
    else
        d2=d(size(d,1),1);
        lastfile=d2.name;
        %Fine values between Data.imbase and .mat
        Data.fileend=str2double(lastfile(strfind(lastfile,Data.outbase)+size(Data.outbase,2):strfind(lastfile,'.mat')-1))-str2double(Data.imfstart)+1;        
    end
else
    Data.fileend=0;
end

%Grid Size - set based on GUI input
if Data.DiaGS=='1'
    Data.GSdia=32;
elseif Data.DiaGS=='2'
    Data.GSdia=64;
elseif Data.DiaGS=='3'
    Data.GSdia=128;
elseif Data.DiaGS=='4'
    Data.GSdia=256;
end

if Data.bkgd =='1'
%% Calculate BackGround:
%{
    Noise is introduced through stationary objects, reflections, and a 
    non-zero backgrond. The background is calculated by comparing all 
    images in the set and taking the minimum value for each pixel.
%}
fprintf('\n-------------- Calculating Background --------------\n')

%Image file location and base (ie. C:\Folder\File-prefix)
if ispc
    imbase=[Data.imdirec '\' Data.imbase];
else
    imbase=[Data.imdirec '/' Data.imbase];
end

%Image sets
I1 = str2double(Data.imfstart):str2double(Data.imfstep):str2double(Data.imfend);
I2 = I1+str2double(Data.imcstep);

if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
    %Load single image to determin image size
    Davis=readimx([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I2(1))]);
    im2=double(Davis.Frames{str2double(Data.imNum)}.Components{1}.Planes{1}');
else
    %All other image files (Same method as for DaVis *im7 files [above])
    % im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I2(1))]));
    if Data.imNum == '1'
        im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(1))]));
    elseif Data.imNum == '2'
        im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I2(1))]));
    else
        error('Currently only uses frames 1 and 2 (unless im7)')
    end
end

%Pre-allocate matrices for the background and to calculate the mean/stdv
Data.bkgspc=100;%length(I1);
bkgd = ones([size(im2),2])*(2^16);
bkgrd2=zeros([size(im2), ceil(length(I1)/Data.bkgspc)]);
%Load every image pair 
j=0;
for q=1:length(I1)
    if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
        %DaVis Image File
        Davis=readimx([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I2(q))]);
        im2=double(Davis.Frames{str2double(Data.imNum)}.Components{1}.Planes{1}');
    else
        %All other image files (Same method as for DaVis *im7 files [above])
        if Data.imNum == '1'
            im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(q))]));
        elseif Data.imNum == '2'
            im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I2(q))]));
        else
            error('Currently only uses frames 1 and 2 (unless im7)')
        end
    end
    %Assign current image to 2nd slot in matrix
    bkgd(:,:,2) = im2;
    %Compare the current min the the new image, keep lowest value
    bkgd(:,:,1) = min(bkgd,[],3);

    %Sum the individual values and squared values for mean and stdv
    % imgSum = imgSum + im2;
    % imgSum2 = imgSum2 + im2.^2;
    % Imax=max(Imax,max(im2(:)));
    if (q/Data.bkgspc-floor(q/Data.bkgspc))==0 || q==length(I1)
        j=j+1;
        bkgrd2(:,:,j)= bkgd(:,:,1);
        bkgd = ones([size(im2),2])*(2^16);
    end
end
%Store background in Data structure for future use
Data.bkgrd = bkgrd2;%bkgd(:,:,1);

%Calculate and store mean
% Data.imgMean = (imgSum./q);
%Calculate and store standard deviation using: 
%http://en.wikipedia.org/wiki/Computational_formula_for_the_variance
% Data.imgStd = sqrt(imgSum2./(q-1)-(q/(q-1))*((imgSum./q).^2));
%{
%Uncomment to make the following plots:
map = gray(2^8);

%Shows Original Image
figure(1)
imshow(im2,map)   

%Shows time minimum background
figure(2)
imshow(Data.bkgrd,map) 

%Shows Figure (1) without background
figure(3)
Temp2=im2-Data.bkgrd;
Temp2(Temp2<0)=0;
imshow(Temp2,map) 
%}
%End Calculage Backgroud
end

%% Run Jobs in Parallel
%{
    If the "Run in Parallel" box is checked in the PIVuncertainty GUI, 
    this section opens matlabpool which enables parallel processing.
    Images are devided between the total number of processors.
%}
if str2double(Data.par)
    fprintf('\n--- Initializing Processor Cores for Parallel Job ----\n')
    poolopen=1;
    
    %Don't open more processors than there are image pairs
    if length(str2double(Data.imfstart):str2double(Data.imfstep):str2double(Data.imfend)) < str2double(Data.parprocessors)
        Data.parprocessors=num2str(length(str2double(Data.imfstart):str2double(Data.imfstep):str2double(Data.imfend)));
    end
    
    try
        %Open Matlab Pool
%         matlabpool('open','local',Data.parprocessors);
        parpool('local',str2double(Data.parprocessors));
    catch
        try
            %If First attepmt doesn't work, close and retry
%             matlabpool close
            delete(gcp('nocreate'));
%             matlabpool('open','local',Data.parprocessors);
            parpool('local',str2double(Data.parprocessors));
        catch
            %If Second attempt doesn't work, continue with single processor
            beep
            disp('Error Running Job in Parallel - Defaulting to Single Processor')
            poolopen=0;
            fprintf('\n-------------- Processing Dataset ------------------\n')
            USMprocessing(Data)
            fprintf('---------------- Job Completed ---------------------\n')
        end
    end
    if poolopen
        %Pool successfully open, divide sets
        I1=str2double(Data.imfstart):str2double(Data.imfstep):str2double(Data.imfend);
        I2=I1+str2double(Data.imcstep);
        Idata = str2double(Data.datafstart):str2double(Data.datafstep):str2double(Data.datafstep);

            fprintf('\n--------------- Processing Dataset -------------------\n')
            spmd
                verstr=version('-release');
                if str2double(verstr(1:4))>=2010
                    I1dist=getLocalPart(codistributed(I1,codistributor('1d',2)));
                    I2dist=getLocalPart(codistributed(I2,codistributor('1d',2)));
                    Idatdist=getLocalPart(codistributed(Idata,codistributor('1d',2)));
                else
                    I1dist=localPart(codistributed(I1,codistributor('1d',2),'convert'));
                    I2dist=localPart(codistributed(I2,codistributor('1d',2),'convert'));
                    Idatdist=localPart(codistributed(Idata,codistributor('1d',2),'convert'));
                end
                USMprocessing(Data,I1dist,I2dist,Idatdist);
            end
            fprintf('----------------- Job Completed ----------------------\n')
            
        if poolopen
%             matlabpool close
            delete(gcp('nocreate'));
        end
    end
else
    %"Run in Parallel" not selected - Default to single processor
    fprintf('\n-------------- Processing Dataset ------------------\n')
    USMprocessing(Data)
    fprintf('---------------- Job Completed ---------------------\n')
end


function USMprocessing(Data,I1,I2,I1data)
    
tic
%%Image file location/base (ie. C:\Folder\File-prefix) and output directory
if ispc
    imbase=[Data.imdirec '\' Data.imbase];
    database=[Data.datadirec '\' Data.database];
    pltdirec=[Data.outdirec '\'];
    Usurf=[Data.surfdirec '\' Data.surfname];
else
    imbase=[Data.imdirec '/' Data.imbase];
    database=[Data.datadirec '/' Data.database];
    pltdirec=[Data.outdirec '/'];
    Usurf=[Data.surfdirec '/' Data.surfname];
end

if nargin<3 %(Not runing in parallel)
%Image indices
I1 = str2double(Data.imfstart):str2double(Data.imfstep):str2double(Data.imfend);
I2 = I1+str2double(Data.imcstep);
%Data indices
I1data = str2double(Data.datafstart):str2double(Data.datafstep):str2double(Data.datafstep);
end

%Physical parameters
Mag = str2double(Data.wrmag);   %(µm/pix)
dt = str2double(Data.wrsep);    %(µs)
 
%Preallocate "time remaining" variable
frametime=zeros(length(I1));

%% EVALUATE IMAGE PAIRS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for q=1:length(I1)
    %Record Start time
    tf=cputime;
    %Set being analyzed
    title=['Frame' sprintf(['%0.' Data.imzeros 'i'],I1(q)) ' and Frame' sprintf(['%0.' Data.imzeros 'i'],I2(q))];
    title2=[Data.database sprintf(['%0.' Data.datazeros 'i'],I1data(q))];
    
    %LOAD IMAGE PAIRS
    if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
        %DaVis Image File
        Davis=readimx([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I2(q))]);
        im2=double(Davis.Frames{str2double(Data.imNum)}.Components{1}.Planes{1}');

    else
        %All other image files (bmp, gif, jpg, png, tif, etc)
        if Data.imNum == '1'
        im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(q))]));
        elseif Data.imNum == '2'
        im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I2(q))]));
        end
    end
   
    %Subtract background and a small % of bitdepth to compensate for aditional noise.
    bitdepth = ceil(log(max(im2(:)))/log(2));
    
    if Data.bkgd =='1'
        kk=ceil(I2(q)/Data.bkgspc);
        if bitdepth ==8
            im2 = im2-Data.bkgrd(:,:,kk);%-12; % 2% of bit depth
        elseif bitdepth == 12
            im2 = im2-Data.bkgrd(:,:,kk);%-20;% 0.5% of bit depth
        else
            im2 = im2-Data.bkgrd(:,:,kk);%-(0.01*2^bitdepth);
        end
    end

%     if Data.batchname(1)=='J';
%         im2=im2(:,1:298); %for J
%     elseif Data.batchname(1)=='F';
%         im2=im2(:,1:404); %for F
%     elseif Data.batchname(1)=='B';
%         im2=im2(:,1:192); %for B
%     end
    %Any negative values set to zero
    im2(im2<0)=0;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Load Position and Displacement values (*.mat/*.VC7 files only)
    if strcmp(Data.dataext,'vc7')==1 || strcmp(Data.dataext,'VC7')==1
    %Create Eval matrix to locate 0 values (not including mask 0s)
        %pre-processed vector field
        davis=readimx([database sprintf(['%0.' Data.datazeros 'i.' Data.dataext],I1data(q))]);
        Grid=davis.Frames{1}.Grids.X;
        [Nx, Ny]= size(davis.Frames{1}.Components{1}.Planes{1});
        
        X=Grid/2:Grid:Grid*Nx;
        Y=Grid/2:Grid:Grid*Ny;
        X = ReplicatePad(X,[Ny-1 0]);
        Y = ReplicatePad(Y',[0 Nx-1]);
        
%         if Data.stereo=='1'
%             if davis.IType==5
%                 error('VC7 FILES ARE STILL IN 3C FORMAT! CONVERT TO 2x2D')
%             else
%             %Position (pixel)
%                 if Data.vcNum=='1'
%                     U=double(davis.Data(:,1:davis.Ny))';
%                     V=double(davis.Data(:,davis.Ny+1:2*davis.Ny))';
%                 else
%                     U=double(davis.Data(:,2*davis.Ny+1:3*davis.Ny))';      
%                     V=double(davis.Data(:,3*davis.Ny+1:4*davis.Ny))';
%                 end
%             end
%             %NEED TO UPDATE DaVis later to see if these are outputs.
%             Eval=zeros(size(U));
%             Eval(U==0)=1;
%             U_vt=zeros(size(U));
%             U_vta=zeros(size(U));
%             U_vtb=zeros(size(U));
%             Qratio=zeros(size(U));
%             EvalQ=zeros(size(U));
%             
%         elseif Data.stereo=='0'
%             if davis.IType==5
%                 error('VC7 FILES ARE STILL IN 3C FORMAT! CONVERT TO 2x2D')
%             elseif davis.IType==2
%                 if size(davis.Data,2)/davis.Ny>2
%                 error('VC7 FILES ARE 3C CONVERTED TO 2x2D. CHECK BOX: Using split VC7 file in GUI.')
%                 else
%                     U=double(davis.Data(1:davis.Nx,1:davis.Ny))';
%                     V=double(davis.Data(:,davis.Ny+1:2*davis.Ny))'; 
%                     Eval=zeros(size(U));
%                     Eval(U==0)=1;
%                     U_vt=zeros(size(U));
%                     U_vta=zeros(size(U));
%                     U_vtb=zeros(size(U));
%                     Qratio=zeros(size(U));
%                     EvalQ=zeros(size(U));
%                 end
%             elseif davis.IType==3

%             VC7=readimx([path_name filename]);

            U0=davis.Frames{1}.Components{1}.Planes{1}';
            V0=davis.Frames{1}.Components{2}.Planes{1}';
            U1=davis.Frames{1}.Components{3}.Planes{1}';
            V1=davis.Frames{1}.Components{4}.Planes{1}';
            U2=davis.Frames{1}.Components{5}.Planes{1}';
            V2=davis.Frames{1}.Components{6}.Planes{1}';
            U3=davis.Frames{1}.Components{7}.Planes{1}';
            V3=davis.Frames{1}.Components{8}.Planes{1}';
            Active_Choice=davis.Frames{1}.Components{9}.Planes{1}';

            AC0=zeros(size(U0));  AC1=AC0;  AC2=AC0;  AC3=AC0;
            AC0(Active_Choice==0)=1;
            AC1(Active_Choice==1)=1;
            AC2(Active_Choice==2)=1;
            AC3(Active_Choice==4)=1;

            U=U0.*AC0 + U1.*AC1 + U2.*AC2 + U3.*AC3;
            V=V0.*AC0 + V1.*AC1 + V2.*AC2 + V3.*AC3;
           
            Eval=double(davis.Frames{1}.Components{10}.Planes{1}');
            U= double(U.*Eval);
            V= double(V.*Eval);
%                 Eval=Eval-1;
%                 Eval(Eval>=0)=0;
%                 Eval(Eval==-1)=1;
% 
%             end
%         end
%           
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     elseif strcmp(Data.dataext,'mat')==1 || strcmp(Data.dataext,'MAT')==1
%         %PRANA Results: MATLAB MAT files (includes Eval matrix)
%         load([database sprintf(['%0.' Data.datazeros 'i.' Data.dataext],I1data(q))])
%         U=U(:,:,1);V=V(:,:,1);Eval=Eval(:,:,1);
% 
%         %Retrieve Peak Ratio and Calculate PT Uncertainty
%         if  exist('C','var')
%             if size(C,3)==3
%             Qratio=C(:,:,2)./C(:,:,3);
%             evalQ=Eval;
%             evalQ(evalQ==0)=-100;
%             evalQ(evalQ>-100)=0;
%             evalQ(evalQ==-100)=1;
%             EvalQ=evalQ;                        
%             
%             %VT Uncertainty [pix], This is Virginia Tech's method and can
%             %be ignored.
%             U_vta = 0.9787*Qratio.^(-1.0);
%             U_vtb = 0.9787*Qratio.^(-1.0);
%             U_vt = sqrt((5.381*exp(-0.5*((Qratio-1)/0.3369).^2)).^2 + (0.8706*Qratio.^-1).^2 + (2.221e-14)^2);
%             else
%                 fprintf('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n')
%                 fprintf('Output required to calculate VT uncertainty not available.\n')
%                 fprintf('In Prana, make sure you have selected to save additional\n')
%                 fprintf('peak information for Peaks 1,2 and check Peak Magnitude\n')
%                 fprintf('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n')
%             evalQ=Eval;
%             evalQ(evalQ==0)=-100;
%             evalQ(evalQ>-100)=0;
%             evalQ(evalQ==-100)=1;
%             EvalQ=evalQ;                        
%             Qratio=NaN; U_vta = NaN; U_vtb = NaN; U_vt = NaN;
%             end
%         end
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    else %All other files
        %Based on format:
            %TITLE = "B00002"
            % VARIABLES = "x", "y", "Vx", "Vy"
            % ZONE T="Frame 0", I=128, J=128
            % 8.000000 8.000000 1.500675 -0.172555
            % 24.000000 8.000000 0.328738 1.074726
            % 40.000000 8.000000 0.000000 0.000000
            % 56.000000 8.000000 0.629379 -0.249157
        
        %This does not seem to work for latest PRANA output    
        D=importdata([database sprintf(['%0.' Data.datazeros 'i.' Data.dataext],I1data(q))], ' ');%,nheaderlines);
        X=D.data(:,1);
        Y=D.data(:,2);
        U=D.data(:,3);
        V=D.data(:,4);
        Eval =zeros(size(U));
        %Only works if bad vectors are not replaced.
        Eval(U==0)=1;
        [X,Y,U,V,Eval]=matrixform(X,Y,U,V,Eval);
        U = double(flipud(U));
        V = double(flipud(V));                
    end
    
    % Remove NaN from Prana (DaVis should already have this done)
    U(isnan(U))=0;
    V(isnan(V))=0;
    
% Convert from physical units to pixles (if able)
    if X(1,1)-floor(X(1,1))==0 %(0 if pixel units, decimal if physical units)
        % Results are in pixel untis, Correct
    else
        %If im7/vc7 files are used/convert to pixels based on data.
        if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
            %Same values entered as found in Files
            if floor(Davis.ScaleX(1)*1000)==floor(Mag)
                % Results are in physical units, convert to Pixel Units
                X= (X-Davis.ScaleX(2))/Davis.ScaleX(1);
                Y= (Y-Davis.ScaleY(2))/Davis.ScaleY(1);  
                if isnan(dt)||dt==0
                   error('Images or PIV results are not in pixel units. No calibration information avaliable in DaVis files.')
                end
                if Data.stereo=='0'    
                U= U/Mag*dt;
                V= V/Mag*dt;
                end
            else
                error('Error:PIVuncertaintyCode','!!!!! Images or PIV results are not in pixel units!!!!!\n(Values entered for conversion to pixel units do not match those found in the im7 files.') 
            end
        else
        error('!!!!! Images or PIV results are not in pixel units !!!!!')    
        end
        
        %Check conversion to pixel units
        if X(1,1)-floor(X(1,1))~=0
            if X(1,1)-floor(X(1,1))<=0.001
               X=floor(X);
               Y=floor(Y);
            else
            error('!!!!! Conversion to pixel units failed !!!!!')
            end
        end
    end
    bcalc = cputime-tf;
    
    % Calculate Particle Diameter and Density (sub-function "FindDia")
    tc=cputime;
    [P_Dia, P_Dens, DiaRatio, P_DiaX, P_DiaY, VecSpace,PkD, PkD2] = DiaDen2(im2,Data,X,Y);
    
    %Time to calculate particle diameter/density
    tcalc = cputime-tc;
    t1=cputime;
    
    %Average bad vectors for shear calculation (sub-function "Fill")
    [U2,V2]=Fill(U,V,Eval,10);
   
    %Compute shear and uncertainty values
    [dudy,dvdx,Bias_k,R_low,R_high,RR_95,Mark,MarkEval] = compute_bias(P_Dens,P_DiaX,P_DiaY,X,Y,U2,V2,Usurf,Eval);
    
    %Time to compute shear and bias
    corrtime=cputime-t1;

    %Write Results (sub-function "write_plt_val")
    t2=cputime;
    time=(q-1);%/Freq;
    
    %Convert to physical units if applicable
    if Mag==1 && dt==1 || Mag==1000 %Values are to remain in pixels     
    else     
        %If vc7 are used and the value read into um/pix matches Davis.ScaleX
        if strcmp(Data.dataext,'vc7')==1 || strcmp(Data.dataext,'VC7')==1 && floor(davis.ScaleX(1)*1000)==floor(Mag)
            X=X*davis.ScaleX(1)+davis.ScaleX(2); 
            Y=Y*davis.ScaleY(1)+davis.ScaleY(2);
        else
            %Otherwise, scales dont match, use entered scale
            X=X*Mag/1000; %[mm]
            Y=Y*Mag/1000; %[mm];
        end
        U=U*Mag/dt;
        V=V*(-Mag)/dt;
        Bias_k=Bias_k*Mag/dt;
        R_low=R_low*Mag/dt;
        R_high=R_high*Mag/dt;
        RR_95=RR_95*(Mag/dt)^2;
    end
      
    %Output data in *.plt file
    if Data.pltout == '1'
        if Data.fileend==0
%             write_plt_val([pltdirec char(Data.outbase) sprintf(['%0.' Data.imzeros 'i.plt' ],Data.fileend+I1data(q))],X,Y,U,V,Eval,Qratio,EvalQ,Bias_k,R_low,R_high,RR_95,dudy,dvdx,P_Dia,P_Dens,DiaRatio,time,title);
            write_plt_val2([pltdirec char(Data.outbase) sprintf(['%0.' Data.imzeros 'i.plt' ],Data.fileend+I1data(q))],X,Y,U,V,Eval,Bias_k,R_low,R_high,title2);

        else
            write_plt_val([pltdirec char(Data.outbase) sprintf(['%0.' Data.imzeros 'i.plt' ],Data.fileend+(str2double(Data.datafstep)-1)+I1data(q))],X,Y,U,V,Eval,Qratio,EvalQ,Bias_k,R_low,R_high,RR_95,dudy,dvdx,P_Dia,P_Dens,DiaRatio,time,title);
        end
    end
    %Output data in*.mat file
    if Data.matout == '1'
        if Data.fileend==0
        save([pltdirec Data.outbase sprintf(['%0.' Data.imzeros 'i.mat'],Data.fileend+I1data(q))],'X','Y','U','V','Eval','Bias_k','R_low','R_high','RR_95','dudy','dvdx','P_Dia','P_Dens','DiaRatio','VecSpace');
        else   
        save([pltdirec Data.outbase sprintf(['%0.' Data.imzeros 'i.mat'],Data.fileend+(str2double(Data.datafstep)-1)+I1data(q))],'X','Y','U','V','Eval','Bias_k','R_low','R_high','RR_95','dudy','dvdx','P_Dia','P_Dens','DiaRatio','VecSpace');
        end
    end
    %compute save time
    savetime=cputime-t2;
    %compute total frame time
    eltime=cputime-tf;
    
    %output status
    fprintf('\n----------------------------------------------------\n')
    fprintf(['Job: ',Data.batchname,'\n'])
    fprintf([title ' Completed (' num2str(q) '/' num2str(length(I1)) ')\n'])
    fprintf('----------------------------------------------------\n')
    fprintf('load data...                     %0.2i:%0.2i.%0.0f\n',floor(bcalc/60),floor(rem(bcalc,60)),rem(bcalc,60)-floor(rem(bcalc,60)))
    fprintf('dia/dens calculation...          %0.2i:%0.2i.%0.0f\n',floor(tcalc/60),floor(rem(tcalc,60)),rem(tcalc,60)-floor(rem(tcalc,60)))
    fprintf('bias calculation...              %0.2i:%0.2i.%0.0f\n',floor(corrtime/60),floor(rem(corrtime,60)),rem(corrtime,60)-floor(rem(corrtime,60)))
    fprintf('save time...                     %0.2i:%0.2i.%0.0f\n',floor(savetime/60),floor(rem(savetime,60)),rem(savetime,60)-floor(rem(savetime,60)))   
    fprintf('total frame time...              %0.2i:%0.2i.%0.0f\n',floor(eltime/60),floor(rem(eltime,60)),rem(eltime,60)-floor(rem(eltime,60)))
    %estimate job completion time
    frametime(q)=eltime;
    comptime=mean(frametime(frametime~=0))*(length(I1)-q);
    fprintf('estimated job completion time... %0.2i:%0.2i:%0.2i\n\n',floor(comptime/3600),floor(rem(comptime,3600)/60),floor(rem(comptime,60)))
end
        
        
%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           END MAIN FUNCTION                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% VECTOR TO MATRIX SUBFUNCTION
function [X,Y,U,V,Eval]=matrixform(x,y,u,v,eval)
%If PIV outputs position/velocity values in vector, convert to matrix form

%find unique x and y grid points
a=sort(unique(x));
b=sort(unique(y));
N=length(x);

%initialize matrices
U=nan(length(b),length(a));
V=nan(length(b),length(a));
Eval=-1*ones(length(b),length(a));

%generate grid matrix
[X,Y]=meshgrid(a,b);

%generate variable matrices (nans where no data available)
for n=1:N
    I=find(b==y(n));
    J=find(a==x(n));
    U(I,J) = u(n);
    V(I,J) = v(n);
    Eval(I,J) = eval(n);
end


%% PLT WRITER SUBFUNCTION (Needed sub-function: matrixform)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function []=write_plt_val(fname,X,Y,U,V,Eval,Qratio, EvalQ,Bias_k,R_low,R_high,RR_95,dudy,dvdx,P_Dia,P_Dens,DiaRatio,T,title)

Ur_minus = (Bias_k.*Bias_k+R_low.*R_low).^0.5;
Ur_plus = (Bias_k.*Bias_k+R_high.*R_high).^0.5;

%remove nans from data, replace with zeros
U(Eval<0)=0;
V(Eval<0)=0;

%find I,J for plt
S = size(U);

%generate text file
fid = fopen(fname,'w');
if fid==-1
    error(['error creating file ',fname])
end

%header lines
fprintf(fid,['TITLE        = "' title '"\n']);
fprintf(fid,'VARIABLES    = "X", "Y", "U", "V", "Mag (m/s)", "Eval", "Qratio", "EvalQ", "b_k_x", "b_k_y","b_rr_x", "b_rr_y", "r_low_x", "r_low_y", "r_high_x", "r_high_y", "U_r-_x", "U_r-_y", "U_r+_x", "U_r+_y", "Error Bar Width (m/s)", "du/dy", "dv/dx", "Diameter", "Density", "DiaRatio"\n');
fprintf(fid,'ZONE T="Time=%0.6f" I=%i J=%i C=BLACK\n',T,S(2),S(1));

%write data
for i=1:S(1)
    for j=1:S(2)
        if isnan(U(i,j)) || isnan(V(i,j))
            %second check to ensure no nans present
            fprintf(fid,'%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n',...
                X(i,j),Y(i,j),0,0,0,-1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
        else
            %valid data points
            fprintf(fid,'%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n',...
                X(i,j),Y(i,j),U(i,j),V(i,j),sqrt(U(i,j)^2+V(i,j)^2),Eval(i,j),Qratio(i,j), EvalQ(i,j),Bias_k(i,j,1),Bias_k(i,j,2),RR_95(i,j,1),RR_95(i,j,2),R_low(i,j,1),R_low(i,j,2),R_high(i,j,1),R_high(i,j,2),Ur_minus(i,j,1),Ur_minus(i,j,2),Ur_plus(i,j,1),Ur_plus(i,j,2),sqrt(Ur_minus(i,j,1)^2+Ur_minus(i,j,2)^2)+sqrt(Ur_plus(i,j,1)^2+Ur_plus(i,j,2)^2),dudy(i,j),dvdx(i,j),P_Dia(i,j),P_Dens(i,j),DiaRatio(i,j));
        end
    end
end
fclose(fid);

%% PLT WRITER SUBFUNCTION (Needed sub-function: matrixform)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function []=write_plt_val2(fname,X,Y,U,V,Eval,Bias_k,R_low,R_high,title)

Ur_minus = (Bias_k.*Bias_k+R_low.*R_low).^0.5;
Ur_plus = (Bias_k.*Bias_k+R_high.*R_high).^0.5;
Ur_minus(Ur_minus<0.012)=0.012;
Ur_plus(Ur_plus<0.012)=0.012;
Ux_minus=Ur_minus(:,:,1);
Uy_minus=Ur_minus(:,:,2);
Ux_plus=Ur_plus(:,:,1);
Uy_plus=Ur_plus(:,:,2);

Ux_minus(U==0)=0;
Uy_minus(U==0)=0;
Ux_plus(U==0)=0;
Uy_plus(U==0)=0;

%remove nans from data, replace with zeros
U(Eval<0)=0;
V(Eval<0)=0;

%find I,J for plt
S = size(U);

%generate text file
fid = fopen(fname,'w');
if fid==-1
    error(['error creating file ',fname])
end

%header lines
fprintf(fid,['TITLE        = "' title '"\n']);
fprintf(fid,'VARIABLES    = "x", "y", "u", "v", "ex-", "ey-", "ex+", "ey+"\n');
fprintf(fid,['ZONE T="' title '" I=%i J=%i\n'],S(2),S(1));

%write data
for i=1:S(1)
    for j=1:S(2)
        if isnan(U(i,j)) || isnan(V(i,j))
            %second check to ensure no nans present
            fprintf(fid,'%4.0f %4.0f %8.6f %8.6f %8.6f %8.6f %8.6f %8.6f\n',...
                X(i,j),Y(i,j),0,0,0,0,0,0,0,0);
        else
            %valid data points
            fprintf(fid,'%4.0f %4.0f %8.6f %8.6f %8.6f %8.6f %8.6f %8.6f\n',...
                X(i,j),Y(i,j),U(i,j),V(i,j),Ux_minus(i,j),Uy_minus(i,j),Ux_plus(i,j),Uy_plus(i,j));
        end
    end
end
fclose(fid);

%% Bias Estimation  (Needed sub-functions: tps, derivative_tps, matrixform)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [u_shear,v_shear,B_k,R_l,R_u,RR_95,Mark,MarkEval] = compute_bias(dens,diaX,diaY,x,y,u,v,Usurf,Eval)

 %OLD CODE (FOR LOOPS)   
%%Loop: estimates Shear
%     for i = 1:size(x,2)
%         for j = 1:size(x,1)
%             %Generate grid to feed into shear calculation.
%             grid_size = 3;              
%             if (i == 1 && j == 1) %Top left Corner
%                 X = x(1:grid_size,1:grid_size);
%                 Y = y(1:grid_size,1:grid_size);
%                 U = u(1:grid_size,1:grid_size);
%                 V = v(1:grid_size,1:grid_size);
%             elseif (i == 1 && j == size(x,1))%Top right corner
%                 X = x(size(x,1)-grid_size+1:size(x,1),1:grid_size);
%                 Y = y(size(x,1)-grid_size+1:size(x,1),1:grid_size);
%                 U = u(size(x,1)-grid_size+1:size(x,1),1:grid_size);
%                 V = v(size(x,1)-grid_size+1:size(x,1),1:grid_size);
%             elseif (i == size(x,2) && j == 1) %Bottom left corner
%                 X = x(1:grid_size,size(x,2)-grid_size+1:size(x,2));
%                 Y = y(1:grid_size,size(x,2)-grid_size+1:size(x,2));
%                 U = u(1:grid_size,size(x,2)-grid_size+1:size(x,2));
%                 V = v(1:grid_size,size(x,2)-grid_size+1:size(x,2));
%             elseif (i == size(x,2) && j == size(x,1)) %Bottom right corner
%                 X = x(size(x,1)-grid_size+1:size(x,1),size(x,2)-grid_size+1:size(x,2));
%                 Y = y(size(x,1)-grid_size+1:size(x,1),size(x,2)-grid_size+1:size(x,2));
%                 U = u(size(x,1)-grid_size+1:size(x,1),size(x,2)-grid_size+1:size(x,2));
%                 V = v(size(x,1)-grid_size+1:size(x,1),size(x,2)-grid_size+1:size(x,2));
%             elseif (i == 1 && j ~= 1 && j ~= size(x,1)) %Top edge
%                 X = x(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,1:grid_size);
%                 Y = y(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,1:grid_size);
%                 U = u(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,1:grid_size);
%                 V = v(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,1:grid_size);
%             elseif (j == 1 && i ~= 1 && i ~= size(x,2)) %Left Edge
%                 X = x(1:grid_size,i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 Y = y(1:grid_size,i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 U = u(1:grid_size,i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 V = v(1:grid_size,i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%             elseif (i == size(x,2) && j ~= 1 && j ~= size(x,1)) %Bottom edge
%                 X = x(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,size(x,2)-grid_size+1:size(x,2));
%                 Y = y(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,size(x,2)-grid_size+1:size(x,2));
%                 U = u(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,size(x,2)-grid_size+1:size(x,2));
%                 V = v(j-(grid_size-1)*0.5:j+(grid_size-1)*0.5,size(x,2)-grid_size+1:size(x,2));
%             elseif (j == size(x,1) && i ~= 1 && i ~= size(x,2)) %Right Edge
%                 X = x(size(x,1)-grid_size+1:size(x,1),i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 Y = y(size(x,1)-grid_size+1:size(x,1),i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 U = u(size(x,1)-grid_size+1:size(x,1),i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%                 V = v(size(x,1)-grid_size+1:size(x,1),i-(grid_size-1)*0.5:i+(grid_size-1)*0.5);
%             else %Internal
%                 grid_size = (grid_size-1)*0.5;
%                 X = x(j-grid_size:j+grid_size,i-grid_size:i+grid_size);
%                 Y = y(j-grid_size:j+grid_size,i-grid_size:i+grid_size);
%                 U = u(j-grid_size:j+grid_size,i-grid_size:i+grid_size);
%                 V = v(j-grid_size:j+grid_size,i-grid_size:i+grid_size);
%             end
% 
%             %Estimate the Gradient using TPS (sub-functions "tps" and "derivative_tps")
%             [X,Y,coef_matu,coef_matv,~,~]=tps(X(:),Y(:),U(:),V(:));
%             [~,dudy,dvdx,~]=derivative_tps(i,j,X,Y,coef_matu,coef_matv);
%             u_shear0(j,i) = dudy;
%             v_shear0(j,i) = dvdx;
%         end
%     end   
%     u_shear0=u_shear0.*Eval;v_shear0=v_shear0.*Eval;
% clearvars -except u_shear0 v_shear0 dens diaX diaY x y u v Usurf Eval    


w = 1; %interrogation for shear is (1+w+w)x(1+w+w)
[i,~,j] = meshgrid(1:size(x,2),1,1:size(x,1));
ipos=1:size(x,2);
jpos=1:size(x,1);

%Particle size interrogation (out of bounds set to bounds)
imin = max(ipos-w,1);
imax = min(ipos+w,size(x,2));
jmin = max(jpos-w,1);
jmax = min(jpos+w,size(x,1));

%Make edge particles size bigger to make the same size as internal particles
imax(imin==1)=1+w+w;
imin(imax==size(x,2))=size(x,2)-w-w;
jmax(jmin==1)=1+w+w;
jmin(jmax==size(x,1))=size(x,1)-w-w; 

u=permute(u',[3 1 2]);
v=permute(v',[3 1 2]);
x=permute(x',[3 1 2]);
y=permute(y',[3 1 2]);

U=[u(1,imin,jmin);u(1,imin,jmin+1);u(1,imin,jmin+2);u(1,imin+1,jmin);u(1,imin+1,jmin+1);u(1,imin+1,jmin+2);u(1,imin+2,jmin);u(1,imin+2,jmin+1);u(1,imin+2,jmin+2);];
V=[v(1,imin,jmin);v(1,imin,jmin+1);v(1,imin,jmin+2);v(1,imin+1,jmin);v(1,imin+1,jmin+1);v(1,imin+1,jmin+2);v(1,imin+2,jmin);v(1,imin+2,jmin+1);v(1,imin+2,jmin+2);];
X=[x(1,imin,jmin);x(1,imin,jmin+1);x(1,imin,jmin+2);x(1,imin+1,jmin);x(1,imin+1,jmin+1);x(1,imin+1,jmin+2);x(1,imin+2,jmin);x(1,imin+2,jmin+1);x(1,imin+2,jmin+2);];
Y=[y(1,imin,jmin);y(1,imin,jmin+1);y(1,imin,jmin+2);y(1,imin+1,jmin);y(1,imin+1,jmin+1);y(1,imin+1,jmin+2);y(1,imin+2,jmin);y(1,imin+2,jmin+1);y(1,imin+2,jmin+2);];

[coef_matu,coef_matv]=tps2(X,Y,U,V);
[~,u_shear,v_shear,~]=derivative_tps2(i,j,X,Y,coef_matu,coef_matv,Eval);

clear x y imin imax jmin jmax ipos jpos w X Y U V coef_matu coef_matv i j

u=(permute(u,[2 3 1])').*Eval;
v=(permute(v,[2 3 1])').*Eval;

%Pre-allocate Arrays
B_k     = zeros([size(u),2])+0.5;
R_l     = zeros([size(u),2])+0.5;
R_u     = zeros([size(u),2])+0.5;
RR_95   = zeros([size(u),2]);

%Load Uncertainty Surface Data
s=load(Usurf);

%"Mark" AN OUTPUT, SIMILAR TO "Eval", shows values that were cropped
Mark = int8(zeros(size(diaX)));
Mark(dens>max(s.density(:))) = 1;
Mark(dens<min(s.density(:))) = -1;
Mark(diaX>max(s.diameter(:))) = 2;
Mark(diaX<min(s.diameter(:))) = -2;
Mark(diaY>max(s.diameter(:))) = 2;
Mark(diaY<min(s.diameter(:))) = -2;
Mark(abs(u_shear)>max(s.shear(:))) = 3;
Mark(abs(u_shear)<min(s.shear(:))) = -3;
Mark(abs(v_shear)>max(s.shear(:))) = 3;
Mark(abs(v_shear)<min(s.shear(:))) = -3;

MarkEval=Mark;
MarkEval(MarkEval~=0)=-1;
MarkEval=MarkEval+1;

    %Locate/mark values outside uncertainty surface (limits pulled from
    %uncertainty surface are used.)
    dens(dens>max(s.density(:)))=max(s.density(:)); 
    dens(dens<min(s.density(:)))=min(s.density(:));
    diaX(diaX>max(s.diameter(:)))=max(s.diameter(:));
    diaX(diaX<min(s.diameter(:)))=min(s.diameter(:)); 
    diaY(diaY>max(s.diameter(:)))=max(s.diameter(:));
    diaY(diaY<min(s.diameter(:)))=min(s.diameter(:));
    u_shear(abs(u_shear)>max(s.shear(:)))=max(s.shear(:));
    u_shear(abs(u_shear)<min(s.shear(:)))=min(s.shear(:));
    v_shear(abs(v_shear)>max(s.shear(:)))=max(s.shear(:));
    v_shear(abs(v_shear)<min(s.shear(:)))=min(s.shear(:));

    %Sub-pixel displacement (no assumed symmetry: values used 0±1.0)
    dx = abs(u-fix(u));
    dy = abs(v-fix(v));
fo=ones(size(dx))*0;
    %Interpolate Bias Uncertainties from Uncertainty surface
    %Calculate the bias uncertainty (u_bar-u_true)
    B_k(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.b_k,dens,diaX,abs(u_shear),dx,fo,'linear',0.5);
    B_k(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.b_k,dens,diaY,abs(v_shear),dy,fo,'linear',0.5);
    %Calculate the r_low uncertainty (u_bar-x_2.5%)
    R_l(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.r_low,dens,diaX,abs(u_shear),dx,fo,'linear',0.5);
    R_l(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.r_low,dens,diaY,abs(v_shear),dy,fo,'linear',0.5);
    %Calculate the r_high uncertainty (x_97.5%-u_bar)
    R_u(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.r_high,dens,diaX,abs(u_shear),dx,fo,'linear',0.5);
    R_u(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.r_high,dens,diaY,abs(v_shear),dy,fo,'linear',0.5); 
    %Calculate the rr uncertainty for the fluctuations (x_95%-u_bar) [introduced by Brandon]
    RR_95(:,:,1) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.rr95,dens,diaX,abs(u_shear),dx,fo,'linear',0.5);
    RR_95(:,:,2) = interpn(s.density,s.diameter,s.shear,s.displacement,s.Fo,s.rr95,dens,diaY,abs(v_shear),dy,fo,'linear',0.5); 


%% TPS                                 (Used by sub-function: compute_bias)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [X,Y,coef_matu,coef_matv,U,V]=tps(X,Y,U,V)
%         P = zeros(length(X),3);
%         for i=1:length(X)
%             P(i,:)=[1,X(i),Y(i)];
%         end
%         B = zeros(length(X),length(X));
%         for i=1:length(X)
%             for j=1:length(X)% index for columns
%                dummy=((X(i)-X(j))^2 +(Y(i)-Y(j) )^2);
%                 B(i,j)=dummy*log(sqrt(dummy+eps));
%             end
%         end
%   G=[B,P;
%      transpose(P),zeros(size(P,2),size(P,2))];
%   coef_matu=G\[U;zeros(size(P,2),1)];
%   coef_matv=G\[V;zeros(size(P,2),1)];

function [coef_matu,coef_matv]=tps2(X,Y,U,V)
P = ones([size(X,1),3,size(X,2),size(X,3)]);
P(:,2,:,:)=X;
P(:,3,:,:)=Y;

dummy=[0,1,4,1,2,5,4,5,8;1,0,1,2,1,2,5,4,5;4,1,0,5,2,1,8,5,4;1,2,5,0,1,4,1,2,5;2,1,2,1,0,1,2,1,2;5,2,1,4,1,0,5,2,1;4,5,8,1,2,5,0,1,4;5,4,5,2,1,2,1,0,1;8,5,4,5,2,1,4,1,0]*(Y(2,1,1)-Y(1,1,1))^2;
B=dummy.*log(sqrt(dummy+eps));
B=repmat(permute(B,[1 2 3 4]),[1 1 size(U,2) size(U,3) ]);

G=  [B,P;
    permute(P,[2 1 3 4]),zeros(size(P,2),size(P,2),size(U,2),size(U,3))];

tempU=[U;zeros(size(P,2), size(P,3),size(P,4))];
tempV=[V;zeros(size(P,2), size(P,3),size(P,4))];

coef_matu=zeros(size(tempU));
coef_matv=coef_matu;
%Takes the most time:
for i=1:size(X,2)
    for j=1:size(X,3)
    coef_matu(:,i,j)=G(:,:,i,j)\tempU(:,i,j);
    coef_matv(:,i,j)=G(:,:,i,j)\tempV(:,i,j);
    end
end


%% Derivative TPS                      (Used by sub-function: compute_bias)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [dudx,dudy,dvdx,dvdy]=derivative_tps(x,y,X,Y,coef_matu,coef_matv)
% % Functions should be called after computing the coefficient matrices 
% % Compute the gradients at x, y for the interpolation over X, Y 
% 
% dudx_interp=0;dvdx_interp=0;dudy_interp=0;dvdy_interp=0;
% 
% for k=1:length(X);
%     dummy=((x-X(k))^2 +(y-Y(k) )^2);
%     dummy1=(1+log(sqrt(dummy+eps)));
%     dudx_interp=(x-X(k))*dummy1*coef_matu(k)+dudx_interp;
%     dudy_interp=(y-Y(k))*dummy1*coef_matu(k)+dudy_interp;
%     dvdx_interp=(x-X(k))*dummy1*coef_matv(k)+dvdx_interp;
%     dvdy_interp=(y-Y(k))*dummy1*coef_matv(k)+dvdy_interp;
% end
% 
% Pddx=[0,1,0];Pddy=[0,0,1];%,x^2,x*y,y^2];%,x^2,x*y,y^2];
% dudx=dudx_interp+Pddx*coef_matu(end-size(Pddx,2)+1:end);
% dvdx=dvdx_interp+Pddx*coef_matv(end-size(Pddx,2)+1:end);  
% dudy=dudy_interp+Pddy*coef_matu(end-size(Pddy,2)+1:end);  
% dvdy=dvdy_interp+Pddy*coef_matv(end-size(Pddy,2)+1:end);  

function [dudx,dudy,dvdx,dvdy]=derivative_tps2(x,y,X,Y,coef_matu,coef_matv,Eval)
% Functions should be called after computing the coefficient matrices 
% Compute the gradients at x, y for the interpolation over X, Y 
x=permute(x,[2 3 1]);
y=permute(y,[2 3 1]);
X=permute(X,[2 3 1]);
Y=permute(Y,[2 3 1]);
coef_matu=permute(coef_matu,[2 3 1]);
coef_matv=permute(coef_matv,[2 3 1]);

dummy=bsxfun(@minus,x,X).^2+bsxfun(@minus,y,Y).^2;
dummy1=(1+log(sqrt(dummy+eps)));
dudx_interp=sum(bsxfun(@minus,x,X).*dummy1.*coef_matu(:,:,1:size(X,3)),3);
dudy_interp=sum(bsxfun(@minus,y,Y).*dummy1.*coef_matu(:,:,1:size(X,3)),3);
dvdx_interp=sum(bsxfun(@minus,x,X).*dummy1.*coef_matv(:,:,1:size(X,3)),3);
dvdy_interp=sum(bsxfun(@minus,y,Y).*dummy1.*coef_matv(:,:,1:size(X,3)),3);
    
dudx=((dudx_interp+coef_matu(:,:,end-1))').*Eval;
dvdx=((dvdx_interp+coef_matv(:,:,end-1))').*Eval;
dudy=((dudy_interp+coef_matu(:,:,end))').*Eval;
dvdy=((dvdy_interp+coef_matv(:,:,end))').*Eval;

%% Average Scheme
 function [u,v]=Fill(u,v,eval,maxct)

%find unique x and y grid points
i=size(u,1);
j=size(u,2);

%Keep originals
U=u.*eval;
V=v.*eval;


for k=1:maxct
    %Average all 4 neighbors
    I=zeros([size(u) 4]);
    I(:,2:j,1)=u(:,1:j-1);
    I(:,1:j-1,2)=u(:,2:j);
    I(2:i,:,3)=u(1:i-1,:);
    I(1:i-1,:,4)=u(2:i,:);
    Temp=sum(I,3)./4;
    %Remove all original U
    Temp=Temp.*(eval-1)*(-1);
    %Fill up touching original U
    u=U+Temp;
    
    %Average all 4 neighbors
    I=zeros([size(v) 4]);
    I(:,2:j,1)=v(:,1:j-1);
    I(:,1:j-1,2)=v(:,2:j);
    I(2:i,:,3)=v(1:i-1,:);
    I(1:i-1,:,4)=v(2:i,:);
    Temp=sum(I,3)./4;
    %Remove all original V
    Temp=Temp.*(eval-1)*(-1);
    %Fill up touching original V
    v=V+Temp;
end

%%
% diff=1; 
% uNew=u;
% vNew=v;
% ct=0;
% while   ct<maxct && diff > 10^-3
%     ct=ct+1;
%     uOld=uNew;
%     vOld=vNew;
%     for i = 1:b
%         for j = 1:a
%             if eval(i,j)~=1
%                  % North Face (u = 0)
%                  if i==1    
%                      %Northwest Corner
%                      if j==1         
%                         u(i,j)=(u(i+1,j)+u(i,j+1))/2;
%                         v(i,j)=(v(i+1,j)+v(i,j+1))/2;
%                      %North East Corner
%                      elseif j==a    
%                         u(i,j)=(u(i+1,j)+u(i,j-1))/2;
%                         v(i,j)=(v(i+1,j)+v(i,j-1))/2;
%                      %North Face (non-corner)
%                      else
%                         u(i,j)=(u(i+1,j)+u(i,j+1)+u(i,j-1))/3;
%                         v(i,j)=(v(i+1,j)+v(i,j+1)+v(i,j-1))/3;
%                      end
%                  % South Face (u = 0)
%                  elseif i==b 
%                      %Southwest Corner
%                      if j==1    
%                         u(i,j)=(u(i-1,j)+u(i,j+1))/2;
%                         v(i,j)=(v(i-1,j)+v(i,j+1))/2;
%                      %Southeast Corner
%                      elseif j==a
%                         u(i,j)=(u(i-1,j)+u(i,j-1))/2;
%                         v(i,j)=(v(i-1,j)+v(i,j-1))/2;
%                      %South Face (non-corner)
%                      else
%                         u(i,j)=(u(i-1,j)+u(i,j-1)+u(i,j+1))/3;
%                         v(i,j)=(v(i-1,j)+v(i,j-1)+v(i,j+1))/3;
%                      end    
%                  else
%                      %West Face
%                      if j==1
%                          u(i,j)=(u(i+1,j)+u(i-1,j)+u(i,j+1))/3;
%                          v(i,j)=(v(i+1,j)+v(i-1,j)+v(i,j+1))/3;
%                      %East Face
%                      elseif j==a
%                          u(i,j)=(u(i+1,j)+u(i-1,j)+u(i,j-1))/3;
%                          v(i,j)=(v(i+1,j)+v(i-1,j)+v(i,j-1))/3;
%                      else
%                          u(i,j)=(u(i+1,j)+u(i-1,j)+u(i,j-1)+u(i,j+1))/4;
%                          v(i,j)=(v(i+1,j)+v(i-1,j)+v(i,j-1)+v(i,j+1))/4;
%                      end
%                  end
%             end
%         end
%     end
%     %Estimate of T0
%     uNew=u;    
%     vNew=u;
%     %keep iterating until residual < 10^-6
%     diffu=abs(uNew-uOld);
%     diffv=abs(vNew-vOld);
%     diff=max(max([diffu diffv]));
% end           



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pad Array                                                               %
%   This function replicates the matrix 'a' so that it becomes size       %
%   'padSize'                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                             
function b = ReplicatePad(a, padSize)
numDims = numel(padSize);
idx   = cell(1,numDims);
for k = 1:numDims
  M = size(a,k);
  p = padSize(k);
  onesVector = ones(1,p);
  idx{k}   = [1:M M*onesVector];
end
 b = a(idx{:});

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             END OF CODE                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}