%LAST UPDATED: 10/29/2014

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   This USU Synthetic Image Generator (USU-SIG) produces images for  %%%
%%%   generating uncertainty surfaces for the uncertainty surface       %%%
%%%   method. This code also generates the file 'Key.m' which is        %%%
%%%   required to generate the surface after the images have been       %%%
%%%   processed. If the key is accidentally deleted, you can run the    %%%
%%%   code up to the point where the Key is saved (~line 160) then stop %%%
%%%                                                                     %%%
%%%   While this code is able to model various flows, only the linear   %%%
%%%   velocity gradient is used in generating the uncertainty images.   %%%
%%%   Unless you know what you are doing, it is recommended that you    %%%
%%%   do not make changes after the text:                               %%%
%%%         "USERS: DO NOT MAKE CHANGES BEYOND THIS POINT!"             %%%
%%%                                                                     %%%
%%%   USER DEFINED PARAMETERS:                                          %%%
%%%   ==============================================================    %%%
%%%     par - Run code in parallel? (yes = '1', no = '0')               %%%
%%%     parprocessors- Number of processors to use                      %%%
%%%     Parameter Space:                                                %%%
%%%         DiaVals - Diameter values (pix)                             %%%
%%%         DenVals - Density values (particles/(32x32) region)         %%%
%%%         ShrVals - Shear values (pixels/pixel)                       %%%
%%%         DspVals - Displacement values (pix)                         %%%
%%%         Fo      - Out-of-plane motion (%)                           %%%
%%%     WindowSize - Final Grid Size (ie 16 or 32 etc)                  %%%
%%%     Overlap - interogation overlap (ie 0, 0.25, 0.50, 0.75, etc)    %%% 
%%%     b_noise - The level of background noise in each image           %%%
%%%         b_noise = 0  no background noise                            %%%
%%%         b_noise = 1  LaVision Imager Intense noise approximation    %%%
%%%         b_noise = 2  Photron FASTCAM APX RS noise approximation     %%%
%%%     rows - The number of rows of pixels in the image               %%%
%%%     cols - The number of columns of pixels in the image            %%%
%%%                                                                     %%%
%%%     Other (not currently in the user defined section):              %%%
%%%         bit_depth - The bit depth of the image                      %%%
%%%         b_noise_mu - The mean of the noise                          %%%
%%%         b_noise_sigma - The standard deviation of the noise         %%%
%%%         d_max - Maximum displacement in the image                   %%%
%%%     v_menu - (HARD CODED)The type of flow to be modeled in the image%%%
%%%         v_menu = 1 linear velocity gradient                         %%%
%%%         v_menu = 2 combined Couette-Poiseuille flow                 %%%
%%%         v_menu = 3 Burgers Vortex                                   %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%If parameters.bit_depth = 12; parameters.ratio=1; 
%can use: wx   = uint32(ceil(parameters.d_tau)); wy   = uint32(ceil(parameters.d_tau))
%If parameters.bit_depth = 16; parameters.ratio=1; Edge can be as big as 2 pix
%In my opition, that is fine since max intensity can be 2^16 (nothing compared to 2)
%bt you can use: wx   = uint32(ceil(parameters.d_tau+1)); wy   = uint32(ceil(parameters.d_tau+1))

% png 25min 54sec 16.5GB
% tif 17min 45sec 41.2GB

function SIG_parallel_New
%Clear Command Window

tic
% Run in Parallel? (no=0, yes=1)
parameters.par='1';
% How many processors?
parameters.parprocessors=5;

%Parameter Space
% parameters.DiaVals=single( [1.0 1.5 2.0 3.0 4.0 6.0] ); %(pixels)
parameters.DiaVals=single( [1.0 1.25 1.5 2.0 3.0 5.0] ); %(pixels)
% parameters.DenVals=single( [3 5 10 20 30 50] ); %(particles/32x32 pixel region)
parameters.DenVals=single( [1 2 4 8 16 32 64 128] ); %(particles/32x32 pixel region)
% parameters.ShrVals=single( [0 0.2 0.4 0.6] );%(pixels/pixel)
parameters.ShrVals=single( [0 0.0125 0.025 0.05 0.1 0.2] );%(pixels/pixel)
% parameters.DspVals=single( [0 0.125 0.25 0.375 0.5 0.625 0.75 0.875 1.0] ); %(pixels)
parameters.DspVals=single( [0 0.125 0.25 0.375 0.5 0.625 0.75 0.875 1.0] ); %(pixels)
parameters.Fo     =single( 0 );

%Final Grid Size:
parameters.WindowSize = 32;
%Overlap
parameters.Overlap = 0.75;

%If even_spaced=1, each WindowSize has the same particle image density
%If even_spaced=0, all particles are randomly placed in the image (row*col)
parameters.even_spaced=1;

%Noise ( No noise: 0, 12-bit camera noise: 1, 10-bit camera noise: 2)
parameters.b_noise = uint16(0);

%Flow Type (Uncertainty surface purposes, only use 1)
% parameters.v_menu = 1;

%Synthetic image size
parameters.rows = 256;
parameters.cols = 80000;

%Output location
imbase='E:\'; %make sure this ends with \
%Folder location (automatically generated)
imfolder='CalImages_4x32x32_75%ov_Round_2\';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%                                                %%%%%%%%%%%%%%%%
%%%%%%%%%%%  USERS: DO NOT MAKE CHANGES BEYOND THIS POINT! %%%%%%%%%%%%%%%%
%%%%%%%%%%%                                                %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%Output Folder name (foler created in code)
parameters.loc=[imbase imfolder];

%Make Sub Folder directory in Output location/Output Folder name
if ~exist(parameters.loc,'file')
    fprintf('Creating Folder...')
    mkdir(parameters.loc)
else
    fprintf('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n')
    fprintf('Folder alread exists.\n')
    fprintf('Continue anyway?')
    choice = input('[y/n]\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n','s');
    if choice == 'y'
    else
        error('Program was ended by user')
    end
end

%Change bit-depth when using noise from 10-bit camera
if parameters.b_noise==2
    %Noise from 10-bit camera
    parameters.bit_depth = 10;
    parameters.ratio=1.75;
else
    %Noise from 12-bit camera
    parameters.bit_depth = 12;  
    parameters.ratio=2;
end

%Givens
parameters.d_max(2) = single(0);

i=uint16(0);
Key=single(zeros(size(parameters.ShrVals,2)*size(parameters.DspVals,2)*size(parameters.DiaVals,2)*size(parameters.DenVals,2)*size(parameters.Fo,2),5));
%Particle Density
for kk = uint8(1:size(parameters.ShrVals,2))
    %Particle Diameter
    for ll = uint8(1:size(parameters.DspVals,2))
        %Flow Gradient
        for jj = uint8(1:size(parameters.DiaVals,2))
            %Particle Displacements
            for ii= uint8(1:size(parameters.DenVals,2))
                %Out-of-plane motion
                for mm= uint8(1:size(parameters.Fo,2))
                    
                    i=i+1;
                    Key(i,:)=[parameters.ShrVals(kk) parameters.DspVals(ll) parameters.DiaVals(jj) parameters.DenVals(ii) parameters.Fo(mm)];
                    
                end
            end
        end        
    end
end

I1=uint32(1:i);

% fprintf('You need approximatly %3.2f GB .\n',parameters.rows*parameters.cols*2*2*single(i)/1000000000)
% fprintf('Continue?')
% choice = input('[y/n]\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n','s');
% if choice == 'y'
% else
%     error('Program was ended by user')
% end
    
clear choice i ii jj kk ll mm imbase imfolder
%Save file to be read in by Uncertainty Surface Generator.
save([parameters.loc 'Key.mat'],'Key');
save([parameters.loc 'parameters.mat'],'parameters');
%% Run Jobs in Parallel
%{
    This section opens matlabpool which enables parallel processing.
    Images are devided between the total number of processors.
%}
if str2double(parameters.par)
    fprintf('\n--- Initializing Processor Cores for Parallel Job ----\n')
    poolopen=1;
    
    %Don't open more processors than there are image pairs
%     if length(1:size(parameters.ShrVals,2)) < (parameters.parprocessors)
%         parameters.parprocessors=(length(1:size(parameters.ShrVals,2)));
%     end
    
    try
        %Open Matlab Pool
%         matlabpool('open','local',parameters.parprocessors);
        parpool('local',parameters.parprocessors);
    catch
        try
            %If First attepmt doesn't work, close and retry
%             matlabpool close
            delete(gcp('nocreate'));
%             matlabpool('open','local',parameters.parprocessors);
            parpool('local',parameters.parprocessors);
        catch
            %If Second attempt doesn't work, continue with single processor
            beep
            disp('Error Running Job in Parallel - Defaulting to Single Processor')
            poolopen=0;
            fprintf('\n-------------- Processing Dataset ------------------\n')
            parameters.par = '0';
            ImgGen(parameters,I1,Key)
            fprintf('---------------- Job Completed ---------------------\n')
        end
    end
    if poolopen
        %Pool successfully open, divide sets
            fprintf('\n--------------- Processing Dataset -------------------\n')
            spmd
                verstr=version('-release');
                if str2double(verstr(1:4))>=2010
                    I1dist=getLocalPart(codistributed(I1,codistributor('1d',2)));
                else
                    I1dist=localPart(codistributed(I1,codistributor('1d',2),'convert'));
                end
                ImgGen(parameters,I1dist,Key);
            end
            fprintf('----------------- Job Completed ----------------------\n')
            
        if poolopen
%             matlabpool close
            delete(gcp('nocreate'));
        end
    end
else
    %"Run in Parallel" not selected - Default to single processor
    fprintf('\n-------------- Processing Dataset ------------------\n')
    parameters.par = '0';
    ImgGen(parameters,I1,Key)
    fprintf('---------------- Job Completed ---------------------\n')
end
comptime=toc;
fprintf('total job completion time... %0.2i:%0.2i:%0.2i',floor(comptime/3600),floor(rem(comptime,3600)/60),floor(rem(comptime,60)))
fprintf('\n----------------------------------------------------\n')
end

function ImgGen(parameters,I1,Key)
tic

%File ouput Parameters
imzeros = '5';
imext = 'png';
base = 'B';

%Total image pairs
NimgPairs=single(size(I1,2));

ct=uint16(0);
for i=I1(1):I1(end)
    ct=ct+1;

    %Shear (pixels/pixel)
    parameters.gradient = Key(I1(ct),1);
    % X-Displacements (pixels)
    parameters.d_max(1) = Key(I1(ct),2);
    %Diameter (pixels)
    parameters.d_tau    = Key(I1(ct),3);
    %Density (particles per pixel)
    parameters.d_p      = Key(I1(ct),4)/(32*32);
    %Fo
    parameters.d_max(3) = Key(I1(ct),5);

    %Generate Images
    [image1,image2] = Image_Generator(parameters);

%     %Image Names
%     im1_name = [base sprintf(['%0.' imzeros 'i.' imext],i*2-1)];
%     im2_name = [base sprintf(['%0.' imzeros 'i.' imext],i*2)];               
% 
%     %Save Images
%     imwrite(flipud(image1),[parameters.loc im1_name])
%     imwrite(flipud(image2),[parameters.loc im2_name])
   
    %Image Names: TIF Much faster if you have room, no compression
    %Needs GB: parameters.rows*parameters.cols*2*2*NimgPairs/1000000000
    im1_name = [base sprintf(['%0.' imzeros 'i.tif'],i*2-1)];
    im2_name = [base sprintf(['%0.' imzeros 'i.tif'],i*2)]; 
    imwrite(flipud(image1),[parameters.loc im1_name],'Compression','none')
    imwrite(flipud(image2),[parameters.loc im2_name],'Compression','none')

    % imwrite(im2,map,[loc im2_name])
    % solution(parameters)
    perc=single(ct)*100/NimgPairs;
    ctoc=toc/single(ct)*(NimgPairs-single(ct));
    fprintf('(%3.2f%%)Completed, Time remaining: %0.2i:%0.2i:%0.2i\n',perc,floor(ctoc/3600),floor(rem(ctoc,3600)/60),floor(rem(ctoc,60)))

end
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
function [im1, im2] = Image_Generator(parameters)
% Make the synthetic images
        
%maximum possible intensity
Io = single((2^parameters.bit_depth)/parameters.ratio);

%laser sheet thickness in pixels (4 sigma)
Zo = single(1);

%Preallocate Image
im1 = single(zeros(parameters.rows,parameters.cols));

% add noise to the image
if (parameters.b_noise == 0)
    parameters.b_noise_mu = 0.0;
    parameters.b_noise_sigma = 0.0;
else
    [im1, parameters]= background_noise(im1, parameters);
end

%compute the number of particle within the image
np = uint32(floor(parameters.d_p*parameters.rows*parameters.cols));

%Preallocate loc
if parameters.even_spaced==1;
    %Makes sure the number of particles are in each interrogation
    loc = single(zeros(np,3));
    ppa=floor(parameters.d_p*parameters.WindowSize*parameters.WindowSize);
    kk=0;
    for ii=1:parameters.rows/parameters.WindowSize
        for jj=1:parameters.cols/parameters.WindowSize
            kk=kk+ppa;
            loc(kk-ppa+1:kk,1:3)=[rand(ppa,1)+(jj-1) rand(ppa,1)+(ii-1) rand(ppa,1)];
        end
    end
    loc(:,1) = loc(:,1)/jj;
    loc(:,2) = loc(:,2)/ii;
elseif parameters.even_spaced==0
  %This gives particle location in x,y,z
  loc = single(rand(np,3));
end

% This gives particle location in x,y,z
loc(:,1) = loc(:,1)*parameters.cols;% x location
loc(:,2) = loc(:,2)*parameters.rows;% y location
loc(:,3) = loc(:,3)*Zo-(Zo/2.0); %z ranges from +/- 2*sigma where sigma is the standard deviation of the gaussian light sheet (Zo = 4 sigma)
loc=sortrows(loc,3);

%-----------------------Make the particle images------------------------
%   M. Raffel: 5.5.1 Synthetic Particle Image Generation
%   I(x,y)=Io*exp[(-(x-xo)^2 -(y-yo)^2)/(0.125*d_tau^2)]
    
ipos = uint32(ceil(loc(:,2)));
jpos = uint32(ceil(loc(:,1)));
wx   = uint32(ceil(parameters.d_tau));
wy   = uint32(ceil(parameters.d_tau));

%Particle size interrogation (out of bounds set to bounds)
imin = uint32(max(ipos-wx,1));
imax = uint32(min(ipos+wx,parameters.rows));
jmin = uint32(max(jpos-wy,1));
jmax = uint32(min(jpos+wy,parameters.cols));

%Make edge particles size bigger to make the same size as internal particles
imax(imin==1)=1+wx+wx;
imin(imax==parameters.rows)=parameters.rows-wx-wx;
jmax(jmin==1)=1+wy+wy;
jmin(jmax==parameters.cols)=parameters.cols-wy-wy;

%Peak intensity
In=Io*exp((-loc(:,3).^2)/(0.125*Zo*Zo));      
const=1/(0.125*parameters.d_tau^2);

% 3rd Try:
% w=1+wy+wy;
% [jj2,ii2] = meshgrid(0:(wx+wx),0:(wy+wy),0:np-1);
% imin2=repmat(permute(imin,[3 2 1]),[w w 1]);   
% jmin2=repmat(permute(jmin,[3 2 1]),[w w 1]); 
% loc1=repmat(permute(loc(:,1),[3 2 1]),[w w 1]);
% loc2=repmat(permute(loc(:,2),[3 2 1]),[w w 1]);
% In1=repmat(permute(In,[3 2 1]),[w w 1]);
% temp=In1.*exp((-(loc2+0.5-single(imin2+ii2)).^2-(loc1+0.5-single(jmin2+jj2)).^2)*const);

[j,~,i] = meshgrid(0:(wx+wx),0,0:(wy+wy));
t1=bsxfun(@plus,jmin,j);
t1=bsxfun(@plus,loc(:,1),0.5-single(t1));
t2=bsxfun(@plus,imin,i);
t2=bsxfun(@plus,loc(:,2),0.5-single(t2));
temp=permute(bsxfun(@times,In,exp((-(t2.*t2)-(t1.*t1))*const)),[3 2 1]);

for k = 1:np
% 1st Try: Loops
%     for i=imin(k):imax(k)
%         for j=jmin(k):jmax(k) 
%             im(i,j) = im(i,j)+ In(k)*exp((-(loc(k,2)+0.5-single(i))^2-(loc(k,1)+0.5-single(j))^2)*const);
%         end
%     end

% 2nd Try: Able to do ii, jj outside of loop to decrease time
%     ii= repmat([imin(k):imax(k)]',size(jmin(k):jmax(k)));
%     jj= repmat(jmin(k):jmax(k),[size(imin(k):imax(k),2) 1]);
%     im1(imin(k):imax(k),jmin(k):jmax(k)) = im1(imin(k):imax(k),jmin(k):jmax(k))+ In(k)*exp((-(loc(k,2)+0.5-single(ii)).^2-(loc(k,1)+0.5-single(jj)).^2)*const);

% 3rd Try: Calculated I(x,y)=Io*exp[(-(x-xo)^2-(y-yo)^2)/(0.125*d_tau^2)] out side of loop
%     (called temp) using repmat and permute (can see older version in EFDLSERVER

% 4th Try: Using bsxfun, decreases time by half compared to 3rd try.
    im1(imin(k):imax(k),jmin(k):jmax(k)) = im1(imin(k):imax(k),jmin(k):jmax(k))+ temp(:,:,k);
end
im1=uint16(im1);





%Compute the displacement at particle location
y = loc(:,2)-floor(parameters.rows/2.0)-(parameters.WindowSize*(1-parameters.Overlap)/2.0)+0.5;

loc(:,1) =loc(:,1)+ y*parameters.gradient + parameters.d_max(1);
loc(:,2) =loc(:,2)+ parameters.d_max(2);
loc(:,3) =loc(:,3)+ parameters.d_max(3);

%particles that leave area due to shear, wrap around
loc(:,1)=mod(loc(:,1),parameters.cols);

if parameters.d_max(3)>0
    %Generate future particles (out of plane motion)
    npFo=floor(single(np)*parameters.d_max(3));
    locFo = zeros(npFo,3);
    locFo = rand(size(locFo));
    locFo(:,1) = locFo(:,1)*parameters.cols;% x location
    locFo(:,2) = locFo(:,2)*parameters.rows;% y location
    locFo(:,3) = locFo(:,3)*parameters.d_max(3)-(Zo/2.0);
    locFo=sortrows(locFo,3);

    temp=zeros(np,1);
    temp(loc(:,3)<(0.5))=1;

    loc=[locFo; loc(1:sum(temp),:);];
    clear locFo temp npFo
end

im2 = single(zeros(parameters.rows,parameters.cols));
% add noise to the image
if (parameters.b_noise == 0)
    parameters.b_noise_mu = 0.0;
    parameters.b_noise_sigma = 0.0;
else
    [im2, parameters]= background_noise(im2, parameters);
end

ipos = uint32(ceil(loc(:,2)));
jpos = uint32(ceil(loc(:,1)));

imin = uint32(max(ipos-wx,1));
imax = uint32(min(ipos+wx,parameters.rows));
jmin = uint32(max(jpos-wy,1));
jmax = uint32(min(jpos+wy,parameters.cols));

imax(imin==1)=1+wx+wx;
imin(imax==parameters.rows)=parameters.rows-wx-wx;
jmax(jmin==1)=1+wy+wy;
jmin(jmax==parameters.cols)=parameters.cols-wy-wy;
        
In=Io*exp((-loc(:,3).^2)/(0.125*Zo*Zo));  
np2=uint32(length(imax));
t1=bsxfun(@plus,jmin,j);
t1=bsxfun(@plus,loc(:,1),0.5-single(t1));
t2=bsxfun(@plus,imin,i);
t2=bsxfun(@plus,loc(:,2),0.5-single(t2));
temp=permute(bsxfun(@times,In,exp((-(t2.*t2)-(t1.*t1))*const)),[3 2 1]);
for k = 1:np2  
    im2(imin(k):imax(k),jmin(k):jmax(k)) = im2(imin(k):imax(k),jmin(k):jmax(k))+ temp(:,:,k);
end
im2=uint16(im2);

im1(im1 > (2^parameters.bit_depth)) = (2^parameters.bit_depth); %0.0625 correspones to 2^12 (saturation level)
im2(im2 > (2^parameters.bit_depth)) = (2^parameters.bit_depth); %0.0625 correspones to 2^12 (saturation level)
   
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [im, parameters]= background_noise(im, parameters)
% This subroutine generate background noise to add to PIV images.

% Determine the type of noise to be used.  The noise source is based
% on the Lavision Imager intense Low Speed camera and the Photonic Fast CAM
% high speed camera.  To get this noise, each camera took pictures of the 
% lens cap (so nothing should be there) and a histogram was made of pixel
% intensity values.  A gaussian was fitted to the histogram and is used as
% the distribution to drawn noise levels from.  The histogram was normalized
% against the bit depth of the camera and is scaled to the bit resolution of 
% the image you are creating.
        
if (parameters.b_noise == 1) 
    parameters.b_noise_mu = 0.01011*(2.0^parameters.bit_depth)/(2^16);
    parameters.b_noise_sigma = 0.000596*(2.0^parameters.bit_depth)/(2^16);
elseif (parameters.b_noise == 2) 
    parameters.b_noise_mu = 0.002834*(2.0^parameters.bit_depth)/(2^16);
    parameters.b_noise_sigma = 0.00355*(2.0^parameters.bit_depth)/(2^16);
end    
        
%Populate image 1 and 2 with random numbers from a uniform distribution
M=size(im,1);N=size(im,2);
im(:,:) = normrnd(parameters.b_noise_mu,parameters.b_noise_sigma,M,N);

im(im1<0)=0;
       
end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [dx]=linear_profile(xyz,parameters)
% % This subroutine calculates the velocity field for a flow with contant
% % gradient.  If the gradient is zero, the the flow is uniform.  Currently
% % only the gradient of u was desired.  Additional gradients can be added here 
% % for more complex flows.
% 
%     y = xyz(:,2)-floor(parameters.rows/2.0)-(parameters.shift/2.0);
%     dx(:,1) = y*parameters.gradient + parameters.d_max(1);
%     dx(:,2) = parameters.d_max(2);
%     dx(:,3) = parameters.d_max(3);
% 
% end
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [dx]= couette_poiseuille(xyz,parameters)
% % This subroutine is used to compute the velocity feild of a combined
% % Couette Poiseuille flow.  Mathematical model of this flow comes from
% % Viscous Fluid Flow by White published by McGraw-Hill in 2006.
%     mu = 0.000635;
%     U = 1.0;
%     %x = xyz(1);
%     y = xyz(:,2)-floor(parameters.rows/2.0)-1;
%     h = floor(parameters.rows/2.0);
% 
%     P_nd = 150.0*(h^2)/(2.0*mu*U);
% 
%     dx(:,1) = U*(0.5*(1.0+y/h)+P_nd*(1.0-(y.^2)/(h^2)));
%     dx(:,2) = 0.0;
%     dx(:,3) = 0.0;
% 
%     y = h/(2.0*P_nd);
%     dx(:,1) = parameters.d_max(1)*dx(:,1)/(U*(0.5*(1+y/h)+P_nd*(1-(y^2)/(h^2))));
% end
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [dx]= burgers_vortex(xyz,parameters)
% % This subroutine is used to calculte the velocity field of a burger
% % vortex.  The mathamtical model for this flow was found in Vortex Flow
% % by Ogawa published by CRC press in 1993.  Dr. Spall has this book.
% 
%     nu = 1.516*10.0^(-5);
%     nu_t = 1000.0*nu;
%     b = 0.7;
%     V_rb = -0.2;
%     V_thetab = 5.0;
% 
%     x = xyz(:,1)-floor(parameters.cols/2.0)-1;
%     y = xyz(:,2)-floor(parameters.rows/2.0)-1;
% 
%     r = sqrt(x.^2+y.^2);
%     theta = atan2(y,x);
% 
%     R_eb = -(b*V_rb)/nu_t;
% 
%     V_theta = V_thetab*((R_eb/(2.0*(1.0-exp(-R_eb/2.0))))*(r/b));
%     V_r = (r*V_rb)/b;
% 
%     dx(:,1) = V_r.*cos(theta)-V_theta.*sin(theta);
%     dx(:,2) = V_r.*sin(theta)+V_theta.*cos(theta);
%     dx(:,3) = 0.0;
% 
%     x = floor(parameters.cols/2.0);
%     y = floor(parameters.rows/2.0);
% 
%     r = sqrt(x^2+y^2);
%     theta = atan2(y,x);
%     R_eb = -(b*V_rb)/nu_t;
% 
%     V_theta = V_thetab*((R_eb/(2.0*(1.0-exp(-R_eb/2.0))))*(r/b));
%     V_r = (r*V_rb)/b;
%     dx(:,1) = parameters.d_max(1)*dx(:,1)/abs(V_r*cos(theta)-V_theta*sin(theta));
%     dx(:,2) = parameters.d_max(2)*dx(:,2)/abs(V_r*sin(theta)+V_theta*cos(theta));
% end 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function []= solution(parameters)
% % This subroutine creates a file containing the displacement for the
% % image created.  Only U and V are recorded, because this study used
% % 2-d PIV, so W velocity was irrelevent.  If you need it you can add it
% % here.
% % This is section creates arrays to store the solution.  The solution
% % should correspond to the vector spacing in a processes image.
%     nx = floor(parameters.cols/8.0)-1;
%     ny = floor(parameters.rows/8.0)-1;
% 
%     x=zeros(ny,nx);y=zeros(ny,nx);u=zeros(ny,nx);v=zeros(ny,nx);
%     for j = 1:nx
%         for i = 1:ny
%             if (j == 1) 
%                 x(i,j) = 8.0;
%             else 
%                 x(i,j) = x(i,j-1) + 8.0;
%             end
%             if (i == 1) 
%                 y(i,j) = 8.0;
%             else 
%                 y(i,j) = y(i-1,j) + 8.0;
%             end
%         end
%     end
%     [y]= flipud(y);
% 
%     % Populate the array with u and v components of velocity. 
%     for j = 1:nx
%         for i = 1:ny
%             if (parameters.v_menu == 1) 
%                 [dx]=linear_profile([x(i,j),y(i,j),0.0],parameters);
%             elseif (parameters.v_menu == 2) 
%                 [dx]= couette_poiseuille([x(i,j),y(i,j),0.0],parameters);
%             elseif (parameters.v_menu == 3) 
%                 [dx]= burgers_vortex([x(i,j),y(i,j),0.0],parameters);
%             end
%             u(i,j) = dx(1);
%             v(i,j) = dx(2);
%         end
%     end
%     [y]= flipud(y);
%     [u]= flipud(u);
%     [v]= flipud(v);
%     
%     %generate text file
%     fname ='solution.dat';
%     fid = fopen(fname,'w');
%     if fid==-1
%         error(['error creating file ',fname])
%     end
%     %header lines
%     fprintf(fid,'VARIABLES    "x", "y", "Vx", "Vy", "|V|"\n');
%     %write data
%     for i=1:ny
%         for j=1:nx
%                 fprintf(fid,'%14.6e %14.6e %14.6e %14.6e %14.6e\n',x(i,j),y(i,j),u(i,j),v(i,j),sqrt(u(i,j)^2+v(i,j)^2));
%         end
%     end
%     fclose(fid);
% end
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%