clear all; close all; clc; dbstop if error;

% This program will plot heat flux measurements from Heat Flux Sensors (HFSs) with that of the TC probe.
% The HFS data are imported from Stats files from BC_Stats_2_2.m and the probe flux is hard-coded.
% It will also plot heat flux from correlations using the measured centerline wall temperature.  It is based
% on SteadySRQ_Plots_HeatFlux_2_3.m.
%
% Blake Lance, 15 September 2015

%% Inputs
Case = 1;       % Case (0 for Forced Convection, 1 for Mixed Convection)
Correct = 0;    % Whether to increase HFS flux by 18.5% to correct for nonuniform thermal resistance

% Air properties from Fluid Mechanics, Cengel and Cimbala
Tinf = 22;      % Free-stream temperature [degC]
Ts = 138.6;     % Heated wall temperature [degC]
Tf = 0.5*(Tinf+Ts); % Film temperature [degC]
Pr = 0.7154;    % Prandtl number at 80C
nu = 2.097E-5;  % Kinematic viscosity at 80C [m^2/s]
beta = 1/(Tf+273.15); % Thermal coefficient of expansion [1/K]
k = 0.02953;    % Thermal conductivity [W/m*K]

folder = uigetdir(pwd,'Select folder that contains thermal BC Stats files');
tic;

% Assign u_inf, unheated starting length, and TC probe heat flux
if Case == 0 % Forced Convection
    u_inf = 4.552; % Free-stream velocity of forced convection at inlet in S7.3 [m/s]
    Xi = 0.3265; % Unheated starting length of forced convection in S7.3.5 [m]
    Flux_TC = [2650;2037;1730]; % Stored in Heat Flux Results.xlsx in Temperature Probe Test7 folder [W/m^2]
    U_Flux_TC = [226.8;144.5;140.4]; % Stored in Heat Flux Results.xlsx in Temperature Probe Test7 folder [W/m^2]
elseif Case == 1 % Mixed Convection
    u_inf = 2.624; % Free-stream velocity of mixed convection at inlet in S7.4 [m/s]
    Xi = 0.4584; % Unheated starting length of mixed convection in S7.4.5 [m]
    Flux_TC = [1899;1397;1185]; % Stored in Heat Flux Results.xlsx in Temperature Probe Test7 folder [W/m^2]
    U_Flux_TC = [164.3;128.2;97.57]; % Stored in Heat Flux Results.xlsx in Temperature Probe Test7 folder [W/m^2]
end

FontSize = 11;
linecolor = {[0.000,  0.447,  0.741]; [0.850,  0.325,  0.098]; [0.929,  0.694,  0.125]};

%% Import Experimental Heat Flux
data_HFS = importdata([folder,'\HeatFlux_Stats.csv']);

x = data_HFS.data(:,1);
Flux_HFS = data_HFS.data(:,4);
U_Flux_HFS = data_HFS.data(:,7);
if Correct == 1; Flux_HFS = 1.185*Flux_HFS; U_Flux_HFS = 1.185*U_Flux_HFS; end

%% Read Wall Temperatures
data_T = importdata([folder,'\HeatedWallTemp_Stats.csv']);
iCL = find(data_T.data(:,3) == 0); % Find centerline TC indeces
xCL = data_T.data(iCL,1); % x values along centerline
TempCL = data_T.data(iCL,4); % Temperature along centerline
% Interpolate centerline temperatures to three HFS locations and convert to deg C
Ts_exp = interp1(xCL,TempCL,x) - 273.15;

%% Calculate Heat Flux From Correlation
xi = linspace(0.001,2+Xi,100)'; % x-values for correlations, starting at unheated starting length [m]
% Ts = interp1(xCL,TempCL,xi,'pchip') - 273.15; % Interpolate TempCL to xi for correlation and convert to deg C
% Interpolate TempCL to xi for correlation and convert to deg C, pad ahead of x=0 with Tinf so that the flux
%   calculation (which starts at the unheated starting length at x=-Xi) is aligned properly in x
Ts = interp1(xCL+Xi,TempCL,xi,'linear',Tinf+273.15) - 273.15;
Re_x = u_inf*xi./nu; % Local Reynolds number

if Case == 0 % Forced Convection
    Nu_x = 0.0287*Re_x.^0.8*Pr./(0.169*Re_x.^(-0.1)*(13.2*Pr-9.25)+0.85); % (12-17) in Kays and Crawford
%     Nu_x = 0.0287*Re_x.^0.8*Pr./(0.0169*Re_x.^(-0.1)*(13.2*Pr-9.25)+0.85); % (12-17) in Kays and Crawford
    Nu_x_UH = abs(Nu_x./(1-(Xi./xi).^(9/10)).^(1/9)); % Nu_x with unheated starting length
    h_x = Nu_x*k./xi; % Local heat transfer coefficient [W/m^2*K]
    h_x_UH = Nu_x_UH*k./xi; % Local heat transfer coefficient [W/m^2*K]
    Flux_corr = h_x.*(Ts-Tinf); % Correlation prediction for heat flux [W/m^2]
    Flux_corr_UH = h_x_UH.*(Ts-Tinf); % Correlation prediction for heat flux [W/m^2]
elseif Case == 1 % Mixed Convection
    g = 9.81;   % Acceleration due to gravity [m/s^2]
    Gr_x = g*beta*(Ts-Tinf).*xi.^3/nu^2; % Local Grashof number
    % Correlations from Studies of Mixed Convection in Vertical Tubes, Jackson
%     Nu_Fx = 0.0287*Re_x.^0.8*Pr^0.6; % Forced convection Nu (Eqn 14.40)
    Nu_Fx = 0.0287*Re_x.^0.8*Pr./(0.169*Re_x.^(-0.1)*(13.2*Pr-9.25)+0.85); % (12-17) in Kays and Crawford
    Nu_Tx = Nu_Fx.*abs(1+10^4*Gr_x./(Re_x.^2.7*Pr^0.5)).^0.46; % Total Nu for opposed (Eqn 6)
    % Update Nu_Tx by using the unheated starting length theory, Incropera Heat Transfer (Eqn 7.43)
    Nu_Tx_UH = abs(Nu_Tx./(1-(Xi./xi).^(9/10)).^(1/9)); % Nu_Tx with unheated starting length
    hx = Nu_Tx*k./xi; % Local heat transfer coefficient [W/m^2*K]
    hx1_UH = Nu_Tx_UH*k./xi; % Local heat transfer coefficient [W/m^2*K]
    Flux_corr = hx.*(Ts-Tinf); % Correlation prediction for heat flux [W/m^2]
    Flux_corr_UH = hx1_UH.*(Ts-Tinf); % Correlation prediction for heat flux [W/m^2]
end

%% Plotting
figure(1); set(gcf,'Units','inches','Position',[4 6 3.5 3]); hold on;
% plot(xi-Xi,Flux_corr,'LineWidth',1.25);
plot(xi-Xi,Flux_corr_UH,'LineWidth',1.25);
errorbar(x,Flux_HFS,U_Flux_HFS,'--','LineWidth',1.25);
errorbar(x,Flux_TC,U_Flux_TC,'-.','LineWidth',1.25); % Plot flux from TC probe
axis([0 1.8 0 3400]);
% hleg1 = legend('Corr','Corr-UH','HFS','TC probe');
hleg1 = legend('Correlation','HFS','TC probe');
set(hleg1,'Location','northeast','Interpreter','latex');
xlabel('$x$ [m]','Interpreter','latex');
ylabel('$q''''$ [W/m$^2$]','Interpreter','latex');
set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex','box','on');
set(findall(gcf,'type','axes'),'FontSize',FontSize);
set(findall(gcf,'type','text'),'FontSize',FontSize);
set(findall(gcf,'type','legend'),'FontSize',FontSize-1);
tightfig;

% Plot temperature
figure(2); set(gcf,'Units','inches','Position',[8 6 3.5 3]); hold on;
plot(xi-Xi,Ts,'LineWidth',1.25);
xlabel('$x$ [m]','Interpreter','latex'); ylabel('$T$ [$^\circ$C]','Interpreter','latex');

% Plot flux without and with unheated starting length calculations
figure(3); set(gcf,'Units','inches','Position',[12 6 3.5 3]); hold on;
plot(xi-Xi,Flux_corr,'LineWidth',1.25);
plot(xi-Xi,Flux_corr_UH,'LineWidth',1.25);
legend('w/o Unheated','w/Unheated');
xlabel('$x$ [m]','Interpreter','latex'); ylabel('$q''''$ [W/m$^2$]','Interpreter','latex');

toc;