clear all; close all; clc; dbstop if error

% This code will perform grid convergence studies on the Transient CFD results using the approach od Roache
% as found in Verification and Validation in Scientific Computing by Oberkampf and Roy, Sec. 8.6. It must be
% run with MATLAB 2014b or later for the plots.
%
% Blake Lance, 4 February 2015

%% Inputs

Nth = 5;    % Frequency of uncertainty bands
Nth_t = 20; % Frequency of plotting u and k in time
converg = 1E-6; % Convergence criterion
pf = 2; % The formal order of the numerical scheme
Nphases = 92;   % The number of time steps used
t = linspace(0,18.2,Nphases); % time [s]
x_HF = [0.1617; 0.7777; 1.3936]; % The x location of the HFS positions [m]
Ny = [100; 50; 25];    % The number of cells in the y direction for Fine, Medium, and Coarse
% Ny = [200; 100; 50];    % The number of cells in the y direction for Fine, Medium, and Coarse

V1 = 7.211398e-07;  % Volume average volume on fine mesh (1M cells)
V2 = 5.898062e-06;  % Volume average volume on medium mesh (125k cells)
V3 = 4.928023e-05;  % Volume average volume on coarse mesh (15625 cells)

% V1 = 8.914464e-8;   % Volume average volume on fine mesh (8M cells)
% V2 = 7.211398e-07;  % Volume average volume on medium mesh (1M cells)
% V3 = 5.898062e-06;  % Volume average volume on coarse mesh (125k cells)

% Select three folders for three simulations containing subfolders for SRQ data
if exist('TransientCFD_GCI_Selection.mat','file')
    load('TransientCFD_GCI_Selection.mat');
else
    SimDir1 = uigetdir('F:\TMCWT CFD\Transient\RealBCs','Select the Simulation Folder for the Fine Mesh');
    SimDir2 = uigetdir('F:\TMCWT CFD\Transient\RealBCs','Select the Simulation Folder for the Medium Mesh');
    SimDir3 = uigetdir('F:\TMCWT CFD\Transient\RealBCs','Select the Simulation Folder for the Coarse Mesh');
    save('TransientCFD_GCI_Selection.mat','SimDir1','SimDir2','SimDir3');
end

tic;

%% Import data in loops into 3D array of [HFS,t] or [HFS,y,t]

% Heat Flux loop
HF1 = zeros(3,Nphases);
HF2 = zeros(3,Nphases);
HF3 = zeros(3,Nphases);
for i=1:Nphases
    HFdata1 = importdata([SimDir1,'\Heat Flux\Heat_Flux_HF_',sprintf('%8.6e',t(i)),'.csv']);
    HFdata2 = importdata([SimDir2,'\Heat Flux\Heat_Flux_HF_',sprintf('%8.6e',t(i)),'.csv']);
    HFdata3 = importdata([SimDir3,'\Heat Flux\Heat_Flux_HF_',sprintf('%8.6e',t(i)),'.csv']);
    
    if i == 1
        x1 = HFdata1.data(:,1);
        x2 = HFdata2.data(:,1);
        x3 = HFdata3.data(:,1);
    end
    
    HF1(:,i) = -interp1(x1,HFdata1.data(:,2),x_HF); % Interp CFD HF results at sensor locations
    HF2(:,i) = -interp1(x2,HFdata2.data(:,2),x_HF); % Interp CFD HF results at sensor locations
    HF3(:,i) = -interp1(x3,HFdata3.data(:,2),x_HF); % Interp CFD HF results at sensor locations
end

% Shear loop
Shear1 = zeros(3,Nphases);
Shear2 = zeros(3,Nphases);
Shear3 = zeros(3,Nphases);
for i=1:Nphases
    Sheardata1 = importdata([SimDir1,'\Shear\Shear_Shear_',sprintf('%8.6e',t(i)),'.csv']);
    Sheardata2 = importdata([SimDir2,'\Shear\Shear_Shear_',sprintf('%8.6e',t(i)),'.csv']);
    Sheardata3 = importdata([SimDir3,'\Shear\Shear_Shear_',sprintf('%8.6e',t(i)),'.csv']);
    
    Shear1(:,i) = interp1(x1,Sheardata1.data(:,2),x_HF); % Interp CFD Shear results at sensor locations
    Shear2(:,i) = interp1(x2,Sheardata2.data(:,2),x_HF); % Interp CFD Shear results at sensor locations
    Shear3(:,i) = interp1(x3,Sheardata3.data(:,2),x_HF); % Interp CFD Shear results at sensor locations
end

% u loop
u1all = zeros(3,Ny(1),Nphases);
u1 = zeros(3,Ny(3),Nphases);
u2 = zeros(3,Ny(3),Nphases);
u3 = zeros(3,Ny(3),Nphases);
for i=1:Nphases
    udata1 = importdata([SimDir1,'\u\u_u_',sprintf('%8.6e',t(i)),'.csv']);
    udata2 = importdata([SimDir2,'\u\u_u_',sprintf('%8.6e',t(i)),'.csv']);
    udata3 = importdata([SimDir3,'\u\u_u_',sprintf('%8.6e',t(i)),'.csv']);
    
    if i == 1
        y1 = udata1.data(:,1);
        y2 = udata2.data(:,1);
        y3 = udata3.data(:,1);
    end
    
    for j=1:3
        u1all(j,:,i) = udata1.data(:,2*j); % Assign all data from fine mesh
        u1(j,:,i) = interp1(y1,udata1.data(:,2*j),y3); % Interp CFD u results to coarse mesh
        u2(j,:,i) = interp1(y2,udata2.data(:,2*j),y3); % Interp CFD u results to coarse mesh
        u3(j,:,i) = interp1(y3,udata3.data(:,2*j),y3); % Interp CFD u results to coarse mesh
    end
end

% k loop
k1all = zeros(3,Ny(1),Nphases);
k1 = zeros(3,Ny(3),Nphases);
k2 = zeros(3,Ny(3),Nphases);
k3 = zeros(3,Ny(3),Nphases);
for i=1:Nphases
    kdata1 = importdata([SimDir1,'\k\k_k_',sprintf('%8.6e',t(i)),'.csv']);
    kdata2 = importdata([SimDir2,'\k\k_k_',sprintf('%8.6e',t(i)),'.csv']);
    kdata3 = importdata([SimDir3,'\k\k_k_',sprintf('%8.6e',t(i)),'.csv']);
    
    for j=1:3
        k1all(j,:,i) = kdata1.data(:,2*j); % Assign all data from fine mesh
        k1(j,:,i) = interp1(y1,kdata1.data(:,2*j),y3); % Interp CFD k results to coarse mesh
        k2(j,:,i) = interp1(y2,kdata2.data(:,2*j),y3); % Interp CFD k results to coarse mesh
        k3(j,:,i) = interp1(y3,kdata3.data(:,2*j),y3); % Interp CFD k results to coarse mesh
    end
end

%% Determine the error between the two meshes

e_HF21 = HF2 - HF1;             e_HF32 = HF3 - HF2;             % Heat Flux error
e_Shear21 = Shear2 - Shear1;    e_Shear32 = Shear3 - Shear2;    % Shear error
e_u21 = u2 - u1;                e_u32 = u3 - u2;                % u error
e_k21 = k2 - k1;                e_k32 = k3 - k2;                % k error

%% Determine the apparent order p and and the factor of safety Fs

r21 = (V2/V1)^(1/3); r32 = (V3/V2)^(1/3); % The refinement in space between meshes
% p = 1/log(r)*abs( log(abs(e32./e21)) + q ); % The apparent order
% q = log(( r21^p - s )/( r32^p - s ));
% s = sign(e32/e21);

%%%%%%%%%%%%%%%%%%%
% Heat Flux
p_HF = pf; % Initial guess for p
for i=1:100
    p_HF_old = p_HF;
    s_HF = sign(e_HF32./e_HF21);
    q_HF = log(( r21.^p_HF - s_HF )./( r32.^p_HF - s_HF ));
    p_HF = 1/log(r21)*abs( log(abs(e_HF32./e_HF21)) + q_HF );
    if abs( p_HF - p_HF_old ) < converg; break; end;
end

Fs_HF = 3.0*ones(3,Nphases); % Initialize to 3.0
% Allow for a smaller factor of safety if the observed order is within 10% of the formal order
Fs_HF(abs( ( p_HF - pf )/pf ) <= 0.1) = 1.25;
% The apparent order of p is bounded within 0.5 and the formal order
p_HF(Fs_HF==3.0) = min(max(0.5,p_HF(Fs_HF==3.0)),pf);

%%%%%%%%%%%%%%%%%%%
% Shear
p_Shear = pf; % Initial guess for p
for i=1:100
    p_Shear_old = p_Shear;
    s_Shear = sign(e_Shear32./e_Shear21);
    q_Shear = log(( r21.^p_Shear - s_Shear )./( r32.^p_Shear - s_Shear ));
    p_Shear = 1/log(r21)*abs( log(abs(e_Shear32./e_Shear21)) + q_Shear );
    if abs( p_Shear - p_Shear_old ) < converg; break; end;
end

Fs_Shear = 3.0*ones(3,Nphases); % Initialize to 3.0
% Allow for a smaller factor of safety if the observed order is within 10% of the formal order
Fs_Shear(abs( ( p_Shear - pf )/pf ) <= 0.1) = 1.25;
% The apparent order of p is bounded within 0.5 and the formal order
p_Shear(Fs_Shear==3.0) = min(max(0.5,p_Shear(Fs_Shear==3.0)),pf);

%%%%%%%%%%%%%%%%%%%
% u
p_u = pf; % Initial guess for p
for i=1:100
    p_u_old = p_u;
    s_u = sign(e_u32./e_u21);
    q_u = log(( r21.^p_u - s_u )./( r32.^p_u - s_u ));
    p_u = 1/log(r21)*abs( log(abs(e_u32./e_u21)) + q_u );
    if abs( p_u - p_u_old ) < converg; break; end;
end

Fs_u = 3.0*ones(3,Ny(3),Nphases); % Initialize to 3.0
% Allow for a smaller factor of safety if the observed order is within 10% of the formal order
Fs_u(abs( ( p_u - pf )/pf ) <= 0.1) = 1.25;
% The apparent order of p is bounded within 0.5 and the formal order
p_u(Fs_u==3.0) = min(max(0.5,p_u(Fs_u==3.0)),pf);

%%%%%%%%%%%%%%%%%%%
% k
p_k = pf; % Initial guess for p
for i=1:100
    p_k_old = p_k;
    s_k = sign(e_k32./e_k21);
    q_k = log(( r21.^p_k - s_k )./( r32.^p_k - s_k ));
    p_k = 1/log(r21)*abs( log(abs(e_k32./e_k21)) + q_k );
    if abs( p_k - p_k_old ) < converg; break; end;
end

Fs_k = 3.0*ones(3,Ny(3),Nphases); % Initialize to 3.0
% Allow for a smaller factor of safety if the observed order is within 10% of the formal order
Fs_k(abs( ( p_k - pf )/pf ) <= 0.1) = 1.25;
% The apparent order of p is bounded within 0.5 and the formal order
p_k(Fs_k==3.0) = min(max(0.5,p_k(Fs_k==3.0)),pf);

%% Calculate the extrapolated values

HF_ext21 = (r21.^p_HF.*HF1 - HF2 )./( r21.^p_HF - 1 ); % Extrapolated Heat Flux from Med and Fine
Shear_ext21 = (r21.^p_Shear.*Shear1 - Shear2 )./( r21.^p_Shear - 1 ); % Extrapolated Shear from Med and Fine
u_ext21 = (r21.^p_u.*u1 - u2 )./( r21.^p_u - 1 ); % Extrapolated u from Med and Fine
k_ext21 = (r21.^p_k.*k1 - k2 )./( r21.^p_k - 1 ); % Extrapolated k from Med and Fine

%% Calculate the non-normalized approximate error and GCI

% The GCI in this form is non-normalized with units of the error
% GCI_fine21 = Fs*abs( phi1 - phi2 )/( r21^p - 1 ); 

GCI_fine_HF21 = Fs_HF.*abs( HF1 - HF2 )./( r21.^p_HF - 1 );
GCI_fine_Shear21 = Fs_Shear.*abs( Shear1 - Shear2 )./( r21.^p_Shear - 1 );
GCI_fine_u21 = Fs_u.*abs( u1 - u2 )./( r21.^p_u - 1 );
GCI_fine_k21 = Fs_k.*abs( k1 - k2 )./( r21.^p_k - 1 );

%% Plot CFD results with uncertainty bands as well as extrapolated results

y3 = 1000*y3; % Convert y3 to mm for plots
figure(1); set(gcf,'units','inches','Position',[1 4 3 6]);
figure(2); set(gcf,'units','inches','Position',[3 2 3.25 6]);
figure(3); set(gcf,'units','inches','Position',[5 4 3 6]);
figure(4); set(gcf,'units','inches','Position',[7 2 3.25 6]);
figure(5); set(gcf,'units','inches','Position',[9 4 3.75 6]);
figure(6); set(gcf,'units','inches','Position',[11 2 3.25 6]);
figure(7); set(gcf,'units','inches','Position',[13 4 3.75 6]);
figure(8); set(gcf,'units','inches','Position',[15 2 3.25 6]);
for i=1:3
    %%%%%%%%%%%%%%%
    % Plot Heat Flux
    figure(1);
    subtightplot(3,1,i,[0,0],0.05,0.05);
    plot(t,HF3(i,:),'LineWidth',1.25); hold on;
    plot(t,HF2(i,:),'--','LineWidth',1.25);
    plot(t,HF1(i,:),'-.','LineWidth',1.25);
    plot(t,HF_ext21(i,:),':','LineWidth',1.25);
    axis([0 t(Nphases) 0 1800]);
    if i == 3; xlabel('$t$ [s]','Interpreter','latex'); end
    ylabel(['$q''''(x_',num2str(i),')$ [W/m$^2$]'],'Interpreter','latex');
    if i == 2; h = legend('Coarse','Medium','Fine','Extrapolated','location','northeast'); 
        set(h,'Interpreter','latex'); end
    if i ~= 3; set(gca,'XTickLabel',''); end
    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
    
    figure(2);
    subtightplot(3,1,i,[0,0],0.05,0.05);
    errorbar(t(1:Nth:end),HF1(i,1:Nth:end),GCI_fine_HF21(i,1:Nth:end),'LineWidth',1.25);
    axis([0 t(Nphases) 0 1800]);
    if i == 3; xlabel('$t$ [s]','Interpreter','latex'); end
    ylabel(['$q''''(x_',num2str(i),')$ [W/m$^2$]'],'Interpreter','latex');
    if i ~= 3; set(gca,'XTickLabel',''); end
    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
    
    %%%%%%%%%%%%%%%
    % Plot Shear
    figure(3);
    subtightplot(3,1,i,[0,0],0.05,0.05);
    plot(t,Shear3(i,:),'LineWidth',1.25); hold on;
    plot(t,Shear2(i,:),'--','LineWidth',1.25);
    plot(t,Shear1(i,:),'-.','LineWidth',1.25);
    plot(t,Shear_ext21(i,:),':','LineWidth',1.25);
    axis([0 t(Nphases) -0.018 0.043]);
    if i == 3; xlabel('$t$ [s]','Interpreter','latex'); end
    ylabel(['$\tau_s(x_',num2str(i),')$ [Pa]'],'Interpreter','latex');
    if i == 2; h = legend('Coarse','Medium','Fine','Extrapolated','location','northeast'); 
        set(h,'Interpreter','latex'); end
    if i ~= 3; set(gca,'XTickLabel',''); end
    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
    
    figure(4);
    subtightplot(3,1,i,[0,0],0.05,0.05);
    errorbar(t(1:Nth:end),Shear1(i,1:Nth:end),GCI_fine_Shear21(i,1:Nth:end),'LineWidth',1.25);
    axis([0 t(Nphases) -0.018 0.043]);
    if i == 3; xlabel('$t$ [s]','Interpreter','latex'); end
    ylabel(['$\tau_s(x_',num2str(i),')$ [Pa]'],'Interpreter','latex');
    if i ~= 3; set(gca,'XTickLabel',''); end
    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
    
    %%%%%%%%%%%%%%%
    % Plot u[HFS,y,t]
    count = 0; % Count for ColorOrderIndex
    for j=1:Nth_t:Nphases
        count = count + 1;
        figure(5);
%         subtightplot(3,1,i,[0,0],0.05,0.05);
        subtightplot(3,1,i,[0,0],0.075,0.175);
        set(gca,'ColorOrderIndex',count); plot(y3,-u3(i,:,j),'LineWidth',1.25); hold on;
        set(gca,'ColorOrderIndex',count); plot(y3,-u2(i,:,j),'--','LineWidth',1.25); 
        set(gca,'ColorOrderIndex',count); plot(y3,-u1(i,:,j),'-.','LineWidth',1.25); 
        set(gca,'ColorOrderIndex',count); plot(y3,-u_ext21(i,:,j),':','LineWidth',1.25);
        axis([0 max(y3) -5 1]);
        if i == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
        ylabel(['$-u(x_',num2str(i),')$ [m/s]'],'Interpreter','latex');
        if i == 2; h = legend('Coarse','Medium','Fine','Extrapolated','location','northeast'); 
            set(h,'Interpreter','latex'); end
        if i ~= 3; set(gca,'XTickLabel',''); end
        set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
        
        figure(6);
        subtightplot(3,1,i,[0,0],0.05,0.05);
        errorbar(y3,-u1(i,:,j),GCI_fine_u21(i,:,j),'LineWidth',1.25); hold on;
        axis([0 max(y3) -5 1]);
        if i == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
        ylabel(['$-u(x_',num2str(i),')$ [m/s]'],'Interpreter','latex');
        if i ~= 3; set(gca,'XTickLabel',''); end
        set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
    end
    
    %%%%%%%%%%%%%%%
    % Plot k[HFS,y,t]
    count = 0; % Count for ColorOrderIndex
    for j=1:Nth_t:Nphases
        count = count + 1;
        figure(7);
%         subtightplot(3,1,i,[0,0],0.05,0.05);
        subtightplot(3,1,i,[0,0],0.075,0.175);
        set(gca,'ColorOrderIndex',count); plot(y3,k3(i,:,j),'LineWidth',1.25); hold on;
        set(gca,'ColorOrderIndex',count); plot(y3,k2(i,:,j),'--','LineWidth',1.25); 
        set(gca,'ColorOrderIndex',count); plot(y3,k1(i,:,j),'-.','LineWidth',1.25); 
        set(gca,'ColorOrderIndex',count); plot(y3,k_ext21(i,:,j),':','LineWidth',1.25);
        axis([0 max(y3) 0 0.175]);
        if i == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
        ylabel(['$k(x_',num2str(i),')$ [m$^2$/s$^2$]'],'Interpreter','latex');
        if i == 2; h = legend('Coarse','Medium','Fine','Extrapolated','location','northeast'); 
            set(h,'Interpreter','latex'); end
        if i ~= 3; set(gca,'XTickLabel',''); end
        set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
        
        figure(8);
        subtightplot(3,1,i,[0,0],0.05,0.05);
        errorbar(y3,k1(i,:,j),GCI_fine_k21(i,:,j),'LineWidth',1.25); hold on;
        axis([0 max(y3) 0 0.175]);
        if i == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
        ylabel(['$k(x_',num2str(i),')$ [m$^2$/s$^2$]'],'Interpreter','latex');
        if i ~= 3; set(gca,'XTickLabel',''); end
        set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
    end
end

for i=1:8
    figure(i);
    set(findall(gcf,'type','axes'),'FontSize',10);
    set(findall(gcf,'type','text'),'FontSize',10);
    tightfig;
end

%% Save values and their uncertainties to plot with PIV and HFS data in TransientPlotsFromStatsAndCFD.m

save('TransientCFD_GCI_Output.mat','HF1','GCI_fine_HF21','Shear1','GCI_fine_Shear21',...
    'y1','y3','u1','u1all','GCI_fine_u21','k1','k1all','GCI_fine_k21');

toc;