clear all; close all; clc; dbstop if error

% This program will read in the Stats.m file from the centerline (z=0) inlet profile, calculate the momentum
% thickness delta2 assuming constant density (valid at inlet), and predict the virtual origin upstream.
%
% Blake Lance, 17 August 2015
%
% Note: it was found that the spatial resolution of the inlet data is insufficient for reliable results.

%% Inputs
Opposed = 1; % Whether the data are from an opposed case
proloc = 30; % Profile location in x to use
% proloc = 26; % Profile location in x to use
nu = 1.516E-5; % Kinematic viscosity at 20degC (Cengel Fluids) [m^2/s]

% Ask user to select input Stats.m file
[f1,p] = uigetfile('.mat');  % Ask user to choose Stat file
tic;    load([p,f1]);

%% Allocate and initialize

w = size(u_ave,2);
u = v_ave;  u(isnan(u)) = 0;    % v_ave in DaVis is u in global coordinates

if Opposed == 1 % Change the sign
    u = -u;
end

%% Find Wall Location
[imin,imax,jmin,jmax] = TrimToMask(u);

xlower = X(1,jmin);
xupper = X(1,jmax);
ylower = Y(imin,1);
yupper = Y(imax,1);

Xshift = X - xlower;    % Shift x_2D array so y=0 is at the left
xupper = xupper - xlower;   % Just changed this order
xlower = xlower - xlower;

%% Flip Xshift if Opposed==1, also shift values if the masked area isn't symmetric
if Opposed==1
    dx = abs( X(1,1) - X(1,2) );  % Spacing between vectors in mm given by DaVis
    Xshift = fliplr(Xshift);
    TrimR = w - jmax + 1;   TrimL = jmin;
    % Add or subtract values so assymetric Xshift will have consistent values
    Xshift = Xshift - dx*(TrimR - TrimL);
end

%% Plot u Profile
figure; plot(Xshift(proloc,:),u(proloc,:));
axis([0 305 0 1.1*max(u(proloc,:))]);
xlabel('$y$ [mm]','Interpreter','latex');
ylabel('$\overline{u}$ [m/s]','Interpreter','latex');
set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
axis square; tightfig;

%% Perform Calculations

% Use center value for free-stream velocity
i_infty = floor(w/2);
u_infty = u(proloc,i_infty); % [m/s]

% Eq 5-6 in Kays and Crawford, assuming constant density (multiplied by 0.5 since the integral counts both
% boundary layers)
delta2 = 0.5*abs(trapz(0.001*Xshift(proloc,:),u(proloc,:)/u_infty.*(1-u(proloc,:)/u_infty))); %[m]
Re_delta2 = u_infty*delta2/nu;

% Determine virtual origin (from Eq 11-22 in Kays and Crawford)
x = delta2^1.25*u_infty^0.25/(0.0157*nu^0.25); % [m]

toc;