clear all; close all; clc; dbstop if error;

% This program will import and plot heat flux data with uncertainty from Stats files from BC_Stats_2_2.m.
% It will also plot heat flux from correlations using the centerline wall temperature.
%
% Blake Lance, 23 June 2015
% Version 2, Blake Lance, 29 June 2015
%   Made code consider both orientations to calculate difference
% Version 2_1, Blake Lance, 30 June 2015
%   Combined aided, opposed, and difference plots into one figure
% Version 2_2, Blake Lance, 1 July 2015
%   Changed heat transfer correlation which gives higher heat flux for opposed than aided (opposite)
% Version 2_3, Blake Lance, 21 August 2015
%   Added unheated starting length to correlation from InletVirtualOrigin.m code

%% Inputs
% Opposed = 1;    % Boolean for Opposed or not
Correct = 0;    % Whether to increase measured flux by 18.5% to correct for nonuniform thermal resistance
u_inf = 2.44;   % Bulk velocity of mixed convection [m/s]

% Air properties from Fluid Mechanics, Cengel and Cimbala
Tinf = 22;      % Free-stream temperature [degC]
Ts = 138.6;     % Heated wall temperature [degC]
Tf = 0.5*(Tinf+Ts); % Film temperature [degC[
Pr = 0.7154;    % Prandtl number at 80C
nu = 2.097E-5;  % Kinematic viscosity at 80C [m^2/s]
beta = 1/(Tf+273.15); % Thermal coefficient of expansion [1/K]
k = 0.02953;    % Thermal conductivity [W/m*K]

% Unheated starting lengths
Xi = [0.4173; % S4_3_1, aided, mixed, z=0, at inlet [m]
    0.4853];  % S7_4_5, opposed, mixed, z=0, at Inlet [m]

FontSize = 11;
linecolor = {[0.000,  0.447,  0.741]; [0.850,  0.325,  0.098]; [0.929,  0.694,  0.125]};

% Ask user to select heat flux data files
[file1,path1] = uigetfile('*.csv','Select Buoyancy-aided Heat Flux Stats');
[file2,path2] = uigetfile('*.csv','Select Buoyancy-opposed Heat Flux Stats');
tic;

%% Import Experimental Heat Flux
data1 = importdata([path1,file1]);
data2 = importdata([path2,file2]);

x1 = data1.data(:,1);
Flux1 = data1.data(:,4);
U_Flux1 = data1.data(:,7);
if Correct == 1; Flux1 = 1.185*Flux1; U_Flux1 = 1.185*U_Flux1; end

x2 = data2.data(:,1);
Flux2 = data2.data(:,4);
U_Flux2 = data2.data(:,7);
if Correct == 1; Flux2 = 1.185*Flux2; U_Flux2 = 1.185*U_Flux2; end

%% Calculate Heat Flux From Correlation
xi = linspace(0.001,2+max(Xi),100); % x-values for correlations [m]
xi1 = xi + Xi(1); % Define new coordinates by adding unheated starting length
xi2 = xi + Xi(2); % Define new coordinates by adding unheated starting length
g = 9.81;   % Acceleration due to gravity [m/s^2]
Re_x = u_inf*xi./nu; % Local Reynolds number
Re = u_inf*0.305/nu; % Bulk Reynolds number
Gr_x = g*beta*(Ts-Tinf)*xi.^3/nu^2; % Local Grashof number

% Correlations from Handbook of Single-Phase Convective Heat Transfer; Kakac, Shah, and Aung
% Other correlations from Studies of Mixed Convection in Vertical Tubes, Jackson
Nu_Fx = 0.0287*Re_x.^(4/5)*Pr^0.6; % Forced convection Nu (Eqn 14.40)
% Nu_Nx = 0.15*Gr_x.^(1/3)*Pr^(1/3)*(1+(0.492/Pr)^(9/16))^(-16/27); % Natural convection Nu (Eqn 14.41)
% Nu_Tx1 = ( Nu_Fx.^3 + Nu_Nx.^3 ).^(1/3); % Total Nu for aided (Eqn 14.11)
% Nu_Tx2 = ( Nu_Fx.^3 - Nu_Nx.^3 ).^(1/3); % Total Nu for opposed (Eqn 14.11)
Nu_Tx1 = Nu_Fx.*abs(1-10^4*Gr_x./(Re_x.^2.7*Pr^0.5)).^0.46; % Total Nu for aided (Eqn 6)
Nu_Tx2 = Nu_Fx.*abs(1+10^4*Gr_x./(Re_x.^2.7*Pr^0.5)).^0.46; % Total Nu for opposed (Eqn 6)
% Update Nu_Tx by using the unheated starting length theory, Incropera Heat Transfer (Eqn 7.43)
Nu_Tx1_UH = abs(Nu_Tx1./(1-(Xi(1)./xi).^(9/10)).^(1/9)); % Nu_Tx with unheated starting length
Nu_Tx2_UH = abs(Nu_Tx2./(1-(Xi(2)./xi).^(9/10)).^(1/9)); % Nu_Tx with unheated starting length
hx1 = Nu_Tx1*k./xi; % Local heat transfer coefficient [W/m^2*K]
hx2 = Nu_Tx2*k./xi; % Local heat transfer coefficient [W/m^2*K]
hx1_UH = Nu_Tx1_UH*k./xi; % Local heat transfer coefficient [W/m^2*K]
hx2_UH = Nu_Tx2_UH*k./xi; % Local heat transfer coefficient [W/m^2*K]
Flux_corr1 = hx1*(Ts-Tinf); % Correlation prediction for heat flux [W/m^2]
Flux_corr2 = hx2*(Ts-Tinf); % Correlation prediction for heat flux [W/m^2]
Flux_corr1_UH = hx1_UH*(Ts-Tinf); % Correlation prediction for heat flux [W/m^2]
Flux_corr2_UH = hx2_UH*(Ts-Tinf); % Correlation prediction for heat flux [W/m^2]

%% Plotting

% Plot Fluxes
figure(1); set(gcf,'Units','inches','Position',[4 6 3.5 3]);
errorbar(x1,Flux1,U_Flux1,'-','Color',linecolor{1},'LineWidth',1.25); hold on;
% plot(xi,Flux_corr1,'--+','LineWidth',1.25);
line_fewer_markers(xi-Xi(1),Flux_corr1_UH,15,'x','Color',linecolor{1},'LineWidth',1.25,'MarkerSize',7);
% plot(xi,Flux_corrM,'LineWidth',1.25);
errorbar(x2,Flux2,U_Flux2,'-.','Color',linecolor{2},'LineWidth',1.25);
% plot(xi,Flux_corr2,'--o','LineWidth',1.25);
line_fewer_markers(xi-Xi(2),Flux_corr2_UH,15,'o','Color',linecolor{2},'LineWidth',1.25,'MarkerSize',7);
plot(x1,Flux2-Flux1,'--','Color',linecolor{3},'LineWidth',1.25);
% plot(xi,Flux_corr1-Flux_corr2,'--*','LineWidth',1.25);
line_fewer_markers(xi-Xi(2),Flux_corr2_UH-interp1(xi,Flux_corr1_UH,xi-(Xi(1)-Xi(2)),'spline'),...
    15,'*','Color',linecolor{3},'LineWidth',1.25,'MarkerSize',7);
axis([0 1.8 0 3200]);
hleg1 = legend('HFS--Aid','Corr--Aid','HFS--Opp','Corr--Opp','HFS--(Opp-Aid)','Corr--(Opp-Aid)');
Pos = get(hleg1,'Position');
set(hleg1,'Location','northeast','Interpreter','latex','Position',[1.075*Pos(1) 1.075*Pos(2) Pos(3) Pos(4)]);
xlabel('$x$ [m]','Interpreter','latex');
ylabel('$q''''$ [W/m$^2$]','Interpreter','latex');

% Plot Nu_Tx and Nu_TxUH together
figure(2); set(gcf,'Units','inches','Position',[8 6 3.5 3]);
plot(xi-Xi(1),Nu_Tx1,'Color',linecolor{1},'LineWidth',1.25); hold on;
plot(xi-Xi(1),Nu_Tx1_UH,'-.','Color',linecolor{1},'LineWidth',1.25);
plot(xi-Xi(2),Nu_Tx2,'Color',linecolor{2},'LineWidth',1.25);
plot(xi-Xi(2),Nu_Tx2_UH,'-.','Color',linecolor{2},'LineWidth',1.25);
hleg2 = legend('Aid - w/o UH','Aid - w/ UH','Opp - w/o UH','Opp - w/UH');
set(hleg2,'Interpreter','latex','Location','Best');
xlabel('$x$ [m]','Interpreter','latex'); ylabel('Nu$_x$','Interpreter','latex');
axis([-inf 1.8 0 950]);

% Plot flux from heated or unheated correlations
figure(3); set(gcf,'Units','inches','Position',[12 6 3.5 3]);
plot(xi-Xi(1),hx1,'Color',linecolor{1},'LineWidth',1.25); hold on;
plot(xi-Xi(1),hx1_UH,'-.','Color',linecolor{1},'LineWidth',1.25);
plot(xi-Xi(2),hx2,'Color',linecolor{2},'LineWidth',1.25);
plot(xi-Xi(2),hx2_UH,'-.','Color',linecolor{2},'LineWidth',1.25);
hleg3 = legend('Aid - w/o UH','Aid - w/ UH','Opp - w/o UH','Opp - w/UH');
set(hleg3,'Interpreter','latex','Location','Best');
xlabel('$x$ [m]','Interpreter','latex'); ylabel('$h_x$ [W/m$^2$]','Interpreter','latex');
axis([-inf 1.8 0 38]);

% Fix up plots
for i=1:3
    figure(i);
    set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
    set(findall(gcf,'type','axes'),'FontSize',FontSize);
    set(findall(gcf,'type','text'),'FontSize',FontSize);
    set(findall(gcf,'type','legend'),'FontSize',FontSize-1);
    tightfig;
end

toc;