clear all; close all; clc;

% This program will read in temperature data from several text files and calculate the heat flux using a
% reference thermal conductivity of air.
%
% Make sure to run the BC_Stats2.m code first to calculate the average wall temperature.
%
% Author:   Blake Lance
% Date:     31 December 2013
%
% 5 September 2014 - Added ability to read wall temperatures at any HFS location
% Version 2, Blake Lance, 18 June 2015
%   Added u_tau array from S7 Mixed IsoT Opposed from linear fit
%   Added interpolation feature for wall temperature at given x location
%   Updated plots
% Version 2_1, Blake Lance, 22 June 2015
%   Considers uncertainty in position as uncertainty in temperature
% Version 2_2, Blake Lance, 23 June 2015
%   Writes temperature and uncertainty to csv file
% Version 2_3, Blake Lance, 20 July 2015
%   Now loops through temperature profiles at all three x locations and plots the data together
%   Changed temperature bias uncertainty to 2.2 degC from standard type K TC and 0.8 degC from DAQ
%   Added profiles for all x locations to nondimensional plot
%   Added variable to change the number of repeats on the analysis

%% Program inputs
N_Pos = 16; % The number of positions and subsequent temperature files
N_Rep = 3;  % The number of data sets to input (often 3, but 1 can be useful)
% HFS = 1;    % If N_Rep == 1, define where in space to look
Export = 0; % Export data to file
Pr = 0.7;    % Prandtl number of air

N = 250;    % The number of samples at each position
% B_T = 0.1*ones(N_Pos,1);    % The bias uncertainty of temperature [deg C]
% B_T = ones(N_Pos,1);    % The bias uncertainty of temperature [deg C]
B_T = sqrt(2.2^2+0.8^2)*ones(N_Pos,1);    % The bias uncertainty of temperature [deg C]
% Uncertainty in relative position, comes from resolution of 1/2 step on motor with 200 steps/rev and a lead
%   screw pitch of 1 mm [m]
U_y = 0.0000025;
U_Ty = 0.1*ones(N_Pos,1); % Initial guess for position uncertainty's influence on temperature uncert. [deg C]
U_k_k = 0.02;   % The relative uncertainty of k (U_k/k)
y_plus_max = 6.6; % The maximum y_plus to consider for the line fit
LineWidth = 1.25; % Plot line width
FontSize = 11; % Plot font size
linestyle = {'-';'-.';'--'};
linecolor = {[0.000,  0.447,  0.741]; [0.850,  0.325,  0.098]; [0.929,  0.694,  0.125];...
    [0.494,  0.184,  0.556]; [0.466,  0.674,  0.188]};
x = [0.1617; 0.7777; 1.3936]; % x-coordinate array [m]

Trim = N; % The number of data points to keep to test convergence

%% Loop through data
data_str = zeros(N_Rep,N,2,N_Pos);
T_str = zeros(N_Rep,N_Pos);
P_T_str = zeros(N_Rep,N_Pos);
U_T_str = zeros(N_Rep,N_Pos);
y_str = zeros(N_Rep,N_Pos);
T_plus_str = zeros(N_Rep,N_Pos);
y_m_str = zeros(N_Rep,N_Pos);
y_plus_str = zeros(N_Rep,N_Pos);
Ts_exp_str = zeros(N_Rep,1);
Ts_fit_str = zeros(N_Rep,1);
dT_dy_str = zeros(N_Rep,1);
start_str = zeros(N_Rep,1);
ending_str = zeros(N_Rep,1);

for k=1:N_Rep
    %% Initialize variables
    data = zeros(N,3,N_Pos);    % All position and temperature data
    
    %% Read data files
    folder = uigetdir;  % select folder that contains temperature files
    
    for i=1:N_Pos
        data(:,:,i) = importdata([folder,'\TC Probe_',int2str(i),'.lvm']);
    end
    
    %% If the first column is NaNs, remove it
    if isnan( data(1,1,1) ); data = data(:,2:3,:); end
    
    %% Determine u_tau by the Series Number
    if isempty(strfind(folder,'7_3_')) == 0
        u_tau_all = [0.1818,0.1948,0.1567]; % u_tau for S7_3, Forced IsoT Opposed
    elseif isempty(strfind(folder,'7_4_')) == 0
        u_tau_all = [0.1093,0.1274,0.1092]; % u_tau for S7_4, Mixed IsoT Opposed
    end
    if N_Rep == 1
        index = strfind(folder,'\');
        HFS = str2double(folder(index(end)-1));
        u_tau_all = u_tau_all(HFS); % Determine x location by the end of the folder name
    end
    
    %% Read wall temperature at HFS location
%     data_temp = importdata([folder,'\Heated_Wall_MEAN_SD_PU.csv']);
    data_temp = importdata([folder,'\HeatedWallTemp_Stats.csv']);
    iCL = find(data_temp.data(:,3) == 0); % Find centerline TC indeces
    xCL = data_temp.data(iCL,1); % x values along centerline
    TempCL = data_temp.data(iCL,4); % Temperature along centerline
    % Interpolate centerline temperatures to current x location and convert to deg C
    Ts_exp = interp1(xCL,TempCL,x(k)) - 273.15;
    Ts_exp_str(k) = Ts_exp;
    
    %% Assign data
    
    % Calculate time-average temperature
    data_ave = squeeze(mean(data(1:Trim,:,:),1))';
    y = data_ave(:,1);
    y_m = 0.001*y;  % Get y array in meters
    T = data_ave(:,2);
    T_str(k,:) = data_ave(:,2);
    
    % Calculate standard deviation of temperature
    T_SD = squeeze(std(data(1:Trim,2,:),1));
    
    % Calculate uncertainty of temperature
    P_T = 1.96*T_SD/sqrt(Trim);    % Precision uncertainty
    P_T_str(k,:) = 1.96*T_SD/sqrt(Trim);    % Precision uncertainty
    U_T = sqrt( B_T.^2 + P_T.^2 + U_Ty.^2 );  % Total uncertainty
    
    %% Determine fluid properties
    
    % Calculate the thermal conductivity of air at each position
    k_air = 6.765E-5*(T+273.15) + 5.886E-3;
    
    % Dynamic viscosity vector by Sutherland's law for air
    mu_0 = 1.716E-5;    % [kg/(m*s)]
    T0 = 273.15;    % [K]
    S = 110.4;  % [K]
    mu = mu_0*((T+273.15)/T0).^(3/2)*( T0 + S )./( (T+273.15) + S );    % [kg/(m*s)]
    
    % Density of air by the ideal gas law
    P = 86000;  % [Pa]
    R = 287.058;    % [J/(kg*K)]
    rho = P./(R*(T+273.15)); % [kg/m^3]
    
    % Kinematic viscosity of air
    nu = mu./rho;   % [m^2/s]
    
    % Specific heat of air c_p at 80 deg C
    c_p = 1008; % [J/(kg*K)]
    
    %% Plot temperature as a function of position
    figure(1); errorbar(y,T,U_T);
    axis([-inf 3 -inf inf]);
    
    % Ask user where reliable data begin and end
    commandwindow; % Take user to Command Window for input
    start = input('Type the index where reliable data begin \n');
    start_str(k) = start;
    ending = 8;    % Initial guess for ending
    close;    % Close figure
    
    if k == 1; tic; end
    
    dT_dy = -5E4; % Initial guess for temperature gradient
    for i=1:100
        %% Consider Position Uncertainty in Temperature Uncertainty
        U_Ty = U_y*abs(dT_dy); % Estimate/update uncertainty of temperature from position
        U_T = sqrt( B_T.^2 + P_T.^2 + U_Ty.^2 );  % Total uncertainty with updated U_Ty
        U_T_str(k,:) = sqrt( B_T.^2 + P_T.^2 + U_Ty.^2 );  % Total uncertainty with updated U_Ty

        %% Fit a line to the reliable temperature data
        
        % % Built-in fit function
        % dT_dy_fit = fit(y_m(start:ending),T(start:ending),'poly1');
        % dT_dy = dT_dy_fit(1);
        % Tw = dT_dy_fit(0);
        
        % Perform Linear Regression for Temperature Gradient
        Delta = sum(1./U_T(start:ending).^2)*sum(y_m(start:ending).^2./U_T(start:ending).^2) ...
            - sum(y_m(start:ending)./U_T(start:ending).^2)^2;
        % T_wall (y-intercept) [deg C]
        Ts_fit = 1/Delta*( sum(y_m(start:ending).^2./U_T(start:ending).^2)*...
            sum(T(start:ending)./U_T(start:ending).^2) ...
            - sum(y_m(start:ending)./U_T(start:ending).^2)*...
            sum(y_m(start:ending).*T(start:ending)./U_T(start:ending).^2) );
        Ts_fit_str(k) = Ts_fit;
        % dT_dy [deg C/m]
        dT_dy = 1/Delta*( sum(1./U_T(start:ending).^2)*...
            sum(y_m(start:ending).*T(start:ending)./U_T(start:ending).^2) ...
            - sum(y_m(start:ending)./U_T(start:ending).^2)*sum(T(start:ending)./U_T(start:ending).^2) );
        dT_dy_str(k) = dT_dy;
        % %% Add temperature line fit to plot
        % plot([y(start),y(ending)], ...
            %[Ts_fit+(y_m(start)-y_m(1))*dT_dy, Ts_fit+(y_m(ending)-y_m(1))*dT_dy],'r');
        % plot(0,Ts_exp,'*'); plot(0,Ts_fit,'r*');
        % xlim([0 1]);    xlabel('y [mm]');   ylabel('T [^\circC]');
        % legend('Experimental Temperatures','Fit to Temperatures','Experimental Wall Temperature',...
        %     'Fit Wall Temperature','location','best');
        % title(['q'''' = ', num2str(Flux,4), ' \pm ', num2str(U_Flux,2),...
        %     ' W/m^2, U_{q''''} = ', num2str(100*U_Flux/Flux,3), ' %']);
        
        %% Correct y positions
        y0 = 0.5*( Ts_exp - Ts_fit )/dT_dy;  % Predict error in y of wall, relaxed a little [m]
        y = y - 1000*y0;    % [mm]
        y_str(k,:) = y - 1000*y0;    % [mm]
        y_m = y_m - y0; % [m]
        y_m_str(k,:) = y_m;
        
        % Calculate y+
        y_plus_str(k,:) = y_m*u_tau_all(k)./nu;   % The offset y0 was previously considered
        
        %% Find the number of data points within y_plus_max
        ending = find(y_plus_str(k,:)<y_plus_max,1,'last');
        ending_str(k) = ending;
        
        if abs( Ts_fit - Ts_exp ) < 1E-12;  break;    end
    end
    
    %% Calculate heat flux and uncertainty
    k_air_ave = mean(k_air(start:ending));
    Flux = -k_air_ave*dT_dy;
    
    % Uncertainty in dT_dy
    U_dT_dy = sqrt(1/Delta*sum(1./U_T.^2));
    % Uncertainty in heat flux
    U_Flux = sqrt( (U_k_k*Flux)^2 + (k_air_ave*U_dT_dy)^2 );
    
    %% Calculate T+
    T_plus_str(k,:) = ( Ts_exp - T )*u_tau_all(k)./(Flux./(rho*c_p));
    
    %% Store data
    data_str(k,:,:,:) = data;
    
    %% Display outputs to screen
    fprintf(['start = ',num2str(start),'\n']);
    fprintf(['ending = ',num2str(ending),'\n']);
    fprintf(['q'''' = ', num2str(Flux,4), ' W/m^2 \n']);
    fprintf(['U_q'''' = ', num2str(U_Flux,3),' W/m^2 \n']);
    
end

%% Plot results

if N_Rep > 1
    % Temporarily store data so it's easy to index when plotting
    y1 = y_str(1,:); y2 = y_str(2,:); y3 = y_str(3,:);
    y_m1 = y_m_str(1,:); y_m2 = y_m_str(2,:); y_m3 = y_m_str(3,:);
    y_plus1 = y_plus_str(1,:); y_plus2 = y_plus_str(2,:); y_plus3 = y_plus_str(3,:);
    T_plus1 = T_plus_str(1,:); T_plus2 = T_plus_str(2,:); T_plus3 = T_plus_str(3,:);
    T1 = T_str(1,:); T2 = T_str(2,:); T3 = T_str(3,:);
    U_T1 = U_T_str(1,:); U_T2 = U_T_str(2,:); U_T3 = U_T_str(3,:);
    Ts_exp1 = Ts_exp_str(1); Ts_exp2 = Ts_exp_str(2); Ts_exp3 = Ts_exp_str(3);
    Ts_fit1 = Ts_fit_str(1); Ts_fit2 = Ts_fit_str(2); Ts_fit3 = Ts_fit_str(3);
    dT_dy1 = dT_dy_str(1); dT_dy2 = dT_dy_str(2); dT_dy3 = dT_dy_str(3);
    start1 = start_str(1); start2 = start_str(2); start3 = start_str(3);
    ending1 = ending_str(1); ending2 = ending_str(2); ending3 = ending_str(3);
else
    % Temporarily store data so it's easy to index when plotting
    y1 = y_str(1,:);
    y_m1 = y_m_str(1,:);
    y_plus1 = y_plus_str(1,:);
    T_plus1 = T_plus_str(1,:);
    T1 = T_str(1,:);
    U_T1 = U_T_str(1,:);
    Ts_exp1 = Ts_exp_str(1);
    Ts_fit1 = Ts_fit_str(1);
    dT_dy1 = dT_dy_str(1);
    start1 = start_str(1);
    ending1 = ending_str(1);
end

% Plot temperature again with corrected positions
figure(1); hold on;
errorbar(y1,T1,U_T1,'LineStyle',linestyle{1},'Color',linecolor{1},'LineWidth',LineWidth);
if N_Rep > 1
    errorbar(y2,T2,U_T2,'LineStyle',linestyle{2},'Color',linecolor{2},'LineWidth',LineWidth);
    errorbar(y3,T3,U_T3,'LineStyle',linestyle{3},'Color',linecolor{3},'LineWidth',LineWidth);
end
set(gcf,'Units','inches','Position',[6 6 3.5 2.75]);
% plot([y(start),y(ending)], [Ts_fit+y_m(start)*dT_dy,Ts_fit+y_m(ending)*dT_dy],...
%     'Color',[0.25 0.25 0.25],'LineWidth',LineWidth);
% plot(0,Ts_exp,'o','Color',[0.635,  0.078,  0.184],'LineWidth',LineWidth);
% plot(0,Ts_fit,'r*','LineWidth',LineWidth);
axis([0 inf 15 147.5]);
xlabel('$y$ [mm]','Interpreter','latex');
ylabel('$T$ [$^\circ$C]','Interpreter','latex');
leg1 = legend('$x_1$','$x_2$','$x_3$');
set(leg1,'Location','northeast','Interpreter','latex');
set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
set(findall(gcf,'type','axes'),'FontSize',FontSize);
set(findall(gcf,'type','text'),'FontSize',FontSize);
set(findall(gcf,'type','legend'),'FontSize',FontSize-1);
box on;
tightfig;

% Plot temperature profile near heated wall
figure(2); hold on; set(gcf,'Units','inches','Position',[10 6 3.5 2.75]);
% errorbar(y(y<10),T(y<10),U_T(y<10),'LineStyle',linestyle{HFS},'Color',linecolor{HFS},'LineWidth',LineWidth);
errorbar(y1(y1<10),T1(y1<10),U_T1(y1<10),'LineStyle',linestyle{1},'Color',linecolor{1},'LineWidth',LineWidth);
if N_Rep > 1
    errorbar(y2(y2<10),T2(y2<10),U_T2(y2<10),...
        'LineStyle',linestyle{2},'Color',linecolor{2},'LineWidth',LineWidth);
    errorbar(y3(y3<10),T3(y3<10),U_T3(y3<10),...
        'LineStyle',linestyle{3},'Color',linecolor{3},'LineWidth',LineWidth);
end
plot([y1(start1),y1(ending1)], [Ts_fit1+y_m1(start1)*dT_dy1,Ts_fit1+y_m1(ending1)*dT_dy1],...
    'Color',[0.25 0.25 0.25],'LineWidth',LineWidth);
plot(0,Ts_exp1,'o','Color',[0.635,  0.078,  0.184],'LineWidth',LineWidth);
if N_Rep > 1; 
    plot([y2(start2),y2(ending2)], [Ts_fit2+y_m2(start2)*dT_dy2,Ts_fit2+y_m2(ending2)*dT_dy2],...
        'Color',[0.25 0.25 0.25],'LineWidth',LineWidth);
    plot(0,Ts_exp2,'o','Color',[0.635,  0.078,  0.184],'LineWidth',LineWidth);
    plot([y3(start3),y3(ending3)], [Ts_fit3+y_m3(start3)*dT_dy3,Ts_fit3+y_m3(ending3)*dT_dy3],...
        'Color',[0.25 0.25 0.25],'LineWidth',LineWidth);
    plot(0,Ts_exp3,'o','Color',[0.635,  0.078,  0.184],'LineWidth',LineWidth);
end
% plot(0,Ts_fit,'r*','LineWidth',LineWidth);
% axis([0 2*y(ending) 50 147.5]);
axis([0 2.3 50 147.5]);
xlabel('$y$ [mm]','Interpreter','latex');
ylabel('$T$ [$^\circ$C]','Interpreter','latex');
if N_Rep > 1
    leg2 = legend('$x_1$','$x_2$','$x_3$','Line Fit','$T_s$');
else
    leg2 = legend('$x_1$','Line Fit','$T_s$');
end
set(leg2,'Location','northeast','Interpreter','latex');
% title(['q'''' = ', num2str(Flux,4), ' \pm ', num2str(U_Flux,2),...
%     ' W/m^2, U_{q''''} = ', num2str(100*U_Flux/Flux,3), ' %']);
set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
set(findall(gcf,'type','axes'),'FontSize',FontSize);
set(findall(gcf,'type','text'),'FontSize',FontSize);
set(findall(gcf,'type','legend'),'FontSize',FontSize-1);
box on;
tightfig;

% Plot temperature on semilog axis to check for transition region
figure(3); 
set(gcf,'Units','inches','Position',[6 1 3.5 3.25]);
semilogx(y_plus1(y_plus1<13.2),Pr*y_plus1(y_plus1<13.2),':',...
    'Color',linecolor{4},'LineWidth',1.5*LineWidth); hold on;
semilogx(y_plus1(y_plus1>12),2.075*log(y_plus1(y_plus1>12))+3.9,':',...
    'Color',linecolor{5},'LineWidth',1.5*LineWidth);
semilogx(y_plus1,T_plus1,'LineStyle',linestyle{1},'Color',linecolor{1},'LineWidth',LineWidth);
if N_Rep > 1
    semilogx(y_plus2,T_plus2,'LineStyle',linestyle{2},'Color',linecolor{2},'LineWidth',LineWidth);
    semilogx(y_plus3,T_plus3,'LineStyle',linestyle{3},'Color',linecolor{3},'LineWidth',LineWidth);
end
axis([0.1 2000 0 28]);
xlabel('$y^+$','Interpreter','latex');
ylabel('$T^+$','Interpreter','latex');
if N_Rep > 1
    leg2 = legend('$T^+ = \: $Pr$ \: y^+$','$T^+ = 2.075$ ln$(y^+) + 3.9$','$x_1$','$x_2$','$x_3$');
else
    leg2 = legend('$T^+ = \: $Pr$ \: y^+$','$T^+ = 2.075$ ln$(y^+) + 3.9$','$x_1$');
end
set(leg2,'Location','northwest','Interpreter','latex');
set(gca,'XMinorTick','on','YMinorTick','on','TickLabelInterpreter','latex');
set(gca,'XTick',[0.1 1 10 100 1000]);
set(findall(gcf,'type','axes'),'FontSize',FontSize);
set(findall(gcf,'type','text'),'FontSize',FontSize);
set(findall(gcf,'type','legend'),'FontSize',FontSize-1);
tightfig;

% % Plot the standard deviation of temperature as a function of position
% figure; plot(data_ave(:,1),T_SD);
% xlabel('y [mm]');   ylabel('SD_T [^\circC]');
%
% % Plot the thermal conductivity of air as a function of position
% figure; plot(data_ave(:,1),k_air);
% xlabel('y [mm]');   ylabel('k_{air} [W/m*K]');

%% Export Data to Files
if Export == 1
    header = {'x[m]','y[m]','z[m]','T[C]','B_T[C]','S_T[C]','U_T[C]'};
    for k=1:N_Rep
        D = [repmat(x(k),N_Pos,1),0.001*y_str(k,:)',zeros(N_Pos,1),T_str(k,:)',B_T,P_T_str(k,:)',U_T_str(k,:)'];
        csvwrite_with_headers([pwd,'\T_x',int2str(k),'.csv'],D,header);
    end
end

toc;