close all; clear all; clc;

% Code to interpolate the PIV data to a fine grid for input to Star-CCM+.
% This code reads the uvw, epsilon, and k files generated from the Inlet
% Analysis code and interpolates the data to a grid defined in the code.
%  The first user input is the opposed variable below.

% Updates:
% June 12, 2014 - Jeff Harris - updated comments
% Sept 17, 2014 - Blake Lance - Added option to pad the sides with zeros
    % Simplified the selection of files
    % Removed padding with zeros
    % Improved plots
    % Changed the definition of z since the incoming data won't be mapped orthogonally
    % Reinstated the orthogonal mapping and changed the operation if Opposed
    % Changed TriScatteredInterp function to scatteredInterpolant for future use
    % Changed to map the top wall data before the interpolation (still has dimpling effect that need to be
    %   corrected, and is only taking the correct data for Opposed)
    % Removed the omega2 option for the transition model
% version 4_2, Sept 25, 2014 - Blake Lance
    % Added section to manually set the velocity profile.  This is to test for Inlet having positive and
    %   negative u velocity such as when in the Opposed orientation and under natural convection.
    % Changed interpolation extrapolation to 'nearest' (removes zeros in the corners)
% version 4_3, Jan 12, 2015 - Blake Lance
    % Since PIV near walls is often biased high, added option to remove BL data and linearly interpolate
% version 4_4, May 26, 2015 - Blake Lance
    % Added profile lines, improved plot appearance.  Requires MATLAB2014b or higher.
    
%% User Input

Export = 1; % Export the output files?
Opposed = 0;% If buoyancy opposed, set opposed to 1
Manual = 0; % Whether to manually set the u velocity
BLdel = 2; % The number of data points to delete in the boundary layer (BL), can be 0

%% Read in the uvw, k, and epsilon files to be interpolated

% Ask user to choose velocity file
[f1,p] = uigetfile('*uvw.csv','Select the *.csv file for the average velocity');
uvwdata = xlsread([p,f1]);
x_o = uvwdata(:,1);  y_o = uvwdata(:,2);  z_o = uvwdata(:,3);
% u_o = uvwdata(:,4);  v_o = uvwdata(:,5);  w_o = uvwdata(:,6);

[f1,~] = uigetfile('*k.csv','Select the *.csv file for k',p);  % Ask user to choose k file
kdata = xlsread([p,f1]);
% k_o = kdata(:,4);

[f1,~] = uigetfile('*epsilon.csv','Select the *.csv file for epsilon',p);  % Ask user to choose epsilon file
epsdata = xlsread([p,f1]);
% eps_o = epsdata(:,4);

tic;

%% Count how many vectors are in a single profile
N = 1; i = 1;
if Opposed == 1
    while y_o(i) > y_o(i+1)
        N = N + 1;
        i = i + 1;
    end
else
    while y_o(i) < y_o(i+1)
        N = N + 1;
        i = i + 1;
    end
end

%% Define y, z, and omega

% Define the y coordinates
N0 = size(y_o,1);
y_1 = y_o(1:N);

% Define the z coordinates
z_1 = y_1 - max(y_1)/2;

omega(1:N0,:) = epsdata;   % Assign the coordinates to omega
omega(1:N0,4) = epsdata(1:N0,4)./kdata(1:N0,4);    % Then assign omega
omega(isnan(omega)) = 0;  % Replace NaNs with zeros

%% Define the Fine Grid

% Use the vector spacing of the PIV data, no fine interpolation
y = y_1;
z = z_1;

% y = [0.00:0.0002:0.3048]';
% z = (-0.1524:((0.1524*2)/(values-1)):0.1524)';

%% Remove PIV data near walls that is biased

% Specify how much of the measured data to write (must be more than the relative BL thickness)
Npart = floor(0.25*N);
Nplanes = round(N0/N); % The number of measured planes in the incoming data
uvwdata(N0+1:N0+2*Npart*Nplanes,1) = 0;   % The x-values are all 0
kdata(N0+1:N0+2*Npart*Nplanes,1) = 0;   % The x-values are all 0

BLi = ones(length(uvwdata),1);  % Boolean array for data to be interpolated
if BLdel > 0
    for i=1:Nplanes
        BLi(2+N*(i-1):BLdel+N*(i-1)+1) = 0;
        BLi((2+N*(i-1):BLdel+N*(i-1)+1)+N-BLdel-2) = 0;
    end
end

%% Map the data near the top wall orthogonally to the two side walls

% Run through the data files and map the top wall data to the left and right walls
for i=1:Nplanes
    TWi = 1+N*(i-1):Npart+N*(i-1);  % Top wall (unheated) indeces for input
    LWo = N0+1+2*Npart*(i-1):N0+Npart+2*Npart*(i-1);    % Left wall (unheated) indeces for output
    RWo = N0+1+Npart+2*Npart*(i-1):N0+2*Npart+2*Npart*(i-1);    % Right wall (unheated) indeces for output
    
    BLi(LWo(2:2+BLdel-1)) = 0;
    BLi(LWo(2:2+BLdel-1)+Npart-BLdel-2) = 0;
    BLi(RWo(2:2+BLdel-1)) = 0;
    BLi(RWo(2:2+BLdel-1)+Npart-BLdel-2) = 0;    

    uvwdata(LWo,2) = -uvwdata(TWi,3) + max(y_1)/2;  % Map z onto y on left
    uvwdata(LWo,3) = -uvwdata(TWi,2) + max(y_1)/2;  % Map y onto z on left
    uvwdata(LWo,4:end) = uvwdata(TWi,4:end);    % Copy the velocity data over to the left
    uvwdata(RWo,2) = uvwdata(TWi,3) + max(y_1)/2;   % Map z onto y on right
    uvwdata(RWo,3) = uvwdata(TWi,2) - max(y_1)/2;   % Map y onto z on right
    uvwdata(RWo,4:end) = uvwdata(TWi,4:end);    % Copy the velocity data over to the right
    
    kdata(LWo,2) = -kdata(TWi,3) + max(y_1)/2;  % Map z onto y on left
    kdata(LWo,3) = -kdata(TWi,2) + max(y_1)/2;  % Map y onto z on left
    kdata(LWo,4:end) = kdata(TWi,4:end);    % Copy the k data over to the left
    kdata(RWo,2) = kdata(TWi,3) + max(y_1)/2;   % Map z onto y on right
    kdata(RWo,3) = kdata(TWi,2) - max(y_1)/2;   % Map y onto z on right
    kdata(RWo,4:end) = kdata(TWi,4:end);    % Copy the k data over to the right
    
    epsdata(LWo,2) = -epsdata(TWi,3) + max(y_1)/2;  % Map z onto y on left
    epsdata(LWo,3) = -epsdata(TWi,2) + max(y_1)/2;  % Map y onto z on left
    epsdata(LWo,4:end) = epsdata(TWi,4:end);    % Copy the epsilon data over to the left
    epsdata(RWo,2) = epsdata(TWi,3) + max(y_1)/2;   % Map z onto y on right
    epsdata(RWo,3) = epsdata(TWi,2) - max(y_1)/2;   % Map y onto z on right
    epsdata(RWo,4:end) = epsdata(TWi,4:end);    % Copy the epsilon data over to the right
    
    omega(LWo,2) = -omega(TWi,3) + max(y_1)/2;  % Map z onto y on left
    omega(LWo,3) = -omega(TWi,2) + max(y_1)/2;  % Map y onto z on left
    omega(LWo,4:end) = omega(TWi,4:end);    % Copy the omega data over to the left
    omega(RWo,2) = omega(TWi,3) + max(y_1)/2;   % Map z onto y on right
    omega(RWo,3) = omega(TWi,2) - max(y_1)/2;   % Map y onto z on right
    omega(RWo,4:end) = omega(TWi,4:end);    % Copy the omega data over to the right
end

%% Interpolate the data to a fine grid

BLi2 = find(BLi==1);    % Find indeces where good data are
[qx,qy] = meshgrid(z,y);

% Fu = scatteredInterpolant(uvwdata(:,3),uvwdata(:,2),uvwdata(:,4),'linear','nearest');
Fu = scatteredInterpolant(uvwdata(BLi2,3),uvwdata(BLi2,2),uvwdata(BLi2,4),'linear','nearest');
u = Fu(qx,qy);
u(isnan(u))=0;
figure;  set(gcf,'units','inches','Position',[2 6 3.25 3]);
contourf(1000*z,1000*y,u,'edgecolor','none'); hold on;
z_uni = unique(z_o); % Determine the measurement planes or unique z values
for i=1:length(z_uni) % Plot measurement planes
%     set(gca,'ColorOrderIndex',7);
    plot(1000*[z_uni(i) z_uni(i)],1000*[min(y_o((i-1)*N+1:i*N)) max(y_o((i-1)*N+1:i*N))],'LineWidth',1.5,...
        'Color',[0.5 0.5 0.5]);
end
axis equal; axis tight;
xlabel('$z$ [mm]','Interpreter','latex');   ylabel('$y$ [mm]','Interpreter','latex');
h1 = colorbar('EastOutside');
set(get(h1,'ylabel'),'String','$u$ [m/s]','Interpreter','latex'); 
set(h1,'TickLabelInterpreter','latex');
ax = gca; axpos = ax.Position; cpos = h1.Position; % Get position info
cpos(3) = 0.5*cpos(3); h1.Position = cpos; ax.Position = axpos; % Change colorbar to half width
set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on');
set(findall(gcf,'type','axes'),'FontSize',10);
set(findall(gcf,'type','text'),'FontSize',10);
set(findall(gcf,'type','colorbar'),'FontSize',10);
tightfig2_1(gcf,-0.6,1);

Fv = scatteredInterpolant(uvwdata(:,3),uvwdata(:,2),uvwdata(:,5),'linear','nearest');
v = Fv(qx,qy);
v(isnan(v))=0;

Fw = scatteredInterpolant(uvwdata(:,3),uvwdata(:,2),uvwdata(:,6),'linear','nearest');
w = Fw(qx,qy);
w(isnan(w))=0;

% Fk = scatteredInterpolant(kdata(:,3),kdata(:,2),kdata(:,4),'linear','nearest');
Fk = scatteredInterpolant(kdata(BLi2,3),kdata(BLi2,2),kdata(BLi2,4),'linear','nearest');
k1 = Fk(qx,qy);
k1(isnan(k1))=0;
figure; set(gcf,'units','inches','Position',[6 5 3.25 3]);
contourf(1000*z,1000*y,k1,'edgecolor','none'); hold on;
for i=1:length(z_uni) % Plot measurement planes
    plot(1000*[z_uni(i) z_uni(i)],1000*[min(y_o((i-1)*N+1:i*N)) max(y_o((i-1)*N+1:i*N))],'LineWidth',1.5,...
        'Color',[0.5 0.5 0.5]);
end
axis equal; axis tight;
xlabel('$z$ [mm]','Interpreter','latex');   ylabel('$y$ [mm]','Interpreter','latex');
h1 = colorbar('EastOutside');
set(get(h1,'ylabel'),'String','$k$ [m$^2$/s$^2$]','Interpreter','latex');  
set(h1,'TickLabelInterpreter','latex');
ax = gca; axpos = ax.Position; cpos = h1.Position; % Get position info
cpos(3) = 0.5*cpos(3); h1.Position = cpos; ax.Position = axpos; % Change colorbar to half width
set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on');
set(findall(gcf,'type','axes'),'FontSize',10);
set(findall(gcf,'type','text'),'FontSize',10);
set(findall(gcf,'type','colorbar'),'FontSize',10);
tightfig2_1(gcf,-0.6,1);

% Feps = scatteredInterpolant(epsdata(:,3),epsdata(:,2),epsdata(:,4),'linear','nearest');
Feps = scatteredInterpolant(epsdata(BLi2,3),epsdata(BLi2,2),epsdata(BLi2,4),'linear','nearest');
eps1 = Feps(qx,qy);
eps1(isnan(eps1))=0;
figure; set(gcf,'units','inches','Position',[10 4 3.25 3]);
contourf(1000*z,1000*y,eps1,'edgecolor','none'); hold on;
for i=1:length(z_uni) % Plot measurement planes
    plot(1000*[z_uni(i) z_uni(i)],1000*[min(y_o((i-1)*N+1:i*N)) max(y_o((i-1)*N+1:i*N))],'LineWidth',1.5,...
        'Color',[0.5 0.5 0.5]);
end
axis equal; axis tight;
xlabel('$z$ [mm]','Interpreter','latex');   ylabel('$y$ [mm]','Interpreter','latex');
h1 = colorbar('EastOutside');
set(get(h1,'ylabel'),'String','$\epsilon$ [m$^2$/s$^3$]','Interpreter','latex');
set(h1,'TickLabelInterpreter','latex');
ax = gca; axpos = ax.Position; cpos = h1.Position; % Get position info
cpos(3) = 0.5*cpos(3); h1.Position = cpos; ax.Position = axpos; % Change colorbar to half width
set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on');
set(findall(gcf,'type','axes'),'FontSize',10);
set(findall(gcf,'type','text'),'FontSize',10);
set(findall(gcf,'type','colorbar'),'FontSize',10);
tightfig2_1(gcf,-0.6,1);

% Fomega = scatteredInterpolant(omega(:,3),omega(:,2),omega(:,4),'linear','nearest');
Fomega = scatteredInterpolant(omega(BLi2,3),omega(BLi2,2),omega(BLi2,4),'linear','nearest');
omega1 = Fomega(qx,qy);
omega1(isnan(omega1))=0;
figure; set(gcf,'units','inches','Position',[14 3 3.25 3]);
contourf(1000*z,1000*y,omega1,'edgecolor','none'); hold on;
for i=1:length(z_uni) % Plot measurement planes
    plot(1000*[z_uni(i) z_uni(i)],1000*[min(y_o((i-1)*N+1:i*N)) max(y_o((i-1)*N+1:i*N))],'LineWidth',1.5,...
        'Color',[0.5 0.5 0.5]);
end
axis equal; axis tight;
xlabel('$z$ [mm]','Interpreter','latex');   ylabel('$y$ [mm]','Interpreter','latex');
h1 = colorbar('EastOutside');
set(get(h1,'ylabel'),'String','$\omega$ [1/s]','Interpreter','latex'); set(h1,'TickLabelInterpreter','latex');
ax = gca; axpos = ax.Position; cpos = h1.Position; % Get position info
cpos(3) = 0.5*cpos(3); h1.Position = cpos; ax.Position = axpos; % Change colorbar to half width
set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on');
set(findall(gcf,'type','axes'),'FontSize',10);
set(findall(gcf,'type','text'),'FontSize',10);
set(findall(gcf,'type','colorbar'),'FontSize',10);
tightfig2_1(gcf,-0.6,1);

%% Manually set the velocity profile (if Manual == 1)
if Manual == 1
    u_max = 1;  % Maximum u velocity [m/s]
    u_min = -1; % Minimum u velocity [m/s]
    u_mean = 0.5;   % Mean u velocity [m/s]
    u_vector = (u_min:(u_max-u_min)/(N-1):u_max)' + u_mean;  % Linear profile from u_max to u_min
    u = repmat(u_vector,1,N);   % Copy the column vector to a 2D matrix
    
    if Opposed == 1 % Change the sign so the wall-side has a negative sign
        u = -u;
    end
    
    v = zeros(N,N);
    w = zeros(N,N);
    
    figure; contourf(1000*z,1000*y,u,'edgecolor','none');
    h1 = colorbar('EastOutside');
    set(get(h1,'ylabel'),'String','u [m/s]')
    axis equal; axis tight; xlabel('z [mm]');   ylabel('y [mm]');   title('Manual u Assignment');
end

%% Now we want to make the left and right sides the same as the top side

% k = k1;
% % Adding the original k to the orthogonal k increases the values slightly but helps the nature of the mapping
% if Opposed == 1 % Top wall at top of array (small indeces)
%     k(:,1:Npart) = k1(:,1:Npart) + k1(1:Npart,:)';  % Write top wall values to right
%     k(:,end-Npart+1:end) = k1(:,end-Npart+1:end) + rot90(k1(1:Npart,:)',2);  % Write top wall values to left
% else    % Top wall at bottom of array (large indeces)
%     k(:,1:Npart) = k1(:,1:Npart) + fliplr(k1(end-Npart+1:end,:)');  % Write top wall values to right
%     k(:,end-Npart+1:end) = k1(:,end-Npart+1:end) + k1(end-Npart+1:end,:)';  % Write top wall values to left
% end
% figure; contourf(1000*z,1000*y,k,'edgecolor','none');
% h1 = colorbar('EastOutside');
% set(get(h1,'ylabel'),'String','k [m^2/s^2]')
% axis equal; axis tight; xlabel('z [mm]');   ylabel('y [mm]');
% title('k with left and right sides same as top');
% 
% eps = eps1;
% if Opposed == 1 % Top wall at top of array (small indeces)
%     eps(:,1:Npart) = eps1(:,1:Npart) + eps1(1:Npart,:)';  % Write top wall values to right
%     eps(:,end-Npart+1:end) = eps1(:,end-Npart+1:end) + rot90(eps1(1:Npart,:)',2);  % Write top wall values to left
% else    % Top wall at bottom of array (large indeces)
%     eps(:,1:Npart) = eps1(:,1:Npart) + fliplr(eps1(end-Npart+1:end,:)');  % Write top wall values to right
%     eps(:,end-Npart+1:end) = eps1(:,end-Npart+1:end) + eps1(end-Npart+1:end,:)';  % Write top wall values to left
% end
% figure; contourf(1000*z,1000*y,eps,'edgecolor','none');
% h1 = colorbar('EastOutside');
% set(get(h1,'ylabel'),'String','\epsilon [m^2/s^3]')
% axis equal; axis tight; xlabel('z [mm]');   ylabel('y [mm]');
% title('\epsilon with left and right sides same as top');
% 
% omega_d = omega1;
% if Opposed == 1 % Top wall at top of array (small indeces)
% % Write top wall values to right
%     omega_d(:,1:Npart) = omega1(:,1:Npart) + omega1(1:Npart,:)';  
% % Write top wall values to left
%     omega_d(:,end-Npart+1:end) = omega1(:,end-Npart+1:end) + rot90(omega1(1:Npart,:)',2);  
% else    % Top wall at bottom of array (large indeces)
% % Write top wall values to right
%     omega_d(:,1:Npart) = omega1(:,1:Npart) + fliplr(omega1(end-Npart+1:end,:)');  
% % Write top wall values to left
%     omega_d(:,end-Npart+1:end) = omega1(:,end-Npart+1:end) + omega1(end-Npart+1:end,:)';  
% end
% figure; contourf(1000*z,1000*y,omega_d,'edgecolor','none');
% h1 = colorbar('EastOutside');
% set(get(h1,'ylabel'),'String','\omega [1/s]')
% axis equal; axis tight; xlabel('z [mm]');   ylabel('y [mm]');
% title('\omega with left and right sides same as top');

%% Enforce data = 0 on the edges

u(1,:) = 0; u(N,:) = 0; u(:,1) = 0; u(:,N) = 0;
v(1,:) = 0; v(N,:) = 0; v(:,1) = 0; v(:,N) = 0;
w(1,:) = 0; w(N,:) = 0; w(:,1) = 0; w(:,N) = 0;
k1(1,:) = 0;    k1(N,:) = 0;    k1(:,1) = 0;    k1(:,N) = 0;
eps1(1,:) = 0;  eps1(N,:) = 0;  eps1(:,1) = 0;  eps1(:,N) = 0;
omega1(1,:) = 0;    omega1(N,:) = 0;    omega1(:,1) = 0;    omega1(:,N) = 0;

%% Write the files

if Export == 1
    Vel=zeros(N*N,6);
    epsilon=zeros(N*N,4);
    turb=zeros(N*N,4);
    omega=zeros(N*N,4);
    
    % Arrange 2D data into output form of X, Y, Z, data,...
    count=0;
    for i=1:N
        for j=1:N
            count=count+1;
            Vel(count,:)=[0,y(i),z(j),u(i,j),v(i,j),w(i,j)];
            turb(count,:)=[0,y(i),z(j),k1(i,j)];
            epsilon(count,:)=[0,y(i),z(j),eps1(i,j)];
            omega(count,:)=[0,y(i),z(j),omega1(i,j)];
        end
    end
    
    % Write the velocity inlet file
    header='X,Y,Z,u,v,w,';
    outid = fopen('Inlet-uvw-interp.csv', 'w+');
    fprintf(outid, '%s', header);
    fprintf(outid, '\n');
    fclose(outid);
    dlmwrite('Inlet-uvw-interp.csv',Vel,'delimiter',',','precision',4,'-append')
    
    % Write the k file for the inlet
    header='X,Y,Z,k,';
    outid3 = fopen('Inlet-k-interp.csv', 'w+');
    fprintf(outid3, '%s', header);
    fprintf(outid3, '\n');
    fclose(outid3);
    dlmwrite('Inlet-k-interp.csv',turb,'delimiter',',','precision',4,'-append')
    
    % Write the epsilon file for the inlet
    header='X,Y,Z,epsilon,';
    outid2 = fopen('Inlet-epsilon-interp.csv', 'w+');
    fprintf(outid2, '%s', header);
    fprintf(outid2, '\n');
    fclose(outid2);
    dlmwrite('Inlet-epsilon-interp.csv',epsilon,'delimiter',',','precision',4,'-append')
    
    % Write the omega files for the inlet
    header='X,Y,Z,w,';
    outid4 = fopen('Inlet-w-interp.csv', 'w+');     % Omega 1 formulation (w = e/k)
    fprintf(outid4, '%s', header);
    fprintf(outid4, '\n');
    fclose(outid4);
    dlmwrite('Inlet-w-interp.csv',omega,'delimiter',',','precision',4,'-append')
end

toc;