clear all; close all; clc; dbstop if error

%% Plot Velocity SRQ Data from PIV and CFD
% This code will read in PIV data from Stats.m files as well as CFD data from *.csv files and make comparison
% plots, including uncertainty on both.  It will have the option to make a movie with single profiles or line
% plots with several profiles at different times.
%
% Blake Lance, 30 January 2015

% Version 2, 5 February
%   Now obtains CFD results from the output of TransientCFD_GCI.m code, including uncertainty
% Version 2_1, 19 February, Added option to combine u and k plots when MovPlot = 1
% Version 2_2, 20 February, Added compatibility for MATLAB 2014b
%                           Now only plots required figures
% Version 2_3, 2 March, Added option to export experimental SRQ data to csv files
% Version 2_4, 3 March, Added export of PIV data from Cam1 (ZoomedIn)
% Version 2_5, 11 March, Added double y plot of CFD HF and PIV shear
% Version 2_6, 5 May, Investigating PIV scale error
% Version 2_7, 20 August, Added option to include CFD results in plots
% Version 2_8, 20 August, Added option to plot u'u' instead of k when CFD results are not included (u'u' is
%       not available from CFD results)
%   Added overlines to u and k on plots

%% User Inputs
MovPlot = 1;   % Enter 0 for Movie or 1 for Plot
Combine = 1;    % Whether to combine the plots into two figures when MovPlot = 1
CFD = 0;        % Whether to include CFD results in plots
plotk = 0;      % Whether to plot k or u'u' (only active when CFD == 0)
Export = 0;     % Whether to export experimental SRQ data to csv files
Nth_y = 20;      % Plot every Nth uncertainty band in space
Nth_t = 20;      % Plot every Nth_t curve in time (should be 20 for accurate legend time stamps)
Nth_t_HF_Shear = 4;   % Plot every Nth_HF_t heat flux uncertainty band in time
Opposed = 1;    % Whether the case is Opposed
t_offset = -1.8;% Time offset [s]
Nphases = 101;
T0 = 24.124 + 273.15;   % Room temperature [K]
P = 87248; % Atmospheric pressure [Pa]
phi0 = 0.10777;  % Relative humidity at room temperature
yp = 5;     % Maximum y+ value for linear fit
Fit = 10;   % The initial number of near-wall velocity data to fit for shear calc

if exist('TransPlotsSelection.mat','file')
    load('TransPlotsSelection.mat');
else
    % Have user specify where the PIV results are located
    CaseDir = uigetfile_n_dir('H:\8_6_x-Transient-HFS1\8_6 - Organized-Cam2\',...
        'Select all the data set folders from 8_6');
    [StatsFile,StatsDir] = uigetfile('*Stats.mat','Select the first Stats file',CaseDir{1});
    % Have user specify where the CFD results are located
    SimDir = uigetdir('F:\TMCWT CFD\Transient\RealBCs','Select the Simulation Folder');
    HF_Dir = 'H:\8_x_x-Transient-BCs\HeatFlux.csv';
    TempDir = 'H:\8_x_x-Transient-BCs\HeatedWallTemp.csv';
    save('TransPlotsSelection.mat','CaseDir','StatsFile','StatsDir','SimDir','HF_Dir','TempDir');
end

% Load CFD results with uncertainties
load('TransientCFD_GCI_Output.mat');

%% Prepare Parameters

tic;

CaseDir = repmat(CaseDir,[3,1]); % Copy paths
CaseDir(2,:) = strrep(CaseDir(2,:),'8_6','8_7');% Find and Replace Serial Numbers
CaseDir(3,:) = strrep(CaseDir(3,:),'8_6','8_8');% Find and Replace Serial Numbers
CaseDir(2,:) = strrep(CaseDir(2,:),'HFS1','HFS2');% Find and Replace Serial Numbers
CaseDir(3,:) = strrep(CaseDir(3,:),'HFS1','HFS3');% Find and Replace Serial Numbers
StatsFile = repmat(StatsFile,[3,1]); % Copy paths
StatsFile(2,:) = strrep(StatsFile(2,:),'8_6','8_7'); % Find and Replace Serial Numbers
StatsFile(3,:) = strrep(StatsFile(3,:),'8_6','8_8'); % Find and Replace Serial Numbers
% Copy paths and convert to cell strings of different lengths can be used
StatsDir = cellstr(repmat(StatsDir,[3,1]));
StatsDir(2,:) = strrep(StatsDir(2,:),'8_6','8_7'); % Find and Replace Serial Numbers
StatsDir(3,:) = strrep(StatsDir(3,:),'8_6','8_8'); % Find and Replace Serial Numbers
StatsDir(2,:) = strrep(StatsDir(2,:),'HFS1','HFS2'); % Find and Replace Serial Numbers
StatsDir(3,:) = strrep(StatsDir(3,:),'HFS1','HFS3'); % Find and Replace Serial Numbers
% Case 8_8 has a different path
StatsDir(3,:) = strrep(StatsDir(3,:),'PIV_MP','SubOverTimeMin_sL=all_NormByAvgRms\PIV_MP'); 

SlashLoc = strfind(CaseDir{1},'\'); % Find the locations of '\' in the data sets
SlashLoc = SlashLoc(end);   % Use the last one
phase = CaseDir{1};
phase = num2str(phase(SlashLoc+1:end));   % Get the data set name
j0 = str2double(phase); % The actual index the PIV starts
t = linspace(t_offset,20+t_offset,Nphases); % Define the time vector starting with thermal BCs
l0 = find(t==0); % This is the index where t=0

if MovPlot == 0
    % Prepare video
    writerObj = VideoWriter('TransSRQ-u-k.mp4','MPEG-4');
    writerObj.FrameRate = 5;
    open(writerObj);
    
    % Prepare Figure(1)
    figure(1); set(gcf,'Position',[1 1 1080 1080],'ToolBar','none','MenuBar','none');
    set(gcf,'Renderer','zbuffer');
else
    if Combine ~= 1
        if Nth_t > 1
            Figs = 1:2;
            figure(1); set(gcf,'units','inches','Position',[2 2 3.25 6]);
            figure(2); set(gcf,'units','inches','Position',[6 2 3.25 6]);          
        else
            Figs = 3:4;
            figure(3); set(gcf,'units','inches','Position',[10 2 3.25 6]);
            figure(4); set(gcf,'units','inches','Position',[14 2 3.25 6]);
        end
    else
        if Nth_t > 1
            Figs = 1:1;
            figure(1); set(gcf,'units','inches','Position',[2 2 6.5 6]);
        else
            Figs = 2:2;
            figure(2); set(gcf,'units','inches','Position',[10 2 6.5 6]);
        end
    end
end

%% Import Heat Flux Data

% Import experimental data
HFdata = importdata(HF_Dir);
x_HF = HFdata.data(:,1); % The x positions of heat flux measurements [m]
HF = HFdata.data(:,find(strcmp(HFdata.colheaders,['q"(W/m^2)[t=',num2str(t(l0)),'s]'])):...
    find(strcmp(HFdata.colheaders,['q"(W/m^2)[t=',num2str(t(end)),'s]'])));
U_HF = HFdata.data(:,find(strcmp(HFdata.colheaders,['U(W/m^2)[t=',num2str(t(l0)),'s]'])):end);

% Assign CFD data
HF_CFD(:,l0:Nphases) = HF1;
U_HF_CFD(:,l0:Nphases) = GCI_fine_HF21;

%% Import HeatedWallTemp Data for mu

TempData = importdata(TempDir);
x_Temp = TempData.data(:,1); % The x positions of thermocouples
z_Temp_i = find(TempData.data(:,3) == 0); % Find the indeces where z == 0
Temp = interp1(unique(x_Temp),...
    TempData.data(z_Temp_i,find(strcmp(TempData.colheaders,['T(K)[t=',num2str(t(l0)),'s]'])):...
    find(strcmp(TempData.colheaders,['T(K)[t=',num2str(t(end)),'s]']))),x_HF);
U_Temp = interp1(unique(x_Temp),...
    TempData.data(z_Temp_i,find(strcmp(TempData.colheaders,['U(K)[t=',num2str(t(l0)),'s]'])):...
    find(strcmp(TempData.colheaders,['U(K)[t=',num2str(t(end)),'s]']))),x_HF);

%% Calculate Air Properties
% See Tsilingiris2007 and Melling97

% Saturation pressure at room temperature [Pa]
P_sat0 = 1000*( 1.901888 - 2.1424081E-1*(T0-273.15) + 1.2057211E-2*(T0-273.15)^2 - ...
    1.3867581E-4*(T0-273.15)^3 + 1.3888951E-6*(T0-273.15)^4 );
omega = 0.622*phi0*P_sat0/( P - phi0*P_sat0 ); % Specific humidity at any temperature
R_mix = 8.31434/((omega+1)/((omega/0.01802)+(1/0.02897))); % Compute the Mixture Gas Constant [m^2/(K*s^2)]
rho = P./(Temp*R_mix);

% Compute the Dynamic Viscosity using Sutherland's Law
a = 1.458E-6; b = 110.4;
mu = a*Temp.^(3/2)./( Temp + b ); % [kg/(m*s)]
U_mu = U_Temp*a.*sqrt(Temp).*(Temp+3*b)./(2*(Temp+b).^2);

%% Import Shear Data

% Import PIV data if it exists
PIV_Shear_exist = exist('TransPIV_Shear.mat','file');
if PIV_Shear_exist ~= 0 % It exists
    load('TransPIV_Shear.mat');
else
    dudy = zeros(3,Nphases);
    U_dudy = zeros(3,Nphases);
    Shear = zeros(3,Nphases);
    U_Shear = zeros(3,Nphases);
end

% Assign CFD data
Shear_CFD(:,l0:Nphases) = Shear1;
U_Shear_CFD(:,l0:Nphases) = GCI_fine_Shear21;

%% Import PIV and CFD Data and Plot in a Loop

leg_h1 = zeros(ceil((Nphases-l0)/Nth_t),1); % Initialize legend handle 1
leg_h2 = zeros(ceil((Nphases-l0)/Nth_t),1); % Initialize legend handle 2
leg_h3 = zeros(ceil((Nphases-l0)/Nth_t),1); % Initialize legend handle 3
count = 0;
for l=l0:Nth_t:Nphases % Loop through phases
    for m=1:size(CaseDir,1) % Loop through positions
        % Replace the case name for the current phase
        SlashLoc_new = strfind(CaseDir{m,l-j0+1},'\'); % Find the locations of '\' in the data sets
        SlashLoc_new = SlashLoc_new(end);   % Use the last one
        phase_new = CaseDir{m,l-j0+1};
        phase_new = num2str(phase_new(SlashLoc_new+1:end));   % Get the data set name
        
        StatsDir_new = strrep(StatsDir{m,:},CaseDir{m,1},CaseDir{m,l-j0+1});
        StatsFile_new = strrep(StatsFile(m,:),phase,phase_new);
        load([StatsDir_new,StatsFile_new]); % Load the data for the current phase
        
        %% Allocate and initialize
        
        w = size(u_ave,2);
        u2 = v_ave;  u2(isnan(u2)) = 0;    % v_ave in DaVis is u in global coordinates
        v2 = u_ave;  v2(isnan(v2)) = 0;    % u_ave in DaVis is v in global coordinates
        U_u2 = Uv;   U_u2(isnan(U_u2)) = 0;    % The time averaged u (global) velocity uncertainty
        U_v2 = Uu;   U_v2(isnan(U_v2)) = 0;    % The time averaged v (global) velocity uncertainty
        
        uu2 = vvt;   uu2(isnan(uu2)) = 0;  % u'u' in global coordinates
        vv2 = uut;   vv2(isnan(vv2)) = 0;  % v'v' in global coordinates
        uv2 = uvt;   uv2(isnan(uv2)) = 0;  % u'v' in global coordinates
        
        U_uu_p2 = Uvv_plus;  U_uu_p2(isnan(U_uu_p2)) = 0;  % Plus uncertainty of u'u' in global coordinates
        U_uu_m2 = Uvv_minus; U_uu_m2(isnan(U_uu_m2)) = 0;  % Minus uncertainty of u'u' in global coordinates
        U_vv_p2 = Uuu_plus;  U_vv_p2(isnan(U_vv_p2)) = 0;  % Plus uncertainty of v'v' in global coordinates
        U_vv_m2 = Uuu_minus; U_vv_m2(isnan(U_vv_m2)) = 0;  % Minus uncertainty of v'v' in global coordinates
        U_uv_p2 = Uuv_plus;  U_uv_p2(isnan(U_uv_p2)) = 0;  % Plus uncertainty of u'v' in global coordinates
        U_uv_m2 = Uuv_minus; U_uv_m2(isnan(U_uv_m2)) = 0;  % Minus uncertainty of u'v' in global coordinates
        
        if Opposed == 1 % Change the signs
            u2 = -u2; uv2 = -uv2;
        end
        
        %% Adaptive trimming to masked area
        [imin,imax,jmin,jmax] = TrimToMask(u2);
        
        xlower = X(1,jmin);
        xupper = X(1,jmax);
        ylower = Y(imin,1);
        if imax==0; imax = 1; end;
        yupper = Y(imax,1);
        
        Xshift = X - xlower;    % Shift x_2D array so y=0 is at the left
        xupper = xupper - xlower;   % Just changed this order
        xlower = xlower - xlower;        
                
        %% Define proloc if it isn't defined
        if exist('proloc','var') ~= 1; proloc = floor(0.5*(imin-imax+1)); end
        
        %% Flip Xshift if Opposed==1, also shift values if the masked area isn't symmetric
        if Opposed==1
            dx = abs( X(1,1) - X(1,2) );  % Spacing between vectors in mm given by DaVis
            Xshift = fliplr(Xshift);
            TrimR = w - jmax + 1;   TrimL = jmin;
            % Add or subtract values so assymetric Xshift will have consistent values
            Xshift = Xshift - dx*(TrimR - TrimL);
        end
        
        %% Calculate the Turbulence Kinetic Energy
        k = 0.5*( uu2(proloc,:) + vv2(proloc,:) + vv2(proloc,:) );
        U_k_p = 0.5*sqrt( U_uu_p2(proloc,:).^2 + U_vv_p2(proloc,:).^2 + U_vv_p2(proloc,:).^2 );
        U_k_m = 0.5*sqrt( U_uu_m2(proloc,:).^2 + U_vv_m2(proloc,:).^2 + U_vv_m2(proloc,:).^2 );
        for i=1:w      % Set the minus uncertainty of k to k so the uncertainty bands stay positive
            if U_k_m(i) > k(i);   U_k_m(i) = k(i);  end;
        end
        
        %% Recreate Xshift for PIV to better match CFD
        Xshift_CFD = linspace(1000*max(y1),Xshift(1,jmax),jmax-jmin+1);
        
        %% Scale factor of PIV to CFD
        C_PIV = ( Xshift(1,1) - Xshift(1,length(Xshift_CFD)) )/max(Xshift_CFD);
        if l == l0
            C_PIV
        end
        
        %% Prepare PIV Data from Cam2 for csv File Export
        
        if Export == 1
            if l==l0 && m == 1; uvw_header2 = 'X(m),Y(m),Z(m)'; end
            if m ==1
                uvw_header2 = [uvw_header2,',',...
                    'u(m/s)[t=',num2str(t(l)),'s],','v(m/s)[t=',num2str(t(l)),'s],',...
                    'u''u''(m^2/s^2)[t=',num2str(t(l)),'s],','v''v''(m^2/m^2)[t=',num2str(t(l)),'s],',...
                    'u''v''(m^2/m^2)[t=',num2str(t(l)),'s],',...
                    'Uu(m/s)[t=',num2str(t(l)),'s],','Uv(m/s)[t=',num2str(t(l)),'s],',...
                    'Uuup(m^2/s^2)[t=',num2str(t(l)),'s],','Uuum(m^2/s^2)[t=',num2str(t(l)),'s],',...
                    'Uvvp(m^2/s^2)[t=',num2str(t(l)),'s],','Uvvm(m^2/s^2)[t=',num2str(t(l)),'s],',...
                    'Uuvp(m^2/s^2)[t=',num2str(t(l)),'s],','Uuvm(m^2/s^2)[t=',num2str(t(l)),'s]'];
            end
            
            rows = (m-1)*(jmax-jmin+1)+1:(m-1)*(jmax-jmin+1)+jmax-jmin+1;
            if l==l0 % Add X,Y,Z values before data
                uvw_Export2(rows,1:16) = ...
                    [repmat(x_HF(m),jmax-jmin+1,1),0.001*Xshift(proloc,jmin:jmax)',...
                    zeros(jmax-jmin+1,1),...
                    u2(proloc,jmin:jmax)',v2(proloc,jmin:jmax)',...
                    uu2(proloc,jmin:jmax)',vv2(proloc,jmin:jmax)',...
                    uv2(proloc,jmin:jmax)',...
                    U_u2(proloc,jmin:jmax)',U_v2(proloc,jmin:jmax)',...
                    U_uu_p2(proloc,jmin:jmax)',U_uu_m2(proloc,jmin:jmax)',...
                    U_vv_p2(proloc,jmin:jmax)',U_vv_m2(proloc,jmin:jmax)',...
                    U_uv_p2(proloc,jmin:jmax)',U_uv_m2(proloc,jmin:jmax)'];
            else % Only add data
                uvw_Export2(rows,4+13*(l-l0):3+13*(l-l0+1)) = ...
                    [u2(proloc,jmin:jmax)',v2(proloc,jmin:jmax)',...
                    uu2(proloc,jmin:jmax)',vv2(proloc,jmin:jmax)',...
                    uv2(proloc,jmin:jmax)',...
                    U_u2(proloc,jmin:jmax)',U_v2(proloc,jmin:jmax)',...
                    U_uu_p2(proloc,jmin:jmax)',U_uu_m2(proloc,jmin:jmax)',...
                    U_vv_p2(proloc,jmin:jmax)',U_vv_m2(proloc,jmin:jmax)',...
                    U_uv_p2(proloc,jmin:jmax)',U_uv_m2(proloc,jmin:jmax)'];
            end
        end
                
        %% Calculate PIV Shear
        
        if PIV_Shear_exist == 0 || Export == 1
            % Import data from Cam1 (zoomed-in data)
            StatsDir_new2 = strrep(StatsDir_new,'Cam2','Cam1');
            if m == 3
                StatsDir_new2 = strrep(StatsDir_new2,'PIV_MP','ShiftCorr_SubOverTimeMin_sL=5\PIV_MP');
                StatsDir_new2 = strrep(StatsDir_new2,'SubOverTimeMin_sL=all_NormByAvgRms','');
            else
                StatsDir_new2 = strrep(StatsDir_new2,'PIV_MP','ShiftCorr_SubOverTimeMin_sL=all\PIV_MP');
            end
            StatsFile_new2 = strrep(StatsFile_new,'Cam2','Cam1');
%             load([StatsDir_new2,StatsFile_new2],'X','v_ave'); % Load data for the current phase
            load([StatsDir_new2,StatsFile_new2]); % Load data for the current phase

            [imin2,imax2,jmin2,jmax2] = TrimToMask(v_ave);
            jmin2 = jmin2 + 1;  % Since good data go to edge, offset by 1
            xlower2 = X(1,jmin2);
            Xshift2 = X - xlower2;    % Shift x_2D array so y=0 is at the left
            if exist('proloc2','var') ~= 1; proloc2 = floor(0.5*(imin2-imax2+1)); end
            if Opposed == 1; v_ave = -v_ave; end
            w2 = size(v_ave,2);
            
            % Flip Xshift if Opposed==1, also shift values if the masked area isn't symmetric
            if Opposed==1
                dx2 = abs( X(1,1) - X(1,2) );  % Spacing between vectors in mm given by DaVis
                Xshift2 = fliplr(Xshift2);
                TrimR2 = w2 - jmax2 + 1;   TrimL2 = jmin2;
                % Add or subtract values so assymetric Xshift will have consistent values
                Xshift2 = Xshift2 - dx2*(TrimR2 - TrimL2);
            end
        end
        if PIV_Shear_exist == 0
            % Calculate velocity gradient and its uncertainty
            [ y0, FitOut, dudy(m,l), U_dudy(m,l) ] = PIV_dudy( Xshift2(proloc2,jmin2:jmax2), ...
                v_ave(proloc2,jmin2:jmax2),U_u2(proloc2,jmin2:jmax2), rho(m,l-l0+1), mu(m,l-l0+1), yp, Fit );
            
            % Plot velocity data and fit
            if m == 2
                figure(5); 
%                 count = count + 1;
                if count == 0; count = 1; end
                set(gca,'ColorOrderIndex',count);
                leg_h3(count) = plot(Xshift2(proloc2,jmin2:jmax2),v_ave(proloc2,jmin2:jmax2),'LineWidth',1.5);
                hold on;
                set(gca,'ColorOrderIndex',count);
                % Appearantly MATLAB sets the width of the uncertainty bands to 2% of the x range, so use less
                errorbar(Xshift2(proloc2,275:2:jmax2),v_ave(proloc2,275:2:jmax2),...
                    U_u2(proloc2,275:2:jmax2),'LineStyle','none','LineWidth',1.5);
                xlabel('$y$ [mm]','Interpreter','latex'); 
                ylabel('$\overline{u}$ [m/s]','Interpreter','latex');
                if count == 5
                    leg_h3(count+1) = plot([Xshift2(proloc2,jmax2), Xshift2(proloc2,jmax2-FitOut+1)],...
                        [y0, y0-dudy(m,l)*(Xshift2(proloc2,jmax2)-Xshift2(proloc2,jmax2-FitOut+1))/1000],...
                        'Color',[0.25 0.25 0.25],'LineWidth',1.5);
                else
                    plot([Xshift2(proloc2,jmax2), Xshift2(proloc2,jmax2-FitOut+1)],...
                        [y0, y0-dudy(m,l)*(Xshift2(proloc2,jmax2)-Xshift2(proloc2,jmax2-FitOut+1))/1000],...
                        'Color',[0.25 0.25 0.25],'LineWidth',1.5);
                end
            end
            
            % Calculate shear and its uncertainty
            Shear(m,l) = mu(m,l-l0+1)*dudy(m,l);
            U_Shear(m,l) = sqrt( (dudy(m,l)*U_mu(m,l-l0+1))^2 + (mu(m,l-l0+1)*U_dudy(m,l))^2 );
            
            if l == Nphases && m == 3; save('TransPIV_Shear.mat','Shear','U_Shear'); end
        end
        
        %%  Prepare PIV Data from Cam1 for csv File Export
        
        if Export == 1
            u1 = v_ave;  u1(isnan(u1)) = 0;    % v_ave in DaVis is u in global coordinates
            v1 = u_ave;  v1(isnan(v1)) = 0;    % u_ave in DaVis is v in global coordinates
            U_u1 = Uv;   U_u1(isnan(U_u1)) = 0;    % The time averaged u (global) velocity uncertainty
            U_v1 = Uu;   U_v1(isnan(U_v1)) = 0;    % The time averaged v (global) velocity uncertainty
            
            uu1 = vvt;   uu1(isnan(uu1)) = 0;  % u'u' in global coordinates
            vv1 = uut;   vv1(isnan(vv1)) = 0;  % v'v' in global coordinates
            uv1 = uvt;   uv1(isnan(uv1)) = 0;  % u'v' in global coordinates
            
            U_uu_p1 = Uvv_plus;  U_uu_p1(isnan(U_uu_p1)) = 0;  % Plus uncertainty of u'u' in global coordinates
            U_uu_m1 = Uvv_minus; U_uu_m1(isnan(U_uu_m1)) = 0;  % Minus uncertainty of u'u' in global coordinates
            U_vv_p1 = Uuu_plus;  U_vv_p1(isnan(U_vv_p1)) = 0;  % Plus uncertainty of v'v' in global coordinates
            U_vv_m1 = Uuu_minus; U_vv_m1(isnan(U_vv_m1)) = 0;  % Minus uncertainty of v'v' in global coordinates
            U_uv_p1 = Uuv_plus;  U_uv_p1(isnan(U_uv_p1)) = 0;  % Plus uncertainty of u'v' in global coordinates
            U_uv_m1 = Uuv_minus; U_uv_m1(isnan(U_uv_m1)) = 0;  % Minus uncertainty of u'v' in global coordinates
            
            if Opposed == 1 % Change the signs
                uv1 = -uv1;
            end
            
            if l==l0 && m == 1; uvw_header1 = 'X(m),Y(m),Z(m)'; end
            if m ==1
                uvw_header1 = [uvw_header1,',',...
                    'u(m/s)[t=',num2str(t(l)),'s],','v(m/s)[t=',num2str(t(l)),'s],',...
                    'u''u''(m^2/s^2)[t=',num2str(t(l)),'s],','v''v''(m^2/m^2)[t=',num2str(t(l)),'s],',...
                    'u''v''(m^2/m^2)[t=',num2str(t(l)),'s],',...
                    'Uu(m/s)[t=',num2str(t(l)),'s],','Uv(m/s)[t=',num2str(t(l)),'s],',...
                    'Uuup(m^2/s^2)[t=',num2str(t(l)),'s],','Uuum(m^2/s^2)[t=',num2str(t(l)),'s],',...
                    'Uvvp(m^2/s^2)[t=',num2str(t(l)),'s],','Uvvm(m^2/s^2)[t=',num2str(t(l)),'s],',...
                    'Uuvp(m^2/s^2)[t=',num2str(t(l)),'s],','Uuvm(m^2/s^2)[t=',num2str(t(l)),'s]'];
            end
            
            rows = (m-1)*(jmax2-jmin2+1)+1:(m-1)*(jmax2-jmin2+1)+jmax2-jmin2+1;
            if l==l0 % Add X,Y,Z values before data
                uvw_Export1(rows,1:16) = [repmat(x_HF(m),jmax2-jmin2+1,1),...
                    0.001*Xshift2(proloc2,jmin2:jmax2)',zeros(jmax2-jmin2+1,1),...
                    u1(proloc2,jmin2:jmax2)',v1(proloc2,jmin2:jmax2)',...
                    uu1(proloc2,jmin2:jmax2)',vv1(proloc2,jmin2:jmax2)',...
                    uv1(proloc2,jmin2:jmax2)',...
                    U_u1(proloc2,jmin2:jmax2)',U_v1(proloc2,jmin2:jmax2)',...
                    U_uu_p1(proloc2,jmin2:jmax2)',U_uu_m1(proloc2,jmin2:jmax2)',...
                    U_vv_p1(proloc2,jmin2:jmax2)',U_vv_m1(proloc2,jmin2:jmax2)',...
                    U_uv_p1(proloc2,jmin2:jmax2)',U_uv_m1(proloc2,jmin2:jmax2)'];
            else % Only add data
                uvw_Export1(rows,4+13*(l-l0):3+13*(l-l0+1)) = ...
                    [u1(proloc2,jmin2:jmax2)',v1(proloc2,jmin2:jmax2)',...
                    uu1(proloc2,jmin2:jmax2)',vv1(proloc2,jmin2:jmax2)',...
                    uv1(proloc2,jmin2:jmax2)',...
                    U_u1(proloc2,jmin2:jmax2)',U_v1(proloc2,jmin2:jmax2)',...
                    U_uu_p1(proloc2,jmin2:jmax2)',U_uu_m1(proloc2,jmin2:jmax2)',...
                    U_vv_p1(proloc2,jmin2:jmax2)',U_vv_m1(proloc2,jmin2:jmax2)',...
                    U_uv_p1(proloc2,jmin2:jmax2)',U_uv_m1(proloc2,jmin2:jmax2)'];
            end
        end
        
        %% Generate Movie or Plot
        
        if MovPlot == 0 % If creating movie, add frames here
            % Plot u
            figure(1);
            subtightplot(3,2,2*m-1,[0,0.1],0.075,0.1); 
            set(gca,'ColorOrderIndex',1); plot(Xshift_CFD,-u2(proloc,jmin:jmax),'LineWidth',1.5); hold on;
            if CFD == 1
                set(gca,'ColorOrderIndex',2); plot(1000*y1,-u1all(m,:,l-l0+1),'-.','LineWidth',1.5);
            end
            set(gca,'ColorOrderIndex',1);
            errorbar(Xshift_CFD(1:Nth_y:end),-u2(proloc,jmin:Nth_y:jmax),U_u2(proloc,jmin:Nth_y:jmax),...
                'LineStyle','none','LineWidth',1.5);
            if CFD == 1
                set(gca,'ColorOrderIndex',2);
                errorbar(y3,-u1(m,:,l-l0+1),GCI_fine_u21(m,:,l-l0+1),'LineStyle','none','LineWidth',1.25);
            end
            axis([min(Xshift_CFD) max(Xshift_CFD) -5.1 1.25]);
            if l == l0 && m == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
            if l == l0; ylabel(['$-\overline{u}(x_',num2str(m),')$ [m/s]'],'Interpreter','latex'); end
            if m == 2; 
                if CFD == 1
                    hu = legend('PIV','CFD','location','southwest'); set(hu,'Interpreter','latex');
                end
            end
            if m ~= 3; set(gca,'XTickLabel',''); end
            set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on');
            set(gca,'nextplot','replacechildren');
            
            % Plot k
            subtightplot(3,2,2*m,[0,0.1],0.075,0.05); 
            set(gca,'ColorOrderIndex',1); 
            if plotk == 1
                plot(Xshift_CFD,k(jmin:jmax),'LineWidth',1.5); hold on;
                if CFD == 1
                    set(gca,'ColorOrderIndex',2); plot(1000*y1,k1all(m,:,l-l0+1),'-.','LineWidth',1.5);
                end
            else
                plot(Xshift_CFD,uu2(proloc,jmin:jmax),'LineWidth',1.5); hold on;
            end
            set(gca,'ColorOrderIndex',1);
            if plotk == 1
                errorbar(Xshift_CFD(1:Nth_y:end),k(jmin:Nth_y:jmax),U_k_m(jmin:Nth_y:jmax),...
                    U_k_p(jmin:Nth_y:jmax),'LineStyle','none','LineWidth',1.5);
                if CFD == 1
                    set(gca,'ColorOrderIndex',2); errorbar(y3,k1(m,:,l-l0+1),GCI_fine_k21(m,:,l-l0+1),...
                        'LineStyle','none','LineWidth',1.25);
                end
            else
                errorbar(Xshift_CFD(1:Nth_y:end),uu2(proloc,jmin:Nth_y:jmax),...
                    U_uu_m2(proloc,jmin:Nth_y:jmax),U_uu_p2(proloc,jmin:Nth_y:jmax),...
                    'LineStyle','none','LineWidth',1.5);
            end
            axis([min(Xshift_CFD) max(Xshift_CFD) 0 0.175]);
            if l == l0 && m == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
            if l == l0; 
                if plotk == 1
                    ylabel(['$\overline{k}(x_',num2str(m),')$ [m$^2$/s$^2$]'],'Interpreter','latex'); 
                else
                    ylabel(['$\overline{u''u''}(x_',num2str(m),')$ [m$^2$/s$^2$]'],'Interpreter','latex'); 
                end
            end
            if m == 2; 
                if CFD == 1
                    hk = legend('PIV','CFD','location','northeast'); set(hk,'Interpreter','latex');
                end
            end
            if m ~= 3; set(gca,'XTickLabel',''); end
            set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on');
            set(gca,'nextplot','replacechildren');

            % Prepare Figure(1) for movie
            h = text(-36,0.54,['$t$ = ',num2str(t(l),'%5.1f'),' s']);
            set(h,'background','w','edgecolor','k','HorizontalAlignment','center','Interpreter','latex');
            set(findall(gcf,'type','axes'),'FontWeight','normal','FontSize',16,'FontName','Times New Roman');
            set(findall(gcf,'type','text'),'FontWeight','normal','FontSize',16,'FontName','Times New Roman');
            if m == 3; writeVideo(writerObj,getframe(gcf)); end
        else % If creating plots, add them here
            if Combine ~= 1 % Make two separate figures
                if Nth_t > 1 % Plot u and k on two figures
                    % Plot u
                    figure(1);
                    if m == 1; count = count + 1; end
                    subtightplot(3,1,m,[0,0],0.05,0.05);
                    set(gca,'ColorOrderIndex',count);
                    leg_h1(count) = plot(Xshift_CFD,-u2(proloc,jmin:jmax),'LineWidth',1.25); hold on;
                    if CFD == 1
                        set(gca,'ColorOrderIndex',count);
                        leg_h2(count) = plot(1000*y1,-u1all(m,:,l-l0+1),'-.','LineWidth',1.25);
                    end
                    set(gca,'ColorOrderIndex',count);
                    errorbar(Xshift_CFD(1:Nth_y:end),-u2(proloc,jmin:Nth_y:jmax),...
                        U_u2(proloc,jmin:Nth_y:jmax),'LineStyle','none','LineWidth',1.25);
                    if CFD == 1
                        set(gca,'ColorOrderIndex',count);
                        errorbar(y3,-u1(m,:,l-l0+1),GCI_fine_u21(m,:,l-l0+1),'LineStyle','none','LineWidth',1.25);
                    end
                    axis([min(Xshift_CFD) max(Xshift_CFD) -5.1 1.25]);
                    if l == l0 && m == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
                    if l == l0; ylabel(['$-\overline{u}(x_',num2str(m),')$ [m/s]'],'Interpreter','latex'); end
                    if m ~= 3; set(gca,'XTickLabel',''); end
                    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')

                    % Plot k or u'u'
                    figure(2);
                    subtightplot(3,1,m,[0,0],0.05,0.05);
                    set(gca,'ColorOrderIndex',count);
                    if plotk == 1
                        plot(Xshift_CFD,k(jmin:jmax),'LineWidth',1.25); hold on;
                        if CFD == 1
                            set(gca,'ColorOrderIndex',count);
                            plot(1000*y1,k1all(m,:,l-l0+1),'-.','LineWidth',1.25);
                        end
                    else
                        plot(Xshift_CFD,uu2(proloc,jmin:jmax),'LineWidth',1.25); hold on;
                    end
                    set(gca,'ColorOrderIndex',count);
                    if plotk == 1
                        errorbar(Xshift_CFD(1:Nth_y:end),k(jmin:Nth_y:jmax),U_k_m(jmin:Nth_y:jmax),...
                            U_k_p(jmin:Nth_y:jmax),'LineStyle','none','LineWidth',1.25);
                        if CFD == 1
                            set(gca,'ColorOrderIndex',count);
                            errorbar(y3,k1(m,:,l-l0+1),GCI_fine_k21(m,:,l-l0+1),...
                                'LineStyle','none','LineWidth',1.25);
                        end
                    else
                        errorbar(Xshift_CFD(1:Nth_y:end),uu2(proloc,jmin:Nth_y:jmax),...
                            U_uu_m2(proloc,jmin:Nth_y:jmax),U_uu_p2(proloc,jmin:Nth_y:jmax),...
                            'LineStyle','none','LineWidth',1.25);
                    end
                    axis([min(Xshift_CFD) max(Xshift_CFD) 0 0.175]);
                    if l == l0 && m == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
                    if l == l0; 
                        if plotk == 1
                            ylabel(['$\overline{k}(x_',num2str(m),')$ [m$^2$/s$^2$]'],'Interpreter','latex'); 
                        else
                            ylabel(['$\overline{u''u''}(x_',num2str(m),')$ [m$^2$/s$^2$]'],...
                                'Interpreter','latex'); 
                        end
                    end
                    if m ~= 3; set(gca,'XTickLabel',''); end
                    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
                else
                    if l == Nphases % This only needs to be plotted at the end
                        % Plot heat flux
                        figure(3);
                        subtightplot(3,1,m,[0,0],0.05,0.05);
                        set(gca,'ColorOrderIndex',1);
                        plot(t(l0:end),HF(m,:),'LineWidth',1.5); hold on;
                        if CFD == 1
                            set(gca,'ColorOrderIndex',2);
                            plot(t(l0:end),HF_CFD(m,l0:end),'-.','LineWidth',1.5);
                        end
                        set(gca,'ColorOrderIndex',1);
                        errorbar(t(l0:Nth_t_HF_Shear:end),HF(m,1:Nth_t_HF_Shear:end),...
                            U_HF(m,1:Nth_t_HF_Shear:end),'LineStyle','none','LineWidth',1.5);
                        if CFD == 1
                            set(gca,'ColorOrderIndex',2);
                            errorbar(t(l0:Nth_t_HF_Shear:end),HF_CFD(m,l0:Nth_t_HF_Shear:end),...
                                U_HF_CFD(m,l0:Nth_t_HF_Shear:end),'LineStyle','none','LineWidth',1.5);
                        end
                        axis([t(l0) t(end) 0 1980]);
                        if m == 3; xlabel('$t$ [s]','Interpreter','latex'); end
                        ylabel(['$q''''(x_',num2str(m),')$ [W/m$^2$]'],'Interpreter','latex');
                        if m == 2;
                            if CFD == 1;
                                hHF = legend('HFS','CFD','location','northeast');
                                set(hHF,'Interpreter','latex');
                            end
                        end
                        if m ~= 3; set(gca,'XTickLabel',''); end
                        set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
                        
                        % Plot shear
                        figure(4)
                        subtightplot(3,1,m,[0,0],0.05,0.05);
                        set(gca,'ColorOrderIndex',1);
                        plot(t(l0:end),Shear(m,l0:end),'LineWidth',1.5); hold on;
                        if CFD == 1
                            set(gca,'ColorOrderIndex',2);
                            plot(t(l0:end),Shear_CFD(m,l0:end),'-.','LineWidth',1.5);
                        end
                        set(gca,'ColorOrderIndex',1);
                        errorbar(t(l0:Nth_t_HF_Shear:end),Shear(m,l0:Nth_t_HF_Shear:end),...
                            U_Shear(m,l0:Nth_t_HF_Shear:end),'LineStyle','none','LineWidth',1.5);
                        if CFD == 1
                            set(gca,'ColorOrderIndex',2);
                            errorbar(t(l0:Nth_t_HF_Shear:end),Shear_CFD(m,l0:Nth_t_HF_Shear:end),...
                                U_Shear_CFD(m,l0:Nth_t_HF_Shear:end),'LineStyle','none','LineWidth',1.5);
                        end
                        axis([t(l0) t(end) -0.015 0.059]);
                        if m == 3; xlabel('$t$ [s]','Interpreter','latex'); end
                        ylabel(['$\tau_s(x_',num2str(m),')$ [Pa]'],'Interpreter','latex');
                        if m == 2; 
                            if CFD == 1;
                                hShear = legend('PIV','CFD','location','northeast');
                                set(hShear,'Interpreter','latex');
                            end
                        end
                        if m ~= 3; set(gca,'XTickLabel',''); end
                        set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
                    end
                end
            else % Combine plots into one figure
                if Nth_t > 1 % Plot u and k on one figure
                    % Plot u
                    figure(1);
                    if m == 1; count = count + 1; end
                    subtightplot(3,2,2*m-1,[0,0.1],0.075,0.075);
                    set(gca,'ColorOrderIndex',count);
                    leg_h1(count) = plot(Xshift_CFD,-u2(proloc,jmin:jmax),'LineWidth',1.25); hold on;
                    if CFD == 1
                        set(gca,'ColorOrderIndex',count);
                        leg_h2(count) = plot(1000*y1,-u1all(m,:,l-l0+1),'-.','LineWidth',1.25);
                    end
                    set(gca,'ColorOrderIndex',count);
                    errorbar(Xshift_CFD(1:Nth_y:end),-u2(proloc,jmin:Nth_y:jmax),...
                        U_u2(proloc,jmin:Nth_y:jmax),'LineStyle','none','LineWidth',1.25);
                    if CFD == 1
                        set(gca,'ColorOrderIndex',count);
                        errorbar(y3,-u1(m,:,l-l0+1),GCI_fine_u21(m,:,l-l0+1),...
                            'LineStyle','none','LineWidth',1.25);
                    end
                    axis([min(Xshift_CFD) max(Xshift_CFD) -5.1 1.25]);
                    if l == l0 && m == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
                    if l == l0; ylabel(['$-\overline{u}(x_',num2str(m),')$ [m/s]'],'Interpreter','latex'); end
                    if m ~= 3; set(gca,'XTickLabel',''); end
                    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
                    
                    % Plot k or u'u'
                    subtightplot(3,2,2*m,[0,0.1],0.075,0.05);
                    set(gca,'ColorOrderIndex',count);
                    if plotk == 1
                        plot(Xshift_CFD,k(jmin:jmax),'LineWidth',1.25); hold on;
                        if CFD == 1
                            set(gca,'ColorOrderIndex',count);
                            plot(1000*y1,k1all(m,:,l-l0+1),'-.','LineWidth',1.25);
                        end
                    else
                        plot(Xshift_CFD,uu2(proloc,jmin:jmax),'LineWidth',1.25); hold on;
                    end
                    set(gca,'ColorOrderIndex',count);
                    if plotk == 1
                        errorbar(Xshift_CFD(1:Nth_y:end),k(jmin:Nth_y:jmax),U_k_m(jmin:Nth_y:jmax),...
                            U_k_p(jmin:Nth_y:jmax),'LineStyle','none','LineWidth',1.25);
                        if CFD == 1
                            set(gca,'ColorOrderIndex',count);
                            errorbar(y3,k1(m,:,l-l0+1),GCI_fine_k21(m,:,l-l0+1),...
                                'LineStyle','none','LineWidth',1.25);
                        end
                    else
                        errorbar(Xshift_CFD(1:Nth_y:end),uu2(proloc,jmin:Nth_y:jmax),...
                            U_uu_m2(proloc,jmin:Nth_y:jmax),U_uu_p2(proloc,jmin:Nth_y:jmax),...
                            'LineStyle','none','LineWidth',1.25);
                    end
                    axis([min(Xshift_CFD) max(Xshift_CFD) 0 0.175]);
                    if l == l0 && m == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
                    if l == l0;
                        if plotk == 1
                            ylabel(['$\overline{k}(x_',num2str(m),')$ [m$^2$/s$^2$]'],'Interpreter','latex');
                        else
                            ylabel(['$\overline{u''u''}(x_',num2str(m),')$ [m$^2$/s$^2$]'],...
                                'Interpreter','latex');
                        end
                    end
                    if m ~= 3; set(gca,'XTickLabel',''); end
                    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
                    
                    % Plot u from both FOVs
                    if PIV_Shear_exist == 0 || Export == 1
                        figure(7);
                        subtightplot(3,1,m,[0,0.1],0.075,0.075);
                        set(gca,'ColorOrderIndex',count);
                        plot(Xshift(proloc,jmin:jmax),-u2(proloc,jmin:jmax)); hold on;
                        set(gca,'ColorOrderIndex',count);
                        plot(Xshift2(proloc2,jmin2:jmax2),-v_ave(proloc2,jmin2:jmax2),'-.');
                        axis([Xshift2(1,jmax2) 10 -5.1 1.25]);
                        if l == l0 && m == 3; xlabel('$y$ [mm]','Interpreter','latex'); end
                        if l == l0; ylabel(['$-\overline{u}(x_',num2str(m),')$ [m/s]'],'Interpreter','latex'); end
                        if m ~= 3; set(gca,'XTickLabel',''); end
                        set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
                    end
                    
                else % Plot HF and shear on one figure
                    if l == Nphases % This only needs to be plotted at the end
                        % Plot heat flux
                        figure(2);
                        subtightplot(3,2,2*m-1,[0,0.1],0.075,0.1);
                        set(gca,'ColorOrderIndex',1);
                        plot(t(l0:end),HF(m,:),'LineWidth',1.5); hold on;
                        if CFD == 1
                            set(gca,'ColorOrderIndex',2);
                            plot(t(l0:end),HF_CFD(m,l0:end),'-.','LineWidth',1.5);
                        end
                        set(gca,'ColorOrderIndex',1);
                        errorbar(t(l0:Nth_t_HF_Shear:end),HF(m,1:Nth_t_HF_Shear:end),...
                            U_HF(m,1:Nth_t_HF_Shear:end),'LineStyle','none','LineWidth',1.5);
                        if CFD == 1
                            set(gca,'ColorOrderIndex',2);
                            errorbar(t(l0:Nth_t_HF_Shear:end),HF_CFD(m,l0:Nth_t_HF_Shear:end),...
                                U_HF_CFD(m,l0:Nth_t_HF_Shear:end),'LineStyle','none','LineWidth',1.5);
                        end
                        axis([t(l0) t(end) 0 2200]);
                        if m == 3; xlabel('$t$ [s]','Interpreter','latex'); end
                        ylabel(['$q''''(x_',num2str(m),')$ [W/m$^2$]'],'Interpreter','latex');
                        if m == 2; 
                            if CFD == 1
                                hHF = legend('HFS','CFD','location','northeast'); 
                                set(hHF,'Interpreter','latex');
                            end
                        end
                        if m ~= 3; set(gca,'XTickLabel',''); end
                        set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
                        
                        % Plot shear
                        subtightplot(3,2,2*m,[0,0.1],0.075,0.1);
                        set(gca,'ColorOrderIndex',1);
                        plot(t(l0:end),Shear(m,l0:end),'LineWidth',1.5); hold on;
                        if CFD == 1
                            set(gca,'ColorOrderIndex',2);
                            plot(t(l0:end),Shear_CFD(m,l0:end),'-.','LineWidth',1.5);
                        end
                        set(gca,'ColorOrderIndex',1);
                        errorbar(t(l0:Nth_t_HF_Shear:end),Shear(m,l0:Nth_t_HF_Shear:end),...
                            U_Shear(m,l0:Nth_t_HF_Shear:end),'LineStyle','none','LineWidth',1.5);
                        if CFD == 1
                            set(gca,'ColorOrderIndex',2);
                            errorbar(t(l0:Nth_t_HF_Shear:end),Shear_CFD(m,l0:Nth_t_HF_Shear:end),...
                                U_Shear_CFD(m,l0:Nth_t_HF_Shear:end),'LineStyle','none','LineWidth',1.5);
                        end
                        axis([t(l0) t(end) -0.015 0.068]);
                        if m == 3; xlabel('$t$ [s]','Interpreter','latex'); end
                        ylabel(['$\tau_s(x_',num2str(m),')$ [Pa]'],'Interpreter','latex');
                        if m == 2; 
                            if CFD == 1
                                hShear = legend('PIV','CFD','location','northeast');
                                set(hShear,'Interpreter','latex');
                            end
                        end
                        if m ~= 3; set(gca,'XTickLabel',''); end
                        set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
                        
                        % Plot normalized CFD HF and PIV Shear
                        figure(6);
                        set(gcf,'Units','inches','Position',[15 2 3.25 6]);
                        subtightplot(3,1,m,[0,0.1],0.075,0.05);
%                         [hAx,hLine1,hLine2] = plotyy(t(l0:end),HF_CFD(m,l0:end)/HF_CFD(m,l0),...
%                             t(l0:end),abs(Shear(m,l0:end)/Shear(m,l0)));
%                         axis(hAx(1),[t(l0) t(end) 0 1]);
%                         axis(hAx(2),[t(l0) t(end) 0 1]);
%                         ylabel(hAx(1),['CFD $q''''(x_',num2str(m),')$ [W/m$^2$]'],'Interpreter','latex');
%                         ylabel(hAx(2),['PIV $|\tau_s(x_',num2str(m),')|$ [Pa]'],'Interpreter','latex');
%                         if m == 3; xlabel('$t$ [s]','Interpreter','latex'); end
%                         if m ~= 3; set(gca,'XTickLabel',''); end
%                         set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
                        set(gca,'ColorOrderIndex',1);
                        plot(t(l0:end),abs(Shear(m,l0:end)/Shear(m,l0)),'-.','LineWidth',1.5); hold on;
                        set(gca,'ColorOrderIndex',2);
                        plot(t(l0:end),HF_CFD(m,l0:end)/HF_CFD(m,l0),'LineWidth',1.5);
                        set(gca,'ColorOrderIndex',1);
%                         errorbar(t(l0:Nth_t_HF_Shear:end),Shear(m,l0:Nth_t_HF_Shear:end),...
%                             U_Shear(m,l0:Nth_t_HF_Shear:end),'LineStyle','none','LineWidth',1.5);
%                         set(gca,'ColorOrderIndex',2);
%                         errorbar(t(l0:Nth_t_HF_Shear:end),HF_CFD(m,l0:Nth_t_HF_Shear:end)/HF_CFD(m,l0),...
%                             U_HF_CFD(m,l0:Nth_t_HF_Shear:end),'LineStyle','none','LineWidth',1.5);
                        axis([t(l0) t(end) 0 1]);
%                         ylabel(['CFD $q''''(x_',num2str(m),')$ [W/m$^2$]'],'Interpreter','latex');
                        if m == 3; xlabel('$t$ [s]','Interpreter','latex'); end
                        if m ~= 3; set(gca,'XTickLabel',''); end
                        set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on')
                        if m == 2; 
                            h6 = legend(['PIV $|\tau_s(x_',num2str(m),')/\tau_s(x_',num2str(m),',t=0)|$'],...
                                ['CFD $q''''(x_',num2str(m),')/q''''(x_',num2str(m),',t=0)$']); 
                            set(h6,'Interpreter','latex','Location','Best');
                        end
                    end
                end
            end
        end
    end
end

if MovPlot == 0
    close(writerObj);
else
    for i=Figs
        figure(i);
        if Nth_t > 1 % Add complex legends on u and k
            ah1 = axes('position',get(gca,'position'), 'visible','off');
            h1 = legend(ah1,leg_h1,'PIV ($t$=0s)','PIV ($t$=4s)','PIV ($t$=8s)',...
                'PIV ($t$=12s)','PIV ($t$=16s)');
            if CFD == 1 % have two legends, one for PIV and one for CFD
%                 ah1 = axes('position',get(gca,'position'), 'visible','off');
%                 h1 = legend(ah1,leg_h1,'PIV ($t$=0s)','PIV ($t$=4s)','PIV ($t$=8s)',...
%                     'PIV ($t$=12s)','PIV ($t$=16s)');
                ah2 = axes('position',get(gca,'position'), 'visible','off');
                h2 = legend(ah2,leg_h2,'CFD ($t$=0s)','CFD ($t$=4s)','CFD ($t$=8s)',...
                    'CFD ($t$=12s)','CFD ($t$=16s)');
%             else % have PIV-only legend in several columns
%                 ah1 = axes('position',get(gca,'position'), 'visible','off');
%                 leg_str = {'PIV ($t$=0s)','PIV ($t$=4s)','PIV ($t$=8s)','PIV ($t$=12s)','PIV ($t$=16s)'};
% %                 h1 = legend(ah1,leg_h1,'PIV ($t$=0s)','PIV ($t$=4s)','PIV ($t$=8s)',...
% %                     'PIV ($t$=12s)','PIV ($t$=16s)');
%                 [h1,h3,~,~] = columnlegend2(ah1,leg_h1,5, leg_str);
%                 set(findall(h3,'Type','text'),'Interpreter','latex','FontSize',10);
            end
            pos1 = get(h1,'Position');
            if CFD == 1; pos2 = get(h2,'Position'); end
            if Combine ~= 1
                set(h1,'Position',[0.08 0.96 pos1(3) pos1(4)],'Interpreter','latex');
                if CFD == 1; set(h2,'Position',[0.48 0.96 pos2(3) pos2(4)],'Interpreter','latex'); end
            else
                set(h1,'Position',[0.3 0.94 pos1(3) pos1(4)],'Interpreter','latex');
                if CFD == 1; set(h2,'Position',[0.5 0.94 pos2(3) pos2(4)],'Interpreter','latex'); end
            end
        end
        set(findall(gcf,'type','axes'),'FontSize',10,'FontName','Times New Roman');
        set(findall(gcf,'type','text'),'FontSize',10,'FontName','Times New Roman');
        tightfig2_1(gcf);
    end
end

if PIV_Shear_exist == 0
    % Fix up velocity fit plot
    figure(5);
    set(gcf,'Units','inches','Position',[14 6 5.5 3]);
    h3 = legend(leg_h3,'PIV ($t$=0s)','PIV ($t$=4s)','PIV ($t$=8s)',...
        'PIV ($t$=12s)','PIV ($t$=16s)','Linear Fit');
    pos3 = get(h3,'Position');
%     set(h3,'location','northoutside','Interpreter','latex','Position',[0.14 0.52 pos3(3) pos3(4)]);
    set(h3,'location','eastoutside','Interpreter','latex');
    set(findall(gcf,'type','axes'),'FontSize',10);
    set(findall(gcf,'type','text'),'FontSize',10);
    axis([0 2.75 -1 3.1]);
    set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on');
    tightfig2(gcf);
end

%% Export Experimental SRQ Results

if Export == 1 % Export results
    % Write uvw data for Cam1
    csvwrite_with_headers('TransientSRQ-uvw-Cam1.csv',uvw_Export1,strsplit(uvw_header1,','));
    % Write uvw data for Cam2
    csvwrite_with_headers('TransientSRQ-uvw-Cam2.csv',uvw_Export2,strsplit(uvw_header2,','));
        
    % Export Shear
    Shear_header = 'X(m),Y(m),Z(m)';
    for l=l0:Nth_t:Nphases
        Shear_header = [Shear_header,',','tau(Pa)[t=',num2str(t(l)),'s]'];
    end
    for l=l0:Nth_t:Nphases
        Shear_header = [Shear_header,',','U_tau(Pa)[t=',num2str(t(l)),'s]'];
    end
    % Append measurements
    ShearExport = [x_HF,zeros(3,2),Shear(:,l0:Nth_t:Nphases),U_Shear(:,l0:Nth_t:Nphases)];
    csvwrite_with_headers('TransientSRQ-Shear.csv',ShearExport,strsplit(Shear_header,','))
end

toc;

